#include "t3f/rng.h"
#include "data.h"
#include "game.h"
#include "game_sprite.h"
#include "effects.h"
#include "game_hazard.h"

void wl_clear_board_flags(void)
{
	int i, j;
	
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			wl_game.board[i][j].flags = 0;
			wl_game.board[i][j].used = false;
		}
	}
}

void wl_board_logic(void)
{
	int i, j, type;
	ALLEGRO_COLOR col_base[3];
	ALLEGRO_COLOR col_outrange[3];
	ALLEGRO_COLOR col_used[3];
	
	col_base[0] = al_map_rgba(162, 180, 213, 204);
	col_outrange[0] = wl_darken_color(col_base[0], 0.6);
	col_used[0] = wl_darken_color(col_base[0], 0.4);
	col_base[1] = al_map_rgba(255, 223, 96, 204);
	col_outrange[1] = wl_darken_color(col_base[1], 0.6);
	col_used[1] = wl_darken_color(col_base[1], 0.4);
	col_base[2] = al_map_rgba(255, 155, 155, 204);
	col_outrange[2] = wl_darken_color(col_base[2], 0.6);
	col_used[2] = wl_darken_color(col_base[2], 0.4);
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			
			if(wl_game.board[i][j].flags & WL_BOARD_FLAG_HAZARD)
			{
				type = 2;
			}
			else if(wl_game.board[i][j].flags & WL_BOARD_FLAG_POWERUP)
			{
				type = 1;
			}
			else
			{
				type = 0;
			}
			/* blank tile where player is standing */
			if(wl_game.player[0].x == j && wl_game.player[0].y == i && wl_game.player[0].state == WL_PLAYER_STATE_NORMAL)
			{
				wl_game.board[i][j].type = 3;
				wl_game.board[i][j].color = col_used[type];
			}
			
			/* blank tile for spaces with no letters */
			else if(wl_game.board[i][j].letter == ' ')
			{
				wl_game.board[i][j].type = 3;
				wl_game.board[i][j].color = col_used[type];
			}
			
			/* in range */
			else if(abs(wl_game.player[0].x - j) < wl_game.player[0].range && abs(wl_game.player[0].y - i) < wl_game.player[0].range)
			{
				wl_game.board[i][j].type = 2;
				wl_game.board[i][j].color = col_base[type];
			}
			
			/* out of range */
			else
			{
				wl_game.board[i][j].type = 0;
				wl_game.board[i][j].color = col_outrange[type];
			}
		}
	}
}

void wl_board_render(void)
{
	int i, j;
	char buf[4] = {0};
	
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			al_draw_tinted_scaled_bitmap(wl_bitmap[WL_BITMAP_TILE], wl_game.board[i][j].color, 0, 0, al_get_bitmap_width(wl_bitmap[WL_BITMAP_TILE]), al_get_bitmap_height(wl_bitmap[WL_BITMAP_TILE]), WL_BOARD_X + j * WL_BOARD_TILE_SIZE, WL_BOARD_Y + i * WL_BOARD_TILE_SIZE, WL_BOARD_TILE_SIZE, WL_BOARD_TILE_SIZE, 0);
		}
	}
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			switch(wl_game.board[i][j].letter)
			{
				/* don't render character for powerup tiles */
				case '?':
				case '!':
				case '_':
				case '0':
				{
					buf[0] = '\0';
					break;
				}
				case 'Q':
				{
					buf[0] = 'Q';
					buf[1] = 'u';
					break;
				}
				default:
				{
					buf[0] = wl_game.board[i][j].letter;
					buf[1] = '\0';
					break;
				}
			}
			al_draw_text(wl_font[WL_FONT_32], al_map_rgba_f(0.0, 0.0, 0.0, 1.0), WL_BOARD_X + j * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2, WL_BOARD_Y + i * WL_BOARD_TILE_SIZE + 1, ALLEGRO_ALIGN_CENTRE, buf);
		}
	}
}

static char wl_board_retrieve_powerup(void)
{
	int i, r;
	char c = 0;
	
	r = rand() % wl_game.powerups;
	switch(wl_game.powerup[r])
	{
		case WL_POWERUP_GURU:
		{
			c = '?';
			break;
		}
		case WL_POWERUP_HOURGLASS:
		{
			c = '_';
			break;
		}
		case WL_POWERUP_LIFE:
		{
			c = '!';
			break;
		}
	}
	for(i = r; i < wl_game.powerups - 1; i++)
	{
		wl_game.powerup[i] = wl_game.powerup[i + 1];
	}
	wl_game.powerups--;
	if(wl_game.powerups <= 0)
	{
		wl_game_generate_powerups(); // generate new list of powerups when we run out
	}
	return c;
}

char wl_board_generate_letter(bool allow_powerups)
{
	float r, a;
	int i;
	
	/* see if we need to generate a powerup */
	if(wl_game.uses_powerups && allow_powerups)
	{
		wl_game.powerup_count--;
		if(wl_game.powerup_count <= 0)
		{
			/* set amount of "time" it takes to generate a powerup */
			wl_game.powerup_count = WL_POWERUP_COUNT + wl_game_use_powerup_randomness();
			
			/* destroy other powerups on the board, we only want one at a time */
			for(i = 0; i < WL_MAX_SPRITES; i++)
			{
				if(wl_game.sprite[i].active)
				{
					if(wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_GURU || wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_HEALTH || wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_SNEAKER)
					{
						wl_explode_sprite(&wl_game.sprite[i]);
						wl_game.board[(int)wl_game.sprite[i].y / WL_BOARD_TILE_SIZE][(int)wl_game.sprite[i].x / WL_BOARD_TILE_SIZE].letter = ' ';
						t3f_play_sample(wl_sample[WL_SAMPLE_DESTROY], 1.0, 0.0, 1.0);
						break;
					}
				}
			}
			
			return wl_board_retrieve_powerup();
		}
	}
	
	/* generate a letter */
	r = t3f_drand(&wl_game.rng_state) * 100.0;
	a = 0.0;
	for(i = 0; i < 26; i++)
	{
		a += wl_letter_probability[i];
		if(r < a)
		{
			return 'A' + i;
		}
	}
	return 'Z';
}

void wl_board_generate_powerup_sprite(int i, int j, int letter)
{
	int s;
	
	switch(letter)
	{
		case '!':
		{
			s = wl_game_add_sprite(WL_SPRITE_TYPE_POWERUP_HEALTH);
			if(s >= 0)
			{
				wl_game.sprite[s].x = j * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].y = i * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].timer = i * WL_MAX_BOARD_WIDTH + j;
			}
			break;
		}
		case '?':
		{
			s = wl_game_add_sprite(WL_SPRITE_TYPE_POWERUP_GURU);
			if(s >= 0)
			{
				wl_game.sprite[s].x = j * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].y = i * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].timer = i * WL_MAX_BOARD_WIDTH + j;
			}
			break;
		}
		case '_':
		{
			s = wl_game_add_sprite(WL_SPRITE_TYPE_POWERUP_SNEAKER);
			if(s >= 0)
			{
				wl_game.sprite[s].x = j * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].y = i * WL_BOARD_TILE_SIZE;
				wl_game.sprite[s].a = 0.0;
				wl_game.sprite[s].va = 0.0;
				wl_game.sprite[s].timer = i * WL_MAX_BOARD_WIDTH + j;
			}
			break;
		}
	}
}

static bool wl_is_player_space(int x, int y)
{
	if(x != wl_game.player[0].x || y != wl_game.player[0].y)
	{
		return false;
	}
	return true;
}

void wl_board_generate_new_letters(void)
{
	int i, j;
	
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			if(wl_game.board[i][j].letter == ' ' && !wl_is_player_space(j, i))
			{
				wl_game.board[i][j].letter = wl_board_generate_letter(true);
				wl_board_generate_powerup_sprite(i, j, wl_game.board[i][j].letter);
			}
		}
	}
}

/* used when continuing the game, generates new tiles for all non-letter tiles */
void wl_board_regenerate(void)
{
	int i, j;
	
	for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
	{
		for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
		{
			switch(wl_game.board[i][j].letter)
			{
				case ' ':
				case '!':
				case '?':
				case '2':
				case '3':
				case '0':
				{
					wl_game.board[i][j].letter = wl_board_generate_letter(false);
					break;
				}
			}
		}
	}
}

void wl_add_board_particle(ALLEGRO_COLOR color, int x, int y, int life)
{
	wl_add_particle(1, WL_PARTICLE_TYPE_FLASH, wl_animation[WL_ANIMATION_PARTICLE], color, WL_BOARD_X + x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + y * WL_BOARD_TILE_SIZE, 0.0, WL_BOARD_TILE_SIZE, life, 0);
}

void wl_generate_board_particles(ALLEGRO_COLOR color, int x, int y, int r)
{
	int i, j, sx, sy, ex, ey;
	
	for(i = 1; i < r; i++)
	{
		sx = x - i;
		sy = y - i;
		ex = x + i;
		ey = y + i;
		for(j = sx; j <= ex; j++)
		{
			if(sy >= 0 && j >= 0 && j < WL_MAX_BOARD_WIDTH)
			{
				wl_add_board_particle(color, j, sy, 10 + i * 2);
			}
			if(ey < WL_MAX_BOARD_HEIGHT && j >= 0 && j < WL_MAX_BOARD_WIDTH)
			{
				wl_add_board_particle(color, j, ey, 10 + i * 2);
			}
		}
		for(j = sy + 1; j <= ey - 1; j++)
		{
			if(sx >= 0 && j >= 0 && j < WL_MAX_BOARD_HEIGHT)
			{
				wl_add_board_particle(color, sx, j, 10 + i * 2);
			}
			if(ex < WL_MAX_BOARD_WIDTH && j >= 0 && j < WL_MAX_BOARD_HEIGHT)
			{
				wl_add_board_particle(color, ex, j, 10 + i * 2);
			}
		}
	}
}
