#include "t3f/t3f.h"
#include "t3f/animation.h"
#include "t3f/music.h"
#include "cinema.h"
#include "music_queue.h"

void wl_process_cinema_event(WL_CINEMA * cp, int event);

WL_CINEMA * wl_create_cinema(void)
{
	WL_CINEMA * cp = NULL;
	int i;

	cp = al_malloc(sizeof(WL_CINEMA));
	if(cp)
	{
		cp->events = 0;
		for(i = 0; i < WL_CINEMA_MAX_BITMAPS; i++)
		{
			cp->bitmap[i] = NULL;
		}
		cp->view = t3f_create_view(0, 0, 640, 480, 320, 240, T3F_FORCE_ASPECT);
		cp->view_callback = NULL;
		cp->view_logic_callback = NULL;
		cp->camera.x = 0.0;
		cp->camera.y = 0.0;
		cp->camera.z = 0.0;
		cp->camera.vx = 0.0;
		cp->camera.vy = 0.0;
		cp->camera.vz = 0.0;
		cp->scene = 0;
		cp->break_points = 0;
	}
	return cp;
}

void wl_destroy_cinema(WL_CINEMA * cp)
{
	int i;

	for(i = 0; i < WL_CINEMA_MAX_BITMAPS; i++)
	{
		if(cp->bitmap[i])
		{
			t3f_destroy_animation(cp->bitmap[i]);
		}
	}
	for(i = 0; i < cp->events; i++)
	{
		al_free(cp->event[i]);
	}
	t3f_destroy_view(cp->view);
	al_free(cp);
}

bool wl_cinema_add_bitmap(WL_CINEMA * cp, ALLEGRO_BITMAP * bp)
{
	return false;
}

void wl_cinema_remove_bitmap(WL_CINEMA * cp, int bitmap)
{
}

bool wl_cinema_add_event(WL_CINEMA * cp, unsigned long tick, int type, WL_CINEMA_EVENT_DATA data, bool break_point, int flags)
{
	cp->event[cp->events] = al_malloc(sizeof(WL_CINEMA_EVENT));
	if(cp->event[cp->events])
	{
		cp->event[cp->events]->tick = tick;
		cp->event[cp->events]->type = type;
		cp->event[cp->events]->flags = flags;
		cp->event[cp->events]->data = data;
		cp->event[cp->events]->break_point = break_point;
		cp->events++;
		if(break_point)
		{
			cp->break_points++;
		}
		return true;
	}
	return false;
}

void wl_cinema_remove_event(WL_CINEMA * cp, int event)
{
	int i;

	al_free(cp->event[event]);
	for(i = event; i < cp->events - 1; i++)
	{
		cp->event[i] = cp->event[i + 1];
	}
	cp->events--;
}

WL_CINEMA * wl_load_cinema(const char * fn)
{
	return NULL;
}

static int get_config_integer(ALLEGRO_CONFIG * cp, const char * section, const char * key)
{
	const char * val;

	val = al_get_config_value(cp, section, key);
	if(val)
	{
		return atoi(val);
	}
	return 0;
}

static bool get_config_float(ALLEGRO_CONFIG * cp, const char * section, const char * key, float * fp)
{
	const char * val;

	val = al_get_config_value(cp, section, key);
	if(val)
	{
		*fp = atof(val);
		return true;
	}
	*fp = 0.0;
	return false;
}

WL_CINEMA * wl_load_cinema_script(const char * fn)
{
	WL_CINEMA * cp;
	WL_CINEMA_EVENT_DATA event_data;
	ALLEGRO_CONFIG * config;
	const char * val;
	char buf[256] = {0};
	int b, e, i, n, t, scene;
	float x, y, z, a, vx, vy, vz, va, s, is, jy, gy;
	int etick = 0;
	bool use_x, use_y, use_z;
	bool use_vx, use_vy, use_vz, use_va;
	float cr, cg, cb, ca;
	bool use_c;
	bool break_point;
	unsigned long tick = 0;
	unsigned long target_tick = 0;
	unsigned long current_tick = 0;
	const char * efn;

	cp = wl_create_cinema();
	if(cp)
	{
		al_stop_timer(t3f_timer);
		cp->entities = 0;
		config = al_load_config_file(fn);
		if(config)
		{

			/* determine the number of events */
			n = 0;
			while(1)
			{
				sprintf(buf, "event %d", n);
				val = al_get_config_value(config, buf, "tick");
				if(!val)
				{
					break;
				}
				else
				{
					n++;
				}
			}

			/* load events */
			for(i = 0; i < n; i++)
			{
				sprintf(buf, "event %d", i);

				/* determine if event is a break point */
				break_point = false;
				val = al_get_config_value(config, buf, "break_point");
				if(val)
				{
					if(!strcasecmp(val, "true"))
					{
						break_point = true;
					}
				}

				tick = get_config_integer(config, buf, "tick");
				current_tick += tick; // use this to keep track of where we are
				target_tick = get_config_integer(config, buf, "target_tick");
				b = get_config_integer(config, buf, "bitmap");
				t = get_config_integer(config, buf, "type");
				e = get_config_integer(config, buf, "entity");
				etick = get_config_integer(config, buf, "tick");
				use_x = get_config_float(config, buf, "x", &x);
				use_y = get_config_float(config, buf, "y", &y);
				use_z = get_config_float(config, buf, "z", &z);
				use_vx = get_config_float(config, buf, "vx", &vx);
				use_vy = get_config_float(config, buf, "vy", &vy);
				use_vz = get_config_float(config, buf, "vz", &vz);
				use_va = get_config_float(config, buf, "va", &va);
				get_config_float(config, buf, "a", &a);
				get_config_float(config, buf, "s", &s);
				get_config_float(config, buf, "cr", &cr);
				get_config_float(config, buf, "cg", &cg);
				get_config_float(config, buf, "cb", &cb);
				get_config_float(config, buf, "jump", &jy);
				get_config_float(config, buf, "gravity", &gy);
				use_c = get_config_float(config, buf, "ca", &ca);
				scene = get_config_integer(config, buf, "scene");
				efn = al_get_config_value(config, buf, "filename");
				switch(t)
				{
					case WL_CINEMA_EVENT_CLEAR:
					{
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_LOAD_SCENE:
					{
						event_data.load_scene.scene = scene;
						event_data.load_scene.config_fn = fn;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_ADD_ENTITY:
					{
						event_data.add_entity.bitmap = b;
						event_data.add_entity.x = x;
						event_data.add_entity.y = y;
						event_data.add_entity.z = z;
						event_data.add_entity.a = a;
						event_data.add_entity.tick = etick;
						if(s > 0.0)
						{
							event_data.add_entity.s = s;
						}

						/* use automatic scaling */
						else if(s < 0.0)
						{
							event_data.add_entity.s = 1.0 / (t3f_project_x(1.0, z) - t3f_project_x(0.0, z));
							event_data.add_entity.x -= (t3f_project_x(x, z) - x) * event_data.add_entity.s;
							event_data.add_entity.y -= (t3f_project_y(y, z) - y) * event_data.add_entity.s;
						}
						else
						{
							event_data.add_entity.s = 1.0;
						}
						if(use_c)
						{
							event_data.add_entity.color = al_map_rgba_f(cr, cg, cb, ca);
						}
						else
						{
							event_data.add_entity.color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
						}
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_CHANGE_ENTITY:
					{
						event_data.change_entity.entity = e;
						event_data.change_entity.bitmap = b;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_REMOVE_ENTITY:
					{
						event_data.remove_entity.entity = e;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_MOVE_ENTITY:
					{
						event_data.move_entity.entity = e;
						event_data.move_entity.tick = current_tick + target_tick;
						event_data.move_entity.ux = use_x;
						event_data.move_entity.uy = use_y;
						event_data.move_entity.uz = use_z;
						event_data.move_entity.x = x;
						event_data.move_entity.y = y;
						event_data.move_entity.z = z;
						event_data.move_entity.jy = jy;
						event_data.move_entity.gy = gy;

						/* use automatic scaling */
						if(s < 0.0)
						{
							is = 1.0 / (t3f_project_x(1.0, z) - t3f_project_x(0.0, z));
							event_data.move_entity.x -= (t3f_project_x(x, z) - x) * is;
							event_data.move_entity.y -= (t3f_project_y(y, z) - y) * is;
						}

						event_data.move_entity.a = a;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						event_data.stop_entity.entity = e;
						wl_cinema_add_event(cp, current_tick + target_tick, WL_CINEMA_EVENT_STOP_ENTITY, event_data, false, 0);
						break;
					}
					case WL_CINEMA_EVENT_ACCEL_ENTITY:
					{
						event_data.accel_entity.entity = e;
						event_data.accel_entity.tick = current_tick + target_tick;
						event_data.accel_entity.uvx = use_vx;
						event_data.accel_entity.uvy = use_vy;
						event_data.accel_entity.uvz = use_vz;
						event_data.accel_entity.uva = use_va;
						event_data.accel_entity.vx = vx;
						event_data.accel_entity.vy = vy;
						event_data.accel_entity.vz = vz;
						event_data.accel_entity.va = va;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_MOVE_CAMERA:
					{
						event_data.move_camera.tick = current_tick + target_tick;
						event_data.move_camera.x = x;
						event_data.move_camera.y = y;
						event_data.move_camera.z = z;
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						wl_cinema_add_event(cp, current_tick + target_tick, WL_CINEMA_EVENT_STOP_CAMERA, event_data, false, 0);
						break;
					}
					case WL_CINEMA_EVENT_FADE_ENTITY:
					{
						event_data.fade_entity.entity = e;
						event_data.fade_entity.tick = current_tick + target_tick;
						event_data.fade_entity.color = al_map_rgba_f(cr, cg, cb, ca);
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						wl_cinema_add_event(cp, current_tick + target_tick, WL_CINEMA_EVENT_STOP_FADE, event_data, false, 0);
						break;
					}
					case WL_CINEMA_EVENT_COLOR_ENTITY:
					{
						event_data.color_entity.entity = e;
						event_data.color_entity.color = al_map_rgba_f(cr, cg, cb, ca);
						wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						break;
					}
					case WL_CINEMA_EVENT_PLAY_MUSIC:
					{
						if(efn)
						{
							strcpy(event_data.play_music.filename, efn);
							wl_cinema_add_event(cp, current_tick, t, event_data, break_point, 0);
						}
						break;
					}
					case WL_CINEMA_EVENT_END:
					{
						cp->length = current_tick;
						break;
					}
				}
			}
		}
		al_destroy_config(config);
		al_start_timer(t3f_timer);
	}

	/* start cinema at first frame */
	cp->tick = 0;

	/* set up first frame so we can render immediately, we'll clear it before
	 * actually processing the first tick */
	cp->first_frame = true;
	for(i = 0; i < cp->events; i++)
	{
		if(cp->event[i]->tick == cp->tick)
		{
			wl_process_cinema_event(cp, i);
		}
	}
	cp->first_frame = false;

	return cp;
}

bool wl_save_cinema(WL_CINEMA * cp, const char * fn)
{
	return true;
}

void wl_set_cinema_view_callbacks(WL_CINEMA * cp, void (*logic_callback)(), void (*render_callback)())
{
	cp->view_logic_callback = logic_callback;
	cp->view_callback = render_callback;
}

static void wl_cinema_remove_entity(WL_CINEMA * cp, int entity)
{
	int i;

	if(entity < cp->entities)
	{
		for(i = entity; i < cp->entities - 1; i++)
		{
			memcpy(&cp->entity[i], &cp->entity[i + 1], sizeof(WL_CINEMA_ENTITY));
		}
		cp->entities--;
	}
}

void wl_process_cinema_event(WL_CINEMA * cp, int event)
{
	int e, i;
	float d, r, g, b, a, dr, dg, db, da;
	unsigned long t;
	ALLEGRO_CONFIG * config = NULL;
	const char * val = NULL;
	char buf[256] = {0};
	char buf2[256] = {0};

	switch(cp->event[event]->type)
	{
		case WL_CINEMA_EVENT_CLEAR:
		{
			cp->entities = 0;
			break;
		}
		case WL_CINEMA_EVENT_LOAD_SCENE:
		{
			al_stop_timer(t3f_timer);
			config = al_load_config_file(cp->event[event]->data.load_scene.config_fn);
			if(config)
			{
				/* destroy old scene bitmaps */
				for(i = 0; i < WL_CINEMA_MAX_BITMAPS; i++)
				{
					if(cp->bitmap[i])
					{
						t3f_destroy_animation(cp->bitmap[i]);
						cp->bitmap[i] = NULL;
					}
				}

				/* load bitmaps for this scene */
				sprintf(buf, "scene %d", cp->event[event]->data.load_scene.scene);
				i = 0;
				for(i = 0; i < WL_CINEMA_MAX_BITMAPS; i++)
				{
					sprintf(buf2, "bitmap %d", i);
					val = al_get_config_value(config, buf, buf2);
					if(val)
					{
						cp->bitmap[i] = t3f_load_animation(val);
						if(!cp->bitmap[i])
						{
							cp->bitmap[i] = t3f_load_animation_from_bitmap(val);
						}
					}
				}
				al_destroy_config(config);
			}
			cp->entities = 0;
			cp->camera.x = 0.0;
			cp->camera.y = 0.0;
			cp->camera.z = 0.0;
			cp->scene = cp->event[event]->data.load_scene.scene;
			al_start_timer(t3f_timer);
			break;
		}
		case WL_CINEMA_EVENT_ADD_ENTITY:
		{
			cp->entity[cp->entities].bitmap = cp->event[event]->data.add_entity.bitmap;
			cp->entity[cp->entities].x = cp->event[event]->data.add_entity.x;
			cp->entity[cp->entities].y = cp->event[event]->data.add_entity.y;
			cp->entity[cp->entities].z = cp->event[event]->data.add_entity.z;
			cp->entity[cp->entities].a = cp->event[event]->data.add_entity.a;
			cp->entity[cp->entities].s = cp->event[event]->data.add_entity.s;
			cp->entity[cp->entities].vx = 0.0;
			cp->entity[cp->entities].vy = 0.0;
			cp->entity[cp->entities].vz = 0.0;
			cp->entity[cp->entities].va = 0.0;
			al_unmap_rgba_f(cp->event[event]->data.add_entity.color, &cp->entity[cp->entities].cr, &cp->entity[cp->entities].cg, &cp->entity[cp->entities].cb, &cp->entity[cp->entities].ca);
			cp->entity[cp->entities].cvr = 0.0;
			cp->entity[cp->entities].cvg = 0.0;
			cp->entity[cp->entities].cvb = 0.0;
			cp->entity[cp->entities].cva = 0.0;
			cp->entity[cp->entities].color = cp->event[event]->data.add_entity.color;
			cp->entity[cp->entities].tick = cp->event[event]->data.add_entity.tick;
			cp->entity[cp->entities].by = cp->entity[cp->entities].y;
			cp->entity[cp->entities].jy = 0.0;
			cp->entity[cp->entities].gy = 0.0;
			cp->entities++;
			break;
		}
		case WL_CINEMA_EVENT_CHANGE_ENTITY:
		{
			cp->entity[cp->event[event]->data.change_entity.entity].bitmap = cp->event[event]->data.change_entity.bitmap;
			break;
		}
		case WL_CINEMA_EVENT_REMOVE_ENTITY:
		{
			wl_cinema_remove_entity(cp, cp->event[event]->data.remove_entity.entity);
			break;
		}
		case WL_CINEMA_EVENT_MOVE_ENTITY:
		{
			e = cp->event[event]->data.move_entity.entity;
			t = cp->event[event]->data.move_entity.tick;

			/* if target tick is the current tick, move entity immediately */
			if(t - cp->event[event]->tick <= 0)
			{
				if(cp->event[event]->data.move_entity.ux)
				{
					cp->entity[e].x = cp->event[event]->data.move_entity.x;
				}
				if(cp->event[event]->data.move_entity.uy)
				{
					cp->entity[e].y = cp->event[event]->data.move_entity.y;
				}
				if(cp->event[event]->data.move_entity.uz)
				{
					cp->entity[e].z = cp->event[event]->data.move_entity.z;
				}
				cp->entity[e].a = cp->event[event]->data.move_entity.a;
			}
			else
			{
				if(cp->event[event]->data.move_entity.ux)
				{
					d = (cp->event[event]->data.move_entity.x - cp->entity[e].x) / (float)(t - cp->event[event]->tick);
					cp->entity[e].vx = d;
				}
				if(cp->event[event]->data.move_entity.uy)
				{
					d = (cp->event[event]->data.move_entity.y - cp->entity[e].y) / (float)(t - cp->event[event]->tick);
					cp->entity[e].vy = d;
				}
				if(cp->event[event]->data.move_entity.uz)
				{
					d = (cp->event[event]->data.move_entity.z - cp->entity[e].z) / (float)(t - cp->event[event]->tick);
					cp->entity[e].vz = d;
				}
				d = (cp->event[event]->data.move_entity.a - cp->entity[e].a) / (float)(t - cp->event[event]->tick);
				cp->entity[e].va = d;
				cp->entity[e].by = cp->entity[e].y;
				cp->entity[e].gy = cp->event[event]->data.move_entity.gy;
				cp->entity[e].jy = cp->event[event]->data.move_entity.jy;
			}
			break;
		}
		case WL_CINEMA_EVENT_ACCEL_ENTITY:
		{
			e = cp->event[event]->data.move_entity.entity;

			if(cp->event[event]->data.accel_entity.uvx)
			{
				cp->entity[e].vx = cp->event[event]->data.accel_entity.vx;
			}
			if(cp->event[event]->data.accel_entity.uvy)
			{
				cp->entity[e].vy = cp->event[event]->data.accel_entity.vy;
			}
			if(cp->event[event]->data.accel_entity.uvz)
			{
				cp->entity[e].vz = cp->event[event]->data.accel_entity.vz;
			}
			if(cp->event[event]->data.accel_entity.uva)
			{
				cp->entity[e].va = cp->event[event]->data.accel_entity.va;
			}
			break;
		}
		case WL_CINEMA_EVENT_MOVE_CAMERA:
		{
			t = cp->event[event]->data.move_camera.tick;

			/* if target tick is the current tick, move camera immediately */
			if(t - cp->event[event]->tick <= 0)
			{
				cp->camera.x = cp->event[event]->data.move_camera.x;
				cp->camera.y = cp->event[event]->data.move_camera.y;
				cp->camera.z = cp->event[event]->data.move_camera.z;
			}
			else
			{
				d = (cp->event[event]->data.move_camera.x - cp->camera.x) / (float)(t - cp->event[event]->tick);
				cp->camera.vx = d;
				d = (cp->event[event]->data.move_camera.y - cp->camera.y) / (float)(t - cp->event[event]->tick);
				cp->camera.vy = d;
				d = (cp->event[event]->data.move_camera.z - cp->camera.z) / (float)(t - cp->event[event]->tick);
				cp->camera.vz = d;
			}
			break;
		}
		case WL_CINEMA_EVENT_FADE_ENTITY:
		{
			e = cp->event[event]->data.fade_entity.entity;
			t = cp->event[event]->data.fade_entity.tick;
			if(e >= 0)
			{
				al_unmap_rgba_f(cp->entity[e].color, &r, &g, &b, &a);
				al_unmap_rgba_f(cp->event[event]->data.fade_entity.color, &dr, &dg, &db, &da);
				d = (dr - r) / (float)(t - cp->event[event]->tick);
				cp->entity[e].cvr = d;
				d = (dg - g) / (float)(t - cp->event[event]->tick);
				cp->entity[e].cvg = d;
				d = (db - b) / (float)(t - cp->event[event]->tick);
				cp->entity[e].cvb = d;
				d = (da - a) / (float)(t - cp->event[event]->tick);
				cp->entity[e].cva = d;
			}
			else
			{
				for(i = 0; i < cp->entities; i++)
				{
					al_unmap_rgba_f(cp->entity[i].color, &r, &g, &b, &a);
					al_unmap_rgba_f(cp->event[event]->data.fade_entity.color, &dr, &dg, &db, &da);
					d = (dr - r) / (float)(t - cp->event[event]->tick);
					cp->entity[i].cvr = d;
					d = (dg - g) / (float)(t - cp->event[event]->tick);
					cp->entity[i].cvg = d;
					d = (db - b) / (float)(t - cp->event[event]->tick);
					cp->entity[i].cvb = d;
					d = (da - a) / (float)(t - cp->event[event]->tick);
					cp->entity[i].cva = d;
				}
			}
			break;
		}
		case WL_CINEMA_EVENT_COLOR_ENTITY:
		{
			e = cp->event[event]->data.color_entity.entity;
			cp->entity[e].color = cp->event[event]->data.color_entity.color;
			al_unmap_rgba_f(cp->entity[e].color, &cp->entity[e].cr, &cp->entity[e].cg, &cp->entity[e].cb, &cp->entity[e].ca);
			break;
		}
		case WL_CINEMA_EVENT_STOP_ENTITY:
		{
			cp->entity[cp->event[event]->data.stop_entity.entity].vx = 0.0;
			cp->entity[cp->event[event]->data.stop_entity.entity].vy = 0.0;
			cp->entity[cp->event[event]->data.stop_entity.entity].vz = 0.0;
			cp->entity[cp->event[event]->data.stop_entity.entity].va = 0.0;
			cp->entity[cp->event[event]->data.stop_entity.entity].gy = 0.0;
			break;
		}
		case WL_CINEMA_EVENT_STOP_FADE:
		{
			e = cp->event[event]->data.color_entity.entity;
			if(e >= 0)
			{
				cp->entity[cp->event[event]->data.stop_entity.entity].cvr = 0.0;
				cp->entity[cp->event[event]->data.stop_entity.entity].cvg = 0.0;
				cp->entity[cp->event[event]->data.stop_entity.entity].cvb = 0.0;
				cp->entity[cp->event[event]->data.stop_entity.entity].cva = 0.0;
			}
			else
			{
				for(i = 0; i < cp->entities; i++)
				{
					cp->entity[i].cvr = 0.0;
					cp->entity[i].cvg = 0.0;
					cp->entity[i].cvb = 0.0;
					cp->entity[i].cva = 0.0;
				}
			}
			break;
		}
		case WL_CINEMA_EVENT_STOP_CAMERA:
		{
			cp->camera.vx = 0.0;
			cp->camera.vy = 0.0;
			cp->camera.vz = 0.0;
			break;
		}
		case WL_CINEMA_EVENT_PLAY_MUSIC:
		{
			if(!cp->first_frame)
			{
				wl_queue_music(cp->event[event]->data.play_music.filename);
				wl_finish_current_song();
			}
			break;
		}
	}
}

bool wl_process_cinema(WL_CINEMA * cp)
{
	int i;
	bool ret = false;

	/* clear entities on first tick */
	if(cp->tick == 0)
	{
		for(i = 0; i < cp->entities; i++)
		{
			wl_cinema_remove_entity(cp, 0);
		}
	}

	for(i = 0; i < cp->events; i++)
	{
		if(cp->event[i]->tick == cp->tick)
		{
			wl_process_cinema_event(cp, i);
			if(cp->event[i]->break_point)
			{
				ret = true;
			}
		}
	}
	cp->tick++;

	/* handle entity movement */
	for(i = 0; i < cp->entities; i++)
	{
		cp->entity[i].x += cp->entity[i].vx;
		cp->entity[i].y += cp->entity[i].vy;
		cp->entity[i].z += cp->entity[i].vz;
		cp->entity[i].a += cp->entity[i].va;
		cp->entity[i].cr += cp->entity[i].cvr;
		cp->entity[i].cg += cp->entity[i].cvg;
		cp->entity[i].cb += cp->entity[i].cvb;
		cp->entity[i].ca += cp->entity[i].cva;
		cp->entity[i].color = al_map_rgba_f(cp->entity[i].cr, cp->entity[i].cg, cp->entity[i].cb, cp->entity[i].ca);

		/* process gravity */
		if(cp->entity[i].gy > 0.0)
		{
			cp->entity[i].vy += cp->entity[i].gy;
			if(cp->entity[i].y >= cp->entity[i].by)
			{
				cp->entity[i].y = cp->entity[i].by;
				cp->entity[i].vy = cp->entity[i].jy;
			}
		}
		cp->entity[i].tick++;

		/* process the logic for the view if this is a view entity */
		if(cp->entity[i].bitmap < 0 && cp->view_logic_callback)
		{
			cp->view_logic_callback();
		}
	}

	/* handle camera movement */
	cp->camera.x += cp->camera.vx;
	cp->camera.y += cp->camera.vy;
	cp->camera.z += cp->camera.vz;

	if(cp->tick >= cp->length)
	{
		ret = true;
	}

	return ret;
}

void wl_render_cinema(WL_CINEMA * cp)
{
	int i;
	float cx, cy;
	float ox, oy;
	float w, h;

	/* render entities */
	for(i = 0; i < cp->entities; i++)
	{

		/* use the view callback in this case, we want to be able to render the
		 * game into this view while the cinema is still running */
		if(cp->entity[i].bitmap < 0)
		{
			if(cp->view_callback)
			{
				ox = t3f_project_x(cp->entity[i].x - cp->camera.x, cp->entity[i].z - cp->camera.z);
				oy = t3f_project_y(cp->entity[i].y - cp->camera.y, cp->entity[i].z - cp->camera.z);
				w = t3f_project_x(cp->entity[i].x - cp->camera.x + t3f_virtual_display_width, cp->entity[i].z - cp->camera.z) - ox;
				h = t3f_project_y(cp->entity[i].y - cp->camera.y + t3f_virtual_display_height, cp->entity[i].z - cp->camera.z) - oy;
				t3f_adjust_view(cp->view, ox, oy, w, h, cp->view->vp_x, cp->view->vp_y, T3F_FORCE_ASPECT);
				t3f_select_view(cp->view);
				cp->view_callback();
				t3f_select_view(NULL);
				al_hold_bitmap_drawing(false);
			}
		}
		else if(cp->bitmap[cp->entity[i].bitmap])
		{
			cx = al_get_bitmap_width(cp->bitmap[cp->entity[i].bitmap]->bitmaps->bitmap[0]) / 2;
			cy = al_get_bitmap_height(cp->bitmap[cp->entity[i].bitmap]->bitmaps->bitmap[0]) / 2;
			t3f_draw_rotated_scaled_animation(cp->bitmap[cp->entity[i].bitmap], cp->entity[i].color, cp->entity[i].tick, cx, cy, cp->entity[i].x - cp->camera.x + cx * cp->entity[i].s, cp->entity[i].y - cp->camera.y + cy * cp->entity[i].s, cp->entity[i].z - cp->camera.z, cp->entity[i].a, cp->entity[i].s, 0);
		}
	}
}
