#ifndef WL_CINEMA_H
#define WL_CINEMA_H

#include "t3f/t3f.h"
#include "t3f/view.h"
#include "t3f/animation.h"

#define WL_CINEMA_MAX_BITMAPS  256
#define WL_CINEMA_MAX_EVENTS  1024
#define WL_CINEMA_MAX_DATA     256
#define WL_CINEMA_MAX_ENTITIES  64

#define WL_CINEMA_EVENT_CLEAR         0
#define WL_CINEMA_EVENT_ADD_ENTITY    1
#define WL_CINEMA_EVENT_REMOVE_ENTITY 2
#define WL_CINEMA_EVENT_MOVE_ENTITY   3
#define WL_CINEMA_EVENT_MOVE_CAMERA   4
#define WL_CINEMA_EVENT_FADE_ENTITY   5
#define WL_CINEMA_EVENT_STOP_FADE     6
#define WL_CINEMA_EVENT_COLOR_ENTITY  7
#define WL_CINEMA_EVENT_LOAD_SCENE    8
#define WL_CINEMA_EVENT_ACCEL_ENTITY  9
#define WL_CINEMA_EVENT_STOP_ENTITY  16
#define WL_CINEMA_EVENT_STOP_CAMERA  17
#define WL_CINEMA_EVENT_CHANGE_ENTITY 18
#define WL_CINEMA_EVENT_PLAY_MUSIC   19
#define WL_CINEMA_EVENT_END         255

typedef struct
{
	
	int bitmap;
	float x, y, z, a, s;
	int tick;
	ALLEGRO_COLOR color;
	
} WL_CINEMA_EVENT_DATA_ADD_ENTITY;

typedef struct
{
	
	int entity;
	
} WL_CINEMA_EVENT_DATA_REMOVE_ENTITY;

typedef struct
{
	
	int entity;
	unsigned long tick;
	float x, y, z, a;
	float jy, gy;
	bool ux, uy, uz;
	
} WL_CINEMA_EVENT_DATA_MOVE_ENTITY;

typedef struct
{
	
	int entity;
	unsigned long tick;
	float vx, vy, vz, va;
	bool uvx, uvy, uvz, uva;
	
} WL_CINEMA_EVENT_DATA_ACCEL_ENTITY;

typedef struct
{
	
	unsigned long tick;
	float x, y, z;
	
} WL_CINEMA_EVENT_DATA_MOVE_CAMERA;

typedef struct
{
	
	int entity;
	
} WL_CINEMA_EVENT_DATA_STOP_ENTITY;

typedef struct
{
	
	int entity;
	unsigned long tick;
	ALLEGRO_COLOR color;
	
} WL_CINEMA_EVENT_DATA_FADE_ENTITY;

typedef struct
{
	
	int entity;
	ALLEGRO_COLOR color;
	
} WL_CINEMA_EVENT_DATA_COLOR_ENTITY;

typedef struct
{
	
	int scene;
	const char * config_fn;
	
} WL_CINEMA_EVENT_DATA_LOAD_SCENE;

typedef struct
{
	
	int entity;
	int bitmap;
	
} WL_CINEMA_EVENT_DATA_CHANGE_ENTITY;

typedef struct
{
	
	char filename[1024];
	
} WL_CINEMA_EVENT_DATA_PLAY_MUSIC;

typedef union
{
	
	WL_CINEMA_EVENT_DATA_ADD_ENTITY add_entity;
	WL_CINEMA_EVENT_DATA_REMOVE_ENTITY remove_entity;
	WL_CINEMA_EVENT_DATA_MOVE_ENTITY move_entity;
	WL_CINEMA_EVENT_DATA_ACCEL_ENTITY accel_entity;
	WL_CINEMA_EVENT_DATA_MOVE_CAMERA move_camera;
	WL_CINEMA_EVENT_DATA_STOP_ENTITY stop_entity;
	WL_CINEMA_EVENT_DATA_FADE_ENTITY fade_entity;
	WL_CINEMA_EVENT_DATA_COLOR_ENTITY color_entity;
	WL_CINEMA_EVENT_DATA_LOAD_SCENE load_scene;
	WL_CINEMA_EVENT_DATA_CHANGE_ENTITY change_entity;
	WL_CINEMA_EVENT_DATA_PLAY_MUSIC play_music;
	
} WL_CINEMA_EVENT_DATA;

typedef struct
{
	
	unsigned long tick;
	int type;
	WL_CINEMA_EVENT_DATA data;
	bool break_point;
	int flags;
	
} WL_CINEMA_EVENT;

typedef struct
{
	
	int bitmap;
	float x, y, z, a, s;
	float vx, vy, vz, va;
	float by, jy, gy;
	ALLEGRO_COLOR color;
	float cr, cg, cb, ca;
	float cvr, cvg, cvb, cva;
	int tick;
	
} WL_CINEMA_ENTITY;

typedef struct
{
	
	float x, y, z;
	float vx, vy, vz;
	
} WL_CINEMA_CAMERA;

typedef struct
{

	T3F_ANIMATION * bitmap[WL_CINEMA_MAX_BITMAPS];
	
	WL_CINEMA_EVENT * event[WL_CINEMA_MAX_EVENTS];
	int events;
	
	void (*view_callback)();
	void (*view_logic_callback)();
	T3F_VIEW * view;
	
	/* cinema playback data */
	unsigned long tick;
	unsigned long length;
	WL_CINEMA_ENTITY entity[WL_CINEMA_MAX_ENTITIES];
	int entities;
	WL_CINEMA_CAMERA camera;
	int scene;
	int break_points;
	bool first_frame;

} WL_CINEMA;

WL_CINEMA * wl_create_cinema(void);
void wl_destroy_cinema(WL_CINEMA * cp);

bool wl_cinema_add_bitmap(WL_CINEMA * cp, ALLEGRO_BITMAP * bp);
void wl_cinema_remove_bitmap(WL_CINEMA * cp, int bitmap);
bool wl_cinema_add_event(WL_CINEMA * cp, unsigned long tick, int type, WL_CINEMA_EVENT_DATA data, bool break_point, int flags);
void wl_cinema_remove_event(WL_CINEMA * cp, int event);

WL_CINEMA * wl_load_cinema(const char * fn);
WL_CINEMA * wl_load_cinema_script(const char * fn);
bool wl_save_cinema(WL_CINEMA * cp, const char * fn);
void wl_set_cinema_view_callbacks(WL_CINEMA * cp, void (*logic_callback)(), void (*render_callback)());

bool wl_process_cinema(WL_CINEMA * cp);
void wl_render_cinema(WL_CINEMA * cp);

#endif
