#include "t3f/t3f.h"
#include <ctype.h>
#include "dictionary.h"

static int wl_count_lines(const char * buffer, int s)
{
	int lines = 0;
	int i;
	
	for(i = 0; i < s; i++)
	{
		if(buffer[i] == '\n')
		{
			lines++;
		}
	}
	return lines;
}

static int word_compare(const void * e1, const void * e2)
{
	const WL_WORD * word1 = e1;
	const WL_WORD * word2 = e2;
	
	return strcmp(word1->data, word2->data);
}

void wl_sort_dictionary(WL_DICTIONARY * dp)
{
	qsort(dp->word, dp->words, sizeof(WL_WORD), word_compare);
}

static void delete_word(WL_DICTIONARY * dp, int word)
{
	int i;
	
	for(i = word; i < dp->words - 1; i++)
	{
		dp->word[i] = dp->word[i + 1];
	}
	dp->words--;
}

void wl_trim_dictionary(WL_DICTIONARY * dp)
{
	int i;
	
	for(i = 0; i < dp->words - 1; i++)
	{
		if(!strcmp(dp->word[i].data, dp->word[i + 1].data))
		{
			if(dp->word[i].rarity < 0)
			{
				delete_word(dp, i);
			}
			else
			{
				delete_word(dp, i + 1);
			}
		}
	}
}

WL_DICTIONARY * wl_load_dictionary(const char * fn)
{
	WL_DICTIONARY * dp = NULL;
	ALLEGRO_FILE * fp = NULL;
	int lines = 0;
	int i, j, s;
	char current_letter = 'A';
	int current_offset = 0;
	char number[16] = {0};
	
	/* allocate the dictionary */
	dp = al_malloc(sizeof(WL_DICTIONARY));
	if(!dp)
	{
		return NULL;
	}
	/* buffer dictionary file */
	fp = al_fopen(fn, "rb");
	if(!fp)
	{
		return NULL;
	}
	s = al_fsize(fp);
	if(s <= 0)
	{
		return NULL;
	}
	dp->file_data = al_malloc(s);
	if(al_fread(fp, dp->file_data, s) < (unsigned int)s)
	{
		return NULL;
	}
	dp->file_pos = 0;

	/* count the words */
	lines = wl_count_lines(dp->file_data, s);
	if(lines <= 0)
	{
		return NULL;
	}
	dp->words = lines;

	/* allocate space for words */
	dp->word = al_malloc(sizeof(WL_WORD) * dp->words);
	if(!dp->word)
	{
		return NULL;
	}
	
	/* point to the words in file_data */
	for(i = 0; i < dp->words; i++)
	{
		
		/* get word data */
		dp->word[i].data = &dp->file_data[dp->file_pos];
		while(dp->file_pos < s && dp->file_data[dp->file_pos] != ' ')
		{
			dp->file_data[dp->file_pos] = toupper(dp->file_data[dp->file_pos]); // convert to upper case
			dp->file_pos++;
		}
		dp->file_data[dp->file_pos] = '\0'; // NULL terminate each word
		dp->file_pos++;
		
		/* get word rarity */
		j = 0;
		while(dp->file_pos < s && dp->file_data[dp->file_pos] != '\n')
		{
			number[j] = dp->file_data[dp->file_pos];
			j++;
			dp->file_pos++;
		}
		number[j] = '\0';
		dp->file_pos++;
		dp->word[i].rarity = atoi(number);
	}
	
	/* calculate offsets to speed up comparisons */
	for(i = 0; i < dp->words; i++)
	{
		if(dp->word[i].data[0] == current_letter)
		{
			dp->offsets[current_offset] = i;
			current_letter++;
			if(current_letter > 'Z')
			{
				break;
			}
			current_offset++;
			if(current_offset >= 26)
			{
				break;
			}
		}
	}
	
	return dp;
}

bool wl_save_dictionary(WL_DICTIONARY * dp, const char * fn)
{
	ALLEGRO_FILE * fp;
	int i;
	char buf[512] = {0};
	
	fp = al_fopen(fn, "wb");
	if(!fp)
	{
		return false;
	}
	for(i = 0; i < dp->words; i++)
	{
		sprintf(buf, "%s %d\n", dp->word[i].data, dp->word[i].rarity);
		al_fputs(fp, buf);
	}
	al_fclose(fp);
	return true;
}

void wl_destroy_dictionary(WL_DICTIONARY * dp)
{
	al_free(dp->file_data);
	al_free(dp->word);
	al_free(dp);
}

int wl_word_in_dictionary(WL_DICTIONARY * dp, const char * wp)
{
	int start;
	int i;
	
	start = dp->offsets[wp[0] - 'A'];
	
	for(i = start; i < dp->words; i++)
	{
		if(!strcasecmp(dp->word[i].data, wp))
		{
			return i;
		}
		if(dp->word[i].data[0] != wp[0])
		{
			break;
		}
	}
	return -1;
}
