#include "t3f/t3f.h"
#include "t3f/music.h"
#include "t3f/sound.h"
#include "t3net/t3net.h"
#include "game_intro.h"
#include "player.h"
#include "board.h"
#include "game_sprite.h"
#include "game_hazard.h"
#include "data.h"
#include "dictionary.h"
#include "particle.h"
#include "effects.h"
#include "awards.h"
#include "leaderboards.h"
#include "text.h"
#include "menu_proc.h"
#include "music_queue.h"
#include "menu.h"
#include "game_tally.h"

void wl_game_generate_powerups(void)
{
	wl_game.powerups = 0;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_GURU;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_HOURGLASS;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_HOURGLASS;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_HOURGLASS;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_LIFE;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_LIFE;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_LIFE;
	wl_game.powerups++;
	wl_game.powerup[wl_game.powerups] = WL_POWERUP_LIFE;
	wl_game.powerups++;
	wl_game.powerup_randomness = WL_POWERUP_VARY * wl_game.powerups;
}

int wl_game_figure_base_points(const char * word)
{
	unsigned int i;
	int m = -2;
	int points = 0;

	for(i = 0; i < strlen(word); i++)
	{
		if(wl_letter_probability[word[i] - 'A'] <= 0.2)
		{
			points += 5;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 2.0)
		{
			points += 4;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 4.0)
		{
			points += 3;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 7.0)
		{
			points += 2;
		}
		else
		{
			points += 1;
		}
		m++;
	}
	points *= m;
	points *= 10;
	return points;
}

int wl_game_figure_total_points(int w)
{
	int s, m;

	s = wl_game_figure_base_points(wl_dictionary->word[w].data);
	if(wl_dictionary->word[w].rarity <= 0)
	{
		m = 5;
	}
	else if(wl_dictionary->word[w].rarity < 5)
	{
		m = 3;
	}
	else if(wl_dictionary->word[w].rarity < 9)
	{
		m = 2;
	}
	else
	{
		m = 1;
	}
	return s * m;
}

int wl_game_use_powerup_randomness(void)
{
	int v;

	/* use up rest of randomness for last powerup */
	if(wl_game.powerups < 2)
	{
		v = wl_game.powerup_randomness;
	}
	else
	{
		/* use a random amount of our randomness */
		if(wl_game.powerup_randomness > 0)
		{
			v = rand() % (WL_POWERUP_VARY * 2);
			if(v > wl_game.powerup_randomness)
			{
				v = wl_game.powerup_randomness;
			}
		}

		/* no more randomness left */
		else
		{
			v = 0;
		}
		/* limit amount of randomness we can use if we aren't using the last bit */
		if(v > WL_POWERUP_VARY * 2)
		{
			v = WL_POWERUP_VARY * 2;
		}
	}

	/* take away randomness we used */
	wl_game.powerup_randomness -= v;
	return v;
}

void wl_game_setup_type(void)
{
	wl_game.uses_powerups = false;
	wl_game.uses_hazards = false;
	wl_game.uses_life = false;
	wl_game.uses_level_progress = false;
	wl_game.uses_invalid = false;
	switch(wl_game.mode)
	{
		case WL_GAME_MODE_STORY:
		{
			wl_game.uses_powerups = true;
			wl_game.uses_hazards = true;
			wl_game.uses_life = true;
			wl_game.uses_level_progress = true;
			wl_game.uses_invalid = true;
			break;
		}
		case WL_GAME_MODE_CASUAL:
		{
			break;
		}
		case WL_GAME_MODE_PUZZLE:
		{
			break;
		}
		case WL_GAME_MODE_GURU:
		{
			wl_game.uses_life = true;
			break;
		}
		case WL_GAME_MODE_PARTY:
		{
			break;
		}
	}
}

bool wl_game_initialize(int mode)
{
	int i, j;

	t3f_srand(&wl_game.rng_state, time(0));
	srand(time(0));
	wl_reset_particles(1);

	/* set up features of the game mode */
	wl_game.music_pos = 0.0;
	wl_game.old_music_pos = 0.0;
	wl_game.music_length = 0.0;
	wl_game.fast_forward = false;
	wl_game.music_fade_level = 0.0;
	wl_game.music_done = false;
	wl_game_setup_type();

	/* when starting a brand new game, regenerate entire contents of game board */
	if(mode != WL_LEVEL_LOAD_MUSIC)
	{
		wl_game.wipe_count = -1;
		for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
		{
			for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
			{
				wl_game.board[i][j].letter = wl_board_generate_letter(false);
				wl_game.board[i][j].type = 0;
				wl_game.board[i][j].flags = 0;
			}
		}
	}
	/* begin wiping the current board at continue/retry */
	else if(wl_game.regenerate)
	{
		wl_game.wipe_count = 0;
//		wl_board_regenerate();
		for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
		{
			for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
			{
				wl_game.board[i][j].flags = 0;
				wl_game.board[i][j].type = 0;
			}
		}
	}
	wl_game.board[WL_BOARD_CENTER_Y][WL_BOARD_CENTER_X].letter = ' ';
	wl_game_generate_powerups();
	wl_game.powerup_count = WL_POWERUP_COUNT + wl_game_use_powerup_randomness();
	wl_game.player[0].level = wl_game.player[0].start_level;
	wl_game.player[0].progress = (wl_game.player[0].start_level - 1) * WL_LEVEL_UP;
	wl_game.player[0].made_progress = false;
	wl_game.player[0].score = 0;
	wl_game.player[0].word_score = 0;
	wl_game.player[0].life = WL_MAX_LIFE;
	wl_game.player[0].lost = false;
	wl_game.player[0].life_fade = 0.0;
	wl_game.player[0].words = 0;
	wl_game.player[0].word_list_count = 0;
	wl_game.player[0].top_word_count = 0;
	wl_game.player[0].word[0] = '\0';
	wl_game.player[0].word_pos = 0;
	wl_game.player[0].word_exists = false;
	wl_game.player[0].x = WL_BOARD_CENTER_X;
	wl_game.player[0].y = WL_BOARD_CENTER_Y;
	wl_game.player[0].sprite.x = wl_game.player[0].x * WL_BOARD_TILE_SIZE;
	wl_game.player[0].sprite.y = wl_game.player[0].y * WL_BOARD_TILE_SIZE;
	wl_game.player[0].sprite.z = -320;
	wl_game.player[0].sprite.vz = WL_TERMINAL_VELOCITY;
	wl_game.player[0].hover_x = WL_BOARD_CENTER_X;
	wl_game.player[0].hover_y = WL_BOARD_CENTER_Y;
	wl_game.player[0].clicked = true;
	wl_game.player[0].range = 0;
	wl_game.player[0].powerups = 0;
	wl_game.player[0].words_this_level = 0;
	wl_game.player[0].counter = 0;
	wl_game.player[0].state = WL_PLAYER_STATE_ENTER;
	wl_game.player[0].sprite.type = WL_SPRITE_TYPE_PLAYER;
	wl_game.player[0].sprite.state = 0;
	wl_game.player[0].sprite.active = true;
	wl_game.player[0].click_queue.clicks = 0;
	wl_game.player[0].sprite.collider = t3f_create_collision_object(8, 8, 32 - 8, 32 - 8, 32, 32, 0);
	wl_game.player[0].sprite.cy = 24 + 1;
	wl_game.player[0].move[0] = wl_game_player_pos_to_move(wl_game.player[0].x, wl_game.player[0].y);
	wl_game.player[0].moves = 1;

	/* remember we already did the tutorial when continuing, retrying, etc. */
	if(mode != WL_LEVEL_LOAD_MUSIC)
	{
		wl_game.player[0].watched_tutorial = false;
	}

	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		wl_game.sprite[i].collider = t3f_create_collision_object(0, 0, 32, 32, 32, 32, 0);
		wl_game.sprite[i].active = false;
	}
	wl_game.guru_active = false;
	wl_game.guru_dir = -1.0;
	wl_game.hazard_count = wl_hazard_time[wl_game.player[0].level - 1] * 60 + rand() % 300 - 150;

	/* add play to profile */
	wl_profiles->entry[wl_game.player[0].profile].games++;
	wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_DEDICATION);

	/* expand profile word list so we can add the words from this game */
	if(!wl_expand_profile(wl_profiles, wl_game.player[0].profile, 1000))
	{
		return false;
	}

	al_stop_timer(t3f_timer);
	wl_load_level(&wl_level[wl_get_level_slot(wl_game.player[0].level - 1)], wl_game.player[0].level, mode, !wl_game.retry, wl_option[WL_OPTION_LOWRES] ? 1 : 0);
	al_start_timer(t3f_timer);

	/* generate initial background shapes */
	if(mode != WL_LEVEL_LOAD_MUSIC)
	{
		for(i = 0; i < WL_MAX_SHAPES; i++)
		{
			wl_bg_shape[i].flags = 0;
			wl_fg_shape[i].flags = 0;
		}
		wl_current_bg_shape = 0;
		wl_generate_shape(&wl_bg_shape[0], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image, 640, 0, 128.0, -1.0 / 4.0);
		while(wl_bg_shape[wl_current_bg_shape].ox > -64.0)
		{
			wl_current_bg_shape++;
			wl_connect_shape(&wl_bg_shape[wl_current_bg_shape], &wl_bg_shape[wl_current_bg_shape - 1], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image);
		}
		wl_current_bg_shape++;
		wl_current_fg_shape = 0;
		wl_generate_shape(&wl_fg_shape[0], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image, 640, 40, 0.0, -1.0 / 4.0);
		while(wl_fg_shape[wl_current_fg_shape].ox > -64.0)
		{
			wl_current_fg_shape++;
			wl_connect_shape(&wl_fg_shape[wl_current_fg_shape], &wl_fg_shape[wl_current_fg_shape - 1], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image);
		}
		wl_current_fg_shape++;
		wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes_used = true;
	}

	wl_word_messages.messages = 0;
	wl_game_messages.messages = 0;

	/* set up HUD */
	wl_game.player[0].progress_indicator = wl_game.player[0].progress;
	wl_game.player[0].life_indicator = 0.0;

	/* set state to start the game */
	wl_game.retry = false;
	wl_game.regenerate = false;
	wl_game.finished = false;
	wl_game.tick = 0;
	wl_game.want_new_level = 0;
	wl_game.word_list_ox = 640.0;
	wl_game.current_player = 0;
	if(wl_game.mode == WL_GAME_MODE_PARTY)
	{
		wl_add_message(&wl_game_messages, WL_MESSAGE_TYPE_SCROLLER, wl_profiles->entry[wl_game.player[wl_game.current_player].profile].name, wl_font[WL_FONT_32], 0);
		wl_game.state = WL_GAME_STATE_PLAYER_WAIT;
	}
	else
	{
		wl_game.state = WL_GAME_STATE_PLAY;
	}
	if(mode == WL_LEVEL_LOAD_NORMAL)
	{
		if(wl_game.mode == WL_GAME_MODE_MUSIC)
		{
			wl_state = WL_STATE_MUSIC;
		}
		else
		{
			wl_state = WL_STATE_GAME;
		}
	}
	return true;
}

void wl_game_scroll_level_name(void)
{
	wl_add_message(&wl_game_messages, WL_MESSAGE_TYPE_SCROLLER, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].name, wl_font[WL_FONT_32], 0);
}

void wl_game_end_story(void)
{
	int i;
	bool exploded = false;
	int bx, by;

	wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
	wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
	if(wl_game.guru_active)
	{
		wl_game.guru_state = 3;
		wl_game.player[0].powerups &= ~WL_POWERUP_GURU;
		wl_game.player[0].counter = 0;
	}

	/* destroy sprites */
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		if(wl_game.sprite[i].active)
		{
			if(!exploded)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_DESTROY], 1.0, 0.0, 1.0);
				exploded = true;
			}

			/* remove tile from beneath sprites that sit on the board */
			if(wl_game.sprite[i].type != WL_SPRITE_TYPE_LASER)
			{
				bx = (int)wl_game.sprite[i].x / WL_BOARD_TILE_SIZE;
				by = (int)wl_game.sprite[i].y / WL_BOARD_TILE_SIZE;
				switch(wl_game.board[by][bx].letter)
				{
					case '?':
					case '!':
					case '_':
					case '0':
					{
						wl_game.board[by][bx].letter = ' ';
						break;
					}
				}
			}
			wl_explode_sprite(&wl_game.sprite[i]);
		}
	}
	if(wl_game.finished)
	{
		al_stop_timer(t3f_timer);
		t3f_render(true);
		if(wl_demo)
		{
			wl_cinema = wl_load_cinema_script("data/scripts/demo_ending.script");
			if(wl_cinema)
			{
				wl_play_music("data/music/regret.ogg");
				wl_set_cinema_view_callbacks(wl_cinema, wl_ending_view_logic, wl_ending_view_render);
				wl_credits_initialize(&wl_credits, 1);
				wl_ending_state = 0;
				wl_select_menu(WL_MENU_DEMO_OVER);
				wl_state = WL_STATE_DEMO_WIN;
			}
		}
		else
		{
			wl_cinema = wl_load_cinema_script("data/scripts/ending.script");
			if(wl_cinema)
			{
				wl_play_music("data/music/ending.ogg");
				wl_set_cinema_view_callbacks(wl_cinema, wl_ending_view_logic, wl_ending_view_render);
				wl_credits_initialize(&wl_credits, 1);
				wl_ending_state = 0;
				wl_state = WL_STATE_GAME_WIN;
			}
		}
		al_start_timer(t3f_timer);
	}
	else
	{
		al_stop_timer(t3f_timer);
		t3f_render(true);
		wl_cinema = wl_load_cinema_script("data/scripts/game_over.script");
		if(!wl_cinema)
		{
			printf("Could not load game over cinema!\n");
			exit(0);
		}
		wl_set_cinema_view_callbacks(wl_cinema, wl_game_view_logic, wl_game_view_render);
		wl_play_music("data/music/regret.ogg");
		al_start_timer(t3f_timer);
		wl_game_update_best_flags();
		if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_AUTO && wl_game.player[0].word_list_count > 0)
		{
			wl_state = WL_STATE_GAME_OVER_WAIT;
			wl_leaderboard_mode = wl_game.mode;
			wl_leaderboard_type = 0;
			wl_leaderboard_style = 1;
			wl_game_upload_scores();
			wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
			wl_select_menu(WL_MENU_LEADERBOARD);
		}
		else if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_ASK && wl_game.player[0].word_list_count > 0)
		{
			wl_state = WL_STATE_GAME_OVER;
			wl_select_menu(WL_MENU_UPLOAD_ASK);
		}
		else
		{
			wl_state = WL_STATE_GAME_OVER;
			wl_select_menu(WL_MENU_GAME_OVER);
		}
	}
}

void wl_game_end(void)
{
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_game_end_story();
	}
	else
	{
		wl_game_tally_score();
		wl_state = WL_STATE_GAME_TALLY;
		if(wl_game.mode == WL_GAME_MODE_PUZZLE)
		{
			wl_tally_tick = 60;
		}
		else
		{
			wl_tally_tick = 0;
		}
		wl_game.player[0].hover_x = -1;
		wl_game.player[0].hover_y = -1;
	}
}

void wl_game_update_best_flags(void)
{
	int i, c, ci = -1, best = 0;
	int flag[4] = {WL_PROFILE_FLAG_UPLOAD_BEST, WL_PROFILE_FLAG_UPLOAD_BEST_C, WL_PROFILE_FLAG_UPLOAD_BEST_P, WL_PROFILE_FLAG_UPLOAD_BEST_G};

	/* find the best word in the word list from the game that just ended */
	for(i = 0; i < wl_game.player[0].word_list_count; i++)
	{
		c = wl_game_figure_total_points(wl_game.player[0].word_list[i]);
		if(c > best)
		{
			ci = i;
			best = c;
		}
	}
	wl_game.current_best_word = -1;
	if(ci >= 0)
	{
		wl_game.current_best_word = wl_game.player[0].word_list[ci];

		/* compare current best word to profile best word to see if we beat our best */
		if(wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode] >= 0)
		{
			if(wl_game_figure_total_points(wl_game.player[0].word_list[ci]) > wl_game_figure_total_points(wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode]))
			{
				wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode] = wl_game.player[0].word_list[ci];
				wl_profiles->entry[wl_game.player[0].profile].flags |= flag[wl_game.mode];
			}
		}
		else // we don't have a best word yet so use the best from this game */
		{
			wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode] = wl_game.player[0].word_list[ci];
			wl_profiles->entry[wl_game.player[0].profile].flags |= flag[wl_game.mode];
		}
	}
}

void wl_game_upload_scores(void)
{
	wl_leaderboard_upload_flags = 0; // reset temporary upload flags
	wl_upload_scores(false);
}

void wl_game_exit(void)
{
	char fnbuf[1024] = {0};
	int i;

	wl_save_profiles(wl_profiles, t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		t3f_destroy_collision_object(wl_game.sprite[i].collider);
	}
	for(i = 0; i < 2; i++)
	{
		wl_free_level(&wl_level[i]);
	}
	wl_reset_particles(0);
	wl_state = WL_STATE_TITLE;
	wl_select_menu(WL_MENU_MAIN);
}

void wl_game_process_bg(void)
{
	bool generate;
	int ps;
	int i;

	generate = true;
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_bg_shape[i].flags)
		{
			wl_process_shape(&wl_bg_shape[i]);
			if(wl_bg_shape[i].ox <= -64.0)
			{
				generate = false;
			}
		}
	}
	if(generate)
	{
		ps = wl_current_bg_shape - 1;
		if(ps < 0)
		{
			ps = WL_MAX_SHAPES - 1;
		}
		wl_connect_shape(&wl_bg_shape[wl_current_bg_shape], &wl_bg_shape[ps], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image);
		wl_current_bg_shape++;
		if(wl_current_bg_shape >= WL_MAX_SHAPES)
		{
			wl_current_bg_shape = 0;
		}
		wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes_used = true;
	}

	generate = true;
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_fg_shape[i].flags)
		{
			wl_process_shape(&wl_fg_shape[i]);
			if(wl_fg_shape[i].ox <= -32.0)
			{
				generate = false;
			}
		}
	}
	if(generate)
	{
		ps = wl_current_fg_shape - 1;
		if(ps < 0)
		{
			ps = WL_MAX_SHAPES - 1;
		}
		wl_connect_shape(&wl_fg_shape[wl_current_fg_shape], &wl_fg_shape[ps], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape[rand() % wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes], &wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shape_image);
		wl_current_fg_shape++;
		if(wl_current_fg_shape >= WL_MAX_SHAPES)
		{
			wl_current_fg_shape = 0;
		}
		wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes_used = true;
	}

	/* process bg image */
	wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].z -= wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].vz;
	wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].a += wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].va;
	if(wl_game.player[0].level - 2 >= 0)
	{
		wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].z -= wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].vz;
		wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].a += wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].va;
	}
}

int wl_get_best_word(void)
{
	int i, amount;
	int best_amount = 0;
	int best_i = -1;

	if(wl_game.player[0].word_list_count <= 0)
	{
		return -1;
	}
	for(i = 0; i < wl_game.player[0].word_list_count; i++)
	{
		amount = wl_game_figure_total_points(wl_game.player[0].word_list[i]);
		if(amount > best_amount)
		{
			best_i = i;
			best_amount = amount;
		}
	}
	return wl_game.player[0].word_list[best_i];
}

void wl_game_guru_logic(void)
{
	float a;

	if(wl_game.guru_active)
	{
		switch(wl_game.guru_state)
		{
			case 0:
			{
				break;
			}
			case 1:
			{
				break;
			}
			case 2:
			{
				if(wl_game.guru_timer < 30)
				{
					wl_game.guru_timer++;
					a = (float)wl_game.guru_timer / 240.0;
					wl_game.guru_color = al_map_rgba_f(a, a, a, a);
				}
				break;
			}
			case 3:
			{
				wl_game.guru_timer--;
				a = (float)wl_game.guru_timer / 240.0;
				wl_game.guru_color = al_map_rgba_f(a, a, a, a);
				if(wl_game.guru_timer <= 0)
				{
					wl_game.guru_active = false;
				}
				break;
			}
		}
	}

	/* make guru move up and down */
	wl_game.guru_z += wl_game.guru_vz;
	wl_game.guru_vz += 0.01 * wl_game.guru_dir;
	if(wl_game.guru_dir > 0.0 && wl_game.guru_vz >= 0.25)
	{
		wl_game.guru_dir = -wl_game.guru_dir;
	}
	else if(wl_game.guru_dir < 0.0 && wl_game.guru_vz <= -0.25)
	{
		wl_game.guru_dir = -wl_game.guru_dir;
	}
}

void wl_game_process_overlay_menu(T3F_GUI * mp)
{
	int i, l;
	float x, s = 32.0;

	wl_last_item = mp->hover_element;
	t3f_process_gui(mp, NULL);

	/* generate particles when changing menu selection */
	if(mp->hover_element >= 0 && wl_last_item != mp->hover_element)
	{
		if(mp->element[mp->hover_element].resource == (void **)&wl_font[WL_FONT_16])
		{
			s = 24.0;
		}
		l = wl_menu_get_element_characters(mp->element[mp->hover_element].data);
		for(i = 0; i < l; i++)
		{
			x = mp->ox + mp->element[mp->hover_element].ox;
			if(mp->element[mp->hover_element].flags & T3F_GUI_ELEMENT_CENTRE)
			{
				x -= (l * s) / 2;
			}
			wl_add_particle(1, WL_PARTICLE_TYPE_MFLASH, wl_animation[WL_ANIMATION_PARTICLE], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), x + i * s, mp->oy + mp->element[mp->hover_element].oy, 0.0, s, 10 + i * 2, 0);
		}
	}
}

void wl_game_logic(void)
{
	int i, x, y;

	if(wl_game.wipe_count >= 0)
	{
		wl_game.wipe_count++;
		if(wl_game.wipe_count >= 2 * 15)
		{
			wl_game.wipe_count = -1;
		}
		else if(wl_game.wipe_count % 2 == 0)
		{
			i = wl_game.wipe_count / 2 - 1;
			y = i;
			for(x = 0; x < i + 1; x++)
			{
				if(x < WL_MAX_BOARD_WIDTH && y < WL_MAX_BOARD_HEIGHT && !(x == 3 && y == 3))
				{
					wl_game.board[y][x].letter = wl_board_generate_letter(false);
					wl_add_board_particle(al_map_rgb_f(1.0, 1.0, 1.0), x, y, 10);
				}
				y--;
			}
		}
	}
	wl_process_particles(1);
	if(wl_game.uses_hazards)
	{
		wl_game_hazard_logic();
	}
	wl_clear_board_flags();
	wl_game_sprite_logic();
	wl_board_logic();
	wl_game_player_control();
	wl_game_player_logic();
	wl_player_rules_logic();
	wl_game_guru_logic();
	wl_game_process_bg();

	wl_message_logic(&wl_game_messages, false);
	wl_message_logic(&wl_word_messages, true);

	/* add time to profile */
	wl_profiles->entry[wl_game.player[0].profile].time++;

	if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		wl_menu_proc_overlay_menu(NULL, 0, NULL);
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
	}
	else
	{
		if(wl_state == WL_STATE_GAME_TALLY)
		{
		}
		else
		{
			wl_game_process_overlay_menu(wl_menu[WL_MENU_GAME_OVERLAY]);
		}
	}
	if(wl_game.tick > 0)
	{
		wl_game.tick--;
	}
}

void wl_game_render_bg(void)
{
	int i;
	ALLEGRO_COLOR color;
	ALLEGRO_BITMAP * bp = NULL;
	int list = 0;

	if(wl_game.tick > 0)
	{
		color = wl_fade_color(wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].color, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].color, (float)(WL_LEVEL_FADE_FRAMES - wl_game.tick) / WL_LEVEL_FADE_FRAMES);
	}
	else
	{
		color = wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].color;
	}

	wl_clear_cached_shapes(0);
	wl_clear_cached_shapes(1);
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_bg_shape[i].flags)
		{
			if(bp == NULL)
			{
				bp = *wl_bg_shape[i].parent_bp;
			}

			/* if we come across a different bitmap, switch lists */
			if(*wl_bg_shape[i].parent_bp != bp)
			{
				bp = *wl_bg_shape[i].parent_bp;
				list = 1 - list;
			}
			wl_cache_shape(list, &wl_bg_shape[i]);
		}
	}
	wl_draw_cached_shapes(0, wl_darken_color(color, 0.75));
	wl_draw_cached_shapes(1, wl_darken_color(color, 0.75));
	wl_clear_cached_shapes(0);
	wl_clear_cached_shapes(1);
	bp = NULL;
	list = 0;
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_fg_shape[i].flags)
		{
			if(bp == NULL)
			{
				bp = *wl_fg_shape[i].parent_bp;
			}

			/* if we come across a different bitmap, switch lists */
			if(*wl_fg_shape[i].parent_bp != bp)
			{
				bp = *wl_fg_shape[i].parent_bp;
				list = 1 - list;
			}
			wl_cache_shape(list, &wl_fg_shape[i]);
		}
	}
	wl_draw_cached_shapes(0, color);
	wl_draw_cached_shapes(1, color);
}

/* render text up to width 'width', removing the last bit and replacing with
 * '...' if it is too long */
static void wl_draw_truncated_textf(T3F_FONT * fp, ALLEGRO_COLOR color, float x, float y, float z, float width, int flags, const char * format, ...)
{
	char buf[1024] = {0};
	bool truncated = false;
	va_list vap;

	va_start(vap, format);
	vsnprintf(buf, 1024, format, vap);
	va_end(vap);

	/* keep taking away the last letter until we are at or below the desired width */
	while(t3f_get_text_width(fp, buf) > width)
	{
		buf[strlen(buf) - 1] = '\0';
		truncated = true;
	}

	/* if we actually truncated, add elipsis */
	if(truncated)
	{
		strncat(buf, "...", 1024);
	}

	t3f_draw_text(fp, color, x, y, z, 0, 0, flags, buf);
}

void wl_game_render_word_list_header(const char * header, float x, float y, bool shadow)
{
	if(shadow)
	{
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), x - 16.0 + 2.0, y + 2.0, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", header);
	}
	else
	{
		t3f_draw_textf(wl_ascii_font, WL_LIFE_FILL_COLOR, x - 16.0 - 1.0, y - 1.0, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", header);
	}
}

/* render a word list word, x refers to the center point between the word and the point display */
void wl_game_render_word_list_word(int word, float x, float y, float a, bool shadow)
{
	if(shadow)
	{
		wl_draw_truncated_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75 * a), x - 16.0 + 2.0, y + 2.0, 0, 192, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
//		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75 * a), x - 16.0 + 2.0, y + 2.0, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
	}
	else
	{
		wl_draw_truncated_textf(wl_ascii_font, al_map_rgba_f(a, a, a, a), x - 16.0, y, 0, 192, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
//		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(a, a, a, a), x - 16.0, y, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
	}
}

void wl_game_render_top_words_list(float ox, bool shadow)
{
	float a, sy, oy;
	int i;

	/* render top words */
	a = 1.0;
	sy = 0.0;
	wl_game_render_word_list_header("Top Words", ox, WL_BOARD_Y + sy, shadow);
	sy += 24;
	if(wl_game.player[wl_game.current_player].top_word_counter > 0)
	{
		oy = 0;
		for(i = 0; i < wl_game.player[wl_game.current_player].top_word_spot; i++)
		{
			a = 1.0;
			wl_game_render_word_list_word(wl_game.player[wl_game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 - oy, a, shadow);
		}

		/* render inserted word */
		a = 1.0 - (float)wl_game.player[wl_game.current_player].top_word_counter / 24.0;
		wl_game_render_word_list_word(wl_game.player[wl_game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 - oy, a, shadow);

		/* offset position of words below inserted word only while the counter is active (> 0) */
		if(wl_game.player[wl_game.current_player].top_word_counter > 0 && wl_game.player[wl_game.current_player].top_word_spot < wl_game.player[wl_game.current_player].top_word_count - 1)
		{
			oy = wl_game.player[wl_game.current_player].top_word_counter - 24;
		}
		for(i = wl_game.player[wl_game.current_player].top_word_spot + 1; i < wl_game.player[wl_game.current_player].top_word_count; i++)
		{
			a = 1.0;

			/* fade out word past end of list */
			if(i >= WL_MAX_TOP_WORDS - 1)
			{
				a = (float)wl_game.player[wl_game.current_player].top_word_counter / 24.0;
			}
			wl_game_render_word_list_word(wl_game.player[wl_game.current_player].top_word[i], ox, WL_BOARD_Y + sy + (i - 1) * 24 - oy, a, shadow);
		}
	}
	else
	{
		oy = 0;
		for(i = 0; i < wl_game.player[wl_game.current_player].top_word_count && i < WL_MAX_TOP_WORDS - 1; i++)
		{
			a = 1.0;
			wl_game_render_word_list_word(wl_game.player[wl_game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 + oy, a, shadow);
		}
	}
}

void wl_game_render_recent_words_list(float ox, bool shadow)
{
	float a, oy;
	int i, start;

	/* render word list */
	a = 1.0;
	oy = 24 * 8;
	start = wl_game.player[0].word_list_count - WL_MAX_WORD_LIST;
	if(wl_game.player[wl_game.current_player].word_list_count > WL_MAX_WORD_LIST && wl_game.player[wl_game.current_player].word_list_counter > 0)
	{
		start--;
		oy = 24 * 8 + wl_game.player[wl_game.current_player].word_list_counter - 24;
	}
	if(start < 0)
	{
		start = 0;
	}
	for(i = start; i < wl_game.player[wl_game.current_player].word_list_count; i++)
	{
		a = 1.0;
		if(wl_game.player[wl_game.current_player].word_list_counter > 0)
		{
			if(wl_game.player[wl_game.current_player].word_list_count > WL_MAX_WORD_LIST && i == start)
			{
				a = (float)wl_game.player[wl_game.current_player].word_list_counter / 24.0;
			}
			else if(i == wl_game.player[wl_game.current_player].word_list_count - 1)
			{
				a = 1.0 - (float)wl_game.player[wl_game.current_player].word_list_counter / 24.0;
			}
		}
		wl_game_render_word_list_word(wl_game.player[wl_game.current_player].word_list[i], ox, WL_BOARD_Y + (i - start) * 24 + oy, a, shadow);
	}

	/* render recent words header last so diappearing words go beneath it */
	a = 1.0;
	oy = 24 * 7;
	wl_game_render_word_list_header("Recent Words", ox, WL_BOARD_Y + oy, shadow);
}

void wl_game_render(void)
{
	int i, li = 0;
	float a, mx, my, ox, oy = 0.0, x, w, fi;
	float life_scale;
	float d;
	ALLEGRO_COLOR c1, c2, c3, c4;
	char helper_text[3] = {0};
	float life_rate;

	/* draw background */
	if(wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].bg_animation)
	{
		t3f_draw_rotated_animation(wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].bg_animation, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 0, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].cx, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].cy, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].ox, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].oy, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].z, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].a, 0);
		if(wl_game.player[0].level > 1 && wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation && wl_game.tick > 0)
		{
			a = (float)(wl_game.tick) / WL_LEVEL_FADE_FRAMES;
			mx = ((float)wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation->frame[0]->width - 640.0) / 2.0;
			my = ((float)wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation->frame[0]->height - 480.0) / 2.0;
			t3f_draw_rotated_animation(wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation, al_map_rgba_f(a, a, a, a), 0, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].cx, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].cy, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].cx - mx, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].cy - my, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].z, wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].a, 0);
		}
	}
	wl_game_render_bg();

	al_hold_bitmap_drawing(true);
	t3f_set_clipping_rectangle(WL_BOARD_X - 36, WL_BOARD_Y - 36, WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 72, WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE + 72);

	wl_board_render();
	if((wl_game.player[0].state == WL_PLAYER_STATE_NORMAL || wl_game.player[0].state == WL_PLAYER_STATE_JUMP) && wl_game.player[0].hover_x >= 0 && wl_game.player[0].hover_y >= 0 && !wl_game.player[0].clicked)
	{
		if(wl_game.player[0].state == WL_PLAYER_STATE_NORMAL && wl_game.player[0].hover_x == wl_game.player[0].x && wl_game.player[0].hover_y == wl_game.player[0].y)
		{
			if((wl_game.uses_invalid && strlen(wl_game.player[0].word) > 0) || wl_game.player[0].word_exists)
			{
				t3f_draw_animation(wl_animation[WL_ANIMATION_SELECTOR], WL_LIFE_FILL_COLOR, wl_game.tick, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
			}
			else
			{
				t3f_draw_animation(wl_animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(1.0, 0.5, 0.5, 1.0), wl_game.tick, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
			}
		}
		else if(wl_game.player[0].in_range)
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), wl_game.tick, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
		}
		else
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(0.8, 0.8, 0.8, 0.8), wl_game.tick, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
		}
	}

	/* draw particles */
	wl_render_particles(1, WL_PARTICLE_TYPE_FLASH);

	/* draw shadows */
	for(i = 0; i < wl_game.sprite_list.sprites; i++)
	{
		wl_sprite_shadow_render(wl_game.sprite_list.sprite[i]);
	}

	/* draw sprites */
	for(i = 0; i < wl_game.sprite_list.sprites; i++)
	{
		wl_sprite_render(wl_game.sprite_list.sprite[i]);
	}

	/* draw particles */
	wl_render_particles(1, WL_PARTICLE_TYPE_SHARD);

	al_hold_bitmap_drawing(false);
	t3f_set_clipping_rectangle(0, 0, 0, 0);

	/* draw guru */
	if(wl_game.guru_active)
	{
		t3f_draw_animation(wl_animation[WL_ANIMATION_GURU], wl_game.guru_color, 0, WL_BOARD_X, WL_BOARD_Y + 16.0 - wl_game.guru_z, 0.0, 0);
	}

	al_hold_bitmap_drawing(true);

	/* draw "HUD" */
	c1 = wl_fade_color(WL_LIFE_BG_COLOR, WL_LIFE_FREEZE_BG_COLOR, wl_game.player[0].life_fade);
	c2 = wl_fade_color(WL_LIFE_FILL_COLOR, WL_LIFE_FREEZE_FILL_COLOR, wl_game.player[0].life_fade);
	if(wl_game.player[0].life <= 10.0 && !(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		c3 = wl_fade_color(c2, t3f_color_white, 1.0 - (float)(wl_tick % 60) / 60.0);
	}
	else
	{
		c3 = c2;
	}
	if(wl_game.player[0].life <= 0.0 && !wl_game.player[0].lost && wl_state == WL_STATE_GAME)
	{
		c4 = wl_fade_color(c1, t3f_color_white, 1.0 - (float)(wl_tick % 60) / 50.0);
	}
	else
	{
		c4 = c1;
	}
//	al_draw_bitmap(wl_bitmap[WL_BITMAP_LIFE], WL_BOARD_X - 32 - 24, WL_BOARD_Y, 0);
	life_scale = WL_MAX_LIFE / (float)al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]);
	ox = WL_BOARD_X / 2 - al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE]) / 2;
	oy = 480 - (480 - (WL_BOARD_Y + WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE)) / 2 - al_get_bitmap_height(wl_bitmap[WL_BITMAP_LEVEL]) / 2;
	al_draw_tinted_bitmap(wl_bitmap[WL_BITMAP_LIFE], c4, ox, WL_BOARD_Y, 0);
	d = (float)al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]) - wl_game.player[0].life_indicator / life_scale;
//	al_draw_bitmap_region(wl_bitmap[WL_BITMAP_LIFE_FILL], 0, d, al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE_FILL]), al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]) - d, WL_BOARD_X - 32 - 24, WL_BOARD_Y + d, 0);
	al_draw_tinted_bitmap_region(wl_bitmap[WL_BITMAP_LIFE_FILL], c3, 0, d, al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE_FILL]), al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]) - d, ox, WL_BOARD_Y + d, 0);
	if(wl_game.uses_life && !(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		if(wl_game.mode == WL_GAME_MODE_GURU)
		{
			life_rate = wl_life_chart[9];
		}
		else
		{
			life_rate = wl_life_chart[wl_game.player[0].level - 1];
		}
		li = 0;
		for(fi = WL_MAX_LIFE - wl_game.player[0].life; fi < WL_MAX_LIFE; fi += life_rate)
		{
//			al_draw_tinted_bitmap(wl_bitmap[WL_BITMAP_LIFE_DEC], al_map_rgba_f(0.0, 0.0, 0.0, 0.125), ox, WL_BOARD_Y + fi / life_scale, 0);
			li++;
		}
	}
	al_draw_bitmap(wl_bitmap[WL_BITMAP_LIFE_OVERLAY], ox, WL_BOARD_Y, 0);
	al_draw_bitmap(wl_bitmap[WL_BITMAP_LEVEL], WL_BOARD_X, oy, 0);
	d = (float)al_get_bitmap_width(wl_bitmap[WL_BITMAP_LEVEL_FILL]) * (float)(wl_game.player[0].progress_indicator % WL_LEVEL_UP) / (float)WL_LEVEL_UP;
	al_draw_bitmap_region(wl_bitmap[WL_BITMAP_LEVEL_FILL], al_get_bitmap_width(wl_bitmap[WL_BITMAP_LEVEL_FILL]) - d - 1, 0, d, al_get_bitmap_height(wl_bitmap[WL_BITMAP_LEVEL_FILL]), WL_BOARD_X + 1, oy, 0);
	if(wl_game.uses_life && !(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		if(wl_game.player[0].life > 0.0)
		{
			if(wl_game.player[0].life <= WL_MAX_LIFE * 0.5)
			{
				a = 0.125;
			}
			else
			{
				a = (1.0 - ((wl_game.player[0].life - (WL_MAX_LIFE * 0.5))) / (WL_MAX_LIFE * 0.5)) * 0.125;
			}
			al_draw_textf(wl_font[WL_FONT_24], al_map_rgba_f(0.0, 0.0, 0.0, a), ox + 16.0, WL_BOARD_Y + (WL_MAX_LIFE - wl_game.player[0].life) / life_scale - al_get_font_line_height(wl_font[WL_FONT_24]), ALLEGRO_ALIGN_CENTER, "%d", li);
		}
	}

	/* draw particles */
	wl_render_particles(1, WL_PARTICLE_TYPE_ICE);
	wl_render_particles(1, WL_PARTICLE_TYPE_STEAM);

	/* render current word */
	x = (float)WL_BOARD_X + (float)WL_BOARD_TILE_SIZE * ((float)WL_MAX_BOARD_WIDTH / 2.0);
	oy = WL_BOARD_Y - 52;
	t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), x + 2, oy + 2, 0, 0, 0, T3F_FONT_ALIGN_CENTER, "( %s )", wl_game.player[0].word);
	if((wl_game.uses_invalid && strlen(wl_game.player[0].word) > 0) || wl_game.player[0].word_exists)
	{
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), x, oy, 0, 0, 0, T3F_FONT_ALIGN_CENTER, "( %s )", wl_game.player[0].word);
	}
	else
	{
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(1.0, 0.5, 0.5, 1.0), x + 1, oy + 1, 0, 0, 0, T3F_FONT_ALIGN_CENTER, "( %s )", wl_game.player[0].word);
	}

	/* render score */
	if(wl_game.word_list_ox >= 640.0)
	{
		w = t3f_get_text_width(wl_ascii_font, "Score: 000000");
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 640 - 16 - w + 2, WL_BOARD_Y - 24 * 2 + 2, 0, 0, 0, 0, "Score:");
		t3f_draw_textf(wl_ascii_font, WL_LIFE_FILL_COLOR, 640 - 16 - w - 1, WL_BOARD_Y - 24 * 2 - 1, 0, 0, 0, 0, "Score:");
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 640 - 16 + 2, WL_BOARD_Y - 24 * 2 + 2, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%d", wl_game.player[wl_game.current_player].score);
		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 640 - 16, WL_BOARD_Y - 24 * 2, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%d", wl_game.player[wl_game.current_player].score);
	}

	if(wl_game.word_list_ox < 640.0)
	{
		al_hold_bitmap_drawing(false);
		al_draw_filled_rectangle(0, 0, 640, 480, al_map_rgba_f(0.0, 0.0, 0.0, ((640.0 - wl_game.word_list_ox) / 320.0) / 2.9));
		al_hold_bitmap_drawing(true);
	}

	wl_game_render_top_words_list(wl_game.word_list_ox, true);
	wl_game_render_top_words_list(wl_game.word_list_ox, false);
	if(wl_game.word_list_ox >= 640.0)
	{
		wl_game_render_recent_words_list(wl_game.word_list_ox, true);
		wl_game_render_recent_words_list(wl_game.word_list_ox, false);
	}

	/* render touch helper */
	if((wl_game.player[0].state == WL_PLAYER_STATE_NORMAL || wl_game.player[0].state == WL_PLAYER_STATE_JUMP) && wl_game.player[0].hover_x >= 0 && wl_game.player[0].hover_y >= 0)
	{
		if(wl_game.player[0].touching && wl_game.player[0].touch_id != 0 && !wl_game.player[0].clicked)
		{
			if(wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter != ' ')
			{
				al_draw_bitmap(wl_bitmap[WL_BITMAP_POPUP], WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE + (WL_BOARD_TILE_SIZE / 2) - al_get_bitmap_width(wl_bitmap[WL_BITMAP_POPUP]) / 2, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE - al_get_bitmap_height(wl_bitmap[WL_BITMAP_POPUP]), 0);

				/* render powerup animation */
				if(wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter == '?')
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_GURU], t3f_color_white, 0, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - al_get_bitmap_width(wl_animation[WL_ANIMATION_POWER_GURU]->bitmaps->bitmap[0]) / 2, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				else if(wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter == '!')
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_HEALTH], t3f_color_white, 0, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - al_get_bitmap_width(wl_animation[WL_ANIMATION_POWER_HEALTH]->bitmaps->bitmap[0]) / 2, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				else if(wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter == '_')
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_HOURGLASS], t3f_color_white, 0, WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - al_get_bitmap_width(wl_animation[WL_ANIMATION_POWER_HOURGLASS]->bitmaps->bitmap[0]) / 2, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				/* otherwise it's not a powerup, render character */
				else
				{
					if(wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter == 'Q')
					{
						sprintf(helper_text, "%s", "Qu");
					}
					else
					{
						sprintf(helper_text, "%c", wl_game.board[wl_game.player[0].hover_y][wl_game.player[0].hover_x].letter);
					}
					al_draw_text(wl_font[WL_FONT_32], al_map_rgba_f(0.0, 0.0, 0.0, 1.0), WL_BOARD_X + wl_game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2, WL_BOARD_Y + wl_game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 6, ALLEGRO_ALIGN_CENTRE, helper_text);
				}
			}
		}
	}

	/* render pause overlay menu */
	if(wl_state == WL_STATE_GAME_TALLY)
	{
	}
	else
	{
		t3f_render_gui(wl_menu[WL_MENU_GAME_OVERLAY]);
	}
	wl_render_particles(1, WL_PARTICLE_TYPE_MFLASH);

	/* draw particles */
	wl_render_particles(1, WL_PARTICLE_TYPE_GLASS);

	wl_render_messages(&wl_game_messages, false);
	wl_render_messages(&wl_word_messages, true);
	al_hold_bitmap_drawing(false);
}

void wl_game_paused_logic(void)
{
	wl_process_cinema(wl_cinema);

	if(wl_cinema->tick >= wl_cinema->length)
	{
		wl_title_process_menu();
	}
}

void wl_game_paused_render(void)
{
	wl_render_cinema(wl_cinema);
	if(wl_cinema->tick >= wl_cinema->length)
	{
		wl_title_render_menu();
	}
}

void wl_game_win_logic(void)
{
	int i;
	int frames = 1;

	if(t3f_key[ALLEGRO_KEY_RIGHT])
	{
		frames = 16;
	}
	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(wl_cinema);
		if(wl_cinema->tick >= wl_cinema->length)
		{
			wl_credits_logic(&wl_credits);
		}
	}
	if(wl_ending_state == 0)
	{
		if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK] || wl_credits.done)
		{
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
			wl_ending_state = 1;
			t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
			t3f_key[ALLEGRO_KEY_BACK] = 0;
			t3f_clear_touch_data();
		}
	}
	else
	{
		if(wl_fade_tick <= 0)
		{
			wl_game_update_best_flags();
			if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_AUTO && wl_game.player[0].word_list_count > 0)
			{
				t3f_stop_music();
				wl_state = WL_STATE_GAME_DONE;
				wl_leaderboard_mode = wl_game.mode;
				wl_leaderboard_type = 0;
				wl_leaderboard_style = 1;
				wl_game_upload_scores();
				wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
				wl_select_menu(WL_MENU_LEADERBOARD);
			}
			else if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_ASK && wl_game.player[0].word_list_count > 0)
			{
				t3f_stop_music();
				wl_state = WL_STATE_GAME_DONE;
				wl_select_menu(WL_MENU_UPLOAD_ASK);
			}
			else
			{
				wl_game_exit();
				wl_select_menu(WL_MENU_MAIN);
				wl_play_music("data/music/theme.ogg");
				wl_state = WL_STATE_TITLE;
			}
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
}

void wl_game_win_render(void)
{
	wl_render_cinema(wl_cinema);
	if(wl_cinema->tick >= wl_cinema->length)
	{
		wl_credits_render(&wl_credits);
	}
}

void wl_demo_win_logic(void)
{
	int i;
	int frames = 1;

	if(t3f_key[ALLEGRO_KEY_RIGHT])
	{
		frames = 16;
	}
	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(wl_cinema);
	}
	if(wl_ending_state == 0)
	{
		if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK] || wl_credits.done)
		{
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
			wl_ending_state = 1;
			t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
			t3f_key[ALLEGRO_KEY_BACK] = 0;
			t3f_clear_touch_data();
		}
		if(wl_cinema->tick >= wl_cinema->length)
		{
			wl_title_process_menu();
		}
	}
	else
	{
		if(wl_fade_tick <= 0)
		{
			wl_game_update_best_flags();
			if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_AUTO && wl_game.player[0].word_list_count > 0)
			{
				t3f_stop_music();
				wl_state = WL_STATE_GAME_DONE;
				wl_leaderboard_mode = wl_game.mode;
				wl_leaderboard_type = 0;
				wl_leaderboard_style = 1;
				wl_game_upload_scores();
				wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
				wl_select_menu(WL_MENU_LEADERBOARD);
			}
			else if(wl_game.player[0].profile > 0 && wl_option[WL_OPTION_INTERNET] == WL_INTERNET_ASK && wl_game.player[0].word_list_count > 0)
			{
				t3f_stop_music();
				wl_state = WL_STATE_GAME_DONE;
				wl_select_menu(WL_MENU_UPLOAD_ASK);
			}
			else
			{
				wl_game_exit();
				wl_select_menu(WL_MENU_MAIN);
				wl_play_music("data/music/theme.ogg");
				wl_state = WL_STATE_TITLE;
			}
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
}

void wl_demo_win_render(void)
{
	wl_render_cinema(wl_cinema);
	if(wl_cinema->tick >= wl_cinema->length)
	{
		wl_title_render_menu();
	}
}

void wl_game_paused_2_logic(void)
{
	/* game logic */
	wl_process_particles(1);
	wl_clear_board_flags();
	wl_game_sprite_logic();
	wl_board_logic();
//	wl_game_player_logic();
	wl_game_process_bg();

	wl_message_logic(&wl_game_messages, false);
	wl_message_logic(&wl_word_messages, true);

	wl_process_particles(0);
	wl_title_process_menu();
}

void wl_game_paused_2_render(void)
{
	wl_game_render();
	al_hold_bitmap_drawing(false);
	al_draw_filled_rectangle(0.0, 0.0, 640.0, 480.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
	al_hold_bitmap_drawing(true);
	wl_title_render_menu();
	al_hold_bitmap_drawing(false);
}
