#ifndef WL_GAME_H
#define WL_GAME_H

#include "t3f/rng.h"
#include "player.h"
#include "sprite.h"
#include "particle.h"
#include "board.h"

#define WL_GAME_STATE_PLAY        0
#define WL_GAME_STATE_PLAYER_WAIT 1
#define WL_GAME_STATE_DONE        2

#define WL_LIFE_BG_COLOR          al_map_rgba(255, 50, 95, 255)
#define WL_LIFE_FILL_COLOR        al_map_rgba(0, 189, 82, 255)
#define WL_LIFE_FREEZE_BG_COLOR   al_map_rgba(101, 188, 255, 255)
#define WL_LIFE_FREEZE_FILL_COLOR al_map_rgba(193, 223, 255, 255)

typedef struct
{
	
	int mode;
	WL_BOARD_TILE board[WL_MAX_BOARD_HEIGHT][WL_MAX_BOARD_WIDTH];
	WL_PLAYER player[WL_MAX_PLAYERS];
	int powerup[WL_MAX_POWERUPS];
	int powerups;
	int powerup_count;
	int powerup_randomness;
	int hazard_count;
	WL_SPRITE sprite[WL_MAX_SPRITES];
	WL_SPRITE_LIST sprite_list;
	bool finished, retry, regenerate;
	
	/* guru stuff */
	bool guru_active;
	float guru_z, guru_vz, guru_dir;
	float guru_x, guru_y;
	int guru_state;
	int guru_timer;
	ALLEGRO_COLOR guru_color;
	
	/* game mode feature toggles */
	bool uses_powerups;
	bool uses_hazards;
	bool uses_life;
	bool uses_level_progress;
	bool uses_invalid;
	
	int time;
	int tick;
	double music_pos, old_music_pos;
	double music_length;
	bool fast_forward;
	float music_fade_level;
	bool music_done;
	float word_list_ox;
	int want_new_level;
	int current_best_word;
	int current_unused_letters;
	int wipe_count; // >= 0 means we are processing a wipe
	T3F_RNG_STATE rng_state;
	int players;
	int current_player;
	int state;
	
} WL_GAME;

void wl_game_generate_powerups(void);
int wl_get_best_word(void);
int wl_game_figure_base_points(const char * word);
int wl_game_figure_total_points(int w);
void wl_generate_board_particles(ALLEGRO_COLOR color, int x, int y, int r);
int wl_game_use_powerup_randomness(void);

void wl_game_setup_type(void);
bool wl_game_initialize(int mode);
void wl_game_scroll_level_name(void);
void wl_game_exit();
void wl_game_end(void);
void wl_game_update_best_flags(void);
void wl_game_upload_scores(void);
void wl_game_process_bg(void);
void wl_game_guru_logic(void);
void wl_game_process_overlay_menu(T3F_GUI * mp);
void wl_game_logic(void);
void wl_game_render_bg(void);
void wl_game_render(void);
void wl_game_paused_logic(void);
void wl_game_paused_render(void);
void wl_game_paused_2_logic(void);
void wl_game_paused_2_render(void);
void wl_game_credits_logic(void);
void wl_game_credits_render(void);
void wl_game_win_logic(void);
void wl_game_win_render(void);
void wl_demo_win_logic(void);
void wl_demo_win_render(void);

#endif
