#include "t3f/music.h"
#include "cinema.h"
#include "data.h"
#include "game_sprite.h"
#include "player.h"
#include "effects.h"
#include "music_queue.h"

void wl_game_intro_logic(void)
{
	int frames = 1;
	int i;

	if(t3f_key[ALLEGRO_KEY_RIGHT])
	{
		frames = 16;
	}
	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(wl_cinema);
	}
	if(wl_cinema->tick >= wl_cinema->length || t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		wl_destroy_cinema(wl_cinema);
		wl_cinema = NULL;
		wl_clear_music_queue();
		wl_game.retry = false;
		wl_game_initialize(WL_LEVEL_LOAD_MUSIC);
		
		/* don't scroll level name if we are going to be showing the tutorial */
		if(!wl_player_needs_tutorial())
		{
			wl_game_scroll_level_name();
		}
		wl_state = WL_STATE_GAME;
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
		t3f_clear_touch_data();
	}
}

static int click_count = 0;

void wl_game_intro_skip_logic(void)
{
	int i;
	bool click = false;
	
	for(i = 0; i < T3F_MAX_TOUCHES; i++)
	{
		if(t3f_touch[i].active)
		{
			click = true;
		}
	}
	if(t3f_key[ALLEGRO_KEY_SPACE] || t3f_key[ALLEGRO_KEY_ENTER])
	{
		click = true;
	}
	if(click)
	{
		click_count++;
	}
	else
	{
		click_count = 0;
	}
	if(click_count == 1)
	{
		while(!wl_process_cinema(wl_cinema))
		{
		}
	}
}

void wl_game_intro_render(void)
{
	if(wl_game.mode == WL_GAME_MODE_STORY && wl_overlay_state != WL_STATE_GAME_TUTORIAL)
	{
		wl_render_cinema(wl_cinema);
	}
	else
	{
		wl_game_render();
	}
}

void wl_game_view_logic(void)
{
	wl_game.player[0].life = 0;
	wl_process_particles(1);
	wl_clear_board_flags();
	wl_game_sprite_logic();
	wl_board_logic();
	if(wl_state != WL_STATE_GAME_INTRO)
	{
		wl_game_player_logic();
	}
	wl_game_guru_logic();
	wl_game_process_bg();
}

void wl_game_view_render(void)
{
	wl_game_render();
}

void wl_game_paused_view_logic(void)
{
	wl_game_process_bg();
}

void wl_game_continue_logic(void)
{
	wl_process_cinema(wl_cinema);
	if(wl_cinema->tick >= wl_cinema->length)
	{
		wl_destroy_cinema(wl_cinema);
		wl_cinema = NULL;
		wl_state = WL_STATE_GAME;
	}
}

void wl_game_quit_logic(void)
{
	if(wl_game.mode == WL_GAME_MODE_STORY && wl_overlay_state != WL_STATE_GAME_TUTORIAL)
	{
		wl_process_cinema(wl_cinema);
		if(wl_cinema->tick >= wl_cinema->length)
		{
			wl_destroy_cinema(wl_cinema);
			wl_cinema = NULL;
			wl_game_exit();
			wl_play_music("data/music/theme.ogg");
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
	else
	{
		/* game logic */
		wl_process_particles(1);
		wl_clear_board_flags();
		wl_game_sprite_logic();
		wl_board_logic();
		wl_game_player_logic();
		wl_game_process_bg();
		if(!wl_fading)
		{
			wl_game_exit();
			wl_play_music("data/music/theme.ogg");
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
}

void wl_game_exit_logic(void)
{
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_process_cinema(wl_cinema);
		if(wl_cinema->tick >= wl_cinema->length)
		{
			wl_destroy_cinema(wl_cinema);
			wl_cinema = NULL;
			wl_state = -1;
			wl_game_exit();
			t3f_exit();
		}
	}
	else
	{
		/* game logic */
		wl_process_particles(1);
		wl_clear_board_flags();
		wl_game_sprite_logic();
		wl_board_logic();
		wl_game_player_logic();
		wl_game_process_bg();
		if(!wl_fading)
		{
			wl_game_exit();
			t3f_exit();
		}
	}
}

void wl_ending_view_logic(void)
{
	if(wl_cinema->scene == 0)
	{
		wl_game_view_logic();
	}
	else
	{
		wl_title_word_logic();
	}
}

void wl_ending_view_render(void)
{
	if(wl_cinema->scene == 0)
	{
		wl_game_view_render();
	}
	else
	{
		wl_title_word_render(wl_font[WL_FONT_24], 0.125);
	}
}
