#include "t3f/t3f.h"
#include "particle.h"
#include "music_queue.h"
#include "effects.h"
#include "data.h"
#include "menu_proc.h"

void wl_game_music_logic(void)
{
	int i;
	int f = 1;

	wl_process_particles(1);
	if(wl_game.fast_forward)
	{
		f = 8;
		if(t3f_get_music_state() == T3F_MUSIC_STATE_PLAYING)
		{
			al_set_audio_stream_speed(t3f_stream, 8.0);
		}
		wl_set_music_fade_speed(WL_MUSIC_FADE_SPEED * 8.0);
	}
	else
	{
		if(t3f_get_music_state() == T3F_MUSIC_STATE_PLAYING)
		{
			al_set_audio_stream_speed(t3f_stream, 1.0);
			t3f_set_music_volume(t3f_get_music_volume());
		}
		wl_set_music_fade_speed(WL_MUSIC_FADE_SPEED);
	}
	if(wl_game.fast_forward && !t3f_touch[0].active)
	{
		wl_game.fast_forward = false;
	}
	for(i = 0; i < f; i++)
	{
		wl_game_process_bg();
	}

	/* add time to profile */
	wl_profiles->entry[wl_game.player[0].profile].time++;

	if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		wl_menu_proc_overlay_menu(NULL, 0, NULL);
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
	}
	if(t3f_get_music_state() == T3F_MUSIC_STATE_PLAYING)
	{

		/* get music length if it hasn't been gotten */
		if(wl_game.music_length <= 0.0)
		{
			wl_game.music_length = al_get_audio_stream_length_secs(t3f_stream);
		}

		wl_game.old_music_pos = wl_game.music_pos;
		wl_game.music_pos = al_get_audio_stream_position_secs(t3f_stream);
		if(wl_game.music_pos < 0.0)
		{
			wl_game.music_pos = 0.0;
		}

		/* if we have looped, go to next track */
		if(wl_get_music_queue_state() == WL_MUSIC_QUEUE_STATE_PLAYING && (wl_game.music_pos < wl_game.old_music_pos || wl_game.music_pos >= wl_game.music_length))
		{
			if((wl_game.player[0].start_level <= 10 && wl_game.player[0].level < 10) || (wl_game.player[0].start_level > 10 && wl_game.player[0].level <= 15))
			{
				wl_game.player[0].level++;
				al_stop_timer(t3f_timer);
				wl_load_level(&wl_level[wl_get_level_slot(wl_game.player[0].level - 1)], wl_game.player[0].level, WL_LEVEL_LOAD_NORMAL, true, wl_option[WL_OPTION_LOWRES] ? 1 : 0);
				al_start_timer(t3f_timer);
				wl_game.tick = WL_LEVEL_FADE_FRAMES;
			}

			/* fade out and go back to title screen */
			else
			{
				wl_finish_current_song();
				wl_game.music_done = true;
			}
		}
	}
	else
	{
		wl_game.music_pos = 0.0;
		wl_game.old_music_pos = 0.0;
		wl_game.music_length = 0.0;
		wl_game.fast_forward = false;
	}
	if(wl_state == WL_STATE_MUSIC && !wl_game.music_done)
	{
		wl_game_process_overlay_menu(wl_menu[WL_MENU_MUSIC_OVERLAY]);
	}
	if(wl_game.tick > 0)
	{
		wl_game.tick -= f;
		if(wl_game.tick < 0)
		{
			wl_game.tick = 0;
		}
	}
	if(wl_game.music_done)
	{
		for(i = 0; i < f; i++)
		{
			wl_game.music_fade_level += 1.0 / (float)WL_LEVEL_FADE_FRAMES;
			if(wl_game.music_fade_level >= 1.0)
			{
				t3f_render(true);
				wl_game_exit();
				wl_play_music("data/music/theme.ogg");
				wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
				break;
			}
		}
	}
}

void wl_game_render_music_static_info(float offx, float offy, ALLEGRO_COLOR color)
{
	float ox, oy;
	char buf[64] = {0};

	ox = 16 + t3f_get_text_width(wl_ascii_font, "Musician: ");
	oy = 12;

	sprintf(buf, "Title: ");
	t3f_draw_text(wl_ascii_font, color, ox + offx, oy + offy, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, buf);
	oy += 24;

	sprintf(buf, "Artist: ");
	t3f_draw_text(wl_ascii_font, color, ox + offx, oy + offy, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, buf);
	oy += 24;

	sprintf(buf, "Music: ");
	t3f_draw_text(wl_ascii_font, color, ox + offx, oy + offy, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, buf);
	oy += 24;

	sprintf(buf, "Musician: ");
	t3f_draw_text(wl_ascii_font, color, ox + offx, oy + offy, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, buf);
}

void wl_game_render_music_info(int level, float offx, float offy, ALLEGRO_COLOR color, float fade)
{
	float ox, oy;
	char buf[64] = {0};

	ox = 16 + t3f_get_text_width(wl_ascii_font, "Musician: ");
	oy = 12;

	sprintf(buf, "Title: ");
	t3f_draw_text(wl_ascii_font, wl_alpha_color(color, fade), ox + offx, oy + offy, 0, 0, 0, 0, wl_level[level].name);
	oy += 24;

	sprintf(buf, "Artist: ");
	t3f_draw_text(wl_ascii_font, wl_alpha_color(color, fade), ox + offx, oy + offy, 0, 0, 0, 0, "Travis Cope");
	oy += 24;

	sprintf(buf, "Music: ");
	t3f_draw_text(wl_ascii_font, wl_alpha_color(color, fade), ox + offx, oy + offy, 0, 0, 0, 0, wl_level[level].music_title);
	oy += 24;

	sprintf(buf, "Musician: ");
	t3f_draw_text(wl_ascii_font, wl_alpha_color(color, fade), ox + offx, oy + offy, 0, 0, 0, 0, wl_level[level].music_artist);
	oy += 24;
}

void wl_game_music_render(void)
{
	float a, mx, my;
	int s;

	/* draw background */
	if(wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].bg_animation)
	{
		s = wl_get_level_slot(wl_game.player[0].level - 1);
		t3f_draw_rotated_animation(wl_level[s].bg_animation, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), 0, wl_level[s].cx, wl_level[s].cy, wl_level[s].ox, wl_level[s].oy, wl_level[s].z, wl_level[s].a, 0);
		if(wl_game.player[0].level > 1 && wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation && wl_game.tick > 0)
		{
			s = wl_get_level_slot(wl_game.player[0].level - 2);
			a = (float)(wl_game.tick) / WL_LEVEL_FADE_FRAMES;
			mx = ((float)wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation->frame[0]->width - 640.0) / 2.0;
			my = ((float)wl_level[wl_get_level_slot(wl_game.player[0].level - 2)].bg_animation->frame[0]->height - 480.0) / 2.0;
			t3f_draw_rotated_animation(wl_level[s].bg_animation, al_map_rgba_f(a, a, a, a), 0, wl_level[s].cx, wl_level[s].cy, wl_level[s].cx - mx, wl_level[s].cy - my, wl_level[s].z, wl_level[s].a, 0);
		}
	}
	wl_game_render_bg();
	if(wl_state == WL_STATE_MUSIC && !wl_game.music_done)
	{
		t3f_render_gui(wl_menu[WL_MENU_MUSIC_OVERLAY]);
		wl_render_particles(1, WL_PARTICLE_TYPE_MFLASH);
	}

	/* render song info */
	wl_game_render_music_static_info(2.0, 2.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.75));
	wl_game_render_music_static_info(0.0, 0.0, WL_LIFE_FILL_COLOR);
	if(wl_game.tick > 0)
	{
		a = (float)wl_game.tick / WL_LEVEL_FADE_FRAMES;
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 2), 2.0, 2.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), a);
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 1), 2.0, 2.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 1.0 - a);
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 2), 0.0, 0.0, t3f_color_white, a);
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 1), 0.0, 0.0, t3f_color_white, 1.0 - a);
	}
	else
	{
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 1), 2.0, 2.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 1.0);
		wl_game_render_music_info(wl_get_level_slot(wl_game.player[0].level - 1), 0.0, 0.0, t3f_color_white, 1.0);
	}
	if(wl_game.music_done)
	{
		al_draw_filled_rectangle(0.0, 0.0, 640.0, 480.0, al_map_rgba_f(0.0, 0.0, 0.0, wl_game.music_fade_level));
	}
}

void wl_game_music_quit_logic(void)
{
	wl_game_process_bg();

	/* add time to profile */
	wl_profiles->entry[wl_game.player[0].profile].time++;

	if(wl_game.tick > 0)
	{
		wl_game.tick--;
	}

	/* see if fade is done and exit game if so */
	if(!wl_fading)
	{
		wl_game_exit();
		wl_play_music("data/music/theme.ogg");
		wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
	}
}
