#include "t3f/t3f.h"
#include "t3f/collision.h"
#include "data.h"

/* qsort function for depth-sorting sprites by y-coordinate */
static int wl_sprite_sorter(const void * item_1, const void * item_2)
{
	WL_SPRITE * sp1 = *(WL_SPRITE **)(item_1);
	WL_SPRITE * sp2 = *(WL_SPRITE **)(item_2);
	
	if(sp1->y + sp1->cy < sp2->y + sp2->cy)
	{
		return -1;
	}
	else if(sp2->y + sp2->cy < sp1->y + sp1->cy)
	{
		return 1;
	}
	return 0;
}

/* sort the sprite list using the above helper function */
void wl_sprite_list_sort(WL_SPRITE_LIST * lp)
{
   qsort(lp->sprite, lp->sprites, sizeof(WL_SPRITE *), wl_sprite_sorter);
}

/* provide some default values to newly created sprites */
void wl_game_initialize_sprite(WL_SPRITE * sp)
{
	switch(sp->type)
	{
		case WL_SPRITE_TYPE_POWERUP_HEALTH:
		case WL_SPRITE_TYPE_POWERUP_GURU:
		case WL_SPRITE_TYPE_POWERUP_SNEAKER:
		case WL_SPRITE_TYPE_100:
		{
			sp->z = 0.0;
			sp->vx = 0.0;
			sp->vy = 0.0;
			sp->vz = -0.75;
			sp->state = 0;
//			t3f_recreate_collision_object(sp->collider, 8, 8, 32 - 8, 32 - 8, 32, 32, 0);
			break;
		}
	}
}

/* active a sprite of the specified type in an unused slot in the sprite[] array */
int wl_game_add_sprite(int type)
{
	int i;
	
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		if(!wl_game.sprite[i].active)
		{
			wl_game.sprite[i].type = type;
			wl_game_initialize_sprite(&wl_game.sprite[i]);
			wl_game.sprite[i].active = true;
			return i;
		}
	}
	return -1;
}

/* build the sprite list */
void wl_game_sprite_logic(void)
{
	int i;
	
	/* process player sprite logic and add it to the sprite list first */
	wl_game.sprite_list.sprites = 0;
	wl_sprite_logic(&wl_game.player[0].sprite);
	wl_game.sprite_list.sprite[wl_game.sprite_list.sprites] = &wl_game.player[0].sprite;
	wl_game.sprite_list.sprites++;
	
	/* process sprite logic for all other sprites and add them to the sprite list */
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		if(wl_game.sprite[i].active)
		{
			wl_sprite_logic(&wl_game.sprite[i]);
			if(wl_game.sprite[i].active)
			{
				wl_game.sprite_list.sprite[wl_game.sprite_list.sprites] = &wl_game.sprite[i];
				wl_game.sprite_list.sprites++;
			}
		}
	}
	
	/* depth-sort sprites */
	wl_sprite_list_sort(&wl_game.sprite_list);
}

