#include "data.h"
#include "text.h"
#include "game.h"
#include "game_sprite.h"
#include "game_hazard.h"

static int wl_game_tutorial_sprite = -1;
static bool touched = false;
static float wl_tutorial_text_offset_x = WL_BOARD_X + 16;
static float wl_tutorial_text_offset_y = WL_BOARD_Y + ((WL_MAX_BOARD_HEIGHT / 2) * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2);
static float wl_tutorial_text_width = WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE - 32;

bool wl_tutorial_click_to_next_step(int step)
{
	bool ret = false;
	if(touched || t3f_key[ALLEGRO_KEY_SPACE])
	{
		if(!wl_game.player[0].clicked)
		{
			wl_tutorial_step = step;
			wl_game.player[0].clicked = true;
			ret = true;
		}
	}
	else
	{
		wl_game.player[0].clicked = false;
	}
	wl_game.player[0].hover_x = -1;
	return ret;
}

void wl_game_tutorial_logic_story(void)
{
	int f, i, w, x, y;
	
	w = wl_game.player[0].word_list_count;
	f = wl_game.player[0].powerups;
	
	switch(wl_tutorial_step)
	{
		case 14:
		case 15:
		case 16:
		case 18:
		case 19:
		case 20:
		case 21:
		case 23:
		case 24:
		case 27:
		case 28:
		case 29:
		case 34:
		case 37:
		case 38:
		case 39:
		case 43:
		{
			wl_tutorial_click_to_next_step(wl_tutorial_step + 1);
			break;
		}
		case 17:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].word_list_count > w)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 22:
		{
			for(i = 0; i < WL_MAX_SPRITES; i++)
			{
				if(!wl_game.sprite[i].active)
				{
					wl_game.sprite[i].type = WL_SPRITE_TYPE_LASER;
					wl_game_initialize_hazard(&wl_game.sprite[i]);
					wl_game_tutorial_sprite = i;
					wl_game.sprite[i].flags = 1;
					break;
				}
			}
			wl_tutorial_step++;
			break;
		}
		case 25:
		{
			/* clear flags so laser will move */
			wl_game.sprite[wl_game_tutorial_sprite].flags = 0;
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].sprite.state == WL_PLAYER_SPRITE_STATE_HURT)
			{
				wl_tutorial_step++;
				for(i = 0; i < WL_MAX_SPRITES; i++)
				{
					if(!wl_game.sprite[i].active)
					{
						wl_game.sprite[i].type = WL_SPRITE_TYPE_LASER;
						wl_game_initialize_hazard(&wl_game.sprite[i]);
						wl_game_tutorial_sprite = i;
						wl_game.sprite[i].flags = 1;
						break;
					}
				}
			}
			else if(!wl_game.sprite[wl_game_tutorial_sprite].active)
			{
				wl_tutorial_step += 2;
			}
			break;
		}
		case 26:
		{
			wl_tutorial_click_to_next_step(25);
			break;
		}
		case 30:
		{
			if(wl_tutorial_click_to_next_step(31))
			{
				x = wl_game.player[0].x - 2 + rand() % 5;
				y = wl_game.player[0].y - 2 + rand() % 5;
				if(x < 0)
				{
					x = 0;
				}
				else if(x >= WL_MAX_BOARD_WIDTH)
				{
					x = WL_MAX_BOARD_WIDTH - 1;
				}
				if(y < 0)
				{
					y = 0;
				}
				else if(y >= WL_MAX_BOARD_HEIGHT)
				{
					y = WL_MAX_BOARD_HEIGHT - 1;
				}
				if(x == wl_game.player[0].x && y == wl_game.player[0].y)
				{
					if(x < 5)
					{
						x += 1;
					}
					else
					{
						x -= 1;
					}
				}
				wl_game.board[y][x].letter = '_';
				wl_board_generate_powerup_sprite(y, x, '_');
			}
			break;
		}
		case 31:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			
			/* if the player tried to make a word, show next tutorial page */
			if(wl_game.player[0].word_list_count > w || wl_game.player[0].sprite.state == WL_PLAYER_SPRITE_STATE_SAD)
			{
				if(f & WL_POWERUP_HOURGLASS)
				{
					wl_tutorial_step = 33;
				}
				/* didn't have the hourglass, so go to step that tells player to get it */
				else
				{
					wl_tutorial_step = 32;
				}
			}
			break;
		}
		case 32:
		{
			wl_tutorial_click_to_next_step(31);
			break;
		}
		case 33:
		{
			if(wl_tutorial_click_to_next_step(34))
			{
				wl_game.player[0].life /= 2;
				wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_SAD;
				wl_game.player[0].sprite.tick = 0;
				wl_game.player[0].sprite.timer = 30;
				t3f_play_sample(wl_sample[WL_SAMPLE_HURT], 1.0, 0.0, 1.0);
			}
			break;
		}
		case 35:
		{
			if(wl_tutorial_click_to_next_step(wl_tutorial_step + 1))
			{
				x = wl_game.player[0].x - 2 + rand() % 5;
				y = wl_game.player[0].y - 2 + rand() % 5;
				if(x < 0)
				{
					x = 0;
				}
				else if(x >= WL_MAX_BOARD_WIDTH)
				{
					x = WL_MAX_BOARD_WIDTH - 1;
				}
				if(y < 0)
				{
					y = 0;
				}
				else if(y >= WL_MAX_BOARD_HEIGHT)
				{
					y = WL_MAX_BOARD_HEIGHT - 1;
				}
				if(x == wl_game.player[0].x && y == wl_game.player[0].y)
				{
					if(x < 5)
					{
						x += 1;
					}
					else
					{
						x -= 1;
					}
				}
				wl_game.board[y][x].letter = '!';
				wl_board_generate_powerup_sprite(y, x, '!');
				for(i = 0; i < WL_MAX_SPRITES; i++)
				{
					if(wl_game.sprite[i].active && wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_HEALTH)
					{
						wl_game_tutorial_sprite = i;
						break;
					}
				}
			}
			break;
		}
		case 36:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(!wl_game.sprite[wl_game_tutorial_sprite].active)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 40:
		{
			if(wl_tutorial_click_to_next_step(wl_tutorial_step + 1))
			{
				x = wl_game.player[0].x - 2 + rand() % 5;
				y = wl_game.player[0].y - 2 + rand() % 5;
				if(x < 0)
				{
					x = 0;
				}
				else if(x >= WL_MAX_BOARD_WIDTH)
				{
					x = WL_MAX_BOARD_WIDTH - 1;
				}
				if(y < 0)
				{
					y = 0;
				}
				else if(y >= WL_MAX_BOARD_HEIGHT)
				{
					y = WL_MAX_BOARD_HEIGHT - 1;
				}
				if(x == wl_game.player[0].x && y == wl_game.player[0].y)
				{
					if(x < 5)
					{
						x += 1;
					}
					else
					{
						x -= 1;
					}
				}
				wl_game.board[y][x].letter = '?';
				wl_board_generate_powerup_sprite(y, x, '?');
				for(i = 0; i < WL_MAX_SPRITES; i++)
				{
					if(wl_game.sprite[i].active && wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_GURU)
					{
						wl_game_tutorial_sprite = i;
						break;
					}
				}
			}
			break;
		}
		case 41:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].word_list_count > w)
			{
				wl_tutorial_step += 2;
			}
			else if((f & WL_POWERUP_GURU) && !(wl_game.player[0].powerups & WL_POWERUP_GURU))
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 42:
		{
			if(wl_tutorial_click_to_next_step(41))
			{
				x = wl_game.player[0].x - 2 + rand() % 5;
				y = wl_game.player[0].y - 2 + rand() % 5;
				if(x < 0)
				{
					x = 0;
				}
				else if(x >= WL_MAX_BOARD_WIDTH)
				{
					x = WL_MAX_BOARD_WIDTH - 1;
				}
				if(y < 0)
				{
					y = 0;
				}
				else if(y >= WL_MAX_BOARD_HEIGHT)
				{
					y = WL_MAX_BOARD_HEIGHT - 1;
				}
				if(x == wl_game.player[0].x && y == wl_game.player[0].y)
				{
					if(x < 5)
					{
						x += 1;
					}
					else
					{
						x -= 1;
					}
				}
				wl_game.board[y][x].letter = '?';
				wl_board_generate_powerup_sprite(y, x, '?');
				for(i = 0; i < WL_MAX_SPRITES; i++)
				{
					if(wl_game.sprite[i].active && wl_game.sprite[i].type == WL_SPRITE_TYPE_POWERUP_GURU)
					{
						wl_game_tutorial_sprite = i;
						break;
					}
				}
			}
			break;
		}
		case 44:
		{
			wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
			wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
			wl_tutorial_step++;
			break;
		}
		case 45:
		{
			wl_tutorial_step = 99;
			break;
		}
	}
}

void wl_game_tutorial_logic_casual(void)
{
	int w;
	
	w = wl_game.player[0].word_list_count;
	
	switch(wl_tutorial_step)
	{
		case 14:
		case 15:
		case 16:
		case 17:
		case 19:
		case 20:
		case 21:
		{
			wl_tutorial_click_to_next_step(wl_tutorial_step + 1);
			break;
		}
		case 18:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].word_list_count > w)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 22:
		{
			wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
			wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
			wl_tutorial_step++;
			break;
		}
		case 23:
		{
			wl_tutorial_step = 99;
			break;
		}
	}
}

void wl_game_tutorial_logic_puzzle(void)
{
	int w;
	
	w = wl_game.player[0].word_list_count;
	
	switch(wl_tutorial_step)
	{
		case 14:
		case 15:
		case 16:
		case 17:
		case 19:
		case 20:
		case 21:
		case 23:
		case 24:
		case 25:
		{
			wl_tutorial_click_to_next_step(wl_tutorial_step + 1);
			break;
		}
		case 18:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].word_list_count > w)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 22:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].x == WL_BOARD_CENTER_X && wl_game.player[0].y == WL_BOARD_CENTER_Y && wl_game.player[0].state == WL_PLAYER_STATE_NORMAL)
			{
				wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
				wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
				wl_tutorial_step++;
			}
			break;
		}
		case 26:
		{
			wl_tutorial_step++;
			break;
		}
		case 27:
		{
			wl_tutorial_step = 99;
			break;
		}
	}
}

void wl_game_tutorial_logic_guru(void)
{
	int w;
	
	w = wl_game.player[0].word_list_count;

	switch(wl_tutorial_step)
	{
		case 14:
		case 15:
		case 16:
		case 17:
		case 18:
		case 20:
		{
			wl_tutorial_click_to_next_step(wl_tutorial_step + 1);
			break;
		}
		case 19:
		{
			wl_game_player_control();
			wl_player_rules_logic();
			if(wl_game.player[0].word_list_count > w)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 21:
		{
			wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
			wl_tutorial_step = 99;
			break;
		}
	}
}

void wl_game_tutorial_logic(void)
{
	int i;
	
	if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		wl_select_menu(WL_MENU_TUTORIAL);
		wl_state = WL_STATE_GAME_PAUSED_2;
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
	}
	else
	{
		wl_game_process_overlay_menu(wl_menu[WL_MENU_GAME_OVERLAY]);
	}
	touched = false;
	if(wl_menu[WL_MENU_GAME_OVERLAY]->hover_element < 0)
	{
		for(i = 0; i < T3F_MAX_TOUCHES; i++)
		{
			if(t3f_touch[i].active)
			{
				touched = true;
			}
		}
	}
	wl_process_particles(1);
	wl_clear_board_flags();
	wl_game_sprite_logic();
	wl_board_logic();
	if(wl_game.mode == WL_GAME_MODE_PUZZLE && wl_tutorial_step != 22)
	{
		wl_game.board[WL_BOARD_CENTER_Y][WL_BOARD_CENTER_X].flags |= WL_BOARD_FLAG_BLOCKED;
	}
	switch(wl_tutorial_step)
	{
		case 4:
		case 10:
		{
			wl_game_player_control();
			break;
		}
	}
	wl_game_player_logic();
	wl_game_guru_logic();
	wl_game_process_bg();
	
	wl_message_logic(&wl_game_messages, false);
	wl_message_logic(&wl_word_messages, true);
	
	/* add time to profile */
	wl_profiles->entry[wl_game.player[0].profile].time++;
	
	/* basics tutorial */
	switch(wl_tutorial_step)
	{
		case 0:
		case 1:
		case 2:
		case 3:
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
		case 11:
		{
			if(touched || t3f_key[ALLEGRO_KEY_SPACE])
			{
				if(!wl_game.player[0].clicked)
				{
					wl_tutorial_step++;
					wl_game.player[0].clicked = true;
				}
			}
			else
			{
				wl_game.player[0].clicked = false;
			}
			wl_game.player[0].hover_x = -1;
			break;
		}
		case 4:
		{
			if(strlen(wl_game.player[0].word) > 0)
			{
				wl_tutorial_step++;
			}
			break;
		}
		case 10:
		{
			if(wl_game.player[0].word_list_count > 0)
			{
				wl_tutorial_step = 13;
			}
			else if(wl_game.player[0].sprite.state == WL_PLAYER_SPRITE_STATE_SAD && wl_game.player[0].sprite.tick == 0)
			{
				wl_tutorial_step = 12;
			}
			break;
		}
		case 12:
		{
			if(touched || t3f_key[ALLEGRO_KEY_SPACE])
			{
				if(!wl_game.player[0].clicked)
				{
					wl_tutorial_step = 9;
					wl_game.player[0].clicked = true;
				}
			}
			else
			{
				wl_game.player[0].clicked = false;
			}
			wl_game.player[0].hover_x = -1;
			break;
		}
		case 13:
		{
			if(touched || t3f_key[ALLEGRO_KEY_SPACE])
			{
				if(!wl_game.player[0].clicked)
				{
					wl_tutorial_step++;
					wl_game.player[0].clicked = true;
				}
			}
			else
			{
				wl_game.player[0].clicked = false;
			}
			wl_game.player[0].hover_x = -1;
			if(wl_game.player[0].profile != 0)
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_BASIC;
			}
			wl_game.player[0].watched_tutorial = true;
			break;
		}
		case 99:
		{
			if(wl_game.player[0].sprite.z < -320)
			{
				if(wl_game.player[0].profile != 0)
				{
					wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_BASIC;
					if(wl_game.mode == WL_GAME_MODE_STORY)
					{
						wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_STORY;
					}
					else if(wl_game.mode == WL_GAME_MODE_CASUAL)
					{
						wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_CASUAL;
					}
					else if(wl_game.mode == WL_GAME_MODE_PUZZLE)
					{
						wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_PUZZLE;
					}
					else if(wl_game.mode == WL_GAME_MODE_GURU)
					{
						wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_GURU;
					}
				}
				wl_game.retry = true;
				wl_game.regenerate = true;
				wl_game_initialize(WL_LEVEL_LOAD_MUSIC);
				if(wl_game.mode == WL_GAME_MODE_STORY)
				{
					wl_game_scroll_level_name();
				}
				wl_state = WL_STATE_GAME;
				wl_game.player[0].watched_tutorial = true;
			}
			break;
		}
		default:
		{
			wl_game_setup_type();
			break;
		}
	}
	
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_game_tutorial_logic_story();
	}
	else if(wl_game.mode == WL_GAME_MODE_CASUAL)
	{
		wl_game_tutorial_logic_casual();
	}
	else if(wl_game.mode == WL_GAME_MODE_PUZZLE)
	{
		wl_game_tutorial_logic_puzzle();
	}
	else if(wl_game.mode == WL_GAME_MODE_GURU)
	{
		wl_game_tutorial_logic_guru();
	}
	if(wl_game.mode == WL_GAME_MODE_PUZZLE && wl_tutorial_step != 22 && wl_game.player[0].hover_x == WL_BOARD_CENTER_X && wl_game.player[0].hover_y == WL_BOARD_CENTER_Y)
	{
		wl_game.player[0].hover_x = -1;
	}
	else if(wl_game.mode == WL_GAME_MODE_CASUAL && wl_tutorial_step < 14)
	{
		wl_game.player[0].progress = 0;
		wl_game.player[0].progress_indicator = 0;
		wl_game.player[0].made_progress = false;
	}
}

void wl_game_tutorial_render_story(void)
{
	switch(wl_tutorial_step)
	{
		case 14:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Let's learn how to play STORY mode.");
			break;
		}
		case 15:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "In STORY mode your life meter (left) drains with each move you make. Exchanged words will refill your life meter in addition to giving you points.");
			break;
		}
		case 16:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Make a word to see what I am talking about.");
			break;
		}
		case 18:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "As you can see, your life meter is recharged when you exchange a word. Better words replenish more of your life.");
			break;
		}
		case 19:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "You may have also noticed the progress meter (bottom) also filled slightly. When the progress meter fills all the way up you will move on to the next level.");
			break;
		}
		case 20:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Now let's talk about hazards. There are a few types of hazards you will encounter on your journey. You typically want to avoid crossing paths with these.");
			break;
		}
		case 21:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "I will send one your way to demonstrate.");
			break;
		}
		case 23:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Notice the tiles that are shaded red? In order to avoid being hit by the hazard you will want to make sure you are not standing on any of these red tiles.");
			break;
		}
		case 24:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Try and avoid the hazard.");
			break;
		}
		case 26:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Oops, you got hit. Let's try that again. Avoid the hazard by moving to a space that is not shaded red.");
			break;
		}
		case 27:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Good job. Avoiding hazards is essential to completing your journey.");
			break;
		}
		case 28:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Lastly, let's cover powerups. You will come across various kinds of powerups as you make your journey. Generally, you will want to acquire these when you come across them.");
			break;
		}
		case 29:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "The first powerup I want to show you is the Hourglass. When you pick up an Hourglass, your life meter will be frozen until you try to exchange a word for points.");
			break;
		}
		case 30:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "The Hourglass will give you the opportunity to make an amazing word. It will also come in handy when your life meter is low. Grab the Hourglass and make a word.");
			break;
		}
		case 32:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Oops, you forgot to get the powerup. Let's try that again.");
			break;
		}
		case 33:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "As you can see, your life meter didn't deplete with each move as it normally does.");
			break;
		}
		case 34:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Next we're going to cover the Health Box, but first, let me take away half of your health. I am doing this to make the effects of the Health Box more apparent.");
			break;
		}
		case 35:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Health Boxes replenish your life meter completely. These come in very handy when your life meter gets low. Grab the Health Box and see what I mean.");
			break;
		}
		case 37:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Using Health Boxes wisely could mean the difference between descending into darkness and reaching enlightenment.");
			break;
		}
		case 38:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "We have one more powerup to talk about. The Light Bulb allows you to channel the Guru. Practically speaking, it will allow you to reach any space on the board in one move.");
			break;
		}
		case 39:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Use the Light Bulb to go after those rare words or escape a sticky situation.");
			break;
		}
		case 40:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Grab the Light Bulb to see what I mean. Try and make a long word.");
			break;
		}
		case 42:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Oops. you tried an invalid word. Let's try that again.");
			break;
		}
		case 43:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Congratulations! You have completed the STORY mode tutorial. Now I will send you on your way. I wish you the best in your quest for enlightenment.");
			break;
		}
	}
}

void wl_game_tutorial_render_casual(void)
{
	switch(wl_tutorial_step)
	{
		case 14:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Let's learn how to play CASUAL mode.");
			break;
		}
		case 15:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "In CASUAL mode it's just you and the game board.");
			break;
		}
		case 16:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Each word you make will fill your meter up one tenth of the way.");
			break;
		}
		case 17:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Make a word to see what I mean.");
			break;
		}
		case 19:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "You will not be able to exchange invalid words in this mode.");
			break;
		}
		case 20:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "That's all there is to it. Use your vocabulary skills to shoot for the high score.");
			break;
		}
		case 21:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Have fun!");
			break;
		}
	}
}

void wl_game_tutorial_render_puzzle(void)
{
	switch(wl_tutorial_step)
	{
		case 14:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Let's learn how to play PUZZLE mode.");
			break;
		}
		case 15:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "In PUZZLE mode your goal is to clear as much of the board as possible.");
			break;
		}
		case 16:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Letters taken from the board are not replaced in this mode.");
			break;
		}
		case 17:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Make a word to see what I mean.");
			break;
		}
		case 19:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Did you notice the @ symbol in the center of the board? That is the Home space.");
			break;
		}
		case 20:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Move to the Home space when you are done playing.");
			break;
		}
		case 21:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Try it out.");
			break;
		}
		case 23:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Be sure to clear as much of the board as you can before going to the Home space to maximize your score.");
			break;
		}
		case 24:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Any unused letters will take away from your final score, so try not to leave too many.");
			break;
		}
		case 25:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "That's all there is to PUZZLE mode. Let's see if you can rack up an insane score.");
			break;
		}
	}
}

void wl_game_tutorial_render_guru(void)
{
	switch(wl_tutorial_step)
	{
		case 14:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "GURU mode is my personal favorite. Let me explain how to play it.");
			break;
		}
		case 15:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "The goal in GURU mode is to go for the high score. In this mode, you will lose life with each move, just as you did in STORY mode.");
			break;
		}
		case 16:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "The game ends when your life meter is depleted.");
			break;
		}
		case 17:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Keep the game going for as long as you can by exchanging rare words.");
			break;
		}
		case 18:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Give it a try.");
			break;
		}
		case 20:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "That's pretty much it. Have fun and enjoy!");
			break;
		}
	}
}

void wl_game_tutorial_render(void)
{
	wl_game_render();
	
	/* basics tutorial */
	switch(wl_tutorial_step)
	{
		case 0:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Welcome to WordLeap!\n\nThis tutorial will teach you how to play the game.");
			break;
		}
		case 1:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "First, let's cover the basics");
			break;
		}
		case 2:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "You are standing on the game board. You are allowed to move to any of the highlighted spaces.");
			break;
		}
		case 3:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Click any highlighted space to move to that space. Try it out.");
			break;
		}
		case 5:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "By moving to a space with a letter, you take the letter that is in that space. You can also drop letters by moving to empty spaces. Your goal is to make words by taking letters from the game board.");
			break;
		}
		case 6:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Letters taken from the board are shown above. Once you have made a word, you can exchange that word for points by clicking the space on which you are standing.");
			break;
		}
		case 7:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Valid words are any real English words that are at least 3 letters long and are not proper nouns.");
			break;
		}
		case 8:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Exchanging invalid words, if the current game mode allows it, will make bad things happen. Avoid doing this if you want things to go well for you.");
			break;
		}
		case 9:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "Try to make a word.");
			break;
		}
		case 11:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "The amount of points a word is worth is determined by the length of the word, the rarity of the letters in that word, and the rarity of the word itself.");
			break;
		}
		case 12:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "You made an invalid word. Try again.");
			break;
		}
		case 13:
		{
			wl_draw_text_box(wl_font[WL_FONT_24], al_map_rgba_f(0.85, 0.85, 0.85, 0.85), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), wl_tutorial_text_offset_x, wl_tutorial_text_offset_y, wl_tutorial_text_width, 0, "And that is basically how you play the game.");
			break;
		}
	}
	
	/* story mode tutorial */
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_game_tutorial_render_story();
	}
	else if(wl_game.mode == WL_GAME_MODE_CASUAL)
	{
		wl_game_tutorial_render_casual();
	}
	else if(wl_game.mode == WL_GAME_MODE_PUZZLE)
	{
		wl_game_tutorial_render_puzzle();
	}
	else if(wl_game.mode == WL_GAME_MODE_GURU)
	{
		wl_game_tutorial_render_guru();
	}
}
