#include "t3f/t3f.h"
#include "t3f/resource.h"
#include "t3_logo.h"
#include "data.h"
#include "video_mode.h"
#include "text_entry.h"
#include "effects.h"
#include "menu_proc.h"
#include "music_queue.h"

void wl_logic(void * data);
void wl_render(void * data);
void wl_event_handler(ALLEGRO_EVENT * event, void * data);

void wl_load_config(void)
{
	ALLEGRO_CONFIG * cp;
	const char * val;
	const ALLEGRO_FILE_INTERFACE * old_interface;
	char fnbuf[1024] = {0};

	old_interface = al_get_new_file_interface();
	al_set_standard_file_interface();

	/* set default options */
	wl_option[WL_OPTION_CONTROLS] = 0;
	wl_option[WL_OPTION_MUSIC_VOLUME] = 100;
	wl_option[WL_OPTION_SOUND_VOLUME] = 100;
	wl_option[WL_OPTION_VIDEO_MODE] = WL_VIDEO_MODE_LARGE_WINDOW;
	if(!wl_video_mode_okay(wl_option[WL_OPTION_VIDEO_MODE]))
	{
		wl_option[WL_OPTION_VIDEO_MODE] = WL_VIDEO_MODE_MEDIUM_WINDOW;
		if(!wl_video_mode_okay(wl_option[WL_OPTION_VIDEO_MODE]))
		{
			wl_option[WL_OPTION_VIDEO_MODE] = WL_VIDEO_MODE_SMALL_WINDOW;
		}
	}
	wl_option[WL_OPTION_INTERNET] = WL_INTERNET_ASK;
	wl_option[WL_OPTION_FIRST_RUN] = 1;
	#ifdef ALLEGRO_ANDROID
		wl_option[WL_OPTION_LOWRES] = 1;
	#else
		wl_option[WL_OPTION_LOWRES] = 0;
	#endif

	/* load config file */
	cp = al_load_config_file(t3f_get_filename(t3f_config_path, "wordleap.cfg", fnbuf, 1024));
	if(cp)
	{
		val = al_get_config_value(cp, NULL, "Controls");
		if(val)
		{
			wl_option[WL_OPTION_CONTROLS] = atoi(val);
			if(wl_option[WL_OPTION_CONTROLS] < 0 || wl_option[WL_OPTION_CONTROLS] > 2)
			{
				wl_option[WL_OPTION_CONTROLS] = 0;
			}
		}
		val = al_get_config_value(cp, NULL, "Music Volume");
		if(val)
		{
			wl_option[WL_OPTION_MUSIC_VOLUME] = atoi(val);
			if(wl_option[WL_OPTION_MUSIC_VOLUME] < 0 || wl_option[WL_OPTION_MUSIC_VOLUME] > 200)
			{
				wl_option[WL_OPTION_MUSIC_VOLUME] = 100;
			}
		}
		val = al_get_config_value(cp, NULL, "Sound Volume");
		if(val)
		{
			wl_option[WL_OPTION_SOUND_VOLUME] = atoi(val);
			if(wl_option[WL_OPTION_SOUND_VOLUME] < 0 || wl_option[WL_OPTION_SOUND_VOLUME] > 200)
			{
				wl_option[WL_OPTION_SOUND_VOLUME] = 100;
			}
		}
		val = al_get_config_value(cp, NULL, "Video Mode");
		if(val)
		{
			wl_option[WL_OPTION_VIDEO_MODE] = atoi(val);
			if(wl_option[WL_OPTION_VIDEO_MODE] < 0 || wl_option[WL_OPTION_VIDEO_MODE] > WL_VIDEO_MODE_FULL_SCREEN || !wl_video_mode_okay(wl_option[WL_OPTION_VIDEO_MODE]))
			{
				wl_option[WL_OPTION_VIDEO_MODE] = WL_VIDEO_MODE_SMALL_WINDOW;
			}
		}
		val = al_get_config_value(cp, NULL, "Upload Scores");
		if(val)
		{
			wl_option[WL_OPTION_INTERNET] = atoi(val);
			if(wl_option[WL_OPTION_INTERNET] < 0 || wl_option[WL_OPTION_INTERNET] > 2)
			{
				wl_option[WL_OPTION_INTERNET] = 1;
			}
		}
		val = al_get_config_value(cp, NULL, "First Run");
		if(val)
		{
			wl_option[WL_OPTION_FIRST_RUN] = atoi(val);
		}
		al_destroy_config(cp);
	}
	t3f_set_music_volume((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0);
	t3f_set_new_music_volume((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0);
	t3f_set_sound_volume((float)wl_option[WL_OPTION_SOUND_VOLUME] / 100.0);
	if(wl_option[WL_OPTION_CONTROLS] == 2)
	{
		wl_mouse_off = true;
	}
	al_set_new_file_interface(old_interface);
}

bool wl_load_bitmap(int bitmap, const char * fn)
{
	wl_bitmap[bitmap] = t3f_load_resource((void **)(&wl_bitmap[bitmap]), T3F_RESOURCE_TYPE_BITMAP, fn, 0, 0, 0);
	if(!wl_bitmap[bitmap])
	{
		printf("Failed to load bitmap %d!\n", bitmap);
		return false;
	}
	return true;
}

bool wl_load_sample(int sample, const char * fn)
{
	wl_sample[sample] = al_load_sample(fn);
	if(!wl_sample[sample])
	{
		printf("Failed to load sample %d!\n", sample);
		return false;
	}
	return true;
}

bool wl_load_animation(int animation, const char * fn)
{
	wl_animation[animation] = t3f_load_animation(fn);
	if(!wl_animation[animation])
	{
		wl_animation[animation] = t3f_load_animation_from_bitmap(fn);
	}
	if(!wl_animation[animation])
	{
		printf("Failed to load animation %d!\n", animation);
		return false;
	}
	return true;
}

bool wl_load_data(void)
{
	ALLEGRO_STATE old_state;
	char fnbuf[1024] = {0};

	/* load animations */
	if(!wl_load_animation(WL_ANIMATION_PLAYER_NORMAL, "data/graphics/player_0.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_HURT, "data/graphics/player_1.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_HAPPY, "data/graphics/player_2.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_SAD, "data/graphics/player_4.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_UP, "data/graphics/player_3u.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_DOWN, "data/graphics/player_3d.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_LEFT, "data/graphics/player_3l.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_RIGHT, "data/graphics/player_3r.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PLAYER_SHADOW, "data/graphics/player_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_GURU, "data/graphics/guru.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_BOMB, "data/graphics/bomb.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_BOMB_SHADOW, "data/graphics/bomb_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_LASER, "data/graphics/laser.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_LASER_SHADOW, "data/graphics/laser_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PARTICLE, "data/graphics/particle.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PARTICLE_2, "data/graphics/particle2.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PARTICLE_3, "data/graphics/particle3.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_PARTICLE_4, "data/graphics/particle4.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_HEALTH, "data/graphics/powerup_health.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_HEALTH_SHADOW, "data/graphics/powerup_health_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_GURU, "data/graphics/powerup_guru.t3a"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_GURU_SHADOW, "data/graphics/powerup_guru_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_HOURGLASS, "data/graphics/powerup_hourglass.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_POWER_HOURGLASS_SHADOW, "data/graphics/powerup_hourglass_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_SPIKE, "data/graphics/spike.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_SPIKE_2, "data/graphics/spike2.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_SPIKE_SHADOW, "data/graphics/spike_shadow.png"))
	{
		return false;
	}
	if(!wl_load_animation(WL_ANIMATION_SELECTOR, "data/graphics/selector.png"))
	{
		return false;
	}

	/* load bitmaps */
	if(!wl_load_bitmap(WL_BITMAP_LIFE, "data/graphics/life.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_LIFE_FILL, "data/graphics/life_fill.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_LIFE_OVERLAY, "data/graphics/life_overlay.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_LEVEL, "data/graphics/level.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_LEVEL_FILL, "data/graphics/level_fill.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_TILE, "data/graphics/tile.png"))
	{
		return false;
	}
	if(!wl_load_bitmap(WL_BITMAP_POPUP, "data/graphics/popup.png"))
	{
		return false;
	}
	al_store_state(&old_state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
	al_set_new_bitmap_flags(ALLEGRO_MIN_LINEAR | ALLEGRO_MAG_LINEAR | ALLEGRO_NO_PRESERVE_TEXTURE);
	wl_bitmap[WL_BITMAP_SCRATCH] = al_create_bitmap(512, 64);
	al_restore_state(&old_state);

	/* create atlas */
	wl_atlas[WL_ATLAS_GAME] = t3f_create_atlas(512, 512);
	if(wl_atlas[WL_ATLAS_GAME])
	{
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_GAME], &wl_bitmap[WL_BITMAP_TILE], T3F_ATLAS_TILE);
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_GAME], &wl_bitmap[WL_BITMAP_POPUP], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_SELECTOR], T3F_ATLAS_TILE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_NORMAL], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_HURT], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_HAPPY], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_SAD], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_UP], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_DOWN], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_LEFT], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_RIGHT], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PLAYER_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_BOMB], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_BOMB_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_LASER], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_LASER_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PARTICLE], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PARTICLE_2], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PARTICLE_3], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_PARTICLE_4], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_HEALTH], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_HEALTH_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_GURU], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_GURU_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_HOURGLASS], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_POWER_HOURGLASS_SHADOW], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_SPIKE], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_SPIKE_2], T3F_ATLAS_SPRITE);
		t3f_add_animation_to_atlas(wl_atlas[WL_ATLAS_GAME], wl_animation[WL_ANIMATION_SPIKE_SHADOW], T3F_ATLAS_SPRITE);
	}

	/* create HUD atlas for slight optimization */
	wl_atlas[WL_ATLAS_HUD] = t3f_create_atlas(512, 512);
	if(wl_atlas[WL_ATLAS_HUD])
	{
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_HUD], &wl_bitmap[WL_BITMAP_LEVEL], T3F_ATLAS_TILE);
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_HUD], &wl_bitmap[WL_BITMAP_LEVEL_FILL], T3F_ATLAS_TILE);
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_HUD], &wl_bitmap[WL_BITMAP_LIFE], T3F_ATLAS_TILE);
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_HUD], &wl_bitmap[WL_BITMAP_LIFE_FILL], T3F_ATLAS_TILE);
		t3f_add_bitmap_to_atlas(wl_atlas[WL_ATLAS_HUD], &wl_bitmap[WL_BITMAP_LIFE_OVERLAY], T3F_ATLAS_TILE);
	}

	/* load fonts */
	wl_font[WL_FONT_24] = t3f_load_resource((void **)(&wl_font[WL_FONT_24]), T3F_RESOURCE_TYPE_FONT, "data/fonts/teen.ttf", 24, 0, 0);
	if(!wl_font[WL_FONT_24])
	{
		printf("Failed to load font %d!\n", WL_FONT_24);
		return false;
	}
	wl_font[WL_FONT_16] = t3f_load_resource((void **)(&wl_font[WL_FONT_16]), T3F_RESOURCE_TYPE_FONT, "data/fonts/teen.ttf", 14, 0, 0);
	if(!wl_font[WL_FONT_16])
	{
		printf("Failed to load font %d!\n", WL_FONT_16);
		return false;
	}
	wl_font[WL_FONT_32] = t3f_load_resource((void **)(&wl_font[WL_FONT_32]), T3F_RESOURCE_TYPE_FONT, "data/fonts/teen.ttf", 40, 0, 0);
	if(!wl_font[WL_FONT_32])
	{
		printf("Failed to load font %d!\n", WL_FONT_32);
		return false;
	}
	wl_font[WL_FONT_12] = t3f_load_resource((void **)(&wl_font[WL_FONT_12]), T3F_RESOURCE_TYPE_FONT, "data/fonts/teen.ttf", 12, 0, 0);
	if(!wl_font[WL_FONT_12])
	{
		printf("Failed to load font %d!\n", WL_FONT_12);
		return false;
	}
	wl_ascii_font = t3f_load_resource((void **)(&wl_ascii_font), T3F_RESOURCE_TYPE_T3F_FONT_LOAD, "data/fonts/teen_outline.png", 0, 0, 0);

	/* load sounds */
	if(!wl_load_sample(WL_SAMPLE_JUMP, "data/sounds/jump.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_LAND, "data/sounds/land.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_BAD_WORD, "data/sounds/bad_word.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_WORD, "data/sounds/word.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_GOOD_WORD, "data/sounds/good_word.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_AMAZING_WORD, "data/sounds/amazing_word.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_SHATTER, "data/sounds/shatter.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_BOMB, "data/sounds/bomb.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_SPIKE, "data/sounds/spike.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_LASER, "data/sounds/laser.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_LIFE_UP, "data/sounds/life_up.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_LIFE_LOW, "data/sounds/life_low.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_GURU, "data/sounds/guru.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_HOURGLASS, "data/sounds/hourglass.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_SWOOSH, "data/sounds/swoosh.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_HURT, "data/sounds/hurt.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_DESTROY, "data/sounds/destroy.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_BEST_WORD, "data/sounds/best_word.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_FREEZE, "data/sounds/freeze.wav"))
	{
		return false;
	}
	if(!wl_load_sample(WL_SAMPLE_THAW, "data/sounds/thaw.wav"))
	{
		return false;
	}

	wl_dictionary = wl_load_dictionary("data/misc/dictionary.txt");
	if(!wl_dictionary)
	{
		printf("Unable to load dictionary!\n");
		return false;
	}
	wl_dictionary_checksum = t3f_checksum_file("data/misc/dictionary.txt");

	wl_profiles = wl_load_profiles(t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
	if(!wl_profiles)
	{
		wl_profiles = wl_create_profiles();
		if(!wl_profiles)
		{
			return false;
		}
	}
	return true;
}

/* detect if we are playing the demo version */
bool wl_is_demo(void)
{
	if(!al_filename_exists("data/scripts/ending.script"))
	{
		return true;
	}
	return false;
}

static bool want_opengl(int argc, char * argv[])
{
	int i;

	for(i = 1; i < argc; i++)
	{
		if(!strcmp(argv[i], "-opengl"))
		{
			return true;
		}
	}
	return false;
}

bool wl_initialize(int argc, char * argv[])
{
	int ogl_flag = want_opengl(argc, argv) ? T3F_USE_OPENGL : 0;

	if(!t3f_initialize(T3F_APP_TITLE, 640, 480, 60.0, wl_logic, wl_render, T3F_USE_KEYBOARD | T3F_USE_MOUSE | T3F_USE_TOUCH | T3F_USE_SOUND | T3F_FORCE_ASPECT | ogl_flag, NULL))
	{
		printf("Failed to initialize program!\n");
		return false;
	}
	#ifdef T3F_NO_UTF8
		t3f_windows_text_to_utf8(T3F_APP_COPYRIGHT, wl_copyright_message, 256);
	#else
		strcpy(wl_copyright_message, T3F_APP_COPYRIGHT);
	#endif
	wl_demo = wl_is_demo();
	if(wl_demo)
	{
		wl_end_level = 4;
	}

	t3f_set_event_handler(wl_event_handler);
	wl_async_queue = al_create_event_queue();
	if(!wl_async_queue)
	{
		return false;
	}
	al_init_user_event_source(&wl_async_source);
	al_register_event_source(wl_async_queue, &wl_async_source);

	wl_load_config();
	wl_initialize_video_mode();
	srand(time(0));
	t3f_srand(&wl_rng_state, time(0));

	if(!wl_load_data())
	{
		return false;
	}

	wl_entered_text_ustr = al_ustr_new("\\_");
	if(!wl_entered_text_ustr)
	{
		return false;
	}

	wl_game_messages.messages = 0;
	wl_title_init();
	if(wl_option[WL_OPTION_FIRST_RUN])
	{
		wl_create_options_menu("INITIAL SETUP", "DONE");
		wl_option[WL_OPTION_FIRST_RUN] = 0;
		wl_select_menu(WL_MENU_OPTIONS);
	}
	wl_play_music("data/music/theme.ogg");
	wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
	return true;
}
