#include "t3f/t3f.h"
#include "t3net/t3net.h"
#include "data.h"
#include "game.h"
#include "menu_proc.h"

static ALLEGRO_THREAD * wl_leaderboard_thread = NULL;
static int wl_leaderboard_retries = 3;
static bool wl_leaderboard_manual_upload = false;
int wl_leaderboard_profile_upload_flags = 0;
int wl_leaderboard_upload_flags = 0;
int wl_leaderboard_thread_counter = 0;

static void * wl_upload_proc(ALLEGRO_THREAD * tp, void *arg)
{
	ALLEGRO_EVENT event;
	int r = wl_leaderboard_retries;
	int i, b, p, s = 0;
	bool upload = true;
	char opt[256] = {0};
	char extra[256] = {0};
	char * mode_name[4] = {"story", "casual", "puzzle", "guru"};
	char * mode_best_name[4] = {"best_word", "best_word_casual", "best_word_puzzle", "best_word_guru"};
	int mode_flag[4] = {WL_PROFILE_FLAG_UPLOAD_STORY, WL_PROFILE_FLAG_UPLOAD_CASUAL, WL_PROFILE_FLAG_UPLOAD_PUZZLE, WL_PROFILE_FLAG_UPLOAD_GURU};
	int mode_best_flag[4] = {WL_PROFILE_FLAG_UPLOAD_BEST, WL_PROFILE_FLAG_UPLOAD_BEST_C, WL_PROFILE_FLAG_UPLOAD_BEST_P, WL_PROFILE_FLAG_UPLOAD_BEST_G};

	sprintf(opt, "%lu", wl_dictionary_checksum);

	/* upload score */
	upload = true;
	if(wl_leaderboard_manual_upload && !(wl_profiles->entry[wl_game.player[0].profile].flags & mode_flag[wl_game.mode]))
	{
		s++; // interpret not uploading as success
		upload = false;
	}
	else if(!wl_leaderboard_manual_upload && (wl_leaderboard_upload_flags & WL_LEADERBOARD_UPLOAD_FLAG_SCORE))
	{
		s++; // count as uploaded if score is already uploaded
		upload = false;
	}
	if(upload)
	{
		if(wl_game.mode == WL_GAME_MODE_PUZZLE)
		{
			snprintf(extra, 256, "%d", wl_game.current_unused_letters);
		}
		else
		{
			strncpy(extra, "", 256);
		}
		for(i = 0; i < r; i++)
		{
			if(t3net_upload_score("http://www.t3-i.com/t3net2/leaderboards/insert.php", "wordleap", "1.0", mode_name[wl_game.mode], opt, wl_profiles->entry[wl_game.player[0].profile].name, wl_game.player[0].score * 3 + 'w' + 'l' + 'e' + 'a' + 'p', extra))
			{
				/* clear profile flag only if this score is the best */
				if(wl_game.player[0].score >= wl_profiles->entry[wl_game.player[0].profile].high_score[wl_game.mode])
				{
					wl_profiles->entry[wl_game.player[0].profile].flags &= ~mode_flag[wl_game.mode];
				}
				wl_leaderboard_upload_flags |= WL_LEADERBOARD_UPLOAD_FLAG_SCORE; // set temporary flag so we can properly retry later on failure
				s++;
				break;
			}
		}
	}

	/* upload best word */
	upload = true;
	if(wl_leaderboard_manual_upload && !(wl_profiles->entry[wl_game.player[0].profile].flags & mode_best_flag[wl_game.mode]))
	{
		s++; // interpret not uploading as success
		upload = false;
	}
	else if(!wl_leaderboard_manual_upload && (wl_leaderboard_upload_flags & WL_LEADERBOARD_UPLOAD_FLAG_WORD))
	{
		s++; // count as uploaded if score is already uploaded
		upload = false;
	}
	if(upload)
	{
		for(i = 0; i < r; i++)
		{
			b = wl_get_best_word();
			if(b >= 0)
			{
				p = wl_game_figure_total_points(b);
				sprintf(extra, "%d", b);
				if(t3net_upload_score("http://www.t3-i.com/t3net2/leaderboards/insert.php", "wordleap", "1.0", mode_best_name[wl_game.mode], opt, wl_profiles->entry[wl_game.player[0].profile].name, p * 3 + 'w' + 'l' + 'e' + 'a' + 'p', extra))
				{
					/* clear profile flag only if this word is the best */
					if(wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode] >= 0 && p >= wl_game_figure_total_points(wl_profiles->entry[wl_game.player[0].profile].best_word[wl_game.mode]))
					{
						wl_profiles->entry[wl_game.player[0].profile].flags &= ~mode_best_flag[wl_game.mode];
					}
					wl_leaderboard_upload_flags |= WL_LEADERBOARD_UPLOAD_FLAG_WORD; // set temporary flag so we can properly retry later on failure
					s++;
					break;
				}
			}
			else
			{
				s++;
			}
		}
	}
	if(s >= 2)
	{
		event.type = ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 's');
	}
	else if(s > 0)
	{
		event.type = ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 'p');
	}
	else
	{
		event.type = ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 'f');
	}
	al_emit_user_event(&wl_async_source, &event, NULL);
	return NULL;
}

bool wl_upload_scores(bool manual)
{
	if(wl_leaderboard_thread)
	{
		al_destroy_thread(wl_leaderboard_thread);
	}
	wl_leaderboard_thread = al_create_thread(wl_upload_proc, NULL);
	if(!wl_leaderboard_thread)
	{
		return false;
	}
	wl_leaderboard_manual_upload = manual;
	wl_leaderboard_thread_counter++;
	al_start_thread(wl_leaderboard_thread);
	return true;
}

static void * wl_download_proc(ALLEGRO_THREAD * tp, void *arg)
{
	ALLEGRO_EVENT event;
	char * mode_text[2][4] =
	{
		{"story", "casual", "puzzle", "guru"},
		{"best_word", "best_word_casual", "best_word_puzzle", "best_word_guru"}
	};
	char opt[256] = {0};

	sprintf(opt, "%lu", wl_dictionary_checksum);
	if(wl_leaderboard)
	{
		t3net_destroy_leaderboard(wl_leaderboard);
	}
	wl_leaderboard = t3net_get_leaderboard("http://www.t3-i.com/t3net2/leaderboards/query.php", "wordleap", "1.0", mode_text[wl_leaderboard_type][wl_leaderboard_mode], opt, 10, 0);
	if(!wl_leaderboard)
	{
		event.type = ALLEGRO_GET_EVENT_TYPE('t', 'r', 'd', 'f');
	}
	else
	{
		event.type = ALLEGRO_GET_EVENT_TYPE('t', 'r', 'd', 's');
	}
	al_emit_user_event(&wl_async_source, &event, NULL);
	return NULL;
}

bool wl_download_scores(void)
{
	if(wl_leaderboard_thread)
	{
		al_destroy_thread(wl_leaderboard_thread);
	}
	wl_leaderboard_thread = al_create_thread(wl_download_proc, NULL);
	if(!wl_leaderboard_thread)
	{
		return false;
	}
	wl_leaderboard_thread_counter++;
	al_start_thread(wl_leaderboard_thread);
	return true;
}

void wl_free_leaderboard_data(void)
{
	if(wl_leaderboard_thread)
	{
		al_destroy_thread(wl_leaderboard_thread);
	}
}
