#include "t3f/t3f.h"
#include "t3f/music.h"
#include "t3f/resource.h"
#include "level.h"
#include "music_queue.h"

int wl_get_level_slot(int level)
{
	return level & 1;
}

/* load level shapes for the specified level */
bool wl_load_level_shapes(WL_LEVEL * lp, int flags)
{
	ALLEGRO_CONFIG * cp = NULL;
	ALLEGRO_CONFIG * scp = NULL;
	const char * val;
	int i;
	char buf[256] = {0};
	bool loaded_shapes = false;
	float source_width, source_height;
	float target_width, target_height;
	float scale_x, scale_y;
	
	cp = al_load_config_file("data/levels/levels.ini");
	if(cp)
	{
		sprintf(buf, "level %d", lp->number);
		
		/* load shapes */
		val = al_get_config_value(cp, buf, "shapes");
		if(val)
		{
			scp = al_load_config_file(val);
			if(scp)
			{
				val = NULL;
				if(flags & WL_LEVEL_LOAD_FLAG_LOWRES)
				{
					val = al_get_config_value(scp, NULL, "bitmap_lowres");
				}
				/* load normal bitmap unless lowres version is requested and found */
				if(!val)
				{
					val = al_get_config_value(scp, NULL, "bitmap");
				}
				if(val)
				{
					lp->shape_image = t3f_load_resource((void **)(&lp->shape_image), T3F_RESOURCE_TYPE_BITMAP, val, 0, 0, 0);
					if(lp->shape_image)
					{
						/* get target size so we can scale our shapes to match the loaded bitmap */
						source_width = al_get_bitmap_width(lp->shape_image);
						source_height = al_get_bitmap_width(lp->shape_image);
						target_width = source_width;
						target_height = source_height;
						val = al_get_config_value(scp, NULL, "target_width");
						if(val)
						{
							target_width = atof(val);
						}
						val = al_get_config_value(scp, NULL, "target_height");
						if(val)
						{
							target_height = atof(val);
						}
						scale_x = source_width / target_width;
						scale_y = source_height / target_height;
						
						val = al_get_config_value(scp, NULL, "shapes");
						if(val)
						{
							lp->shapes = atoi(val);
							for(i = 0; i < lp->shapes; i++)
							{
								sprintf(buf, "shape %d", i);
								val = al_get_config_value(scp, buf, "x");
								if(val)
								{
									lp->shape[i].x = atof(val) * scale_x;
								}
								val = al_get_config_value(scp, buf, "y");
								if(val)
								{
									lp->shape[i].y = atof(val) * scale_y;
								}
								val = al_get_config_value(scp, buf, "w");
								if(val)
								{
									lp->shape[i].target_width = atof(val);
									lp->shape[i].width = lp->shape[i].target_width * scale_x;
								}
								val = al_get_config_value(scp, buf, "h");
								if(val)
								{
									lp->shape[i].target_height = atof(val);
									lp->shape[i].height = lp->shape[i].target_height * scale_y;
								}
							}
							loaded_shapes = true;
							lp->shapes_used = false;
						}
					}
				}
				al_destroy_config(scp);
			}
		}
		al_destroy_config(cp);
	}
	return loaded_shapes;
}

bool wl_load_level(WL_LEVEL * lp, int level, int mode, bool load_music, int flags)
{
	ALLEGRO_CONFIG * cp = NULL;
	char buf[256] = {0};
	const char * val;
	float r = 0.0, g = 0.0, b = 0.0, a = 1.0;
	
	cp = al_load_config_file("data/levels/levels.ini");
	if(cp)
	{
		sprintf(buf, "level %d", level);
		
		/* start playing the music */
		if(mode == WL_LEVEL_LOAD_NORMAL || mode == WL_LEVEL_LOAD_MUSIC)
		{
			if(load_music)
			{
				val = al_get_config_value(cp, buf, "music");
				if(val)
				{
					wl_queue_music(val);
					wl_finish_current_song();
				}
			}
		}
		
		/* load everything else unless we just want the music */
		if(mode == WL_LEVEL_LOAD_NORMAL || mode == WL_LEVEL_LOAD_INTRO)
		{
			wl_free_level(lp);
			val = NULL;
			if(flags & WL_LEVEL_LOAD_FLAG_LOWRES)
			{
				val = al_get_config_value(cp, buf, "bg_lowres");
			}
			/* load normal bitmap unless lowres version is requested and found */
			if(!val)
			{
				val = al_get_config_value(cp, buf, "bg");
			}
			if(val)
			{
				lp->bg_animation = t3f_load_animation(val);
				if(!lp->bg_animation)
				{
					lp->bg_animation = t3f_load_animation_from_bitmap(val);
					if(!lp->bg_animation)
					{
						return false;
					}
				}
			}
			else
			{
				return false;
			}
			val = al_get_config_value(cp, buf, "name");
			if(val)
			{
				strcpy(lp->name, val);
			}
			lp->number = level;
			val = al_get_config_value(cp, buf, "color_r");
			if(val)
			{
				r = atof(val);
			}
			val = al_get_config_value(cp, buf, "color_g");
			if(val)
			{
				g = atof(val);
			}
			val = al_get_config_value(cp, buf, "color_b");
			if(val)
			{
				b = atof(val);
			}
			val = al_get_config_value(cp, buf, "color_a");
			if(val)
			{
				a = atof(val);
			}
			lp->color = al_map_rgba_f(r, g, b, a);
			
			lp->z = 0.0;
			lp->vz = 0.0;
			val = al_get_config_value(cp, buf, "zoom");
			if(val)
			{
				lp->vz = atof(val);
			}
			
			lp->a = 0.0;
			lp->va = 0.0;
			val = al_get_config_value(cp, buf, "rotate");
			if(val)
			{
				lp->va = atof(val);
			}
			
			lp->cx = lp->bg_animation->frame[0]->width / 2;
			val = al_get_config_value(cp, buf, "cx");
			if(val)
			{
				lp->cx = atof(val);
			}
			lp->cy = lp->bg_animation->frame[0]->height / 2;
			val = al_get_config_value(cp, buf, "cy");
			if(val)
			{
				lp->cy = atof(val);
			}
			lp->ox = 320.0;
			val = al_get_config_value(cp, buf, "ox");
			if(val)
			{
				lp->ox = atof(val);
			}
			lp->oy = 240.0;
			val = al_get_config_value(cp, buf, "oy");
			if(val)
			{
				lp->oy = atof(val);
			}
			
			/* load music info */
			val = al_get_config_value(cp, buf, "music artist");
			if(val)
			{
				strncpy(lp->music_artist, val, 64);
			}
			else
			{
				strcpy(lp->music_artist, "");
			}

			val = al_get_config_value(cp, buf, "music title");
			if(val)
			{
				strncpy(lp->music_title, val, 64);
			}
			else
			{
				strcpy(lp->music_title, "");
			}
			
			/* load shapes */
			val = al_get_config_value(cp, buf, "shapes");
			if(val)
			{
				if(!wl_load_level_shapes(lp, flags))
				{
					return false;
				}
			}
		}
		al_destroy_config(cp);
	}
	return true;
}

void wl_free_level(WL_LEVEL * lp)
{
	if(lp->bg_animation)
	{
		t3f_destroy_animation(lp->bg_animation);
		lp->bg_animation = NULL;
	}
	if(lp->shape_image)
	{
		t3f_destroy_resource(lp->shape_image);
		lp->shape_image = NULL;
	}
}
