#include "t3f/t3f.h"
#include "t3f/memory.h"
#include "t3f/music.h"
#include "t3f/sound.h"
#include "t3f/resource.h"
#include "t3net/t3net.h"
#include "t3_logo.h"
#include "data.h"
#include "init.h"
#include "exit.h"
#include "dictionary.h"
#include "shape.h"
#include "title.h"
#include "game.h"
#include "game_over.h"
#include "game_intro.h"
#include "game_music.h"
#include "effects.h"
#include "menu_proc.h"
#include "game_tally.h"
#include "game_tutorial.h"
#include "memory.h"
#include "music_queue.h"
#include "menu_proc.h"
#include "video_mode.h"
#include "text_entry.h"
#include "privacy.h"

/* user pressed close button (or ESC key in certain places) */
void wl_close(void)
{
	switch(wl_state)
	{
		case WL_STATE_TITLE:
		case WL_STATE_GAME_PAUSED:
		case WL_STATE_GAME_PAUSED_2:
		case WL_STATE_GAME_OVER:
		case WL_STATE_GAME_OVER_WAIT:
		case WL_STATE_GAME_DONE:
		{
			wl_last_menu = wl_current_menu;
			wl_current_menu = WL_MENU_QUIT;
			break;
		}

		case WL_STATE_PRIVACY:
		{
			wl_pause_state = WL_STATE_PRIVACY;
//			wl_overlay_state = wl_state;
			wl_current_menu = WL_MENU_QUIT;
			wl_state = WL_STATE_PAUSED;
			break;
		}
		/* bring up pause menu when in-game */
		case WL_STATE_GAME:
		case WL_STATE_GAME_TALLY:
		{
			wl_exit_state = wl_state;
			if(wl_game.mode == WL_GAME_MODE_STORY)
			{
				al_stop_timer(t3f_timer);
				wl_cinema = wl_load_cinema_script("data/scripts/game_over.script");
				wl_set_cinema_view_callbacks(wl_cinema, wl_game_paused_view_logic, wl_game_render);
				al_start_timer(t3f_timer);
				wl_current_menu = WL_MENU_GAME_EXIT;
				wl_state = WL_STATE_GAME_PAUSED;
			}
			else
			{
				wl_overlay_state = wl_state;
				wl_current_menu = WL_MENU_GAME_EXIT;
				wl_state = WL_STATE_GAME_PAUSED_2;
			}
			break;
		}

		case WL_STATE_GAME_INTRO:
		case WL_STATE_MUSIC:
		{
			wl_pause_state = wl_state;
			wl_state = WL_STATE_PAUSED;
			wl_current_menu = WL_MENU_GAME_EXIT;
			t3f_pause_music();
			wl_set_music_fade_speed(0.0);
			break;
		}

		/* cinema pause states */
		case WL_STATE_GAME_WIN:
		case WL_STATE_DEMO_WIN:
		case WL_STATE_TITLE_CREDITS:
		case WL_STATE_GAME_CONTINUE:
		case WL_STATE_GAME_TUTORIAL:
		{
			wl_pause_state = wl_state;
			wl_state = WL_STATE_PAUSED;
			wl_current_menu = WL_MENU_GAME_EXIT;
			break;
		}

		/* ignore close button in these cases */
		case WL_STATE_PAUSED:
		case WL_STATE_GAME_QUIT:
		case WL_STATE_TO_GAME:
		{
			break;
		}
		default:
		{
			t3f_exit();
			break;
		}
	}
}

void wl_event_handler(ALLEGRO_EVENT * event, void * data)
{
	ALLEGRO_STATE old_state;

	switch(event->type)
	{
		case ALLEGRO_EVENT_MOUSE_AXES:
		{
			if(wl_option[WL_OPTION_CONTROLS] != 2)
			{
				wl_mouse_off = false;
			}
			t3f_event_handler(event);
			break;
		}
		case ALLEGRO_EVENT_DISPLAY_CLOSE:
		{
			wl_close();
			break;
		}
		case ALLEGRO_EVENT_DISPLAY_HALT_DRAWING:
		{
			al_destroy_bitmap(wl_bitmap[WL_BITMAP_SCRATCH]);
			t3f_event_handler(event);
			break;
		}
		case ALLEGRO_EVENT_DISPLAY_RESUME_DRAWING:
		{
			/* let T3F reload resources before reloading shapes */
			t3f_event_handler(event);
			al_store_state(&old_state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
			al_set_new_bitmap_flags(ALLEGRO_MIN_LINEAR | ALLEGRO_MAG_LINEAR | ALLEGRO_NO_PRESERVE_TEXTURE);
			wl_bitmap[WL_BITMAP_SCRATCH] = al_create_bitmap(512, 64);
			al_restore_state(&old_state);
			break;
		}

		/* pass the event through to T3F for handling by default */
		default:
		{
			t3f_event_handler(event);
			break;
		}
	}
}

static int old_display_width = 0;
static int old_display_height = 0;

/* we have to monitor the display size to check for changes since we don't
   get the correct size immediately after a resize event under Linux */
static void work_around_linux_resize_bug(void)
{
	if(al_get_display_width(t3f_display) != old_display_width || al_get_display_height(t3f_display) != old_display_height)
	{
		t3f_adjust_view(t3f_default_view, 0, 0, al_get_display_width(t3f_display), al_get_display_height(t3f_display), 320, 240, t3f_flags);
	}
	old_display_width = al_get_display_width(t3f_display);
	old_display_height = al_get_display_height(t3f_display);
}

void wl_state_logic(int state)
{
	work_around_linux_resize_bug();
	switch(state)
	{
		case WL_STATE_INTRO:
		{
			t3_logo_logic();
			if(t3_logo_done)
			{
				t3_logo_finish();
				wl_play_music("data/music/theme.ogg");
				wl_state = WL_STATE_TITLE;
				wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
				t3f_clear_touch_data();
			}
			break;
		}
		case WL_STATE_TITLE:
		{
			wl_title_logic();
			break;
		}
		case WL_STATE_TITLE_CREDITS:
		{
			wl_title_credits_logic();
			break;
		}
		case WL_STATE_TITLE_EXIT:
		{
			wl_title_exit_logic();
			break;
		}
		case WL_STATE_TO_GAME:
		{
			wl_title_to_game_logic();
			break;
		}
		case WL_STATE_GAME_INTRO:
		{
			wl_game_intro_logic();
			if(wl_cinema && wl_cinema->break_points)
			{
				wl_game_intro_skip_logic();
			}
			break;
		}
		case WL_STATE_GAME:
		{
			wl_game_logic();
			break;
		}
		case WL_STATE_MUSIC:
		{
			wl_game_music_logic();
			break;
		}
		case WL_STATE_GAME_TUTORIAL:
		{
			wl_game_tutorial_logic();
			break;
		}
		case WL_STATE_GAME_OVER:
		{
			wl_game_over_logic();
			break;
		}
		case WL_STATE_GAME_PAUSED:
		{
			wl_game_paused_logic();
			break;
		}
		case WL_STATE_GAME_PAUSED_2:
		{
			wl_game_paused_2_logic();
			break;
		}
		case WL_STATE_GAME_CONTINUE:
		{
			wl_game_continue_logic();
			break;
		}
		case WL_STATE_GAME_QUIT:
		{
			wl_game_quit_logic();
			break;
		}
		case WL_STATE_MUSIC_QUIT:
		{
			wl_game_music_quit_logic();
			break;
		}
		case WL_STATE_GAME_EXIT:
		{
			wl_game_exit_logic();
			break;
		}
		case WL_STATE_GAME_WIN:
		{
			wl_game_win_logic();
			break;
		}
		case WL_STATE_DEMO_WIN:
		{
			wl_demo_win_logic();
			break;
		}
		case WL_STATE_GAME_OVER_WAIT:
		{
			wl_game_over_wait_logic();
			break;
		}
		case WL_STATE_GAME_TALLY:
		{
			wl_game_tally_logic();
			break;
		}
		case WL_STATE_GAME_DONE:
		{
			wl_game_done_logic();
			break;
		}
		case WL_STATE_PAUSED:
		{
			wl_title_process_menu();
			break;
		}
		case WL_STATE_PRIVACY:
		{
			wl_privacy_logic();
			break;
		}
		default:
		{
			break;
		}
	}
}

void wl_logic(void * data)
{
	/* disable mouse input if keyboard used */
	if(t3f_key[ALLEGRO_KEY_UP] || t3f_key[ALLEGRO_KEY_DOWN] || t3f_key[ALLEGRO_KEY_LEFT] || t3f_key[ALLEGRO_KEY_RIGHT])
	{
		if(wl_option[WL_OPTION_CONTROLS] != 1)
		{
			wl_mouse_off = true;
		}
	}
	wl_process_music_queue();
	wl_state_logic(wl_state);
	wl_process_effects();
	wl_message_logic(&wl_award_messages, false);
	wl_tick++;
}

void wl_state_render(int state)
{
	switch(state)
	{
		case WL_STATE_INTRO:
		{
			t3_logo_render();
			break;
		}
		case WL_STATE_TITLE:
		{
			wl_title_render();
			break;
		}
		case WL_STATE_TITLE_CREDITS:
		{
			wl_title_credits_render();
			break;
		}
		case WL_STATE_TITLE_EXIT:
		{
			wl_title_exit_render();
			break;
		}
		case WL_STATE_TO_GAME:
		{
			wl_title_to_game_render();
			break;
		}
		case WL_STATE_GAME_INTRO:
		{
			wl_game_intro_render();
			break;
		}
		case WL_STATE_GAME:
		{
			wl_game_render();
			break;
		}
		case WL_STATE_MUSIC:
		{
			wl_game_music_render();
			break;
		}
		case WL_STATE_GAME_TUTORIAL:
		{
			wl_game_tutorial_render();
			break;
		}
		case WL_STATE_GAME_OVER:
		{
			wl_game_over_render();
			break;
		}
		case WL_STATE_GAME_PAUSED:
		{
			wl_game_paused_render();
			break;
		}
		case WL_STATE_GAME_PAUSED_2:
		{
			wl_game_paused_2_render();
			break;
		}
		case WL_STATE_GAME_CONTINUE:
		{
			wl_game_intro_render();
			break;
		}
		case WL_STATE_GAME_QUIT:
		case WL_STATE_GAME_EXIT:
		{
			wl_game_intro_render();
			break;
		}
		case WL_STATE_MUSIC_QUIT:
		{
			wl_game_music_render();
			break;
		}
		case WL_STATE_GAME_WIN:
		{
			wl_game_win_render();
			break;
		}
		case WL_STATE_DEMO_WIN:
		{
			wl_demo_win_render();
			break;
		}
		case WL_STATE_GAME_OVER_WAIT:
		{
			wl_game_over_wait_render();
			break;
		}
		case WL_STATE_GAME_TALLY:
		{
			wl_game_tally_render();
			break;
		}
		case WL_STATE_GAME_DONE:
		{
			wl_game_done_render();
			break;
		}
		case WL_STATE_PRIVACY:
		{
			wl_privacy_render();
			break;
		}
		default:
		{
			break;
		}
	}
}

void wl_render(void * data)
{
	/* when exiting, render exit state instead of current state */
	if(wl_state == WL_STATE_TITLE_EXIT)
	{
		wl_state_render(wl_exit_state);
	}
	else if(wl_state == WL_STATE_PAUSED)
	{
		wl_state_render(wl_pause_state);
		al_hold_bitmap_drawing(false);
		al_draw_filled_rectangle(0.0, 0.0, 640.0, 480.0, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
		al_hold_bitmap_drawing(true);
		wl_title_render_menu();
		al_hold_bitmap_drawing(false);
	}
	else
	{
		wl_state_render(wl_state);
	}
	wl_render_effects();
	al_hold_bitmap_drawing(true);
	wl_render_messages(&wl_award_messages, false);
	al_hold_bitmap_drawing(false);
}

int main(int argc, char * argv[])
{
	if(!wl_initialize(argc, argv))
	{
		return -1;
	}
	t3f_run();
	wl_exit();
	t3f_finish();
	return 0;
}
