#include "t3f/gui.h"
#include "data.h"
#include "effects.h"
#include "text.h"

T3F_GUI_DRIVER wl_gui_driver;

int wl_menu_get_element_characters(const char * text)
{
	const ALLEGRO_USTR * ustr = NULL;
	ALLEGRO_USTR_INFO info;
	int i, l, u;

	ustr = al_ref_cstr(&info, text);
	if(!ustr)
	{
		return 0;
	}
	l = al_ustr_length(ustr);
	for(i = 0; i < al_ustr_length(ustr); i++)
	{
		u = al_ustr_get(ustr, i);
		if(u == '\\')
		{
			l--;
		}
	}
	return l;
}

int wl_menu_get_element_characters_u(ALLEGRO_USTR * ustr)
{
	int i, l, u;

	l = al_ustr_length(ustr);
	for(i = 0; i < al_ustr_length(ustr); i++)
	{
		u = al_ustr_get(ustr, i);
		if(u == '\\')
		{
			l--;
		}
	}
	return l;
}

static float wl_menu_get_element_width(T3F_GUI_ELEMENT * ep)
{
	int charwidth = 32;

	/* use small font */
	if(ep->resource == (void **)&wl_font[WL_FONT_16])
	{
		charwidth = 24;
	}

	return charwidth * wl_menu_get_element_characters((char *)ep->data);
}

static float wl_menu_get_element_height(T3F_GUI_ELEMENT * ep)
{
	int charheight = 32;

	/* use small font */
	if(ep->resource == (void **)&wl_font[WL_FONT_16])
	{
		charheight = 24;
	}

	return charheight;
}

static void wl_menu_render_element(T3F_GUI * pp, int i, bool hover)
{
	ALLEGRO_COLOR color;
	ALLEGRO_FONT * f;
	const ALLEGRO_USTR * ustr = NULL;
	ALLEGRO_USTR_INFO info;
	char * text;
	int x, w;
	int j, pj, l, fl, u;
	float y_offset = 0.0;

	if(hover || (pp->element[i].flags & T3F_GUI_ELEMENT_STATIC))
	{
		color = al_map_rgba(162, 180, 213, 204);
	}
	else
	{
		color = wl_darken_color(al_map_rgba(162, 180, 213, 204), 0.6);
	}
	text = (char *)(pp->element[i].data);
	if(pp->element[i].resource == (void **)&wl_font[WL_FONT_16])
	{
		w = 24;
		f = wl_font[WL_FONT_16];
		y_offset = 4.0;
	}
	else
	{
		w = 32;
		f = wl_font[WL_FONT_24];
		y_offset = 2.0;
	}
	if(!text)
	{
		return;
	}
	ustr = al_ref_cstr(&info, text);
	if(!ustr)
	{
		return;
	}
	l = al_ustr_length(ustr);
	fl = l;
	for(j = 0; j < l; j++)
	{
		u = al_ustr_get(ustr, j);

		/* subtract 1 for each '\' character since it is not going to be rendered */
		if(u == '\\')
		{
			fl--;
		}
	}

	x = pp->element[i].ox;
	if(pp->element[i].flags & T3F_GUI_ELEMENT_CENTRE)
	{
		x = pp->element[i].ox - (fl * w) / 2;
	}
	for(j = 0; j < fl; j++)
	{
		al_draw_tinted_scaled_bitmap(wl_bitmap[WL_BITMAP_TILE], color, 0, 0, al_get_bitmap_width(wl_bitmap[WL_BITMAP_TILE]), al_get_bitmap_height(wl_bitmap[WL_BITMAP_TILE]), pp->ox + x + j * w, pp->oy + pp->element[i].oy, w, w, 0);
	}
	pj = 0;
	for(j = 0; j < l; j++)
	{
		u = al_ustr_get(ustr, j);
		if(u == '\\')
		{
			j++;
			u = al_ustr_get(ustr, j);
			switch(u)
			{
				case '.':
				{
					al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, "...");
					break;
				}
				case '_':
				{
					if((wl_tick / 15) % 2 == 0)
					{
						al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, "_");
					}
					break;
				}
				case '|':
				{
					al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, "| |");
					break;
				}
				case '>':
				{
					al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, ">>");
					break;
				}
				case '+':
				{
					al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, ">|");
					break;
				}
				case '-':
				{
					al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, "|<");
					break;
				}
			}
		}
		else
		{
			al_ustr_set_chr(wl_menu_ustr, 0, u);
			al_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, ALLEGRO_ALIGN_CENTRE, al_cstr(wl_menu_ustr));
		}
		pj++;
	}
}

void wl_set_menu_driver(void)
{
	wl_gui_driver.get_element_width = wl_menu_get_element_width;
	wl_gui_driver.get_element_height = wl_menu_get_element_height;
	wl_gui_driver.render_element = wl_menu_render_element;
	t3f_set_gui_driver(&wl_gui_driver);
}
