#include "t3f/t3f.h"
#include "t3f/music.h"
#include "t3f/sound.h"
#include "t3f/android.h"
#include "t3net/t3net.h"
#include "data.h"
#include "title.h"
#include "credits.h"
#include "menu_proc.h"
#include "particle.h"
#include "game.h"
#include "text_entry.h"
#include "effects.h"
#include "game_intro.h"
#include "music_queue.h"
#include "menu.h"
#include "text_entry.h"
#include "video_mode.h"

void wl_save_config(void); // declare here for now instead of making main.h

int wl_selected_profile = 0;
static int wl_level_page = 0;
static bool wl_menu_caps = false;
static char * wl_options_title = NULL;
static char * wl_options_done_text = NULL;
static bool wl_multiplayer = false;

int wl_menu_proc_nop(void * data, int i, void * p)
{
	return 1;
}

int wl_menu_proc_setup_auto(void * data, int i, void * p)
{
	wl_option[WL_OPTION_INTERNET] = 0;
	wl_select_menu(WL_MENU_MAIN);
	return 1;
}

int wl_menu_proc_setup_manual(void * data, int i, void * p)
{
	wl_option[WL_OPTION_INTERNET] = 1;
	wl_select_menu(WL_MENU_MAIN);
	return 1;
}

/* main menu */
int wl_menu_proc_main_game(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME);
	return 1;
}

int wl_menu_proc_main_options(void * data, int i, void * p)
{
	wl_create_options_menu("OPTIONS", "BACK");
	wl_select_menu(WL_MENU_OPTIONS);
	return 1;
}

int wl_menu_proc_main_credits(void * data, int i, void * p)
{
	wl_mouse_clicked = true;
	wl_credits_initialize(&wl_credits, 0);
	wl_state = WL_STATE_TITLE_CREDITS;
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_main_privacy(void * data, int i, void * p)
{
	t3f_clear_touch_data();
	wl_state = WL_STATE_PRIVACY;
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_main_quit(void * data, int i, void * p)
{
	if(wl_current_menu != WL_MENU_QUIT)
	{
		wl_last_menu = wl_current_menu;
	}
	wl_select_menu(WL_MENU_QUIT);
	return 1;
}

int wl_menu_proc_main_quit_yes(void * data, int i, void * p)
{
	if(wl_pause_state == WL_STATE_PRIVACY)
	{
		t3f_clear_touch_data();
		wl_state = wl_pause_state;
	}
	wl_exit_state = wl_state;
	wl_state = WL_STATE_TITLE_EXIT;
	wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	return 1;
}

int wl_menu_proc_main_quit_no(void * data, int i, void * p)
{
	if(wl_pause_state == WL_STATE_PRIVACY)
	{
		t3f_clear_touch_data();
		wl_state = wl_pause_state;
	}
	wl_select_menu(wl_last_menu);
	return 1;
}

/* game menu */
int wl_menu_proc_game_play(void * data, int i, void * p)
{
//	wl_select_menu(WL_MENU_PLAYERS);
	return wl_menu_proc_game_players_single_player(data, i, p);
}

int wl_menu_proc_game_profiles(void * data, int i, void * p)
{
	wl_create_profiles_menu();
	wl_select_menu(WL_MENU_PROFILES);
	return 1;
}

int wl_menu_proc_game_leaderboards(void * data, int i, void * p)
{
	wl_leaderboard_mode = 0;
	wl_leaderboard_type = 0;
	wl_leaderboard_style = 0;
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_download_scores();
	wl_select_menu(WL_MENU_LEADERBOARD);
	return 1;
}

int wl_menu_proc_game_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_MAIN);
	return 1;
}

/* players menu */
int wl_menu_proc_game_players_single_player(void * data, int i, void * p)
{
	wl_game.players = 1;
	wl_multiplayer = false;
	wl_create_play_menu();
	wl_select_menu(WL_MENU_PLAY);
	return 1;
}

int wl_menu_proc_game_players_multiplayer(void * data, int i, void * p)
{
	wl_game.players = 0;
	wl_multiplayer = true;
	wl_select_menu(WL_MENU_MULTIPLAYER);
	return 1;
}

int wl_menu_proc_game_players_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME);
	return 1;
}

/* multiplayer menu */
int wl_menu_proc_game_multiplayer_party(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_PARTY;
	wl_level_page = 0;
	wl_create_party_play_menu();
	wl_select_menu(WL_MENU_PARTY_PLAY);
	return 1;
}

int wl_menu_proc_game_multiplayer_social(void * data, int i, void * p)
{
	return 1;
}

int wl_menu_proc_game_multiplayer_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PLAYERS);
	return 1;
}

/* play menu */
void wl_create_play_menu(void)
{
	int i, pos;

	if(wl_menu[WL_MENU_PLAY])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PLAY]);
	}
	wl_menu[WL_MENU_PLAY] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PLAY])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY], NULL, "SELECT PROFILE", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;
		for(i = 0; i < wl_profiles->entries; i++)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY], wl_menu_proc_play_profile, wl_profiles->entry[i].name, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			pos += 32;
		}
		if(wl_profiles->entries < 10)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY], wl_menu_proc_play_new, "NEW PLAYER", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			pos += 40;
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY], wl_menu_proc_play_cancel, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PLAY], 0, 480);
	}
}

int wl_menu_proc_play_profile(void * data, int i, void * p)
{
	T3F_GUI * pp = (T3F_GUI *)p;
	if(wl_multiplayer)
	{
		wl_game.player[wl_game.players].profile = pp->hover_element - 1;
		wl_game.players++;
		wl_create_party_play_menu();
		wl_select_menu(WL_MENU_PARTY_PLAY);
	}
	else
	{
		wl_game.player[0].profile = pp->hover_element - 1;
		wl_create_play_mode_menu();
		wl_select_menu(WL_MENU_PLAY_MODE);
	}
	return 1;
}

static void wl_menu_proc_play_new_ok_finish(void)
{
	char fnbuf[1024] = {0};

	memcpy(wl_profiles->entry[wl_profiles->entries].name, wl_entered_text, 256);
	wl_profiles->entries++;
	wl_save_profiles(wl_profiles, t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
	if(wl_multiplayer)
	{
		wl_game.player[wl_game.players].profile = wl_profiles->entries - 1;
		wl_game.players++;
		wl_create_party_play_menu();
		wl_select_menu(WL_MENU_PARTY_PLAY);
	}
	else
	{
		wl_game.player[0].profile = wl_profiles->entries - 1;
		wl_create_play_mode_menu();
		wl_select_menu(WL_MENU_PLAY_MODE);
		wl_create_play_menu();
	}
}

#ifdef ALLEGRO_ANDROID
	static void wl_edit_box_new_account_callback(void * data)
	{
		wl_menu_proc_play_new_ok_finish();
	}
#endif

int wl_menu_proc_play_new(void * data, int i, void * p)
{
	if(wl_profiles->entries < WL_MAX_PROFILES)
	{
		#ifndef ALLEGRO_ANDROID
			wl_entering_text = 1;
			wl_entering_text_pos = 0;
			al_ustr_assign_cstr(wl_entered_text_ustr, "\\_");
			al_ustr_to_buffer(wl_entered_text_ustr, wl_entered_text, 256);
			wl_create_name_entry_menu();
			wl_select_menu(WL_MENU_NEW_PROFILE);
			wl_tick = 0;
			t3f_clear_keys();
		#else
			strcpy(wl_entered_text, "");
			t3f_open_edit_box("Enter Profile Name", wl_entered_text, 256, "CapWords", wl_edit_box_new_account_callback, NULL);
		#endif
	}
	return 1;
}

int wl_menu_proc_play_cancel(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME);
	return 1;
}

int wl_menu_proc_play_new_name(void * data, int i, void * p)
{
	return 1;
}

int wl_menu_proc_play_new_ok(void * data, int i, void * p)
{
	wl_entering_text = 0;
	if(wl_profiles->entries < WL_MAX_PROFILES && !wl_is_string_empty())
	{
		wl_clear_profile(&wl_profiles->entry[wl_profiles->entries]);
		al_ustr_set_chr(wl_entered_text_ustr, wl_entering_text_pos, '\0');
		al_ustr_to_buffer(wl_entered_text_ustr, wl_entered_text, 256);
		wl_menu_proc_play_new_ok_finish();
	}
	return 1;
}

int wl_menu_proc_play_new_cancel(void * data, int i, void * p)
{
	wl_entering_text = 0;
	if(wl_multiplayer)
	{
		wl_select_menu(WL_MENU_PARTY_PLAY);
	}
	else
	{
		wl_select_menu(WL_MENU_PLAY);
	}
	return 1;
}

static void wl_get_level_name(int level, char * buf)
{
	ALLEGRO_CONFIG * cp = NULL;
	const char * val;

	cp = al_load_config_file("data/levels/levels.ini");
	if(cp)
	{
		sprintf(buf, "level %d", level);
		val = al_get_config_value(cp, buf, "name");
		if(val)
		{
			strcpy(buf, val);
		}
		al_destroy_config(cp);
	}
}

void wl_create_play_select_menu(void)
{
	int i, l, p;
	char buf[256] = {0};
	int pos = 0;
	bool unlocked = false;

	if(wl_menu[WL_MENU_PLAY_SELECT])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PLAY_SELECT]);
	}
	wl_menu[WL_MENU_PLAY_SELECT] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PLAY_SELECT])
	{
		/* figure out which profile has the most progress */
		p = 0;
		for(i = 0; i < wl_game.players; i++)
		{
			if(wl_profiles->entry[wl_game.player[i].profile].progress > wl_profiles->entry[wl_game.player[0].profile].progress)
			{
				p = i;
			}
			if(wl_profiles->entry[wl_game.player[i].profile].flags & WL_PROFILE_FLAG_UNLOCKED)
			{
				unlocked = true;
			}
			if(wl_profiles->entry[wl_game.player[i].profile].awards & WL_PROFILE_AWARD_WIN)
			{
				unlocked = true;
			}
		}

		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], NULL, "SELECT LEVEL", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;
		if(wl_game.mode == WL_GAME_MODE_STORY)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_level, "INTRO", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			pos += 32;
		}
		else
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], NULL, "", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		}
		if(wl_level_page == 0)
		{
			l = wl_profiles->entry[wl_game.player[p].profile].progress / WL_LEVEL_UP + 1;
			if(l > wl_end_level || unlocked)
			{
				l = wl_end_level;
			}
			for(i = 0; i < l; i++)
			{
				wl_get_level_name(i + 1, buf);
				t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_level, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				pos += 32;
			}

			/* add bonus link if player has finished game */
			if(!wl_demo && wl_game.mode != WL_GAME_MODE_STORY && unlocked)
			{
				t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_level_toggle_page, "Bonus Levels >>", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				pos += 32;
			}
		}
		else
		{
			for(i = 0; i < 6; i++)
			{
				wl_get_level_name(10 + i + 1, buf);
				t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_level, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				pos += 32;
			}
			t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_level_toggle_page, "<< Main Levels", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			pos += 32;
		}
		pos += 8;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_SELECT], wl_menu_proc_play_select_cancel, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PLAY_SELECT], 0, 480);
	}
}

int wl_menu_proc_play_select_level(void * data, int i, void * p)
{
	T3F_GUI * pp = (T3F_GUI *)p;

	wl_game.player[0].start_level = pp->hover_element - 1;
	if(wl_level_page == 1)
	{
		wl_game.player[0].start_level += 10;
	}
	wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	wl_overlay_state = 0;
	wl_state = WL_STATE_TO_GAME;
	return 1;
}

int wl_menu_proc_play_select_level_toggle_page(void * data, int i, void * p)
{
	wl_level_page = 1 - wl_level_page;
	wl_create_play_select_menu();
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_play_select_cancel(void * data, int i, void * p)
{
	switch(wl_game.mode)
	{
		case WL_GAME_MODE_PARTY:
		case WL_GAME_MODE_SOCIAL:
		{
			wl_select_menu(WL_MENU_MULTIPLAYER);
			break;
		}
		default:
		{
			wl_create_play_mode_menu();
			wl_select_menu(WL_MENU_PLAY_MODE);
			break;
		}
	}
	return 1;
}

/* game mode menu */
int wl_menu_proc_play_mode_story(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_STORY;
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_play_mode_casual(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_CASUAL;
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_play_mode_puzzle(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_PUZZLE;
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_play_mode_guru(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_GURU;
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_play_mode_music(void * data, int i, void * p)
{
	wl_game.mode = WL_GAME_MODE_MUSIC;
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_play_mode_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PLAY);
	return 1;
}

/* party play menu */
void wl_create_party_play_menu(void)
{
	int i, pos;

	if(wl_menu[WL_MENU_PARTY_PLAY])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PARTY_PLAY]);
	}
	wl_menu[WL_MENU_PARTY_PLAY] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PARTY_PLAY])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PARTY_PLAY], NULL, "PLAYERS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;
		for(i = 0; i < wl_game.players; i++)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PARTY_PLAY], wl_menu_proc_party_play_profile, wl_profiles->entry[wl_game.player[i].profile].name, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			pos += 32;
		}
		if(wl_game.players < 10)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PARTY_PLAY], wl_menu_proc_party_play_add, "ADD PLAYER", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			pos += 40;
		}
		if(wl_game.players > 0)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PARTY_PLAY], wl_menu_proc_party_play_ok, "OK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			pos += 40;
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PARTY_PLAY], wl_menu_proc_party_play_cancel, "CANCEL", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PARTY_PLAY], 0, 480);
	}
}

int wl_menu_proc_party_play_profile(void * data, int i, void * p)
{
	return 1;
}

int wl_menu_proc_party_play_add(void * data, int i, void * p)
{
	wl_create_play_menu();
	wl_select_menu(WL_MENU_PLAY);
	return 1;
}

int wl_menu_proc_party_play_ok(void * data, int i, void * p)
{
	wl_level_page = 0;
	wl_create_play_select_menu();
	wl_select_menu(WL_MENU_PLAY_SELECT);
	return 1;
}

int wl_menu_proc_party_play_cancel(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_MULTIPLAYER);
	return 1;
}

/* options menu */
int wl_menu_proc_options_controls_left(void * data, int i, void * p)
{
	wl_option[WL_OPTION_CONTROLS]--;
	if(wl_option[WL_OPTION_CONTROLS] < 0)
	{
		wl_option[WL_OPTION_CONTROLS] = 2;
	}
	if(wl_option[WL_OPTION_CONTROLS] == 2)
	{
		wl_mouse_off = true;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_controls_right(void * data, int i, void * p)
{
	wl_option[WL_OPTION_CONTROLS]++;
	if(wl_option[WL_OPTION_CONTROLS] > 2)
	{
		wl_option[WL_OPTION_CONTROLS] = 0;
	}
	if(wl_option[WL_OPTION_CONTROLS] == 2)
	{
		wl_mouse_off = true;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_music_left(void * data, int i, void * p)
{
	wl_option[WL_OPTION_MUSIC_VOLUME] -= 5;
	if(wl_option[WL_OPTION_MUSIC_VOLUME] < 0)
	{
		wl_option[WL_OPTION_MUSIC_VOLUME] = 0;
	}
	t3f_set_music_volume((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0);
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_music_right(void * data, int i, void * p)
{
	wl_option[WL_OPTION_MUSIC_VOLUME] += 5;
	if(wl_option[WL_OPTION_MUSIC_VOLUME] > 200)
	{
		wl_option[WL_OPTION_MUSIC_VOLUME] = 200;
	}
	t3f_set_music_volume((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0);
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_sound_left(void * data, int i, void * p)
{
	wl_option[WL_OPTION_SOUND_VOLUME] -= 5;
	if(wl_option[WL_OPTION_SOUND_VOLUME] < 0)
	{
		wl_option[WL_OPTION_SOUND_VOLUME] = 0;
	}
	t3f_set_sound_volume((float)wl_option[WL_OPTION_SOUND_VOLUME] / 100.0);
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_sound_right(void * data, int i, void * p)
{
	wl_option[WL_OPTION_SOUND_VOLUME] += 5;
	if(wl_option[WL_OPTION_SOUND_VOLUME] > 200)
	{
		wl_option[WL_OPTION_SOUND_VOLUME] = 200;
	}
	t3f_set_sound_volume((float)wl_option[WL_OPTION_SOUND_VOLUME] / 100.0);
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_video_left(void * data, int i, void * p)
{
	while(1)
	{
		wl_option[WL_OPTION_VIDEO_MODE]--;
		if(wl_option[WL_OPTION_VIDEO_MODE] < 0)
		{
			wl_option[WL_OPTION_VIDEO_MODE] = WL_VIDEO_MODE_FULL_SCREEN;
		}
		if(wl_video_mode_okay(wl_option[WL_OPTION_VIDEO_MODE]))
		{
			break;
		}
	}
	wl_initialize_video_mode();
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_video_right(void * data, int i, void * p)
{
	while(1)
	{
		wl_option[WL_OPTION_VIDEO_MODE]++;
		if(wl_option[WL_OPTION_VIDEO_MODE] > WL_VIDEO_MODE_FULL_SCREEN)
		{
			wl_option[WL_OPTION_VIDEO_MODE] = 0;
		}
		if(wl_video_mode_okay(wl_option[WL_OPTION_VIDEO_MODE]))
		{
			break;
		}
	}
	wl_initialize_video_mode();
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_texture_left(void * data, int i, void * p)
{
	wl_option[WL_OPTION_LOWRES]--;
	if(wl_option[WL_OPTION_LOWRES] < 0)
	{
		wl_option[WL_OPTION_LOWRES] = 1;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_texture_right(void * data, int i, void * p)
{
	wl_option[WL_OPTION_LOWRES]++;
	if(wl_option[WL_OPTION_LOWRES] > 1)
	{
		wl_option[WL_OPTION_LOWRES] = 0;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_clear_backbuffer_left(void * data, int i, void * p)
{
	if(t3f_option[T3F_OPTION_RENDER_MODE] == T3F_RENDER_MODE_ALWAYS_CLEAR)
	{
		t3f_set_option(T3F_OPTION_RENDER_MODE, T3F_RENDER_MODE_NORMAL);
	}
	else
	{
		t3f_set_option(T3F_OPTION_RENDER_MODE, T3F_RENDER_MODE_ALWAYS_CLEAR);
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_clear_backbuffer_right(void * data, int i, void * p)
{
	return wl_menu_proc_options_clear_backbuffer_left(data, i, p);
}

int wl_menu_proc_options_internet_left(void * data, int i, void * p)
{
	wl_option[WL_OPTION_INTERNET]--;
	if(wl_option[WL_OPTION_INTERNET] < 0)
	{
		wl_option[WL_OPTION_INTERNET] = 2;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_internet_right(void * data, int i, void * p)
{
	wl_option[WL_OPTION_INTERNET]++;
	if(wl_option[WL_OPTION_INTERNET] > 2)
	{
		wl_option[WL_OPTION_INTERNET] = 0;
	}
	wl_create_options_menu(NULL, NULL);
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_options_back(void * data, int i, void * p)
{
	wl_save_config();
	wl_select_menu(WL_MENU_MAIN);
	return 1;
}

void wl_create_options_menu(char * title, char * done_text)
{
	char * control_text[3] = {"AUTO", "MOUSE ONLY", "KEYBOARD ONLY"};
	char volume_text[16] = {0};
	char * internet_text[3] = {"AUTO", "ASK", "MANUAL"};
	char * video_text[5] = {"SMALL WINDOW", "MEDIUM WINDOW", "LARGE WINDOW", "RESIZABLE WINDOW", "FULL SCREEN"};
	#ifdef ALLEGRO_ANDROID
		char * texture_text[5] = {"HIGH", "LOW"};
	#endif
	char buf[32] = {0};
	float oy = 0.0;
	int aflags = T3F_GUI_ELEMENT_AUTOHIDE;

	if(!title)
	{
		title = wl_options_title;
	}
	else
	{
		wl_options_title = title;
	}
	if(!done_text)
	{
		done_text = wl_options_done_text;
	}
	else
	{
		wl_options_done_text = done_text;
	}
	if(wl_menu[WL_MENU_OPTIONS])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_OPTIONS]);
	}
	#ifdef ALLEGRO_ANDROID
		aflags = 0;
	#endif
	wl_menu[WL_MENU_OPTIONS] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_OPTIONS])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, title, (void **)&wl_font[WL_FONT_24], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		oy += 48;
		#ifndef ALLEGRO_ANDROID
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "CONTROLS", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
			oy += 32;
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, control_text[wl_option[WL_OPTION_CONTROLS]], (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Control Method");
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_controls_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_controls_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			oy += 32;
		#endif
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "MUSIC VOLUME", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		oy += 32;
		sprintf(volume_text, "%d%%", wl_option[WL_OPTION_MUSIC_VOLUME]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, volume_text, (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Volume of Music Playback");
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_music_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_music_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		oy += 32;
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "SOUND VOLUME", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		oy += 32;
		sprintf(volume_text, "%d%%", wl_option[WL_OPTION_SOUND_VOLUME]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, volume_text, (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Volume of Sound Effects Playback");
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_sound_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_sound_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		oy += 32;
		#ifndef ALLEGRO_ANDROID
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "VIDEO MODE", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
			oy += 32;
			sprintf(buf, "%s", video_text[wl_option[WL_OPTION_VIDEO_MODE]]);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, buf, (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Video Mode Setting");
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_video_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_video_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			oy += 32;
		#endif
		#ifdef ALLEGRO_ANDROID
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "TEXTURE QUALITY", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
			oy += 32;
			sprintf(buf, "%s", texture_text[wl_option[WL_OPTION_LOWRES]]);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, buf, (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Texture Quality Setting");
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_texture_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_texture_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			oy += 32;
		#endif
		#ifdef ALLEGRO_ANDROID
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "CLEAR BACKBUFFER", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
			oy += 32;
			sprintf(buf, "%s", t3f_option[T3F_OPTION_RENDER_MODE] == T3F_RENDER_MODE_ALWAYS_CLEAR ? "ALWAYS" : "NEVER");
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, buf, (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Clear the Screen Each Frame");
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_clear_backbuffer_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_clear_backbuffer_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
			oy += 32;
		#endif
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, "UPLOAD SCORES", (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		oy += 32;
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], NULL, internet_text[wl_option[WL_OPTION_INTERNET]], (void **)&wl_font[WL_FONT_16], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_OPTIONS], "Score Upload Setting");
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_internet_left, "<", (void **)&wl_font[WL_FONT_16], 320 - 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_internet_right, ">", (void **)&wl_font[WL_FONT_16], 320 + 240, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | aflags);
		oy += 40;
		t3f_add_gui_text_element(wl_menu[WL_MENU_OPTIONS], wl_menu_proc_options_back, done_text, (void **)&wl_font[WL_FONT_24], 320, oy, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_OPTIONS], 0, 480);
	}

}

/* profiles menu */
void wl_create_profiles_menu(void)
{
	int i;
	int pos = 0;

	if(wl_menu[WL_MENU_PROFILES])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PROFILES]);
	}
	wl_menu[WL_MENU_PROFILES] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILES])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILES], NULL, "SELECT PROFILE", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;
		for(i = 0; i < wl_profiles->entries; i++)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILES], wl_menu_proc_profiles_view, wl_profiles->entry[i].name, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			pos += 32;
		}
		pos += 8;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILES], wl_menu_proc_profiles_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PROFILES], 0, 480);
	}
}

int wl_menu_proc_profiles_view(void * data, int i, void * p)
{
	T3F_GUI * pp = (T3F_GUI *)p;

	wl_create_profile_menu(pp->hover_element - 1);
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

int wl_menu_proc_profiles_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME);
	return 1;
}

static bool wl_used(int profile, int word)
{
	int i;

	for(i = 0; i < word; i++)
	{
		if(wl_profiles->entry[profile].word[i] == wl_profiles->entry[profile].word[word])
		{
			return false;
		}
	}
	return true;
}

static int wl_profile_count_unique_words(int profile)
{
	unsigned int i;
	int count = 0;

	for(i = 0; i < wl_profiles->entry[profile].words; i++)
	{
		if(wl_used(profile, i))
		{
			count++;
		}
	}
	return count;
}

void wl_create_profile_menu(int profile)
{
	char buf[256] = {0};
	int pos = 0;

	wl_selected_profile = profile;
	if(wl_menu[WL_MENU_PROFILE])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PROFILE]);
	}
	wl_menu[WL_MENU_PROFILE] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILE])
	{
		sprintf(buf, "%s", wl_profiles->entry[profile].name);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], NULL, buf, (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		pos += 48;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_high_scores, "HIGH SCORES", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE], "See Your Personal Bests");
		pos += 48;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_stats, "STATS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE], "Various Game Stats");
		pos += 48;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_awards, "AWARDS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE], "View Your Awards");
		pos += 48;
		if(!(wl_profiles->entry[wl_selected_profile].flags & WL_PROFILE_FLAG_UNLOCKED) && !(wl_profiles->entry[wl_selected_profile].awards & WL_PROFILE_AWARD_WIN))
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_unlock, "UNLOCK CONTENT", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE], "Unlock All Levels and Game Modes");
			pos += 48;
		}
		if(wl_selected_profile > 0)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_delete, "DELETE PROFILE", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE], "Delete this Profile (Cannot Be Undone)");
			pos += 48;
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE], wl_menu_proc_profile_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PROFILE], 0, 480);
	}
}

int wl_menu_proc_profile_high_scores(void * data, int i, void * p)
{
	char buf[256] = {0};
	int pos = 0;
	int j, l;

	if(wl_menu[WL_MENU_PROFILE_SCORES])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PROFILE_SCORES]);
	}
	wl_menu[WL_MENU_PROFILE_SCORES] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILE_SCORES])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], NULL, "HIGH SCORES", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;
		sprintf(buf, "STORY          %6d", wl_profiles->entry[wl_selected_profile].high_score[WL_GAME_MODE_STORY]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;
		sprintf(buf, "CASUAL         %6d", wl_profiles->entry[wl_selected_profile].high_score[WL_GAME_MODE_CASUAL]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;
		sprintf(buf, "PUZZLE         %6d", wl_profiles->entry[wl_selected_profile].high_score[WL_GAME_MODE_PUZZLE]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;
		sprintf(buf, "GURU           %6d", wl_profiles->entry[wl_selected_profile].high_score[WL_GAME_MODE_GURU]);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 40;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], NULL, "BEST WORDS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;

		/* story mode best word */
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY] >= 0)
		{
			sprintf(buf, "STORY ");
			l = strlen(wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY]].data);
			if(l <= 15)
			{
				for(j = 0; j < 15 - l; j++)
				{
					strcat(buf, " ");
				}
				strcat(buf, wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY]].data);
			}
			else
			{
				for(j = 0; j < 14; j++)
				{
					buf[6 + j] = wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY]].data[j];
				}
				buf[20] = '\\';
				buf[21] = '.';
			}
		}
		else
		{
			sprintf(buf, "STORY             N/A");
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY] >= 0)
		{
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_STORY]].data);
		}
		pos += 32;

		/* casual mode best word */
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL] >= 0)
		{
			sprintf(buf, "CASUAL ");
			l = strlen(wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL]].data);
			if(l <= 14)
			{
				for(j = 0; j < 14 - l; j++)
				{
					strcat(buf, " ");
				}
				strcat(buf, wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL]].data);
			}
			else
			{
				for(j = 0; j < 13; j++)
				{
					buf[7 + j] = wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL]].data[j];
				}
				buf[20] = '\\';
				buf[21] = '.';
			}
		}
		else
		{
			sprintf(buf, "CASUAL            N/A");
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL] >= 0)
		{
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_CASUAL]].data);
		}
		pos += 32;

		/* PUZZLE mode best word */
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE] >= 0)
		{
			sprintf(buf, "PUZZLE ");
			l = strlen(wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE]].data);
			if(l <= 14)
			{
				for(j = 0; j < 14 - l; j++)
				{
					strcat(buf, " ");
				}
				strcat(buf, wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE]].data);
			}
			else
			{
				for(j = 0; j < 13; j++)
				{
					buf[7 + j] = wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE]].data[j];
				}
				buf[20] = '\\';
				buf[21] = '.';
			}
		}
		else
		{
			sprintf(buf, "PUZZLE            N/A");
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE] >= 0)
		{
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_PUZZLE]].data);
		}
		pos += 32;

		/* guru mode best word */
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU] >= 0)
		{
			sprintf(buf, "GURU ");
			l = strlen(wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU]].data);
			if(l <= 14)
			{
				for(j = 0; j < 16 - l; j++)
				{
					strcat(buf, " ");
				}
				strcat(buf, wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU]].data);
			}
			else
			{
				for(j = 0; j < 15; j++)
				{
					buf[7 + j] = wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU]].data[j];
				}
				buf[20] = '\\';
				buf[21] = '.';
			}
		}
		else
		{
			sprintf(buf, "GURU              N/A");
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		if(wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU] >= 0)
		{
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_dictionary->word[wl_profiles->entry[wl_selected_profile].best_word[WL_GAME_MODE_GURU]].data);
		}
		pos += 32;
		pos += 8;

		if(wl_selected_profile > 0 && (wl_profiles->entry[wl_selected_profile].flags & (WL_PROFILE_FLAG_UPLOAD_STORY | WL_PROFILE_FLAG_UPLOAD_CASUAL | WL_PROFILE_FLAG_UPLOAD_PUZZLE | WL_PROFILE_FLAG_UPLOAD_GURU | WL_PROFILE_FLAG_UPLOAD_BEST | WL_PROFILE_FLAG_UPLOAD_BEST_C | WL_PROFILE_FLAG_UPLOAD_BEST_P | WL_PROFILE_FLAG_UPLOAD_BEST_G)))
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_profile_upload, "UPLOAD SCORES", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_PROFILE_SCORES], "Upload Your Best Scores");
			pos += 48;
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_SCORES], wl_menu_proc_profile_high_scores_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PROFILE_SCORES], 0, 480);

		wl_select_menu(WL_MENU_PROFILE_SCORES);
	}
//	wl_create_awards_menu(wl_selected_profile);
	return 1;
}

int wl_menu_proc_profile_high_scores_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

int wl_menu_proc_profile_stats(void * data, int i, void * p)
{
	char buf[256] = {0};
	char buf2[256] = {0};
	int pos = 0;
	int h, m, s;

	if(wl_menu[WL_MENU_PROFILE_STATS])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PROFILE_STATS]);
	}
	wl_menu[WL_MENU_PROFILE_STATS] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILE_STATS])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], NULL, "STATS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;

		/* played games */
		sprintf(buf, "GAMES PLAYED   %5d", wl_profiles->entry[wl_selected_profile].games);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* play time */
		h = wl_profiles->entry[wl_selected_profile].time / 216000;
		m = (wl_profiles->entry[wl_selected_profile].time / 3600) % 60;
		s = (wl_profiles->entry[wl_selected_profile].time / 60) % 60;
		if(wl_profiles->entry[wl_selected_profile].time / 216000 > 0)
		{
			sprintf(buf2, "%3d:%02d:%02d", h, m, s);
		}
		else
		{
			sprintf(buf2, "    %02d:%02d", m, s);
		}
		sprintf(buf, "PLAY TIME  %s", buf2);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total words */
		sprintf(buf, "WORDS         %6lu", wl_profiles->entry[wl_selected_profile].words);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* unique words */
		sprintf(buf, "UNIQUE WORDS  %6d", wl_profile_count_unique_words(wl_selected_profile));
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total moves */
		sprintf(buf, "MOVES         %6lu", wl_profiles->entry[wl_selected_profile].moves);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total undos */
		sprintf(buf, "UNDOS         %6lu", wl_profiles->entry[wl_selected_profile].undos);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* wrong guesses */
		sprintf(buf, "INVALID WORDS %6lu", wl_profiles->entry[wl_selected_profile].wrongs);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 40;

		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_STATS], wl_menu_proc_profile_high_scores_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_PROFILE_STATS], 0, 480);

		wl_select_menu(WL_MENU_PROFILE_STATS);
	}
	return 1;
}

int wl_menu_proc_profile_awards(void * data, int i, void * p)
{
	wl_create_awards_menu(wl_selected_profile);
	wl_select_menu(WL_MENU_AWARDS);
	return 1;
}

bool wl_menu_upload_helper(void)
{
	wl_game.mode++;
	if(wl_game.mode <= WL_GAME_MODE_GURU)
	{
		wl_game.player[0].profile = wl_selected_profile;
		wl_game.player[0].score = wl_profiles->entry[wl_selected_profile].high_score[wl_game.mode];
		wl_game.player[0].word_list[0] = wl_profiles->entry[wl_selected_profile].best_word[wl_game.mode];
		wl_game.player[0].word_list_count = 1;
		wl_upload_scores(true);
		return true;
	}
	return false;
}

int wl_menu_proc_profile_upload(void * data, int i, void * p)
{
	wl_leaderboard_profile_upload_flags = wl_profiles->entry[wl_selected_profile].flags;
	wl_leaderboard_mode = -1;
	wl_game.mode = -1;
	wl_menu_upload_helper();
	wl_select_menu(WL_MENU_EMPTY);
	return 1;
}

int wl_menu_proc_profile_unlock(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE_UNLOCK);
	return 1;
}

int wl_menu_proc_profile_unlock_yes(void * data, int i, void * p)
{
	char fnbuf[1024] = {0};

	wl_profiles->entry[wl_selected_profile].flags |= WL_PROFILE_FLAG_UNLOCKED;
	wl_create_profile_menu(wl_selected_profile);
	wl_save_profiles(wl_profiles, t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

int wl_menu_proc_profile_unlock_no(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

int wl_menu_proc_profile_delete(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE_DELETE);
	return 1;
}

int wl_menu_proc_profile_delete_yes(void * data, int i, void * p)
{
	char fnbuf[1024] = {0};
	int j;

	for(j = wl_selected_profile; j < wl_profiles->entries - 1; j++)
	{
		memcpy(&wl_profiles->entry[j], &wl_profiles->entry[j + 1], sizeof(WL_PROFILE));
	}
	wl_profiles->entries--;
	wl_save_profiles(wl_profiles, t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
	wl_create_profiles_menu();
	wl_select_menu(WL_MENU_PROFILES);
	return 1;
}

int wl_menu_proc_profile_delete_no(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

int wl_menu_proc_profile_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILES);
	return 1;
}

void wl_create_awards_menu(int profile)
{
	int i;
	int bit = 0;
	char * names[32] = {"Enlightened", " Marathon  ", "  Hundred  ", "Dedication ", "  Stride   ", "  Quickie  ", "   Loser   ", " All Clear ", "  Big Fan  ", "Foul Mouth "};
	char * hints[32] = {"Finish Story Mode", "Finish Story Mode In One Go", "Make One Hundred Words In One Game", "Play One Hundred Games", "Make 20 Words In One Level", "Finish a Level In Less then 10 Words", "Lose the Game Without Making Any Words", "Clear Entire Board In Puzzle Mode", "Name Main Characters from Food Boy the Movie", "Make Several Vulgar Words"};
	int hint_array[32] = {1}; // 1 means show hint, 2 means received award, 0 means not revealed yet
	char buf[256] = {0};
	int pos = 0;
	bool received;

	if(wl_menu[WL_MENU_AWARDS])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_AWARDS]);
	}
	wl_menu[WL_MENU_AWARDS] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_AWARDS])
	{

		/* see what hints we are giving */
		bit = WL_PROFILE_AWARD_WIN;
		for(i = 0; i < 8; i++)
		{
			if(wl_profiles->entry[profile].awards & bit)
			{
				hint_array[i] = 2;
				if(i > 0)
				{
					if(hint_array[i - 1] == 0)
					{
						hint_array[i - 1] = 1;
					}
				}
				if(i < 7)
				{
					if(hint_array[i + 1] == 0)
					{
						hint_array[i + 1] = 1;
					}
				}
			}
			bit = bit << 1;
		}

		/* secret awards are only revealed once gotten */
		for(i = 8; i < 10; i++)
		{
			if(wl_profiles->entry[profile].awards & bit)
			{
				hint_array[i] = 2;
			}
			bit = bit << 1;
		}

		t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], NULL, "AWARDS", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		pos += 48;
		bit = WL_PROFILE_AWARD_WIN;
		for(i = 0; i < 8; i++)
		{
			if(hint_array[i])
			{
				received = hint_array[i] == 2;
				sprintf(buf, "%s%s%s", received ? "* " : "X ", names[i], received ? " *" : " X");
				t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, received ? al_map_rgba_f(1.0, 1.0, 1.0, 1.0) : al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				t3f_describe_last_gui_element(wl_menu[WL_MENU_AWARDS], hints[i]);
			}
			else
			{
				if(hint_array[i])
				{
					t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], wl_menu_proc_nop, "???", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
					t3f_describe_last_gui_element(wl_menu[WL_MENU_AWARDS], hints[i]);
				}
				else
				{
					t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], wl_menu_proc_nop, "???", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				}
			}
			pos += 32;
		}

		/* secret awards */
		for(i = 8; i < 10; i++)
		{
			if(hint_array[i])
			{
				sprintf(buf, "%s%s", hint_array[i] == 2 ? "*" : "", names[i]);
				t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], wl_menu_proc_nop, buf, (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				t3f_describe_last_gui_element(wl_menu[WL_MENU_AWARDS], hints[i + 8]);
				pos += 32;
			}
		}
		pos += 8;
		t3f_add_gui_text_element(wl_menu[WL_MENU_AWARDS], wl_menu_proc_awards_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(wl_menu[WL_MENU_AWARDS], 0, 480);
	}
}

int wl_menu_proc_awards_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE);
	return 1;
}

static void wl_truncate_string(ALLEGRO_USTR * sp, int length)
{
	if(al_ustr_length(sp) > (unsigned int)length)
	{
		al_ustr_set_chr(sp, al_ustr_offset(sp, length - 1), '\\');
		al_ustr_set_chr(sp, al_ustr_offset(sp, length), '.');
		al_ustr_truncate(sp, al_ustr_offset(sp, length + 1));
	}
}

/* leaderboard menu */
int wl_menu_proc_leaderboard_previous_mode(void * data, int i, void * p)
{
	wl_leaderboard_mode--;
	if(wl_leaderboard_mode < 0)
	{
		wl_leaderboard_mode = 3;
	}
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_download_scores();
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_leaderboard_next_mode(void * data, int i, void * p)
{
	wl_leaderboard_mode++;
	if(wl_leaderboard_mode > 3)
	{
		wl_leaderboard_mode = 0;
	}
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_download_scores();
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

int wl_menu_proc_leaderboard_toggle_type(void * data, int i, void * p)
{
	wl_leaderboard_type--;
	if(wl_leaderboard_type < 0)
	{
		wl_leaderboard_type = 1;
	}
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_download_scores();
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
	return 1;
}

bool wl_create_leaderboard_load_menu(int mode, int type)
{
	char * mode_text[] = {"STORY MODE", "CASUAL MODE", "PUZZLE MODE", "GURU MODE"};
	char * type_text[] = {"HIGH SCORE", "BEST WORDS"};
	int i;
	int pos = 0;

	if(wl_menu[WL_MENU_LEADERBOARD])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_LEADERBOARD]);
	}
	wl_menu[WL_MENU_LEADERBOARD] = t3f_create_gui(0, 0);
	if(!wl_menu[WL_MENU_LEADERBOARD])
	{
		return false;
	}
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, mode_text[mode], (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
/*	if(wl_leaderboard_style == 0)
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_previous_mode, "<", (void **)&wl_font[WL_FONT_16], 320 - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_next_mode, ">", (void **)&wl_font[WL_FONT_16], 320 + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	} */
	pos += 32;
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, type_text[type], (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
/*	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_toggle_type, "<", (void **)&wl_font[WL_FONT_16], 320 - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_toggle_type, ">", (void **)&wl_font[WL_FONT_16], 320 + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE); */
	pos += 48;
	for(i = 0; i < 10; i++)
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, "...", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;
	}
	pos += 16;
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, "", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(wl_menu[WL_MENU_LEADERBOARD], 0, 480);
	return true;
}

bool wl_create_leaderboard_menu(int mode, int type)
{
	char * mode_text[] = {"STORY MODE", "CASUAL MODE", "PUZZLE MODE", "GURU MODE"};
	char * type_text[] = {"HIGH SCORE", "BEST WORDS"};
	char buf[256] = {0};
	ALLEGRO_USTR * part[6];
//	char part[4][32] = {{0}};
	int i, j;
	int pos = 0;
	int spaces;

	if(wl_menu[WL_MENU_LEADERBOARD])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_LEADERBOARD]);
	}
	wl_menu[WL_MENU_LEADERBOARD] = t3f_create_gui(0, 0);
	if(!wl_menu[WL_MENU_LEADERBOARD])
	{
		t3net_destroy_leaderboard(wl_leaderboard);
		wl_leaderboard = NULL;
		return false;
	}
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, mode_text[mode], (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	if(wl_leaderboard_style == 0)
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_previous_mode, "<", (void **)&wl_font[WL_FONT_16], 320 - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_next_mode, ">", (void **)&wl_font[WL_FONT_16], 320 + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	}
	pos += 32;
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, type_text[type], (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_toggle_type, "<", (void **)&wl_font[WL_FONT_16], 320 - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboard_toggle_type, ">", (void **)&wl_font[WL_FONT_16], 320 + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	pos += 48;
	for(i = 0; i < wl_leaderboard->entries; i++)
	{
		part[0] = al_ustr_new(wl_leaderboard->entry[i]->name);
		part[1] = al_ustr_new(wl_leaderboard->entry[i]->name);
		wl_truncate_string(part[0], 10);
		if(type == 1)
		{
			part[2] = al_ustr_new(wl_dictionary->word[atoi(wl_leaderboard->entry[i]->extra)].data);
			part[3] = al_ustr_new(wl_dictionary->word[atoi(wl_leaderboard->entry[i]->extra)].data);
		}
		else
		{
			sprintf(buf, "%lu", (wl_leaderboard->entry[i]->score - 'w' - 'l' - 'e' - 'a' - 'p') / 3);
			part[2] = al_ustr_new(buf);
			sprintf(buf, "%lu", (wl_leaderboard->entry[i]->score - 'w' - 'l' - 'e' - 'a' - 'p') / 3);
			part[3] = al_ustr_new(buf);
		}
		wl_truncate_string(part[2], 10);

		/* add spaces to even things out */
		spaces = 21 - (wl_menu_get_element_characters_u(part[0]) + wl_menu_get_element_characters_u(part[2]));
		part[4] = al_ustr_dup(part[0]);
		for(j = 0; j < spaces; j++)
		{
			al_ustr_append_cstr(part[4], " ");
		}
		al_ustr_append(part[4], part[2]);

		if(wl_leaderboard_style == 1 && i == wl_leaderboard_spot)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, (char *)al_cstr(part[4]), (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		}
		else
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, (char *)al_cstr(part[4]), (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		}
		part[5] = al_ustr_dup(part[1]);
		al_ustr_append_cstr(part[5], " - ");
		al_ustr_append(part[5], part[3]);

		/* append unused tile count for puzzle mode */
		if(mode == WL_GAME_MODE_PUZZLE && type == 0 && strlen(wl_leaderboard->entry[i]->extra) > 0)
		{
			al_ustr_append_cstr(part[5], " (");
			al_ustr_append_cstr(part[5], wl_leaderboard->entry[i]->extra);
			al_ustr_append_cstr(part[5], " Unused)");
		}
		t3f_describe_last_gui_element(wl_menu[WL_MENU_LEADERBOARD], (char *)al_cstr(part[5]));
		for(j = 0; j < 6; j++)
		{
			al_ustr_free(part[j]);
		}

		pos += 32;
	}
	for(i = wl_leaderboard->entries; i < 10; i++)
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], NULL, "...", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;
	}
	pos += 16;
	t3f_add_gui_text_element(wl_menu[WL_MENU_LEADERBOARD], wl_menu_proc_leaderboards_done, "DONE", (void **)&wl_font[WL_FONT_16], 320, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(wl_menu[WL_MENU_LEADERBOARD], 0, 480);
	return true;
}

int wl_menu_proc_leaderboards_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME);
	return 1;
}

int wl_menu_proc_leaderboards_done(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_TITLE)
	{
		wl_select_menu(WL_MENU_GAME);
	}
	else if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		if(wl_state == WL_STATE_GAME_OVER)
		{
			wl_select_menu(WL_MENU_GAME_OVER);
		}
		else
		{
			wl_game_exit();
			wl_play_music("data/music/theme.ogg");
		}
	}
	else
	{
		wl_select_menu(WL_MENU_RETRY);
	}
	return 1;
}

int wl_menu_proc_game_over_retry(void * data, int i, void * p)
{
	wl_destroy_cinema(wl_cinema);
	al_stop_timer(t3f_timer);
	wl_cinema = wl_load_cinema_script("data/scripts/continue.script");
	if(wl_cinema)
	{
		wl_set_cinema_view_callbacks(wl_cinema, wl_game_view_logic, wl_game_view_render);
		al_start_timer(t3f_timer);
		wl_game.player[0].start_level = wl_game.player[0].level;
		wl_state = WL_STATE_GAME_INTRO;
	}
	wl_reset_particles(0);
	wl_game.regenerate = true;
	return 1;
}

int wl_menu_proc_game_over_give_up(void * data, int i, void * p)
{
	wl_destroy_cinema(wl_cinema);
	al_stop_timer(t3f_timer);
	wl_cinema = wl_load_cinema_script("data/scripts/quit.script");
	if(wl_cinema)
	{
		wl_reset_particles(0);
		wl_menu[wl_current_menu]->hover_element = -1;
		wl_reset_particles(0);
		wl_set_cinema_view_callbacks(wl_cinema, wl_game_paused_view_logic, wl_game_render);
		al_start_timer(t3f_timer);
		wl_state = WL_STATE_GAME_QUIT;
		wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	}
	return 1;
}

int wl_menu_proc_demo_over_yes(void * data, int i, void * p)
{
	#ifdef T3F_ANDROID_GP
		t3f_open_url("market://details?id=com.t3i.wordleap");
	#else
		t3f_open_url("http://www.t3-i.com/pages/link.php?id=16");
	#endif
	return 1;
}

int wl_menu_proc_demo_over_no(void * data, int i, void * p)
{
	wl_ending_state = 1;
	return 1;
}

int wl_menu_proc_game_paused_continue(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_PAUSED)
	{
		wl_state = wl_pause_state;
	}
	else if(wl_state == WL_STATE_GAME_PAUSED_2)
	{
		wl_state = wl_overlay_state;
	}
	else if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_destroy_cinema(wl_cinema);
		al_stop_timer(t3f_timer);
		wl_cinema = wl_load_cinema_script("data/scripts/continue.script");
		if(wl_cinema)
		{
			wl_set_cinema_view_callbacks(wl_cinema, wl_game_paused_view_logic, wl_game_render);
			al_start_timer(t3f_timer);
			wl_state = WL_STATE_GAME_CONTINUE;
		}
		wl_reset_particles(0);
	}
	else
	{
		wl_state = wl_exit_state;
	}
	if(wl_state == WL_STATE_GAME_INTRO || wl_game.mode == WL_GAME_MODE_MUSIC)
	{
		t3f_resume_music();
		wl_set_music_fade_speed(WL_MUSIC_FADE_SPEED);
	}
	wl_game.player[0].clicked = true;
	return 1;
}

int wl_menu_proc_game_paused_exit(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_PAUSED)
	{
		wl_exit_state = wl_pause_state;
		wl_state = WL_STATE_TITLE_EXIT;
		wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	}
	else if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		wl_destroy_cinema(wl_cinema);
		al_stop_timer(t3f_timer);
		wl_cinema = wl_load_cinema_script("data/scripts/quit.script");
		if(wl_cinema)
		{
			wl_reset_particles(0);
			wl_menu[wl_current_menu]->hover_element = -1;
			wl_reset_particles(0);
			wl_set_cinema_view_callbacks(wl_cinema, wl_game_paused_view_logic, wl_game_render);
			al_start_timer(t3f_timer);
			wl_state = WL_STATE_GAME_EXIT;
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
		}
	}
	else
	{
		wl_state = WL_STATE_GAME_EXIT;
		wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	}
	return 1;
}

void wl_create_play_mode_menu(void)
{
	int pos = 1;

	if(wl_menu[WL_MENU_PLAY_MODE])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_PLAY_MODE]);
	}
	wl_menu[WL_MENU_PLAY_MODE] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PLAY_MODE])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], NULL, "GAME TYPE", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_story, "STORY", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Story Mode");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_casual, "CASUAL", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Make 10 Words");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_puzzle, "PUZZLE", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Try to Clear the Board");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_guru, "GURU", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Survive As Long As You Can");
		pos++;

		/* don't unlock music mode in demo */
		if(!wl_demo)
		{
			if((wl_profiles->entry[wl_game.player[0].profile].awards & WL_PROFILE_AWARD_WIN) || (wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_UNLOCKED))
			{
				t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_music, "SCENE", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
				t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Enjoy the Sights and Sounds");
				pos++;
			}
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAY_MODE], wl_menu_proc_play_mode_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_PLAY_MODE], "Back to Previous Menu");
		pos++;
		t3f_center_gui(wl_menu[WL_MENU_PLAY_MODE], 0, 480);
	}
}

int wl_menu_proc_game_paused_quit(void * data, int i, void * p)
{
	if(wl_pause_state == WL_STATE_MUSIC)
	{
		wl_state = WL_STATE_MUSIC_QUIT;
	}
	else
	{
		wl_state = WL_STATE_GAME_QUIT;
	}
	wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	return 1;
}

int wl_menu_proc_game_retry(void * data, int i, void * p)
{
	wl_game.retry = true;
	wl_game.regenerate = true;
	wl_game_initialize(WL_LEVEL_LOAD_MUSIC);
	wl_state = WL_STATE_GAME;
	return 1;
}

int wl_menu_proc_game_quit(void * data, int i, void * p)
{
	wl_state = WL_STATE_GAME_QUIT;
	wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	wl_select_menu(WL_MENU_MAIN);
	return 1;
}

int wl_menu_proc_tutorial_skip_yes(void * data, int i, void * p)
{
	if(wl_game.player[0].profile != 0)
	{
		wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_BASIC;
		switch(wl_game.mode)
		{
			case WL_GAME_MODE_STORY:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_STORY;
				break;
			}
			case WL_GAME_MODE_CASUAL:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_CASUAL;
				break;
			}
			case WL_GAME_MODE_PUZZLE:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_TUTORIAL_PUZZLE;
				break;
			}
		}
	}
	wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
	wl_game.player[0].state = WL_PLAYER_STATE_EXIT;
	wl_tutorial_step = 99;
	wl_state = WL_STATE_GAME_TUTORIAL;
	wl_game.player[0].clicked = true;
	return 1;
}

int wl_menu_proc_tutorial_skip_no(void * data, int i, void * p)
{
	wl_state = WL_STATE_GAME_TUTORIAL;
	wl_game.player[0].clicked = true;
	return 1;
}

int wl_menu_proc_overlay_menu(void * data, int i, void * p)
{
	wl_overlay_state = wl_state;
	if(wl_game.mode == WL_GAME_MODE_STORY && wl_state != WL_STATE_GAME_TUTORIAL)
	{
		al_stop_timer(t3f_timer);
		t3f_render(true);
		wl_cinema = wl_load_cinema_script("data/scripts/game_over.script");
		if(wl_cinema)
		{
			wl_set_cinema_view_callbacks(wl_cinema, wl_game_paused_view_logic, wl_game_render);
			al_start_timer(t3f_timer);
			wl_select_menu(WL_MENU_GAME_PAUSED);
			wl_state = WL_STATE_GAME_PAUSED;
		}
	}
	else if(wl_state == WL_STATE_MUSIC)
	{
		wl_select_menu(WL_MENU_GAME_PAUSED_2);
		wl_pause_state = wl_state;
		wl_state = WL_STATE_PAUSED;
		t3f_pause_music();
		wl_set_music_fade_speed(0.0);
	}
	else
	{
		wl_select_menu(WL_MENU_GAME_PAUSED_2);
		wl_state = WL_STATE_GAME_PAUSED_2;
	}
	wl_game.player[0].clicked = true;
	wl_menu[WL_MENU_GAME_OVERLAY]->hover_element = -1;
	wl_menu[WL_MENU_MUSIC_OVERLAY]->hover_element = -1;
	return 1;
}

int wl_menu_proc_softkey(void * data, int i, void * p)
{
	char * upper_keys = "!@#$%^&*()\bQWERTYUIOPASDFGHJKLZXCVBNM";
	char * lower_keys = "1234567890\bqwertyuiopasdfghjklzxcvbnm";
	T3F_GUI * pp = (T3F_GUI *)p;

	if(pp->hover_element - 2 < (int)strlen(upper_keys))
	{
		if(wl_menu_caps)
		{
			t3f_add_key(upper_keys[pp->hover_element - 2]);
		}
		else
		{
			t3f_add_key(lower_keys[pp->hover_element - 2]);
		}
	}
	else
	{
		wl_menu_caps = !wl_menu_caps;
		wl_create_name_entry_menu();
	}
	return 1;
}

void wl_create_name_entry_menu(void)
{
	int y;

	if(wl_menu[WL_MENU_NEW_PROFILE])
	{
		t3f_destroy_gui(wl_menu[WL_MENU_NEW_PROFILE]);
	}
	wl_menu[WL_MENU_NEW_PROFILE] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_NEW_PROFILE])
	{
		y = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], NULL, "ENTER NAME", (void **)&wl_font[WL_FONT_24], 320, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		y += 48;
		t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_play_new_name, wl_entered_text, (void **)&wl_font[WL_FONT_24], 320, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		y += 48;
/*		#ifdef T3F_ANDROID
			if(wl_menu_caps)
			{
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "!", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "@", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "#", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "$", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "%", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "^", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "&", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "*", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "(", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, ")", (void **)&wl_font[WL_FONT_24], pos + 32 * 9, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "<", (void **)&wl_font[WL_FONT_24], pos + 32 * 10, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "Q", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "W", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "E", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "R", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "T", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "Y", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "U", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "I", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "O", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "P", (void **)&wl_font[WL_FONT_24], pos + 32 * 9, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "A", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "S", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "D", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "F", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "G", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "H", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "J", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "K", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "L", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "Z", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "X", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "C", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "V", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "B", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "N", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "M", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "^", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 48;
			}
			else
			{
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "1", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "2", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "3", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "4", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "5", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "6", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "7", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "8", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "9", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "0", (void **)&wl_font[WL_FONT_24], pos + 32 * 9, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "<", (void **)&wl_font[WL_FONT_24], pos + 32 * 10, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "q", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "w", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "e", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "r", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "t", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "y", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "u", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "i", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "o", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "p", (void **)&wl_font[WL_FONT_24], pos + 32 * 9, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "a", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "s", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "d", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "f", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "g", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "h", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "j", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "k", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "l", (void **)&wl_font[WL_FONT_24], pos + 32 * 8, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 32;
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "z", (void **)&wl_font[WL_FONT_24], pos, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "x", (void **)&wl_font[WL_FONT_24], pos + 32 * 1, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "c", (void **)&wl_font[WL_FONT_24], pos + 32 * 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "v", (void **)&wl_font[WL_FONT_24], pos + 32 * 3, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "b", (void **)&wl_font[WL_FONT_24], pos + 32 * 4, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "n", (void **)&wl_font[WL_FONT_24], pos + 32 * 5, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "m", (void **)&wl_font[WL_FONT_24], pos + 32 * 6, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_softkey, "^", (void **)&wl_font[WL_FONT_24], pos + 32 * 7, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
				y += 48;
			}
		#endif */
		t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_play_new_ok, "OK", (void **)&wl_font[WL_FONT_24], 320, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		y += 48;
		t3f_add_gui_text_element(wl_menu[WL_MENU_NEW_PROFILE], wl_menu_proc_play_new_cancel, "CANCEL", (void **)&wl_font[WL_FONT_24], 320, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		#ifdef ALLEGRO_ANDROID
			t3f_center_gui(wl_menu[WL_MENU_NEW_PROFILE], 0, 240);
		#else
			t3f_center_gui(wl_menu[WL_MENU_NEW_PROFILE], 0, 480);
		#endif
	}
}

int wl_menu_proc_upload_retry(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_TITLE)
	{
		wl_upload_scores(true);
	}
	else
	{
		wl_upload_scores(false);
	}
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_select_menu(WL_MENU_LEADERBOARD);
	return 1;
}

int wl_menu_proc_upload_cancel(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_TITLE)
	{
		wl_select_menu(WL_MENU_PROFILE_SCORES);
	}
	else
	{
		if(wl_game.mode == WL_GAME_MODE_STORY)
		{
			wl_select_menu(WL_MENU_GAME_OVER);
			wl_state = WL_STATE_GAME_OVER;
		}
		else
		{
			wl_select_menu(WL_MENU_RETRY);
			wl_state = WL_STATE_GAME_DONE;
		}
	}
	return 1;
}

int wl_menu_proc_download_retry(void * data, int i, void * p)
{
	wl_download_scores();
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_select_menu(WL_MENU_LEADERBOARD);
	return 1;
}

int wl_menu_proc_download_cancel(void * data, int i, void * p)
{
	if(wl_state == WL_STATE_TITLE)
	{
		wl_select_menu(WL_MENU_GAME);
	}
	else
	{
		if(wl_game.mode == WL_GAME_MODE_STORY)
		{
			wl_select_menu(WL_MENU_GAME_OVER);
			wl_state = WL_STATE_GAME_OVER;
		}
		else
		{
			wl_select_menu(WL_MENU_RETRY);
			wl_state = WL_STATE_GAME_DONE;
		}
	}
	return 1;
}

int wl_menu_proc_overlay_fast_forward(void * data, int i, void * p)
{
	wl_game.fast_forward = true;
	return 1;
}

int wl_menu_proc_overlay_next_track(void * data, int i, void * p)
{
	if((wl_game.player[0].start_level <= 10 && wl_game.player[0].level < 10) || (wl_game.player[0].start_level > 10 && wl_game.player[0].level <= 15))
	{
		wl_clear_music_queue();
		wl_game.player[0].start_level = wl_game.player[0].level + 1;
		wl_game_initialize(WL_LEVEL_LOAD_NORMAL);
	}
	else
	{
		wl_finish_current_song();
		wl_game.music_done = true;
	}
	wl_game.fast_forward = false;
	return 1;
}

int wl_menu_proc_overlay_previous_track(void * data, int i, void * p)
{
	if(wl_game.player[0].start_level <= 10 && wl_game.player[0].level >= 0)
	{
		wl_clear_music_queue();
		wl_game.player[0].start_level = wl_game.player[0].level - 1;
		if(wl_game.player[0].start_level < 1)
		{
			wl_game.player[0].start_level = 1;
		}
		wl_game_initialize(WL_LEVEL_LOAD_NORMAL);
	}
	else if(wl_game.player[0].start_level > 10 && wl_game.player[0].level >= 10)
	{
		wl_clear_music_queue();
		wl_game.player[0].start_level = wl_game.player[0].level - 1;
		if(wl_game.player[0].start_level < 11)
		{
			wl_game.player[0].start_level = 11;
		}
		wl_game_initialize(WL_LEVEL_LOAD_NORMAL);
	}
	else
	{
		wl_finish_current_song();
		wl_game.music_done = true;
	}
	wl_game.fast_forward = false;
	return 1;
}

int wl_menu_proc_upload_yes(void * data, int i, void * p)
{
	wl_leaderboard_mode = wl_game.mode;
	wl_leaderboard_type = 0;
	wl_leaderboard_style = 1;
	wl_game_upload_scores();
	wl_create_leaderboard_load_menu(wl_leaderboard_mode, wl_leaderboard_type);
	wl_select_menu(WL_MENU_LEADERBOARD);
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		if(wl_state != WL_STATE_GAME_DONE)
		{
			wl_state = WL_STATE_GAME_OVER_WAIT;
		}
	}
	else
	{
		wl_state = WL_STATE_GAME_DONE;
	}
	return 1;
}

int wl_menu_proc_upload_no(void * data, int i, void * p)
{
	if(wl_game.mode == WL_GAME_MODE_STORY)
	{
		if(wl_state != WL_STATE_GAME_DONE)
		{
			wl_select_menu(WL_MENU_GAME_OVER);
		}
		else
		{
			wl_select_menu(WL_MENU_MAIN);
			wl_play_music("data/music/theme.ogg");
			wl_state = WL_STATE_TITLE;
		}
	}
	else
	{
		wl_select_menu(WL_MENU_RETRY);
	}
	return 1;
}
