#include "t3f/t3f.h"
#include "data.h"
#include "defines.h"
#include "message.h"
#include "text.h"

void wl_add_message(WL_MESSAGES * mp, int type, char * text, void * dp, int flags)
{
	ALLEGRO_FONT * fp;
	T3F_FONT * tfp;

	if(mp->messages < WL_MAX_MESSAGES)
	{
		mp->message[mp->messages].type = type;
		switch(type)
		{
			case WL_MESSAGE_TYPE_SCROLLER:
			{
				fp = (ALLEGRO_FONT *)dp;
				strcpy(mp->message[mp->messages].text, text);
				mp->message[mp->messages].x = 240.0 - al_get_text_width(fp, text) / 2;
				mp->message[mp->messages].y = 240 - al_get_font_line_height(fp) / 2;
				mp->message[mp->messages].vx = 0.0;
				mp->message[mp->messages].vy = 0.0;
				while(mp->message[mp->messages].x < 640.0)
				{
					mp->message[mp->messages].vx += 0.5;
					mp->message[mp->messages].x += mp->message[mp->messages].vx;
				}
				mp->message[mp->messages].vx = -mp->message[mp->messages].vx;
				mp->message[mp->messages].dp = dp;
				mp->message[mp->messages].state = 0;
				mp->message[mp->messages].tick = 0;
				mp->message[mp->messages].flags = flags;
				break;
			}
			case WL_MESSAGE_TYPE_POPUP:
			{
				fp = (ALLEGRO_FONT *)dp;
				strcpy(mp->message[mp->messages].text, text);
				mp->message[mp->messages].x = 320.0;
				mp->message[mp->messages].y = -30.0;
				mp->message[mp->messages].tick = 0;
				mp->message[mp->messages].dp = dp;
				mp->message[mp->messages].flags = flags;
				break;
			}
			case WL_MESSAGE_TYPE_FLOAT_UP:
			{
				tfp = (T3F_FONT *)dp;
				strcpy(mp->message[mp->messages].text, text);
				mp->message[mp->messages].x = WL_BOARD_X + wl_game.player[0].sprite.x + WL_BOARD_TILE_SIZE / 2;
				mp->message[mp->messages].y = WL_BOARD_Y + wl_game.player[0].sprite.y - t3f_get_font_line_height(tfp) * 2;
				mp->message[mp->messages].tick = 0;
				mp->message[mp->messages].dp = dp;
				mp->message[mp->messages].flags = flags;
				break;
			}
			case WL_MESSAGE_TYPE_FLOAT_UP_2:
			{
				tfp = (T3F_FONT *)dp;
				strcpy(mp->message[mp->messages].text, text);
				mp->message[mp->messages].x = WL_BOARD_X + wl_game.player[0].sprite.x + WL_BOARD_TILE_SIZE / 2;
				mp->message[mp->messages].y = WL_BOARD_Y + wl_game.player[0].sprite.y - t3f_get_font_line_height(tfp);
				mp->message[mp->messages].tick = 0;
				mp->message[mp->messages].dp = dp;
				mp->message[mp->messages].flags = flags;
				break;
			}
		}
		mp->messages++;
	}
}

static void wl_delete_message(WL_MESSAGES * mp, int m)
{
	int i;
	
	for(i = m; i < mp->messages - 1; i++)
	{
		memcpy(&mp->message[i], &mp->message[i + 1], sizeof(WL_MESSAGE));
	}
	mp->messages--;
}

void wl_message_logic(WL_MESSAGES * mp, bool multi)
{
	int m = 1;
	int i;
	
	if(multi)
	{
		m = mp->messages;
	}
	
	if(mp->messages > 0)
	{
		for(i = m - 1; i >= 0; i--)
		{
			switch(mp->message[i].type)
			{
				case WL_MESSAGE_TYPE_SCROLLER:
				{
					switch(mp->message[i].state)
					{
						case 0:
						{
							mp->message[i].x += mp->message[i].vx;
							mp->message[i].vx += 0.5;
							if(mp->message[i].vx >= 0.0)
							{
								mp->message[i].state = 1;
							}
							break;
						}
						case 1:
						{
							mp->message[i].tick++;
							if(mp->message[i].tick >= 120)
							{
								mp->message[i].state = 2;
							}
							break;
						}
						case 2:
						{
							mp->message[i].vx -= 0.5;
							mp->message[i].x += mp->message[0].vx;
							if(mp->message[i].x + al_get_text_width((ALLEGRO_FONT *)mp->message[i].dp, mp->message[i].text) < 0.0)
							{
								wl_delete_message(mp, i);
							}
							break;
						}
					}
					break;
				}
				case WL_MESSAGE_TYPE_POPUP:
				{
					if(mp->message[i].tick < 60)
					{
						mp->message[i].y += 0.5;
						mp->message[i].tick++;
					}
					else
					{
						mp->message[i].tick++;
						if(mp->message[i].tick >= 180)
						{
							mp->message[i].y -= 0.5;
							if(mp->message[i].y < -30)
							{
								wl_delete_message(mp, i);
							}
						}
					}
					break;
				}
				case WL_MESSAGE_TYPE_FLOAT_UP:
				{
					if(mp->message[i].tick < 60)
					{
						mp->message[i].y -= 0.5;
					}
					mp->message[i].tick++;
					if(mp->message[i].tick >= 60)
					{
						wl_delete_message(mp, i);
					}
					break;
				}
				case WL_MESSAGE_TYPE_FLOAT_UP_2:
				{
					if(mp->message[i].tick < 60)
					{
						mp->message[i].y -= 0.5;
					}
					mp->message[i].tick++;
					if(mp->message[i].tick >= 60)
					{
						wl_delete_message(mp, i);
					}
					break;
				}
			}
		}
	}
}

void wl_render_messages(WL_MESSAGES * mp, bool multi)
{
	float a;
	int m = 1;
	int i;
	
	if(multi)
	{
		m = mp->messages;
	}
	
	if(mp->messages > 0)
	{
		for(i = 0; i < m; i++)
		{
			switch(mp->message[i].type)
			{
				case WL_MESSAGE_TYPE_SCROLLER:
				{
					wl_draw_text((ALLEGRO_FONT *)mp->message[i].dp, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.75), mp->message[i].x, mp->message[i].y, 3, 3, WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_SHADOW, mp->message[i].text);
					break;
				}
				case WL_MESSAGE_TYPE_POPUP:
				{
					wl_draw_text((ALLEGRO_FONT *)mp->message[i].dp, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.75), mp->message[i].x, mp->message[i].y, 3, 3, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_SHADOW, mp->message[i].text);
					break;
				}
				case WL_MESSAGE_TYPE_FLOAT_UP:
				{
					if(mp->message[i].tick >= 30)
					{
						a = 1.0 - (float)(mp->message[i].tick - 30) / 30.0;
					}
					else
					{
						a = 1.0;
					}
					t3f_draw_text((T3F_FONT *)mp->message[i].dp, al_map_rgba_f(0.0, 0.0, 0.0, a * 0.75), mp->message[i].x + 2.0, mp->message[i].y + 2.0, 0, 0, 0, T3F_FONT_ALIGN_CENTER, mp->message[i].text);
					t3f_draw_text((T3F_FONT *)mp->message[i].dp, al_map_rgba_f(a, a, a, a), mp->message[i].x, mp->message[i].y, 0, 0, 0, T3F_FONT_ALIGN_CENTER, mp->message[i].text);
//					wl_draw_text((ALLEGRO_FONT *)mp->message[i].dp, al_map_rgba_f(a, a, a, a), al_map_rgba_f(0.0, 0.0, 0.0, a), al_map_rgba_f(0.0, 0.0, 0.0, a * 0.5), mp->message[i].x, mp->message[i].y, 2.0, 2.0, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_SHADOW, mp->message[i].text);
					break;
				}
				case WL_MESSAGE_TYPE_FLOAT_UP_2:
				{
					if(mp->message[i].tick >= 30)
					{
						a = 1.0 - (float)(mp->message[i].tick - 30) / 30.0;
					}
					else
					{
						a = 1.0;
					}
					t3f_draw_text((T3F_FONT *)mp->message[i].dp, al_map_rgba_f(0.0, 0.0, 0.0, a * 0.75), mp->message[i].x + 2.0, mp->message[i].y + 2.0, 0, 0, 0, T3F_FONT_ALIGN_CENTER, mp->message[i].text);
					t3f_draw_text((T3F_FONT *)mp->message[i].dp, al_map_rgba_f(a, a, a, a), mp->message[i].x, mp->message[i].y, 0, 0, 0, T3F_FONT_ALIGN_CENTER, mp->message[i].text);
//					wl_draw_text((ALLEGRO_FONT *)mp->message[i].dp, al_map_rgba_f(a, a, a, a), al_map_rgba_f(0.0, 0.0, 0.0, a), al_map_rgba_f(0.0, 0.0, 0.0, a * 0.5), mp->message[i].x, mp->message[i].y, 2.0, 2.0, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_SHADOW, mp->message[i].text);
					break;
				}
			}
		}
	}
}
