#include "t3f/music.h"
#include "music_queue.h"
#include "data.h"

static char wl_music_queue_fn[WL_MUSIC_QUEUE_MAX_SIZE][256];
static int wl_music_queue_size = 0;
static int wl_music_queue_state = WL_MUSIC_QUEUE_STATE_OFF;
static float wl_music_fade_level = 1.0;
static float wl_music_fade_speed = 1.0 / WL_LEVEL_FADE_FRAMES;

static bool wl_delete_music_from_queue(int index)
{
	int i;
	
	if(index < wl_music_queue_size)
	{
		/* move all music past index to previous slot */
		for(i = index; i < wl_music_queue_size - 1; i++)
		{
			strncpy(wl_music_queue_fn[i], wl_music_queue_fn[i + 1], 256);
		}
		wl_music_queue_size--; // decrease queue size
		return true;
	}
	return false;
}

/* play next song in the queue */
static bool wl_play_queued_music(void)
{
	if(wl_music_queue_size > 0)
	{
		t3f_set_new_music_volume((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0);
		t3f_play_music(wl_music_queue_fn[0]);
		wl_delete_music_from_queue(0);
		return true;
	}
	return false;
}

/* clear queue */
void wl_clear_music_queue(void)
{
	wl_music_queue_size = 0;
	wl_music_queue_state = WL_MUSIC_QUEUE_STATE_OFF;
}

/* interrupt the queue with a new song, use for everything but in-game */
bool wl_play_music(const char * fn)
{
	wl_clear_music_queue();
	return t3f_play_music(fn);
}

/* queue up a song */
bool wl_queue_music(const char * fn)
{
	if(wl_music_queue_size < WL_MUSIC_QUEUE_MAX_SIZE)
	{
		strncpy(wl_music_queue_fn[wl_music_queue_size], fn, 256);
		wl_music_queue_size++;
		return true;
	}
	return false;
}

/* begin fading current song in preparation for the next song */
void wl_finish_current_song(void)
{
	if(wl_music_queue_state == WL_MUSIC_QUEUE_STATE_PLAYING)
	{
		wl_music_fade_level = 1.0;
		wl_music_queue_state = WL_MUSIC_QUEUE_STATE_TRANSITIONING;
	}
}

void wl_process_music_queue(void)
{
	switch(wl_music_queue_state)
	{
		/* if music is off and there is music in the queue, begin playback */
		case WL_MUSIC_QUEUE_STATE_OFF:
		{
			if(wl_music_queue_size > 0)
			{
				if(wl_play_queued_music())
				{
					wl_music_queue_state = WL_MUSIC_QUEUE_STATE_PLAYING;
				}
			}
			break;
		}
		
		/* fade out current music and start next song when done */
		case WL_MUSIC_QUEUE_STATE_TRANSITIONING:
		{
			wl_music_fade_level -= wl_music_fade_speed;
			
			/* if we have finished fading, play the next song */
			if(wl_music_fade_level <= 0)
			{
				if(wl_play_queued_music())
				{
					wl_music_queue_state = WL_MUSIC_QUEUE_STATE_PLAYING;
				}
				else
				{
					t3f_stop_music();
					wl_music_queue_state = WL_MUSIC_QUEUE_STATE_OFF;
				}
			}
			
			/* otherwise, set the new volume level */
			else
			{
				t3f_set_music_volume(wl_music_fade_level * ((float)wl_option[WL_OPTION_MUSIC_VOLUME] / 100.0));
			}
			break;
		}
	}
}

int wl_get_music_queue_state(void)
{
	return wl_music_queue_state;
}

void wl_set_music_fade_speed(float speed)
{
	wl_music_fade_speed = speed;
}
