#include "t3f/animation.h"
#include "defines.h"
#include "particle.h"

WL_PARTICLE wl_particle[2][WL_MAX_PARTICLES];
int wl_current_particle[2] = {0};

void wl_initialize_particle(WL_PARTICLE * pp, int type, T3F_ANIMATION * ap, ALLEGRO_COLOR color, float x, float y, float z, float s, int life, int flags)
{
	pp->type = type;
	pp->animation = ap;
	pp->color = color;
	pp->x = x;
	pp->y = y;
	pp->z = z;
	pp->s = s;
	pp->start_life = life;
	pp->life = life;
	pp->flags = flags;
	pp->active = true;
}

void wl_process_particle(WL_PARTICLE * pp)
{
	if(pp->active)
	{
		switch(pp->type)
		{
			case WL_PARTICLE_TYPE_FLASH:
			case WL_PARTICLE_TYPE_MFLASH:
			{
				pp->life--;
				if(pp->life <= 0)
				{
					pp->active = false;
				}
				break;
			}
			case WL_PARTICLE_TYPE_GLASS:
			{
				pp->x += pp->vx;
				pp->y += pp->vy;
				pp->vy += 0.5;
				pp->z += pp->vz;
				pp->life--;
				if(pp->life <= 0)
				{
					pp->active = false;
				}
				break;
			}
			case WL_PARTICLE_TYPE_SHARD:
			{
				pp->x += pp->vx;
				pp->y += pp->vy;
				pp->z += pp->vz;
				pp->vz += 0.5;
				if(pp->vz > WL_TERMINAL_VELOCITY)
				{
					pp->vz = WL_TERMINAL_VELOCITY;
				}
				pp->life--;
				if(pp->life <= 0 || pp->z > 0.0)
				{
					pp->active = false;
				}
				break;
			}
			case WL_PARTICLE_TYPE_ICE:
			{
				pp->x += pp->vx;
				pp->y += pp->vy;
				pp->vy += 0.5;
				if(pp->vy > WL_TERMINAL_VELOCITY)
				{
					pp->vy = WL_TERMINAL_VELOCITY;
				}
				pp->z += pp->vz;
				pp->life--;
				if(pp->y > 480.0)
				{
					pp->active = false;
				}
				break;
			}
			case WL_PARTICLE_TYPE_STEAM:
			{
				pp->x += pp->vx;
				pp->y += pp->vy;
				pp->z += pp->vz;
				pp->life--;
				if(pp->life <= 0)
				{
					pp->active = false;
				}
				break;
			}
		}
	}
}

void wl_render_particle(WL_PARTICLE * pp)
{
	ALLEGRO_COLOR color;
	float r, g, b, a, aa, s;

	if(pp->active)
	{
		switch(pp->type)
		{
			case WL_PARTICLE_TYPE_FLASH:
			case WL_PARTICLE_TYPE_MFLASH:
			{
				if(pp->life < 10)
				{
					al_unmap_rgb_f(pp->color, &r, &g, &b);
					a = ((float)pp->life / 10.0) / 2.0;
					color = al_map_rgba_f(r * a, g * a, b * a, a * a);
					s = (float)pp->s / (float)al_get_bitmap_width(pp->animation->bitmaps->bitmap[0]);
					t3f_draw_scaled_animation(pp->animation, color, 0, pp->x, pp->y, 0.0, s, 0);
				}
				break;
			}
			case WL_PARTICLE_TYPE_GLASS:
			{
				al_unmap_rgba_f(pp->color, &r, &g, &b, &a);
				aa = ((float)pp->life / (float)pp->start_life) * a;
				color = al_map_rgba_f(r * aa, g * aa, b * aa, aa);
				t3f_draw_animation(pp->animation, color, 0, pp->x, pp->y, pp->z, 0);
				break;
			}
			case WL_PARTICLE_TYPE_SHARD:
			{
				al_unmap_rgb_f(pp->color, &r, &g, &b);
				a = (float)pp->life / (float)pp->start_life;
				color = al_map_rgba_f(r * a, g * a, b * a, a * a);
				t3f_draw_animation(pp->animation, color, 0, WL_BOARD_X + pp->x, WL_BOARD_Y + pp->y + pp->z, 0, 0);
				break;
			}
			case WL_PARTICLE_TYPE_ICE:
			{
				al_unmap_rgb_f(pp->color, &r, &g, &b);
				a = (float)pp->life / (float)pp->start_life;
				color = al_map_rgba_f(r * a, g * a, b * a, a * a);
				t3f_draw_animation(pp->animation, color, 0, pp->x, pp->y, pp->z, 0);
				break;
			}
			case WL_PARTICLE_TYPE_STEAM:
			{
				al_unmap_rgb_f(pp->color, &r, &g, &b);
				a = (float)pp->life / (float)pp->start_life;
				color = al_map_rgba_f(r * a, g * a, b * a, a * a);
				t3f_draw_animation(pp->animation, color, 0, pp->x, pp->y, pp->z, 0);
				break;
			}
		}
	}
}

void wl_reset_particles(int list)
{
	int i;

	for(i = 0; i < WL_MAX_PARTICLES; i++)
	{
		wl_particle[list][i].active = false;
	}
	wl_current_particle[list] = 0;
}

int wl_add_particle(int list, int type, T3F_ANIMATION * ap, ALLEGRO_COLOR color, float x, float y, float z, float s, int life, int flags)
{
	int r = wl_current_particle[list];

	wl_initialize_particle(&wl_particle[list][wl_current_particle[list]], type, ap, color, x, y, z, s, life, flags);
	wl_current_particle[list]++;
	if(wl_current_particle[list] >= WL_MAX_PARTICLES)
	{
		wl_current_particle[list] = 0;
	}
	return r;
}

void wl_process_particles(int list)
{
	int i;

	for(i = 0; i < WL_MAX_PARTICLES; i++)
	{
		wl_process_particle(&wl_particle[list][i]);
	}
}

void wl_render_particles(int list, int type)
{
	int i;

	for(i = 0; i < WL_MAX_PARTICLES; i++)
	{
		if(type < 0 || wl_particle[list][i].type == type)
		{
			wl_render_particle(&wl_particle[list][i]);
		}
	}
}
