#include "t3f/sound.h"
#include "t3f/rng.h"
#include "data.h"
#include "game.h"
#include "board.h"
#include "awards.h"
#include "game_intro.h"

bool wl_player_needs_tutorial(void)
{
	/* bring up the relevant tutorial */
	if(!wl_game.player[0].watched_tutorial)
	{
		if(!(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_BASIC))
		{
			return true;
		}
		else
		{
			if(wl_game.mode == WL_GAME_MODE_STORY && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_STORY))
			{
				return true;
			}
			else if(wl_game.mode == WL_GAME_MODE_CASUAL && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_CASUAL))
			{
				return true;
			}
			else if(wl_game.mode == WL_GAME_MODE_PUZZLE && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_PUZZLE))
			{
				return true;
			}
			else if(wl_game.mode == WL_GAME_MODE_GURU && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_GURU))
			{
				return true;
			}
		}
	}
	return false;
}

static void wl_player_use_click(WL_PLAYER_CLICK_QUEUE * qp)
{
	int i;

	if(qp->clicks > 0)
	{
		for(i = 0; i < qp->clicks - 1; i++)
		{
			memcpy(&qp->click[i], &qp->click[i + 1], sizeof(WL_PLAYER_CLICK));
		}
		qp->clicks--;
	}
}

void wl_game_player_add_word_to_list(int word, int points)
{
	int i;
	bool top = false;

	/* update main word list */
	wl_game.player[wl_game.current_player].word_list[wl_game.player[0].word_list_count] = word;
	wl_game.player[wl_game.current_player].word_list_count++;
	wl_game.player[wl_game.current_player].word_list_counter = 24;

	/* update top word list */
	for(i = 0; i < wl_game.player[wl_game.current_player].top_word_count; i++)
	{
		if(wl_game_figure_total_points(wl_game.player[wl_game.current_player].top_word[i]) < points)
		{
			wl_game.player[wl_game.current_player].top_word_spot = i;
			wl_game.player[wl_game.current_player].top_word_index = word;
			top = true;
			if(i == 0)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_BEST_WORD], 1.0, 0.0, 1.0);
			}
			break;
		}
	}

	if(wl_game.player[wl_game.current_player].top_word_count < WL_MAX_TOP_WORDS)
	{
		/* add word to end of list if it hasn't already been inserted */
		if(!top)
		{
			wl_game.player[wl_game.current_player].top_word_spot = wl_game.player[0].top_word_count;
			wl_game.player[wl_game.current_player].top_word_index = word;
			wl_game.player[wl_game.current_player].top_word[wl_game.player[wl_game.current_player].top_word_count] = word;
		}
		wl_game.player[wl_game.current_player].top_word_count++;
		top = true;
	}

	/* actually insert word */
	if(top)
	{
		for(i = WL_MAX_TOP_WORDS - 1; i > wl_game.player[wl_game.current_player].top_word_spot; i--)
		{
			wl_game.player[wl_game.current_player].top_word[i] = wl_game.player[wl_game.current_player].top_word[i - 1];
		}
		wl_game.player[wl_game.current_player].top_word[wl_game.player[wl_game.current_player].top_word_spot] = wl_game.player[wl_game.current_player].top_word_index;
	}

	/* if we got a top word, start the counter */
	if(top)
	{
		/* don't start counter if we are using the last spot since that is just
		 * a buffer for words falling off the list */
		if(wl_game.player[wl_game.current_player].top_word_spot < WL_MAX_TOP_WORDS - 1)
		{
			wl_game.player[wl_game.current_player].top_word_counter = 24;
		}
	}
}

static void wl_game_player_activate_guru(void)
{
	wl_game.guru_state = 2;
	wl_game.guru_timer = 0;
	wl_game.guru_x = wl_game.player[0].sprite.x;
	wl_game.guru_y = wl_game.player[0].sprite.y;
	wl_game.guru_z = 64.0;
	wl_game.guru_vz = 0.0;
	wl_game.guru_color = al_map_rgba_f(0.0, 0.0, 0.0, 0.0);
	wl_game.guru_active = true;
	wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = '0';
}

void wl_game_player_add_life(float amount)
{
	wl_game.player[0].life += amount;
	if(wl_game.player[0].life > WL_MAX_LIFE)
	{
		wl_game.player[0].life = WL_MAX_LIFE;
	}
	wl_game.player[0].lost = false;
}

/* subtract life, only make player lose if life is already 0.0 */
void wl_game_player_subtract_life(float amount)
{
	/* subtract life until it reaches 0.0 */
	if(wl_game.player[0].life > 0.0)
	{
		wl_game.player[0].life -= amount;
		if(wl_game.player[0].life < 0.0)
		{
			wl_game.player[0].life = 0.0;
		}
	}

	/* if we are already at 0.0, lose */
	else
	{
		wl_game.player[0].lost = true;
	}
}

int wl_game_player_pos_to_move(int x, int y)
{
	return y * WL_MAX_BOARD_WIDTH + x;
}

int wl_game_player_move_to_x(int pos)
{
	return pos % WL_MAX_BOARD_WIDTH;
}

int wl_game_player_move_to_y(int pos)
{
	return pos / WL_MAX_BOARD_WIDTH;
}

static void wl_game_player_jump(int x, int y)
{
	wl_game.player[0].sprite.vx = ((float)WL_BOARD_TILE_SIZE * (float)(x - wl_game.player[0].x)) / 21.0;
	wl_game.player[0].sprite.vy = ((float)WL_BOARD_TILE_SIZE * (float)(y - wl_game.player[0].y)) / 21.0;
	wl_game.player[0].sprite.vz = -5.0;
	wl_game.player[0].x = x;
	wl_game.player[0].y = y;
	wl_game.player[0].state = WL_PLAYER_STATE_JUMP;
	wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_NORMAL;
	wl_game.player[0].sprite.tick = 0;
	t3f_play_sample(wl_sample[WL_SAMPLE_JUMP], 1.0, 0.0, 1.0);
}

static void wl_game_player_replace_letter(void)
{
	if(wl_game.player[0].word_pos > 0)
	{
		wl_game.board[wl_game.player[0].y][wl_game.player[0].x].type = 2;
		wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = wl_game.player[0].word[wl_game.player[0].word_pos - 1];
		wl_game.player[0].word_pos--;
		wl_game.player[0].word[wl_game.player[0].word_pos] = '\0';
		if(wl_game.player[0].word_pos > 0 && wl_game.player[0].word[wl_game.player[0].word_pos - 1] == 'Q')
		{
			wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = 'Q';
			wl_game.player[0].word_pos--;
			wl_game.player[0].word[wl_game.player[0].word_pos] = '\0';
		}
	}
	else
	{
		wl_game.board[wl_game.player[0].y][wl_game.player[0].x].type = 3;
		wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
	}
}

static void wl_game_player_use_valid_word(int w)
{
	int s, m;
	char buf[16];
	int old_progress;

	wl_game.player[wl_game.current_player].words_this_level++;
	wl_generate_board_particles(al_map_rgb_f(1.0, 1.0, 1.0), wl_game.player[0].x, wl_game.player[0].y, 10);
	if(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS)
	{
		wl_game.player[0].powerups &= ~WL_POWERUP_HOURGLASS;
	}
	s = wl_game_figure_total_points(w);
	if(s >= 3000)
	{
		m = 5;
		t3f_play_sample(wl_sample[WL_SAMPLE_AMAZING_WORD], 1.0, 0.0, 1.0);
		wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_HAPPY;
		wl_game.player[0].sprite.tick = 0;
		wl_add_message(&wl_word_messages, WL_MESSAGE_TYPE_FLOAT_UP, "Amazing Word!", wl_ascii_font, 0);
	}
	else if(s >= 2000)
	{
		m = 3;
		t3f_play_sample(wl_sample[WL_SAMPLE_GOOD_WORD], 1.0, 0.0, 1.0);
		wl_add_message(&wl_word_messages, WL_MESSAGE_TYPE_FLOAT_UP, "Great Word!", wl_ascii_font, 0);
	}
	else if(s >= 1000)
	{
		m = 2;
		t3f_play_sample(wl_sample[WL_SAMPLE_GOOD_WORD], 1.0, 0.0, 1.0);
	}
	else
	{
		m = 1;
		t3f_play_sample(wl_sample[WL_SAMPLE_WORD], 1.0, 0.0, 1.0);
	}
	sprintf(buf, "%d", s);
	wl_add_message(&wl_word_messages, WL_MESSAGE_TYPE_FLOAT_UP_2, buf, wl_ascii_font, 0);
	wl_game.player[wl_game.current_player].score += s;
	wl_game.player[wl_game.current_player].word_score = wl_game.player[wl_game.current_player].score;
	if(wl_game.uses_life)
	{
		wl_game_player_add_life(strlen(wl_game.player[0].word) * m);
	}
	if(wl_game.uses_level_progress)
	{
		old_progress = wl_game.player[0].progress;
		wl_game.player[0].progress += strlen(wl_game.player[0].word) * m;
		if(old_progress % WL_LEVEL_UP > wl_game.player[0].progress % WL_LEVEL_UP)
		{
			t3f_play_sample(wl_sample[WL_SAMPLE_LIFE_UP], 1.0, 0.0, 1.0);
			wl_game_player_add_life(WL_MAX_LIFE / 4.0);
		}
		if(wl_game.player[0].progress > wl_profiles->entry[wl_game.player[0].profile].progress)
		{
			wl_profiles->entry[wl_game.player[0].profile].progress = wl_game.player[0].progress;
		}
	}
	else if(wl_game.mode == WL_GAME_MODE_CASUAL)
	{
		wl_game.player[0].progress += WL_LEVEL_UP / 10;
	}
	if(wl_game.mode != WL_GAME_MODE_PUZZLE)
	{
		wl_board_generate_new_letters();
	}
	wl_game.player[0].word[0] = '\0';
	wl_game.player[0].word_pos = 0;
	wl_game.player[0].words++;
	wl_game.player[0].move[0] = wl_game_player_pos_to_move(wl_game.player[0].x, wl_game.player[0].y);
	wl_game.player[0].moves = 1;
	wl_game_player_add_word_to_list(w, s);
	wl_award_logic(&wl_profiles->entry[wl_game.player[wl_game.current_player].profile], WL_PROFILE_AWARD_HUNDRED);
	wl_award_logic(&wl_profiles->entry[wl_game.player[wl_game.current_player].profile], WL_PROFILE_AWARD_BIG_FAN);
	wl_award_logic(&wl_profiles->entry[wl_game.player[wl_game.current_player].profile], WL_PROFILE_AWARD_FOUL);
	wl_award_logic(&wl_profiles->entry[wl_game.player[wl_game.current_player].profile], WL_PROFILE_AWARD_STRIDE);

	/* add word to profile */
	wl_profiles->entry[wl_game.player[wl_game.current_player].profile].word[wl_profiles->entry[wl_game.player[wl_game.current_player].profile].words] = w;
	wl_profiles->entry[wl_game.player[wl_game.current_player].profile].words++;

	/* adjust high score when appropriate */
	if(wl_game.player[wl_game.current_player].score > wl_profiles->entry[wl_game.player[wl_game.current_player].profile].high_score[wl_game.mode])
	{
		wl_profiles->entry[wl_game.player[wl_game.current_player].profile].high_score[wl_game.mode] = wl_game.player[wl_game.current_player].score;
		switch(wl_game.mode)
		{
			case WL_GAME_MODE_STORY:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_UPLOAD_STORY;
				break;
			}
			case WL_GAME_MODE_CASUAL:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_UPLOAD_CASUAL;
				break;
			}
			case WL_GAME_MODE_PUZZLE:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_UPLOAD_PUZZLE;
				break;
			}
			case WL_GAME_MODE_GURU:
			{
				wl_profiles->entry[wl_game.player[0].profile].flags |= WL_PROFILE_FLAG_UPLOAD_GURU;
				break;
			}
		}
	}
}

static void wl_game_player_use_invalid_word(void)
{
	wl_generate_board_particles(al_map_rgb_f(1.0, 0.0, 0.0), wl_game.player[0].x, wl_game.player[0].y, 10);
	if(wl_game.uses_life)
	{
		wl_game_player_subtract_life(strlen(wl_game.player[0].word));
	}
	wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_SAD;
	wl_game.player[0].sprite.tick = 0;
	wl_game.player[0].sprite.timer = 30;
	if(wl_game.mode != WL_GAME_MODE_PUZZLE)
	{
		wl_board_generate_new_letters();
	}
	wl_game.player[0].word[0] = '\0';
	wl_game.player[0].word_pos = 0;
	wl_game.player[0].powerups = 0;
	wl_game.player[0].move[0] = wl_game_player_pos_to_move(wl_game.player[0].x, wl_game.player[0].y);
	wl_game.player[0].moves = 1;

	t3f_play_sample(wl_sample[WL_SAMPLE_BAD_WORD], 1.0, 0.0, 1.0);

	/* add wrong guess to profile */
	wl_profiles->entry[wl_game.player[wl_game.current_player].profile].wrongs++;
}

static void wl_game_player_use_word(void)
{
	int w;

	if(strlen(wl_game.player[0].word) > 2)
	{
		w = wl_word_in_dictionary(wl_dictionary, wl_game.player[0].word);
		if(w >= 0)
		{
			wl_game_player_use_valid_word(w);
		}
		else
		{
			if(wl_game.uses_invalid)
			{
				wl_game_player_use_invalid_word();
			}
		}
	}
	else if(strlen(wl_game.player[wl_game.current_player].word) > 0)
	{
		if(wl_game.uses_invalid)
		{
			wl_game_player_use_invalid_word();
		}
	}
	if(wl_game.guru_active)
	{
		wl_game.guru_state = 3;
		wl_game.player[0].powerups &= ~WL_POWERUP_GURU;
		wl_game.player[0].counter = 0;
	}
	wl_game.player[0].clicked = true;
	wl_game.player[0].word_exists = false;
	if(wl_game.mode == WL_GAME_MODE_PARTY)
	{
		wl_game.current_player++;
		if(wl_game.current_player >= wl_game.players)
		{
			wl_game.current_player = 0;
			wl_game.player[0].progress += WL_LEVEL_UP / 10;
		}
		if(wl_game.player[0].progress < WL_LEVEL_UP)
		{
			wl_add_message(&wl_game_messages, WL_MESSAGE_TYPE_SCROLLER, wl_profiles->entry[wl_game.player[wl_game.current_player].profile].name, wl_font[WL_FONT_32], 0);
		}
		else
		{
			wl_game.state = WL_GAME_STATE_DONE;
		}
	}
}

static void wl_game_player_move(void)
{
	float life_rate;

	/* put home space down in puzzle mode when moving away from center */
	if(wl_game.mode == WL_GAME_MODE_PUZZLE && wl_game.player[0].x == WL_BOARD_CENTER_X && wl_game.player[0].y == WL_BOARD_CENTER_Y)
	{
		wl_game.board[WL_BOARD_CENTER_Y][WL_BOARD_CENTER_X].letter = '@';
	}

	/* put back letter */
	else if(wl_game.board[wl_game.player[0].click_queue.click[0].by][wl_game.player[0].click_queue.click[0].bx].letter == ' ')
	{
		wl_game_player_replace_letter();

		/* add undo to profile */
		wl_profiles->entry[wl_game.player[0].profile].undos++;
	}

	/* remove letter */
	else
	{
		if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter != '@')
		{
			wl_game.board[wl_game.player[0].y][wl_game.player[0].x].type = 3;
			wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
		}
	}

	/* make player jump */
	wl_game_player_jump(wl_game.player[0].click_queue.click[0].bx, wl_game.player[0].click_queue.click[0].by);
	if(wl_game.uses_life && !(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		if(wl_game.mode == WL_GAME_MODE_GURU)
		{
			life_rate = wl_life_chart[9];
		}
		else
		{
			life_rate = wl_life_chart[wl_game.player[0].level - 1];
		}
		wl_game_player_subtract_life(life_rate);
	}
}

static bool wl_mouse_in_board(float mouse_x, float mouse_y)
{
	if(mouse_x - WL_BOARD_X < 0)
	{
		return false;
	}
	else if(mouse_x - WL_BOARD_X >= WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE)
	{
		return false;
	}
	else if(mouse_y - WL_BOARD_Y < 0)
	{
		return false;
	}
	else if(mouse_y - WL_BOARD_Y >= WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE)
	{
		return false;
	}
	return true;
}

void wl_game_player_control(void)
{
	/* how much life per move for each level */
	int i;
	float mouse_x, mouse_y;
	int mouse_b;
	bool touch_release = false;

	/* handle keyboard input */
	if(wl_mouse_off)
	{
		if(wl_game.player[0].hover_x < 0 || wl_game.player[0].hover_y < 0)
		{
			wl_game.player[0].hover_x = wl_game.player[0].x;
			wl_game.player[0].hover_y = wl_game.player[0].y;
		}
	}
	if(t3f_key[ALLEGRO_KEY_UP])
	{
		if(wl_game.player[0].hover_y > 0)
		{
			wl_game.player[0].hover_y--;
		}
		t3f_key[ALLEGRO_KEY_UP] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_DOWN])
	{
		if(wl_game.player[0].hover_y < WL_MAX_BOARD_HEIGHT - 1)
		{
			wl_game.player[0].hover_y++;
		}
		t3f_key[ALLEGRO_KEY_DOWN] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_LEFT])
	{
		if(wl_game.player[0].hover_x > 0)
		{
			wl_game.player[0].hover_x--;
		}
		t3f_key[ALLEGRO_KEY_LEFT] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_RIGHT])
	{
		if(wl_game.player[0].hover_x < WL_MAX_BOARD_WIDTH - 1)
		{
			wl_game.player[0].hover_x++;
		}
		t3f_key[ALLEGRO_KEY_RIGHT] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_SPACE])
	{
		wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].bx = wl_game.player[0].hover_x;
		wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].by = wl_game.player[0].hover_y;
		wl_game.player[0].click_queue.clicks++;
		t3f_key[ALLEGRO_KEY_SPACE] = 0;
	}
//	if(t3f_key[ALLEGRO_KEY_L])
//	{
//		wl_game.player[0].progress += WL_LEVEL_UP;
//		t3f_key[ALLEGRO_KEY_L] = 0;
//	}
//	if(t3f_key[ALLEGRO_KEY_G])
//	{
//		wl_game.player[0].powerups |= WL_POWERUP_GURU;
//		wl_game_player_activate_guru();
//	}

	/* store mouse info in local variables */
	mouse_x = t3f_mouse_x;
	mouse_y = t3f_mouse_y;
	mouse_b = 0;

	/* store touch input in local variables */
	for(i = 0; i < T3F_MAX_TOUCHES; i++)
	{
		if(t3f_touch[i].active)
		{
			mouse_x = t3f_touch[i].x;
			mouse_y = t3f_touch[i].y;
			mouse_b = 1;
			wl_game.player[0].touching = true;
			wl_game.player[0].touch_id = i;
			break;
		}
		else if(t3f_touch[i].released)
		{
			mouse_x = t3f_touch[i].x;
			mouse_y = t3f_touch[i].y;
			wl_game.player[0].touch_id = i;

			/* only alter touch state if we are clicking inside the board area */
			if(i == 0 || wl_mouse_in_board(mouse_x, mouse_y))
			{
				t3f_touch[i].released = false;
			}
			touch_release = true;
			break;
		}
	}
	if(!mouse_b)
	{
		wl_game.player[0].touching = false;
	}

	/* store clicks in queue */
	if(!wl_mouse_off)
	{
		if(wl_mouse_in_board(mouse_x, mouse_y))
		{
			wl_game.player[0].hover_x = (mouse_x - WL_BOARD_X) / WL_BOARD_TILE_SIZE;
			wl_game.player[0].hover_y = (mouse_y - WL_BOARD_Y) / WL_BOARD_TILE_SIZE;
			wl_game.player[0].board_x = (mouse_x - WL_BOARD_X) / WL_BOARD_TILE_SIZE;
			wl_game.player[0].board_y = (mouse_y - WL_BOARD_Y) / WL_BOARD_TILE_SIZE;
			if(touch_release && !wl_game.player[0].clicked)
			{
				wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].x = mouse_x;
				wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].y = mouse_y;
				wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].bx = (mouse_x - WL_BOARD_X) / WL_BOARD_TILE_SIZE;
				wl_game.player[0].click_queue.click[wl_game.player[0].click_queue.clicks].by = (mouse_y - WL_BOARD_Y) / WL_BOARD_TILE_SIZE;
				wl_game.player[0].click_queue.clicks++;
				wl_game.player[0].clicked = true;
			}
		}
		else
		{
			wl_game.player[0].hover_x = -1;
			wl_game.player[0].hover_y = -1;
			wl_game.player[0].board_x = -1;
			wl_game.player[0].board_y = -1;
		}
		if(!mouse_b)
		{
			wl_game.player[0].clicked = false;
		}
	}
	/* see if we are within range for the current position */
	if(wl_game.player[0].hover_x >= 0 && wl_game.player[0].hover_y >= 0 && abs(wl_game.player[0].x - wl_game.player[0].hover_x) < wl_game.player[0].range && abs(wl_game.player[0].y - wl_game.player[0].hover_y) < wl_game.player[0].range)
	{
		wl_game.player[0].in_range = true;
	}
	else
	{
		wl_game.player[0].in_range = false;
	}

	switch(wl_game.player[0].state)
	{
		case WL_PLAYER_STATE_NORMAL:
		{
			/* process next click */
			if(wl_game.player[0].click_queue.clicks > 0)
			{
				/* see if the click is in range */
				if((abs(wl_game.player[0].x - wl_game.player[0].click_queue.click[0].bx) < wl_game.player[0].range && abs(wl_game.player[0].y - wl_game.player[0].click_queue.click[0].by) < wl_game.player[0].range) && !(wl_game.board[wl_game.player[0].click_queue.click[0].by][wl_game.player[0].click_queue.click[0].bx].flags & WL_BOARD_FLAG_BLOCKED))
				{

					/* clicked on self to use current word */
					if(wl_game.player[0].click_queue.click[0].bx == wl_game.player[0].x && wl_game.player[0].click_queue.click[0].by == wl_game.player[0].y)
					{
						wl_game_player_use_word();
					}
					/* clicked space other than player space */
					else
					{
						wl_game_player_move();
					}
				}

				/* if click is outside of range, remove all clicks from the queue,
				 * player probably wanted a letter that was out of range */
				else
				{
					wl_game.player[0].click_queue.clicks = 0;
				}
			}
			wl_player_use_click(&wl_game.player[0].click_queue);
			break;
		}
	}
}

void wl_game_player_logic(void)
{
	int base_range = 3;
	int i, p;
	bool might_swoosh = false;

	if(wl_state != WL_STATE_GAME && wl_state != WL_STATE_GAME_TUTORIAL)
	{
		base_range = 0;
	}
	if(wl_game.mode == WL_GAME_MODE_PUZZLE)
	{
		base_range = WL_MAX_BOARD_WIDTH;
	}

	/* adjust range if needed */
	if(wl_game.player[0].powerups & WL_POWERUP_GURU)
	{
		if(wl_game.player[0].range < WL_MAX_BOARD_WIDTH)
		{
			wl_game.player[0].counter++;
			if(wl_game.player[0].counter % 4 == 0)
			{
				wl_game.player[0].range++;
			}
		}
	}
	else
	{
		if(wl_game.player[0].range > base_range)
		{
			wl_game.player[0].counter++;
			if(wl_game.player[0].counter % 4 == 0)
			{
				wl_game.player[0].range--;
			}
		}
		else if(wl_game.player[0].range < base_range)
		{
			wl_game.player[0].counter++;
			if(wl_game.player[0].counter % 4 == 0)
			{
				wl_game.player[0].range++;
			}
		}
	}

	switch(wl_game.player[0].state)
	{
		case WL_PLAYER_STATE_ENTER:
		{
			if(wl_game.player[0].sprite.z < WL_SPRITE_SWOOSH_HEIGHT)
			{
				might_swoosh = true;
			}
			wl_game.player[0].sprite.z += wl_game.player[0].sprite.vz;
			if(might_swoosh && wl_game.player[0].sprite.z >= WL_SPRITE_SWOOSH_HEIGHT)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_SWOOSH], 1.0, 0.0, 1.0);
			}
			if(wl_game.player[0].sprite.z >= 0)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_LAND], 1.0, 0.0, 1.0);
				wl_game.player[0].sprite.z = 0;
				wl_game.player[0].state = WL_PLAYER_STATE_NORMAL;

				if(wl_player_needs_tutorial())
				{
					/* bring up the relevant tutorial */
					if(!(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_BASIC))
					{
						wl_tutorial_step = 0;
						wl_game_setup_type();
						wl_game.uses_life = false;
						wl_game.uses_level_progress = false;
						wl_state = WL_STATE_GAME_TUTORIAL;
					}
					else
					{
						if(wl_game.mode == WL_GAME_MODE_STORY && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_STORY))
						{
							wl_tutorial_step = 14;
							wl_state = WL_STATE_GAME_TUTORIAL;
						}
						else if(wl_game.mode == WL_GAME_MODE_CASUAL && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_CASUAL))
						{
							wl_tutorial_step = 14;
							wl_state = WL_STATE_GAME_TUTORIAL;
						}
						else if(wl_game.mode == WL_GAME_MODE_PUZZLE && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_PUZZLE))
						{
							wl_tutorial_step = 14;
							wl_state = WL_STATE_GAME_TUTORIAL;
						}
						else if(wl_game.mode == WL_GAME_MODE_GURU && !(wl_profiles->entry[wl_game.player[0].profile].flags & WL_PROFILE_FLAG_TUTORIAL_GURU))
						{
							wl_tutorial_step = 14;
							wl_state = WL_STATE_GAME_TUTORIAL;
						}
					}
				}
			}
			break;
		}
		case WL_PLAYER_STATE_EXIT:
		{
			if(wl_game.player[0].sprite.z >= WL_SPRITE_SWOOSH_HEIGHT)
			{
				might_swoosh = true;
			}
			wl_game.player[0].sprite.z -= WL_TERMINAL_VELOCITY;
			if(might_swoosh && wl_game.player[0].sprite.z < WL_SPRITE_SWOOSH_HEIGHT)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_SWOOSH], 1.0, 0.0, 1.0);
			}
			break;
		}
		case WL_PLAYER_STATE_NORMAL:
		{
			wl_game.player[0].sprite.x = wl_game.player[0].x * WL_BOARD_TILE_SIZE;
			wl_game.player[0].sprite.y = wl_game.player[0].y * WL_BOARD_TILE_SIZE;
			wl_game.player[0].sprite.z = 0;
			break;
		}
		case WL_PLAYER_STATE_JUMP:
		{
			if(!wl_mouse_off)
			{
//				wl_game.player[0].hover_x = -1;
//				wl_game.player[0].hover_y = -1;
			}
			wl_game.player[0].sprite.x += wl_game.player[0].sprite.vx;
			wl_game.player[0].sprite.y += wl_game.player[0].sprite.vy;
			wl_game.player[0].sprite.z += wl_game.player[0].sprite.vz;
			if(wl_game.player[0].sprite.z >= 0.0)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_LAND], 1.0, 0.0, 1.0);
				if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter == '?')
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_GURU], 1.0, 0.0, 1.0);
					wl_game.player[0].powerups |= WL_POWERUP_GURU;

					/* activate guru */
					wl_game_player_activate_guru();
				}
				else if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter == '!')
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_LIFE_UP], 1.0, 0.0, 1.0);
					wl_game.player[0].life = WL_MAX_LIFE;
					wl_game.player[0].lost = false;
					wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = '0';
				}
				else if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter == '_')
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_HOURGLASS], 1.0, 0.0, 1.0);
					wl_game.player[0].powerups |= WL_POWERUP_HOURGLASS;
					wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = '0';
				}
				else if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter == '@' && wl_game.mode == WL_GAME_MODE_PUZZLE)
				{
					if(wl_state != WL_STATE_GAME_TUTORIAL)
					{
						wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_ALL_CLEAR);
						wl_game_end();
					}
				}
				else if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter == 'Q')
				{
					wl_game.player[0].word[wl_game.player[0].word_pos] = 'Q';
					wl_game.player[0].word_pos++;
					wl_game.player[0].word[wl_game.player[0].word_pos] = '\0';
					wl_game.player[0].word[wl_game.player[0].word_pos] = 'U';
					wl_game.player[0].word_pos++;
					wl_game.player[0].word[wl_game.player[0].word_pos] = '\0';
					wl_game.player[0].move[wl_game.player[0].moves] = wl_game_player_pos_to_move(wl_game.player[0].x, wl_game.player[0].y);
					wl_game.player[0].moves++;
				}
				else if(wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter != ' ' && wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter != '0')
				{
					wl_game.player[0].word[wl_game.player[0].word_pos] = wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter;
					wl_game.player[0].word_pos++;
					wl_game.player[0].word[wl_game.player[0].word_pos] = '\0';
					wl_game.player[0].move[wl_game.player[0].moves] = wl_game_player_pos_to_move(wl_game.player[0].x, wl_game.player[0].y);
					wl_game.player[0].moves++;
				}
				wl_game.board[wl_game.player[0].y][wl_game.player[0].x].letter = ' ';
				/* add move to profile */
				wl_profiles->entry[wl_game.player[0].profile].moves++;

				/* we set state to EXIT in some cases so only revert to NORMAL if we are still JUMPing */
				if(wl_game.player[0].state == WL_PLAYER_STATE_JUMP)
				{
					wl_game.player[0].state = WL_PLAYER_STATE_NORMAL;
				}
				if(strlen(wl_game.player[0].word) >= 3 && wl_word_in_dictionary(wl_dictionary, wl_game.player[0].word) >= 0)
				{
					wl_game.player[0].word_exists = true;
				}
				else
				{
					wl_game.player[0].word_exists = false;
				}
			}
			wl_game.player[0].sprite.vz += 0.5;
			break;
		}
	}

	/* animate HUD */
	if(wl_game.player[0].life_indicator < wl_game.player[0].life)
	{
		wl_game.player[0].life_indicator += 1.0;
		if(wl_game.player[0].life_indicator > wl_game.player[0].life)
		{
			wl_game.player[0].life_indicator = wl_game.player[0].life;
		}
	}
	else if(wl_game.player[0].life_indicator > wl_game.player[0].life)
	{
		wl_game.player[0].life_indicator -= 1.0;
		if(wl_game.player[0].life_indicator < wl_game.player[0].life)
		{
			wl_game.player[0].life_indicator = wl_game.player[0].life;
		}
	}
	if(wl_game.player[0].progress_indicator < wl_game.player[0].progress)
	{
		wl_game.player[0].progress_indicator++;
		wl_game.player[0].made_progress = true;
	}
	else if(wl_game.player[0].progress_indicator > wl_game.player[0].progress)
	{
		wl_game.player[0].progress_indicator--;
	}

	/* life meter graphical effect */
	if(!(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		/* only process fading when fade level is > 0.0 to prevent triggering
		 * extra particle generation */
		if(wl_game.player[0].life_fade > 0.0)
		{
			if(wl_game.player[0].life_fade >= 1.0)
			{
				/* generate steam particles when color fade starts */
				for(i = 0; i < 32; i++)
				{
					p = wl_add_particle(1, WL_PARTICLE_TYPE_STEAM, wl_animation[WL_ANIMATION_PARTICLE_4], al_map_rgba_f(1.0, 1.0, 1.0, 0.75), WL_LIFE_BAR_X + t3f_drand(&wl_rng_state) * al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE_FILL]), WL_LIFE_BAR_Y + t3f_drand(&wl_rng_state) * al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]), 0.0, 1.0, 60, 0);
					wl_particle[1][p].vz = -2.5;
					wl_particle[1][p].vy = -t3f_drand(&wl_rng_state);
					wl_particle[1][p].vx = t3f_drand(&wl_rng_state) - 0.5;
					t3f_play_sample(wl_sample[WL_SAMPLE_THAW], 1.0, 0.0, 1.0);
				}
			}
			wl_game.player[0].life_fade -= 4.0 * (1.0 / 60.0);
			if(wl_game.player[0].life_fade <= 0.0)
			{
				wl_game.player[0].life_fade = 0.0;
			}
		}
	}
	else
	{
		/* only process fading when fade level is < 1.0 to prevent triggering
		 * extra particle generation */
		if(wl_game.player[0].life_fade < 1.0)
		{
			wl_game.player[0].life_fade += 4.0 * (1.0 / 60.0);
			if(wl_game.player[0].life_fade >= 1.0)
			{
				wl_game.player[0].life_fade = 1.0;

				/* generate ice particles when color fade completes */
				for(i = 0; i < 32; i++)
				{
					p = wl_add_particle(1, WL_PARTICLE_TYPE_ICE, wl_animation[WL_ANIMATION_PARTICLE_3], al_map_rgba_f(0.9, 0.9, 1.0, 0.75), WL_LIFE_BAR_X + t3f_drand(&wl_rng_state) * al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE_FILL]), WL_LIFE_BAR_Y + t3f_drand(&wl_rng_state) * al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]), 0.0, 1.0, 60, 0);
					wl_particle[1][p].vz = -0.5;
					wl_particle[1][p].vy = 0.0;
					wl_particle[1][p].vx = t3f_drand(&wl_rng_state) - 0.5;
					t3f_play_sample(wl_sample[WL_SAMPLE_FREEZE], 1.0, 0.0, 1.0);
				}
			}
		}
	}

	/* animate word lists */
	if(wl_game.player[0].word_list_counter > 0)
	{
		wl_game.player[0].word_list_counter--;
	}
	if(wl_game.player[0].top_word_counter > 0)
	{
		wl_game.player[0].top_word_counter--;
	}

	/* play low life warning sound */
	if(wl_game.uses_life && wl_state == WL_STATE_GAME)
	{
		if(wl_game.player[0].life <= 10.0 && !(wl_game.player[0].powerups & WL_POWERUP_HOURGLASS) && wl_tick % 60 == 0)
		{
			t3f_play_sample(wl_sample[WL_SAMPLE_LIFE_LOW], 1.0, 0.0, 1.0);
		}
	}
}

static bool wl_game_has_single_bg(void)
{
	ALLEGRO_BITMAP * bp = NULL;
	int i;

	if(!wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].shapes_used)
	{
		return false;
	}
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_bg_shape[i].flags)
		{
			if(bp == NULL)
			{
				bp = *wl_bg_shape[i].parent_bp;
			}

			/* if we come across a different bitmap, switch lists */
			if(*wl_bg_shape[i].parent_bp != bp)
			{
				return false;
			}
		}
	}
	bp = NULL;
	for(i = 0; i < WL_MAX_SHAPES; i++)
	{
		if(wl_fg_shape[i].flags)
		{
			if(bp == NULL)
			{
				bp = *wl_fg_shape[i].parent_bp;
			}

			/* if we come across a different bitmap, switch lists */
			if(*wl_fg_shape[i].parent_bp != bp)
			{
				return false;
			}
		}
	}
	return true;
}

void wl_player_rules_logic(void)
{
	int i, j, p;

	/* when the progress bar is full, generate "glass" particles and load the next level */
	if(wl_game.player[0].made_progress)
	{
		if(wl_game.player[0].progress_indicator % WL_LEVEL_UP == 0)
		{
			t3f_play_sample(wl_sample[WL_SAMPLE_SHATTER], 1.0, 0.0, 1.0);
			for(i = 0; i < al_get_bitmap_width(wl_bitmap[WL_BITMAP_LEVEL_FILL]) / al_get_bitmap_width(wl_animation[WL_ANIMATION_PARTICLE_2]->bitmaps->bitmap[0]); i++)
			{
				for(j = 0; j < al_get_bitmap_height(wl_bitmap[WL_BITMAP_LEVEL_FILL]) / al_get_bitmap_height(wl_animation[WL_ANIMATION_PARTICLE_2]->bitmaps->bitmap[0]); j++)
				{
					p = wl_add_particle(1, WL_PARTICLE_TYPE_GLASS, wl_animation[WL_ANIMATION_PARTICLE_2], al_map_rgba(172, 212, 255, 141), WL_BOARD_X + i * al_get_bitmap_width(wl_animation[WL_ANIMATION_PARTICLE_2]->bitmaps->bitmap[0]), 480 - (480 - (WL_BOARD_Y + WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE)) / 2 - al_get_bitmap_height(wl_bitmap[WL_BITMAP_LEVEL]) / 2 + j * al_get_bitmap_height(wl_animation[WL_ANIMATION_PARTICLE_2]->bitmaps->bitmap[0]), 0.0, (float)WL_BOARD_TILE_SIZE, 20 + rand() % 20, 0);
					wl_particle[1][p].vx = 0.0;
					wl_particle[1][p].vy = t3f_drand(&wl_rng_state) * -4.0;
					wl_particle[1][p].vz = -(t3f_drand(&wl_rng_state) * 10.0 + 1.0);
				}
			}
			if(wl_game.mode == WL_GAME_MODE_STORY)
			{
				if(wl_game.player[0].level < wl_end_level)
				{
					wl_game.want_new_level++;
					wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_QUICKIE);
				}
				else
				{
					wl_game.finished = true;
					wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_WIN);
					wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_MARATHON);
					wl_game_end();
				}
			}
			else if(wl_game.mode == WL_GAME_MODE_CASUAL || wl_game.mode == WL_GAME_MODE_PARTY)
			{
				wl_game_end();
			}
		}
		wl_game.player[0].made_progress = false;
	}

	if(wl_game.want_new_level && wl_game_has_single_bg())
	{
		wl_game.player[0].level++;
		al_stop_timer(t3f_timer);
		wl_load_level(&wl_level[wl_get_level_slot(wl_game.player[0].level - 1)], wl_game.player[0].level, WL_LEVEL_LOAD_NORMAL, true, wl_option[WL_OPTION_LOWRES] ? 1 : 0);
		wl_add_message(&wl_game_messages, WL_MESSAGE_TYPE_SCROLLER, wl_level[wl_get_level_slot(wl_game.player[0].level - 1)].name, wl_font[WL_FONT_32], 0);
		al_start_timer(t3f_timer);
		wl_game.tick = WL_LEVEL_FADE_FRAMES;
		wl_game.player[0].words_this_level = 0;
		wl_game.want_new_level--;
	}

	/* see if the game is over */
	if(wl_game.uses_life && wl_game.player[0].state != WL_PLAYER_STATE_ENTER)
	{
		if(wl_game.player[0].life <= 0.0 && wl_game.player[0].lost)
		{
			wl_game.player[0].life = 0.0;
			if(wl_game.player[0].state == WL_PLAYER_STATE_NORMAL && wl_state != WL_STATE_GAME_TUTORIAL)
			{
				wl_award_logic(&wl_profiles->entry[wl_game.player[0].profile], WL_PROFILE_AWARD_LOSER);
				wl_game_end();
			}
		}
	}
}
