#ifndef WL_PLAYER_H
#define WL_PLAYER_H

#include "defines.h"
#include "sprite.h"

#define WL_PLAYER_MAX_CLICKS 64
#define WL_PLAYER_MAX_MOVES  64

typedef struct
{
	
	int x, y;
	int bx, by;
	
} WL_PLAYER_CLICK;

typedef struct
{
	
	WL_PLAYER_CLICK click[WL_PLAYER_MAX_CLICKS];
	int clicks;
	
} WL_PLAYER_CLICK_QUEUE;

typedef struct
{
	
	int profile; // profile selected for this player
	bool watched_tutorial;
	
	WL_SPRITE sprite;
	
	/* board related data */
	int x, y;
	int hover_x, hover_y;
	int board_x, board_y;
	bool in_range;
	
	/* controls */
	WL_PLAYER_CLICK_QUEUE click_queue;
	bool clicked;
	bool touching;
	int touch_id;
	
	/* progress, life, etc. for current game */
	int start_level, level;
	unsigned long progress;
	int score, word_score;
	float life;
	int words;
	char word[WL_MAX_WORD_SIZE];
	int word_pos;
	bool word_exists;
	int word_list[WL_MAX_WORDS];
	int word_list_count;
	int word_list_counter;
	int top_word[WL_MAX_TOP_WORDS];
	int top_word_count;
	int top_word_counter;
	int top_word_spot;
	int top_word_index;
	int range;
	int powerups; // flags for active powerups
	int state;
	int counter;
	bool lost;
	float life_fade;
	
	/* awards stuff */
	int words_this_level;
	
	/* HUD stuff */
	float life_indicator;
	unsigned int progress_indicator;
	bool made_progress;
	
	/* social stuff */
	char move[WL_PLAYER_MAX_MOVES];
	int moves;
	
} WL_PLAYER;

bool wl_player_needs_tutorial(void);
void wl_game_player_add_life(float amount);
void wl_game_player_subtract_life(float amount);
int wl_game_player_pos_to_move(int x, int y);
int wl_game_player_move_to_x(int pos);
int wl_game_player_move_to_y(int pos);
void wl_game_player_control(void);
void wl_game_player_logic(void);
void wl_player_rules_logic(void);

#endif
