#include "t3f/t3f.h"
#include "profile.h"

void wl_clear_profile(WL_PROFILE * pp)
{
	int i;

	pp->word = NULL;
	pp->word_size = 0;
	pp->words = 0;
	pp->time = 0;
	pp->moves = 0;
	pp->undos = 0;
	pp->wrongs = 0;
	pp->games = 0;
	for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
	{
		pp->high_score[i] = 0;
	}
	for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
	{
		pp->best_word[i] = -1;
	}
	pp->progress = 0;
	pp->awards = 0;
	pp->flags = 0;
}

WL_PROFILES * wl_create_profiles(void)
{
	WL_PROFILES * pp = NULL;

	pp = al_malloc(sizeof(WL_PROFILES));
	if(pp)
	{
		wl_clear_profile(&pp->entry[0]);
		strcpy(pp->entry[0].name, "GUEST");
		pp->entries = 1;
	}
	return pp;
}

void wl_destroy_profiles(WL_PROFILES * pp)
{
	int i;

	for(i = 0; i < pp->entries; i++)
	{
		if(pp->entry[i].word)
		{
			al_free(pp->entry[i].word);
		}
	}
	al_free(pp);
}

WL_PROFILES * wl_load_profiles(const char * fn)
{
	ALLEGRO_FILE * fp = NULL;
	WL_PROFILES * pp = NULL;
	char header[16] = {0};
	int i;
	unsigned int j;
	const ALLEGRO_FILE_INTERFACE * fi;

	fi = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = al_fopen(fn, "rb");
	if(fp)
	{
		pp = al_malloc(sizeof(WL_PROFILES));
		if(pp)
		{
			al_fread(fp, header, 16);
			if(header[0] == 'W' && header[1] == 'L' && header[2] == 'P')
			{
				pp->entries = al_fread32le(fp);
				for(i = 0; i < pp->entries; i++)
				{
					pp->entry[i].word = NULL;
					al_fread(fp, pp->entry[i].name, 256);
					pp->entry[i].words = al_fread32le(fp);
					pp->entry[i].word_size = pp->entry[i].words;
					wl_expand_profile(pp, i, 0);
					for(j = 0; j < pp->entry[i].words; j++)
					{
						pp->entry[i].word[j] = al_fread32le(fp);
					}
					pp->entry[i].time = al_fread32le(fp);
					pp->entry[i].moves = al_fread32le(fp);
					pp->entry[i].undos = al_fread32le(fp);
					pp->entry[i].wrongs = al_fread32le(fp);
					pp->entry[i].progress = al_fread32le(fp);
					pp->entry[i].games = al_fread32le(fp);
					for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
					{
						pp->entry[i].high_score[j] = al_fread32le(fp);
					}
					for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
					{
						pp->entry[i].best_word[j] = al_fread32le(fp);
						if(header[15] == 2)
						{
							pp->entry[i].best_word[j] = -1;
						}
					}
					pp->entry[i].awards = al_fread32le(fp);
					pp->entry[i].flags = al_fread32le(fp);
					if(header[15] < 4)
					{
						pp->entry[i].flags &= ~WL_PROFILE_FLAG_UPLOAD_BEST;
						pp->entry[i].flags &= ~WL_PROFILE_FLAG_UPLOAD_BEST_C;
						pp->entry[i].flags &= ~WL_PROFILE_FLAG_UPLOAD_BEST_P;
						pp->entry[i].flags &= ~WL_PROFILE_FLAG_UPLOAD_BEST_G;
					}
				}
				for(i = pp->entries - 1; i >= 0; i--)
				{
					if(strlen(pp->entry[i].name) == 0)
					{
						wl_delete_profile(pp, i);
					}
				}
			}
			else
			{
				al_free(pp);
				pp = NULL;
			}
		}
		al_fclose(fp);
	}
	al_set_new_file_interface(fi);
	return pp;
}

void wl_save_profiles(WL_PROFILES * pp, const char * fn)
{
	ALLEGRO_FILE * fp = NULL;
	char header[16] = {'W', 'L', 'P'};
	int i;
	unsigned int j;
	const ALLEGRO_FILE_INTERFACE * fi;

	fi = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = al_fopen(fn, "wb");
	if(fp)
	{
		header[15] = 4;
		al_fwrite(fp, header, 16);
		al_fwrite32le(fp, pp->entries);
		for(i = 0; i < pp->entries; i++)
		{
			al_fwrite(fp, pp->entry[i].name, 256);
			al_fwrite32le(fp, pp->entry[i].words);
			for(j = 0; j < pp->entry[i].words; j++)
			{
				al_fwrite32le(fp, pp->entry[i].word[j]);
			}
			al_fwrite32le(fp, pp->entry[i].time);
			al_fwrite32le(fp, pp->entry[i].moves);
			al_fwrite32le(fp, pp->entry[i].undos);
			al_fwrite32le(fp, pp->entry[i].wrongs);
			al_fwrite32le(fp, pp->entry[i].progress);
			al_fwrite32le(fp, pp->entry[i].games);
			for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
			{
				al_fwrite32le(fp, pp->entry[i].high_score[j]);
			}
			for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
			{
				al_fwrite32le(fp, pp->entry[i].best_word[j]);
			}
			al_fwrite32le(fp, pp->entry[i].awards);
			al_fwrite32le(fp, pp->entry[i].flags);
		}
		al_fclose(fp);
	}
	al_set_new_file_interface(fi);
}

bool wl_expand_profile(WL_PROFILES * pp, int entry, int amount)
{
	int * word = NULL;
	unsigned int i;

	if(pp->entry[entry].word_size + amount > 0)
	{
		word = al_malloc(sizeof(int) * pp->entry[entry].word_size + amount);
		if(!word)
		{
			return false;
		}
		pp->entry[entry].word_size += amount;

		/* if we have words, copy them and free the old ones */
		if(pp->entry[entry].word)
		{
			for(i = 0; i < pp->entry[entry].words; i++)
			{
				word[i] = pp->entry[entry].word[i];
			}
			al_free(pp->entry[entry].word);
		}
		pp->entry[entry].word = word;
	}

	return true;
}

void wl_delete_profile(WL_PROFILES * pp, int entry)
{
	int j;

	for(j = entry; j < pp->entries - 1; j++)
	{
		memcpy(&pp->entry[j], &pp->entry[j + 1], sizeof(WL_PROFILE));
	}
	pp->entries--;
}
