#include "t3f/view.h"
#include "shape.h"

static WL_SHAPE * wl_cached_shape[2][WL_MAX_SHAPES] = {{NULL}};
static int wl_cached_shapes[2] = {0};

void wl_generate_shape(WL_SHAPE * sp, WL_LEVEL_SHAPE * lsp, ALLEGRO_BITMAP ** parent_bp, float x, float y, float z, float slope)
{
	sp->parent_bp = parent_bp;
	sp->ox = x;
	sp->oy = y;
	sp->oz = z;
	sp->vx = 1.0;
	sp->vy = slope;
	sp->slope = slope;
	sp->flags = 1;
	sp->vertex[0].x = 0;
	sp->vertex[0].y = lsp->target_height;
	sp->vertex[0].z = 0.0;
	sp->vertex[0].u = lsp->x;
	sp->vertex[0].v = (float)lsp->y + lsp->height - 0.5;
	sp->vertex[1].x = 0;
	sp->vertex[1].y = 0;
	sp->vertex[1].z = 0.0;
	sp->vertex[1].u = lsp->x;
	sp->vertex[1].v = lsp->y + 0.5;
	sp->vertex[2].x = lsp->target_width;
	sp->vertex[2].y = lsp->target_width * slope;
	sp->vertex[2].z = 0.0;
	sp->vertex[2].u = lsp->x + lsp->width;
	sp->vertex[2].v = lsp->y + 0.5;
	sp->vertex[3].x = lsp->target_width;
	sp->vertex[3].y = lsp->target_width * slope;
	sp->vertex[3].z = 0.0;
	sp->vertex[3].u = lsp->x + lsp->width;
	sp->vertex[3].v = lsp->y + 0.5;
	sp->vertex[4].x = lsp->target_width;
	sp->vertex[4].y = lsp->target_height + lsp->target_width * slope;
	sp->vertex[4].z = 0.0;
	sp->vertex[4].u = lsp->x + lsp->width;
	sp->vertex[4].v = lsp->y + lsp->height - 0.5;
	sp->vertex[5].x = 0;
	sp->vertex[5].y = lsp->target_height;
	sp->vertex[5].z = 0.0;
	sp->vertex[5].u = lsp->x;
	sp->vertex[5].v = lsp->y + lsp->height - 0.5;
	sp->vertices = 6;
}

void wl_connect_shape(WL_SHAPE * sp1, WL_SHAPE * sp2, WL_LEVEL_SHAPE * lsp, ALLEGRO_BITMAP ** parent_bp)
{
	wl_generate_shape(sp1, lsp, parent_bp, sp2->ox - lsp->target_width, sp2->oy - lsp->target_width * sp2->vy, sp2->oz, sp2->slope);
}

void wl_process_shape(WL_SHAPE * sp)
{
	sp->ox += sp->vx;
	if(sp->ox >= 640.0 + 64.0)
	{
		sp->flags = 0;
	}
	sp->oy += sp->vy;
}

void wl_draw_shape(WL_SHAPE * sp, ALLEGRO_COLOR color, int flags)
{
	ALLEGRO_VERTEX vertex[WL_MAX_VERTICES];
	int vertices = 0;
	int i;
	
	for(i = 0; i < sp->vertices; i++)
	{
		vertex[i].x = t3f_project_x(sp->vertex[i].x + sp->ox, sp->vertex[i].z + sp->oz);
		vertex[i].y = t3f_project_y(sp->vertex[i].y + sp->oy, sp->vertex[i].z + sp->oz);
		vertex[i].z = 0.0;
		vertex[i].u = sp->vertex[i].u;
		vertex[i].v = sp->vertex[i].v;
		vertex[i].color = color;
		vertices++;
	}
	al_draw_prim(vertex, NULL, *sp->parent_bp, 0, vertices, ALLEGRO_PRIM_TRIANGLE_LIST);
}

void wl_cache_shape(int list, WL_SHAPE * sp)
{
	wl_cached_shape[list][wl_cached_shapes[list]] = sp;
	wl_cached_shapes[list]++;
}

void wl_draw_cached_shapes(int list, ALLEGRO_COLOR color)
{
	ALLEGRO_VERTEX vertex[WL_MAX_VERTICES * WL_MAX_SHAPES];
	int vertices = 0;
	int i, j;
	
	for(i = 0; i < wl_cached_shapes[list]; i++)
	{
		for(j = 0; j < wl_cached_shape[list][i]->vertices; j++)
		{
			vertex[vertices].x = t3f_project_x(wl_cached_shape[list][i]->vertex[j].x + wl_cached_shape[list][i]->ox, wl_cached_shape[list][i]->vertex[j].z + wl_cached_shape[list][i]->oz);
			vertex[vertices].y = t3f_project_y(wl_cached_shape[list][i]->vertex[j].y + wl_cached_shape[list][i]->oy, wl_cached_shape[list][i]->vertex[j].z + wl_cached_shape[list][i]->oz);
			vertex[vertices].z = 0.0;
			vertex[vertices].u = wl_cached_shape[list][i]->vertex[j].u;
			vertex[vertices].v = wl_cached_shape[list][i]->vertex[j].v;
			vertex[vertices].color = color;
			vertices++;
		}
	}
	if(vertices > 0)
	{
		al_draw_prim(vertex, NULL, *wl_cached_shape[list][0]->parent_bp, 0, vertices, ALLEGRO_PRIM_TRIANGLE_LIST);
	}
}

void wl_clear_cached_shapes(int list)
{
	wl_cached_shapes[list] = 0;
}
