#include "t3f/sound.h"
#include "defines.h"
#include "board.h"
#include "sprite.h"
#include "data.h"
#include "game.h"
#include "board.h"
#include "text.h"
#include "game_hazard.h"

void wl_sprite_powerup_common_logic(WL_SPRITE * sp)
{
	t3f_move_collision_object_xy(sp->collider, sp->x, sp->y);
	sp->z += sp->vz;
	if(sp->state == 0)
	{
		sp->vz += 0.075;
		if(sp->vz > 0.5)
		{
			sp->state = 1;
		}
	}
	else
	{
		sp->vz -= 0.075;
		if(sp->vz < -0.5)
		{
			sp->state = 0;
		}
	}
}

void wl_sprite_logic(WL_SPRITE * sp)
{
	int i, j, x, y, ix, iy;
	int sx, ex, sy, ey;
	bool might_swoosh = false;
	bool exploded = false;

	switch(sp->type)
	{
		case WL_SPRITE_TYPE_PLAYER:
		{
			t3f_move_collision_object_xy(sp->collider, sp->x, sp->y);
			if(sp->timer > 0)
			{
				sp->timer--;
			}
			sp->tick++;
			switch(sp->state)
			{
				case WL_PLAYER_SPRITE_STATE_HAPPY:
				case WL_PLAYER_SPRITE_STATE_HURT:
				case WL_PLAYER_SPRITE_STATE_SAD:
				{
					if(sp->tick >= 30)
					{
						sp->state = WL_PLAYER_SPRITE_STATE_NORMAL;
						sp->tick = 0;
					}
					break;
				}
				case WL_PLAYER_SPRITE_STATE_UP:
				{
					if(sp->tick >= 30)
					{
						sp->state = WL_PLAYER_SPRITE_STATE_DOWN;
						sp->tick = 0;
					}
					break;
				}
				case WL_PLAYER_SPRITE_STATE_LEFT:
				{
					if(sp->tick >= 30)
					{
						sp->state = WL_PLAYER_SPRITE_STATE_RIGHT;
						sp->tick = 0;
					}
					break;
				}
				case WL_PLAYER_SPRITE_STATE_DOWN:
				case WL_PLAYER_SPRITE_STATE_RIGHT:
				{
					if(sp->tick >= 30)
					{
						sp->state = WL_PLAYER_SPRITE_STATE_NORMAL;
						sp->tick = 0;
					}
					break;
				}
				default:
				{
					if(sp->tick >= 180)
					{
						sp->state = WL_PLAYER_SPRITE_STATE_UP + (rand() % 2) * 2;
						sp->tick = 0;
					}
					break;
				}
			}
			break;
		}
		case WL_SPRITE_TYPE_BOMB:
		{
			if(sp->state == 0)
			{
				if(sp->z < WL_SPRITE_SWOOSH_HEIGHT)
				{
					might_swoosh = true;
				}
				sp->z += sp->vz;
				if(might_swoosh && sp->z >= WL_SPRITE_SWOOSH_HEIGHT)
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_SWOOSH], 1.0, t3f_get_sound_position(wl_game.player[0].sprite.x, wl_game.player[0].sprite.y, sp->x, sp->y), 1.0);
				}
				if(sp->z >= 0.0)
				{
					if(sp->vz >= WL_TERMINAL_VELOCITY)
					{
						t3f_play_sample(wl_sample[WL_SAMPLE_LAND], 1.0, 0.0, 1.0);
						sp->vz = -3.0;
					}
					else
					{
						t3f_play_sample(wl_sample[WL_SAMPLE_LAND], 0.5, 0.0, 1.0);
						sp->state = 1;
					}
					sp->z = 0.0;
				}
				sp->vz += 0.5;
				if(sp->vz > WL_TERMINAL_VELOCITY)
				{
					sp->vz = WL_TERMINAL_VELOCITY;
				}
			}
			else
			{
				/* remove letter from under bomb */
				wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].letter = '0';

				sp->timer--;
				/* check collision with player */
				t3f_recreate_collision_object(sp->collider, 4, 4, 32 - 8, 32 - 8, 32, 32, 0);
				t3f_move_collision_object_xy(sp->collider, sp->x, sp->y);

				/* when bomb blows up */
				if(sp->timer <= 0 || (wl_game.player[0].sprite.z >= sp->z - 1 && sp->z >= wl_game.player[0].sprite.z - 1 && t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider)))
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_BOMB], 1.0, t3f_get_sound_position(wl_game.player[0].sprite.x, wl_game.player[0].sprite.y, sp->x, sp->y), 1.0);

					/* damage player */
					if((abs(wl_game.player[0].x - (int)sp->x / WL_BOARD_TILE_SIZE) < 3 && abs(wl_game.player[0].y - (int)sp->y / WL_BOARD_TILE_SIZE) < 3))
					{
						t3f_play_sample(wl_sample[WL_SAMPLE_HURT], 1.0, 0.0, 1.0);
						wl_game_player_subtract_life(20.0);
						if(wl_game.player[0].life < 0)
						{
							wl_game.player[0].life = 0;
						}
						wl_game.player[0].sprite.timer = 30;
//						wl_game.player[0].word_pos = 0;
//						wl_game.player[0].word[0] = '\0';
//						wl_game.player[0].word_exists = false;
						wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_HURT;
						wl_game.player[0].sprite.tick = 0;
					}

					/* randomize letters */
					x = (int)sp->x / WL_BOARD_TILE_SIZE - 2;
					y = (int)sp->y / WL_BOARD_TILE_SIZE - 2;
					for(i = 0; i < 25; i++)
					{
						ix = x + (i % 5);
						iy = y + (i / 5);
						if(ix >= 0 && ix < WL_MAX_BOARD_WIDTH && iy >= 0 && iy < WL_MAX_BOARD_HEIGHT && wl_game.board[iy][ix].letter != ' ')
						{
							wl_game.board[iy][ix].letter = wl_board_generate_letter(false);
						}
					}

					/* destroy spikes */
					for(i = 0; i < WL_MAX_SPRITES; i++)
					{
						if(wl_game.sprite[i].active && wl_game.sprite[i].type == WL_SPRITE_TYPE_SPIKE)
						{
							if(wl_game.sprite[i].z >= 0.0 && abs((int)wl_game.sprite[i].x / WL_BOARD_TILE_SIZE - (int)sp->x / WL_BOARD_TILE_SIZE) < 3 && abs((int)wl_game.sprite[i].y / WL_BOARD_TILE_SIZE - (int)sp->y / WL_BOARD_TILE_SIZE) < 3)
							{
								wl_explode_sprite(&wl_game.sprite[i]);
								if(!exploded)
								{
									t3f_play_sample(wl_sample[WL_SAMPLE_DESTROY], 1.0, 0.0, 1.0);
									exploded = true;
								}
							}
						}
					}

					wl_explode_sprite(sp);
					wl_generate_board_particles(al_map_rgba_f(1.0, 0.5, 0.0, 1.0), (int)sp->x / WL_BOARD_TILE_SIZE, (int)sp->y / WL_BOARD_TILE_SIZE, 3);
					sp->active = false;
				}
			}
			sp->tick++;
			if(sp->timer >= 240)
			{
				sp->ftick += 1.0;
			}
			else if(sp->timer >= 180)
			{
				sp->ftick += 1.5;
			}
			else if(sp->timer >= 120)
			{
				sp->ftick += 2.0;
			}
			else if(sp->timer >= 60)
			{
				sp->ftick += 2.5;
			}
			else
			{
				sp->ftick += 3.0;
			}

			/* color tiles so player is aware of blast radius */
			sx = (int)sp->x / WL_BOARD_TILE_SIZE - 2;
			ex = sx + 5;
			if(sx < 0)
			{
				sx = 0;
			}
			if(ex > WL_MAX_BOARD_WIDTH)
			{
				ex = WL_MAX_BOARD_WIDTH;
			}
			sy = (int)sp->y / WL_BOARD_TILE_SIZE - 2;
			ey = sy + 5;
			if(sy < 0)
			{
				sy = 0;
			}
			if(ey > WL_MAX_BOARD_HEIGHT)
			{
				ey = WL_MAX_BOARD_HEIGHT;
			}
			for(i = sy; i < ey; i++)
			{
				for(j = sx; j < ex; j++)
				{
					wl_game.board[i][j].flags |= WL_BOARD_FLAG_HAZARD;
				}
			}

			/* mark tile as used */
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].used = true;
			break;
		}
		case WL_SPRITE_TYPE_LASER:
		{
			sp->timer++;
			if(sp->state == 0)
			{
				t3f_recreate_collision_object(sp->collider, 4, 12, 32 - 4, 32 - 12, 32, 32, 0);
				for(i = 0; i < WL_MAX_BOARD_WIDTH; i++)
				{
					wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][i].flags |= WL_BOARD_FLAG_HAZARD;
				}
			}
			else
			{
				t3f_recreate_collision_object(sp->collider, 12, 4, 32 - 12, 32 - 4, 32, 32, 0);
				for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
				{
					wl_game.board[i][(int)sp->x / WL_BOARD_TILE_SIZE].flags |= WL_BOARD_FLAG_HAZARD;
				}
			}
			if(sp->flags == 0)
			{
				sp->x += sp->vx;
				if((sp->vx < 0.0 && sp->x < -36 - wl_animation[WL_ANIMATION_LASER]->frame[0]->width) || (sp->vx > 0.0 && sp->x > WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 36))
				{
					sp->active = false;
				}
				sp->y += sp->vy;
				if((sp->vy < 0.0 && sp->y < -36 - wl_animation[WL_ANIMATION_LASER]->frame[0]->height) || (sp->vy > 0.0 && sp->y > WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE + 36))
				{
					sp->active = false;
				}
				t3f_move_collision_object_xy(sp->collider, sp->x, sp->y);
				if(t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider))
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_HURT], 1.0, 0.0, 1.0);
					wl_game_player_subtract_life(10.0);
					if(wl_game.player[0].life < 0)
					{
						wl_game.player[0].life = 0;
					}
					wl_game.player[0].sprite.timer = 30;
					wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_HURT;
					wl_game.player[0].sprite.tick = 0;
					wl_explode_sprite(sp);
					sp->active = false;
				}
			}
			break;
		}
		case WL_SPRITE_TYPE_SPIKE:
		{
			if(sp->z < WL_SPRITE_SWOOSH_HEIGHT)
			{
				might_swoosh = true;
			}
			sp->z += sp->vz;
			if(might_swoosh && sp->z >= WL_SPRITE_SWOOSH_HEIGHT)
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_SWOOSH], 1.0, t3f_get_sound_position(wl_game.player[0].sprite.x, wl_game.player[0].sprite.y, sp->x, sp->y), 1.0);
			}
			if(sp->z >= 0.0)
			{
				if(sp->state == 0)
				{
					t3f_play_sample(wl_sample[WL_SAMPLE_SPIKE], 1.0, t3f_get_sound_position(wl_game.player[0].sprite.x, wl_game.player[0].sprite.y, sp->x, sp->y), 1.0);
				}
				sp->state = 1;
				sp->z = 0.0;
				sp->vz = 0.0;

				/* remove letter from under spike if necessary */
				if(wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].letter != ' ')
				{
					wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].letter = '0';
				}
			}
			sp->vz += 0.5;
			if(sp->vz > WL_TERMINAL_VELOCITY)
			{
				sp->vz = WL_TERMINAL_VELOCITY;
			}

			/* check collision with player */
			t3f_recreate_collision_object(sp->collider, 8, 8, 16, 16, 32, 32, 0);
			t3f_move_collision_object_xy(sp->collider, sp->x, sp->y);
			if(wl_game.player[0].sprite.z >= sp->z - 1 && sp->z >= wl_game.player[0].sprite.z - 1 && t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider))
			{
				t3f_play_sample(wl_sample[WL_SAMPLE_HURT], 1.0, 0.0, 1.0);
				wl_game_player_subtract_life(10.0);
				if(wl_game.player[0].life < 0)
				{
					wl_game.player[0].life = 0;
				}
				wl_game.player[0].sprite.timer = 30;
				wl_game.player[0].sprite.state = WL_PLAYER_SPRITE_STATE_HURT;
				wl_game.player[0].sprite.tick = 0;
				wl_explode_sprite(sp);
			}

			/* color tile */
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].flags |= WL_BOARD_FLAG_HAZARD;

			/* mark tile as used */
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].used = true;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_HEALTH:
		{
			wl_sprite_powerup_common_logic(sp);
			if(wl_game.board[sp->timer / WL_MAX_BOARD_HEIGHT][sp->timer % WL_MAX_BOARD_WIDTH].letter != '!')
			{
				sp->active = false;
			}
			else if(t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider))
			{
			}
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].flags |= WL_BOARD_FLAG_POWERUP;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_GURU:
		{
			sp->tick++;
			wl_sprite_powerup_common_logic(sp);
			if(wl_game.board[sp->timer / WL_MAX_BOARD_HEIGHT][sp->timer % WL_MAX_BOARD_WIDTH].letter != '?')
			{
				sp->active = false;
			}
			else if(t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider))
			{
			}
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].flags |= WL_BOARD_FLAG_POWERUP;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_SNEAKER:
		{
			wl_sprite_powerup_common_logic(sp);

			/* rotation logic */
			if(sp->flags == 0)
			{
				sp->a += ALLEGRO_PI / 128.0;
				if(sp->a >= ALLEGRO_PI / 16)
				{
					sp->flags = 1;
				}
			}
			else
			{
				sp->a -= ALLEGRO_PI / 128.0;
				if(sp->a <= -ALLEGRO_PI / 16)
				{
					sp->flags = 0;
				}
			}

			if(wl_game.board[sp->timer / WL_MAX_BOARD_HEIGHT][sp->timer % WL_MAX_BOARD_WIDTH].letter != '_')
			{
				sp->active = false;
			}
			else if(t3f_check_object_collision(sp->collider, wl_game.player[0].sprite.collider))
			{
			}
			wl_game.board[(int)sp->y / WL_BOARD_TILE_SIZE][(int)sp->x / WL_BOARD_TILE_SIZE].flags |= WL_BOARD_FLAG_POWERUP;
			break;
		}
		case WL_SPRITE_TYPE_100:
		{
			sp->z -= 1.0;
			sp->tick++;
			if(sp->tick > 30)
			{
				sp->active = false;
			}
			break;
		}
	}
}

void wl_sprite_shadow_render(WL_SPRITE * sp)
{
	float a, s;
	float cx, cy;

	a = 1.0 + sp->z / 320.0;
	if(a < 0.0)
	{
		a = 0.0;
	}
	switch(sp->type)
	{
		case WL_SPRITE_TYPE_PLAYER:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_SHADOW], al_map_rgba_f(a, a, a, a), 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_BOMB:
		{
			s = 1.0 - 0.05 * fmod(sp->ftick, 30.0) / 30.0;
			t3f_draw_scaled_animation(wl_animation[WL_ANIMATION_BOMB_SHADOW], al_map_rgba_f(a, a, a, a), sp->tick, WL_BOARD_X + sp->x + 16.0 - 16.0 * s, WL_BOARD_Y + sp->y + 16.0 - 16.0 * s + WL_SPRITE_Y_OFFSET + sp->z + 2, 0.0, s, 0);
//			t3f_draw_animation(wl_animation[WL_ANIMATION_BOMB_SHADOW], al_map_rgba_f(a, a, a, a), 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + 2, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_LASER:
		{
			if(sp->state == 1)
			{
				a = ALLEGRO_PI / 2.0;
			}
			else
			{
				a = 0.0;
			}
			cx = al_get_bitmap_width(wl_animation[WL_ANIMATION_LASER_SHADOW]->bitmaps->bitmap[0]) / 2;
			cy = al_get_bitmap_height(wl_animation[WL_ANIMATION_LASER_SHADOW]->bitmaps->bitmap[0]) / 2;
			t3f_draw_rotated_animation(wl_animation[WL_ANIMATION_LASER_SHADOW], t3f_color_white, 0, cx, cy, WL_BOARD_X + sp->x + cx, WL_BOARD_Y + sp->y + cx + WL_SPRITE_Y_OFFSET + 8, 0.0, a, 0);
			break;
		}
		case WL_SPRITE_TYPE_SPIKE:
		{
			if(sp->state == 0)
			{
				t3f_draw_animation(wl_animation[WL_ANIMATION_SPIKE_SHADOW], al_map_rgba_f(a, a, a, a), 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET - 5, 0.0, 0);
			}
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_HEALTH:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_HEALTH_SHADOW], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_GURU:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_GURU_SHADOW], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_SNEAKER:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_HOURGLASS_SHADOW], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_100:
		{
			break;
		}
	}
}

void wl_sprite_render(WL_SPRITE * sp)
{
	float a = 0.0, cx, cy, s;
	ALLEGRO_COLOR color;

	switch(sp->type)
	{
		case WL_SPRITE_TYPE_PLAYER:
		{
			if(sp->timer > 0)
			{
				a = (float)sp->timer / 30.0;
				color = al_map_rgba_f(1.0, 1.0 - a, 1.0 - a, 1.0);
			}
			else
			{
				color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
			}
			switch(sp->state)
			{
				case WL_PLAYER_SPRITE_STATE_NORMAL:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_NORMAL], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_HURT:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_HURT], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_HAPPY:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_HAPPY], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_SAD:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_SAD], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_UP:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_UP], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_DOWN:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_DOWN], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_LEFT:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_LEFT], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
				case WL_PLAYER_SPRITE_STATE_RIGHT:
				{
					t3f_draw_animation(wl_animation[WL_ANIMATION_PLAYER_RIGHT], color, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
					break;
				}
			}
			break;
		}
		case WL_SPRITE_TYPE_BOMB:
		{
//			t3f_draw_animation(wl_animation[WL_ANIMATION_BOMB], t3f_color_white, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
			s = 1.0 - 0.05 * fmod(sp->ftick, 30.0) / 30.0;
			t3f_draw_scaled_animation(wl_animation[WL_ANIMATION_BOMB], t3f_color_white, sp->tick, WL_BOARD_X + sp->x + 16.0 - 16.0 * s, WL_BOARD_Y + sp->y + 16.0 - 16.0 * s + WL_SPRITE_Y_OFFSET + sp->z, 0.0, s, 0);
			break;
		}
		case WL_SPRITE_TYPE_LASER:
		{
			if(sp->state == 1)
			{
				a = ALLEGRO_PI / 2.0;
			}
			cx = al_get_bitmap_width(wl_animation[WL_ANIMATION_LASER]->bitmaps->bitmap[0]) / 2;
			cy = al_get_bitmap_height(wl_animation[WL_ANIMATION_LASER]->bitmaps->bitmap[0]) / 2;
			t3f_draw_rotated_animation(wl_animation[WL_ANIMATION_LASER], t3f_color_white, 0, cx, cy, WL_BOARD_X + sp->x + cx, WL_BOARD_Y + sp->y + cx + WL_SPRITE_Y_OFFSET + sp->z, 0.0, a, 0);
			if(sp->timer % 16 < 8)
			{
				color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
			}
			else
			{
				color = al_map_rgba_f(1.0, 0.0, 0.0, 1.0);
			}
			if(sp->x < -36 - wl_animation[WL_ANIMATION_LASER]->frame[0]->width)
			{
				wl_draw_text(wl_font[WL_FONT_32], color, t3f_color_black, al_map_rgba(0, 0, 0, 192), WL_BOARD_X - 12, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 2, 2, WL_TEXT_FLAG_SHADOW | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_CENTER, "!");
			}
			else if(sp->x > WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 36.0)
			{
				wl_draw_text(wl_font[WL_FONT_32], color, t3f_color_black, al_map_rgba(0, 0, 0, 192), WL_BOARD_X + WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 12, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET, 2, 2, WL_TEXT_FLAG_SHADOW | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_CENTER, "!");
			}
			else if(sp->y < -36 - wl_animation[WL_ANIMATION_LASER]->frame[0]->height)
			{
				wl_draw_text(wl_font[WL_FONT_32], color, t3f_color_black, al_map_rgba(0, 0, 0, 192), WL_BOARD_X + sp->x + 24, WL_BOARD_Y - al_get_font_line_height(wl_font[WL_FONT_32]) + 4, 2, 2, WL_TEXT_FLAG_SHADOW | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_CENTER, "!");
			}
			else if(sp->y > WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE + 36.0)
			{
				wl_draw_text(wl_font[WL_FONT_32], color, t3f_color_black, al_map_rgba(0, 0, 0, 192), WL_BOARD_X + sp->x + 24, WL_BOARD_Y + WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE - 4, 2, 2, WL_TEXT_FLAG_SHADOW | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_CENTER, "!");
			}
			break;
		}
		case WL_SPRITE_TYPE_SPIKE:
		{
			if(sp->state == 0)
			{
				t3f_draw_animation(wl_animation[WL_ANIMATION_SPIKE], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
			}
			else
			{
				t3f_draw_animation(wl_animation[WL_ANIMATION_SPIKE_2], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z + 11.0, 0.0, 0);
			}
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_HEALTH:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_HEALTH], t3f_color_white, 0, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, 0);
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_GURU:
		{
			t3f_draw_animation(wl_animation[WL_ANIMATION_POWER_GURU], t3f_color_white, sp->tick, WL_BOARD_X + sp->x, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z - 4.0, 0.0, 0);
//			al_draw_text(wl_font[WL_FONT_24], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), WL_BOARD_X + sp->x + 16, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, ALLEGRO_ALIGN_CENTRE, "?");
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_SNEAKER:
		{
			cx = wl_animation[WL_ANIMATION_POWER_HOURGLASS]->frame[0]->width / 2;
			cy = wl_animation[WL_ANIMATION_POWER_HOURGLASS]->frame[0]->height / 2;
			t3f_draw_rotated_animation(wl_animation[WL_ANIMATION_POWER_HOURGLASS], t3f_color_white, 0, cx, cy, cx + WL_BOARD_X + sp->x, cy + WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0.0, sp->a, 0);
//			wl_draw_text(wl_font[WL_FONT_24], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), t3f_color_black, t3f_color_black, WL_BOARD_X + sp->x + 16, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0, 0, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_OUTLINE, "x2");
//			al_draw_text(wl_font[WL_FONT_24], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), WL_BOARD_X + sp->x + 16, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, ALLEGRO_ALIGN_CENTRE, "x2");
			break;
		}
		case WL_SPRITE_TYPE_100:
		{
			a = 1.0 - (float)sp->tick / 30.0;
			t3f_draw_text(wl_ascii_font, al_map_rgba_f(a, a, a, a), WL_BOARD_X + sp->x + 16, WL_BOARD_Y + sp->y + WL_SPRITE_Y_OFFSET + sp->z, 0, 0, 0, T3F_FONT_ALIGN_CENTER, "-100");
			break;
		}
	}
}
