#include "t3f/t3f.h"
#include "defines.h"
#include "data.h"

const int text_box_margin = 4;

void wl_draw_text(ALLEGRO_FONT * fp, ALLEGRO_COLOR color, ALLEGRO_COLOR outline_color, ALLEGRO_COLOR shadow_color, float x, float y, float sx, float sy, int flags, const char * text)
{
	bool held = al_is_bitmap_drawing_held();
	ALLEGRO_STATE old_state;
	float ox = x;
	
	al_hold_bitmap_drawing(false); // clear drawing cache
	al_store_state(&old_state, ALLEGRO_STATE_TARGET_BITMAP);
	
	al_set_target_bitmap(wl_bitmap[WL_BITMAP_SCRATCH]);
	al_clear_to_color(al_map_rgba_f(0.0, 0.0, 0.0, 0.0));
	al_hold_bitmap_drawing(true);
	
	/* render outline */
	if(flags & WL_TEXT_FLAG_OUTLINE)
	{
		al_draw_text(fp, outline_color, 0, 1, 0, text);
		al_draw_text(fp, outline_color, 2, 1, 0, text);
		al_draw_text(fp, outline_color, 1, 0, 0, text);
		al_draw_text(fp, outline_color, 1, 2, 0, text);
	}
	
	/* render text */
	al_draw_text(fp, color, 1, 1, 0, text);
	al_hold_bitmap_drawing(false);

	al_restore_state(&old_state);
	al_hold_bitmap_drawing(held);
	
	/* render to screen */
	if(flags & WL_TEXT_FLAG_CENTER)
	{
		ox = x - al_get_text_width(fp, text) / 2;
	}
	if(flags & WL_TEXT_FLAG_SHADOW)
	{
		al_draw_tinted_bitmap(wl_bitmap[WL_BITMAP_SCRATCH], shadow_color, ox + sx, y + sy, 0);
	}
	al_draw_tinted_bitmap(wl_bitmap[WL_BITMAP_SCRATCH], color, ox, y, 0);
}

void wl_draw_text_box(ALLEGRO_FONT * fp, ALLEGRO_COLOR bg_color, ALLEGRO_COLOR fg_color, float x, float y, float width, int flags, const char * text)
{
	char line[20][256];
	int line_count = 0;
	char current_line[256];
	int current_line_pos = 0;
	int current_line_start_pos = 0;
	int last_space = -1;
	int i;
	float box_y;
	int box_height;
	
	/* divide text into lines */
	for(i = 0; i < (int)strlen(text); i++)
	{
		current_line[current_line_pos] = text[i];
		current_line[current_line_pos + 1] = '\0';
		if(text[i] == ' ')
		{
			last_space = current_line_pos;
		}
		current_line_pos++;
		
		/* copy line since we encountered a manual new line */
		if(text[i] == '\n')
		{
			current_line[current_line_pos] = '\0';
			strcpy(line[line_count], current_line);
			current_line_start_pos += i + 1;
			line_count++;
			current_line_pos = 0;
			current_line[current_line_pos] = '\0';
		}
		
		/* copy this line to our list of lines because it is long enough */
		else if(al_get_text_width(fp, current_line) > width - text_box_margin * 2)
		{
			current_line[last_space] = '\0';
			strcpy(line[line_count], current_line);
			current_line_start_pos += last_space + 1;
			while(text[i] != ' ' && i >= 0)
			{
				i--;
			}
			line_count++;
			current_line_pos = 0;
			current_line[current_line_pos] = '\0';
		}
	}
	strcpy(line[line_count], current_line);
	line_count++;
	
	box_height = al_get_font_line_height(fp) * line_count + text_box_margin * 2;
	box_y = y - box_height / 2;
	al_hold_bitmap_drawing(false);
	al_draw_filled_rectangle(x, box_y, x + width, box_y + box_height, bg_color);
	al_draw_rectangle(x + 0.5, box_y + 0.5, x + width + 0.5, box_y + box_height + 0.5, t3f_color_black, 2.0);
	al_hold_bitmap_drawing(true);
	for(i = 0; i < line_count; i++)
	{
		al_draw_text(fp, fg_color, x + text_box_margin, box_y + text_box_margin + i * al_get_font_line_height(fp), 0, line[i]);
	}
	if(line_count > 0 && (wl_tick / 15) % 2 == 0)
	{
		al_draw_text(fp, fg_color, x + width - text_box_margin - al_get_text_width(fp, "_"), box_y + text_box_margin + (line_count - 1) * al_get_font_line_height(fp), 0, "_");
	}
	al_hold_bitmap_drawing(false);
}
