#include "t3f/gui.h"
#include "t3f/sound.h"
#include "t3f/music.h"
#include "t3f/resource.h"
#include "t3f/rng.h"
#include "t3f/draw.h"
#include "data.h"
#include "menu.h"
#include "menu_proc.h"
#include "text_entry.h"
#include "effects.h"
#include "game_intro.h"
#include "text.h"

WL_TITLE_TILE wl_title_tile[8];
WL_TITLE_WORD wl_title_word[WL_MAX_TITLE_WORDS];
T3F_ATLAS * wl_title_atlas = NULL;
static char * wl_title_text = "wordleap";

void wl_generate_title_word(WL_TITLE_WORD * wp)
{
	wp->word = rand() % wl_dictionary->words;
	wp->x = t3f_drand(&wl_rng_state) * 800.0 - 160.0;
	wp->vx = t3f_drand(&wl_rng_state) * 2.0 - 1.0;
	wp->y = t3f_drand(&wl_rng_state) * 640 - 160;
	wp->vy = t3f_drand(&wl_rng_state) * 2.0 - 1.0;
	wp->a = 0.0;
	wp->va = 0.01;
	wp->time = rand() % 180 + 60;
}

void wl_title_tile_logic(WL_TITLE_TILE * tp)
{
	tp->y += tp->vy;
	tp->vy += 0.125;
	if(tp->y >= WL_TITLE_LOGO_Y)
	{
		tp->y = WL_TITLE_LOGO_Y;
		tp->vy = -1.25;
	}
}

/* helper for moving between menus */
void wl_select_menu(int menu)
{
	wl_current_menu = menu;
	wl_menu[wl_current_menu]->hover_element = -1;
	wl_reset_particles(0);
}

void wl_title_init(void)
{
	unsigned int i;
	char fn[1024]= {0};
	int pos = 0;

	wl_set_menu_driver();
	wl_menu_ustr = al_ustr_new_from_buffer(wl_menu_ustr_buffer, 256);

	wl_menu[WL_MENU_EMPTY] = t3f_create_gui(0, 0);

	wl_menu[WL_MENU_MAIN] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_MAIN])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_main_game, "GAME", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_MAIN], "Play the Game");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_main_options, "OPTIONS", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_MAIN], "Change Game Options");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_main_credits, "CREDITS", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_MAIN], "See Who Made the Game");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_main_privacy, "PRIVACY", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_MAIN], "See Privacy Information");
		pos++;
		if(wl_demo)
		{
			t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_demo_over_yes, "PURCHASE", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
			t3f_describe_last_gui_element(wl_menu[WL_MENU_MAIN], "Get the Full Version of the Game");
			pos++;
		}
		t3f_add_gui_text_element(wl_menu[WL_MENU_MAIN], wl_menu_proc_main_quit, "QUIT", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_MAIN], WL_TITLE_MENU_TOP, WL_TITLE_MENU_BOTTOM);
	}

	wl_menu[WL_MENU_GAME] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME], wl_menu_proc_game_play, "PLAY", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME], "Play the Game");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME], wl_menu_proc_game_profiles, "PROFILES", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME], "View Profiles");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME], wl_menu_proc_game_leaderboards, "LEADERBOARDS", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME], "View Leaderboards");
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME], wl_menu_proc_game_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_GAME], WL_TITLE_MENU_TOP, WL_TITLE_MENU_BOTTOM);
	}

	wl_menu[WL_MENU_PLAYERS] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PLAYERS])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAYERS], NULL, "PLAYERS", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAYERS], wl_menu_proc_game_players_single_player, "SINGLE PLAYER", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAYERS], wl_menu_proc_game_players_multiplayer, "MULTIPLAYER", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_PLAYERS], wl_menu_proc_game_players_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_PLAYERS], WL_TITLE_MENU_TOP, WL_TITLE_MENU_BOTTOM);
	}

	wl_menu[WL_MENU_MULTIPLAYER] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_MULTIPLAYER])
	{
		pos = 0;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MULTIPLAYER], NULL, "GAME TYPE", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MULTIPLAYER], wl_menu_proc_game_multiplayer_party, "PARTY", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MULTIPLAYER], wl_menu_proc_game_multiplayer_social, "SOCIAL", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos++;
		t3f_add_gui_text_element(wl_menu[WL_MENU_MULTIPLAYER], wl_menu_proc_game_multiplayer_back, "BACK", (void **)&wl_font[WL_FONT_24], 320, 48 * pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_MULTIPLAYER], WL_TITLE_MENU_TOP, WL_TITLE_MENU_BOTTOM);
	}

	wl_create_play_menu();

	wl_menu[WL_MENU_GAME_OVER] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME_OVER])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_OVER], NULL, "", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_OVER], wl_menu_proc_game_over_retry, "RETRY", (void **)&wl_font[WL_FONT_24], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_OVER], wl_menu_proc_game_over_give_up, "GIVE UP", (void **)&wl_font[WL_FONT_24], 320, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_GAME_OVER], 0, 480);
	}

	wl_menu[WL_MENU_DEMO_OVER] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_DEMO_OVER])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_DEMO_OVER], NULL, "", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_DEMO_OVER], wl_menu_proc_demo_over_yes, "GET THE FULL GAME", (void **)&wl_font[WL_FONT_16], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_DEMO_OVER], wl_menu_proc_demo_over_no, "KEEP PLAYING THE DEMO", (void **)&wl_font[WL_FONT_16], 320, 48 * 5 + 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_DEMO_OVER], 0, 480);
	}

	wl_menu[WL_MENU_GAME_PAUSED] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME_PAUSED])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_PAUSED], NULL, "", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_PAUSED], wl_menu_proc_game_over_give_up, "QUIT GAME", (void **)&wl_font[WL_FONT_24], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_PAUSED], "Quit the Game");
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_PAUSED], wl_menu_proc_game_paused_continue, "CONTINUE", (void **)&wl_font[WL_FONT_24], 320, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_PAUSED], "Back to the Game In Progress");
		t3f_center_gui(wl_menu[WL_MENU_GAME_PAUSED], 0, 480);
	}

	wl_menu[WL_MENU_GAME_PAUSED_2] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME_PAUSED_2])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_PAUSED_2], wl_menu_proc_game_paused_quit, "QUIT", (void **)&wl_font[WL_FONT_24], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_PAUSED_2], "Quit the Game");
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_PAUSED_2], wl_menu_proc_game_paused_continue, "CONTINUE", (void **)&wl_font[WL_FONT_24], 320, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_PAUSED_2], "Back to the Game In Progress");
		t3f_center_gui(wl_menu[WL_MENU_GAME_PAUSED_2], 0, 480);
	}

	wl_menu[WL_MENU_TUTORIAL] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_TUTORIAL])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_TUTORIAL], NULL, "SKIP TUTORIAL?", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_TUTORIAL], wl_menu_proc_tutorial_skip_yes, "YES", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_TUTORIAL], "Skip to the Game");
		t3f_add_gui_text_element(wl_menu[WL_MENU_TUTORIAL], wl_menu_proc_tutorial_skip_no, "NO", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_TUTORIAL], "Continue Tutorial");
		t3f_center_gui(wl_menu[WL_MENU_TUTORIAL], 0, 480);
	}

	wl_menu[WL_MENU_GAME_EXIT] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME_EXIT])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_EXIT], NULL, "", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_EXIT], wl_menu_proc_game_paused_exit, "EXIT TO OS", (void **)&wl_font[WL_FONT_24], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_EXIT], "Exit the Game");
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_EXIT], wl_menu_proc_game_paused_continue, "KEEP PLAYING", (void **)&wl_font[WL_FONT_24], 320, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_describe_last_gui_element(wl_menu[WL_MENU_GAME_EXIT], "Back to the Game In Progress");
		t3f_center_gui(wl_menu[WL_MENU_GAME_EXIT], 0, 480);
	}

	wl_menu[WL_MENU_QUIT] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_QUIT])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_QUIT], NULL, "REALLY QUIT?", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_QUIT], wl_menu_proc_main_quit_yes, "YES", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_QUIT], wl_menu_proc_main_quit_no, "NO", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_QUIT], 0, 480);
	}

	wl_menu[WL_MENU_RETRY] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_RETRY])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_RETRY], wl_menu_proc_game_retry, "PLAY AGAIN", (void **)&wl_font[WL_FONT_24], 320, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_RETRY], wl_menu_proc_game_quit, "QUIT", (void **)&wl_font[WL_FONT_24], 320, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_RETRY], 0, 480);
	}

	wl_menu[WL_MENU_PROFILE_DELETE] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILE_DELETE])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_DELETE], NULL, "DELETE PROFILE?", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_DELETE], wl_menu_proc_profile_delete_yes, "YES", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_DELETE], wl_menu_proc_profile_delete_no, "NO", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_PROFILE_DELETE], 0, 480);
	}

	wl_menu[WL_MENU_PROFILE_UNLOCK] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_PROFILE_UNLOCK])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_UNLOCK], NULL, "UNLOCK CONTENT?", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_UNLOCK], wl_menu_proc_profile_unlock_yes, "YES", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_PROFILE_UNLOCK], wl_menu_proc_profile_unlock_no, "NO", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_PROFILE_UNLOCK], 0, 480);
	}

	wl_menu[WL_MENU_GAME_OVERLAY] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_GAME_OVERLAY])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_GAME_OVERLAY], wl_menu_proc_overlay_menu, "\\|", (void **)&wl_font[WL_FONT_24], 20, 20, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
	}

	wl_menu[WL_MENU_UPLOAD_FAILED] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_UPLOAD_FAILED])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_FAILED], NULL, "SCORE UPLOAD FAILED", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_FAILED], wl_menu_proc_upload_retry, "RETRY", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_FAILED], wl_menu_proc_upload_cancel, "CANCEL", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_UPLOAD_FAILED], 0, 480);
	}

	wl_menu[WL_MENU_DOWNLOAD_FAILED] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_DOWNLOAD_FAILED])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_DOWNLOAD_FAILED], NULL, "DOWNLOAD FAILED", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_DOWNLOAD_FAILED], wl_menu_proc_download_retry, "RETRY", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_DOWNLOAD_FAILED], wl_menu_proc_download_cancel, "CANCEL", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_DOWNLOAD_FAILED], 0, 480);
	}

	wl_menu[WL_MENU_MUSIC_OVERLAY] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_MUSIC_OVERLAY])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_MUSIC_OVERLAY], wl_menu_proc_overlay_previous_track, "\\-", (void **)&wl_font[WL_FONT_24], 640 - 16 - 32 * 4, 480 - 16 - 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
		t3f_add_gui_text_element(wl_menu[WL_MENU_MUSIC_OVERLAY], wl_menu_proc_overlay_menu, "\\|", (void **)&wl_font[WL_FONT_24], 640 - 16 - 32 * 3, 480 - 16 - 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
		t3f_add_gui_text_element(wl_menu[WL_MENU_MUSIC_OVERLAY], wl_menu_proc_overlay_fast_forward, "\\>", (void **)&wl_font[WL_FONT_24], 640 - 16 - 32 * 2, 480 - 16 - 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_ON_TOUCH);
		t3f_add_gui_text_element(wl_menu[WL_MENU_MUSIC_OVERLAY], wl_menu_proc_overlay_next_track, "\\+", (void **)&wl_font[WL_FONT_24], 640 - 16 - 32 * 1, 480 - 16 - 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 0);
	}

	wl_menu[WL_MENU_UPLOAD_ASK] = t3f_create_gui(0, 0);
	if(wl_menu[WL_MENU_UPLOAD_ASK])
	{
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_ASK], NULL, "UPLOAD SCORE?", (void **)&wl_font[WL_FONT_24], 320, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_ASK], wl_menu_proc_upload_yes, "YES", (void **)&wl_font[WL_FONT_24], 320, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_add_gui_text_element(wl_menu[WL_MENU_UPLOAD_ASK], wl_menu_proc_upload_no, "NO", (void **)&wl_font[WL_FONT_24], 320, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		t3f_center_gui(wl_menu[WL_MENU_UPLOAD_ASK], 0, 480);
	}

	/* create title words */
	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		wl_generate_title_word(&wl_title_word[i]);
	}

	/* create title tiles */
	for(i = 0; i < strlen(wl_title_text); i++)
	{
		sprintf(fn, "data/graphics/logo_%c.png", wl_title_text[i]);
		wl_title_tile[i].bp = t3f_load_resource((void **)(&wl_title_tile[i].bp), T3F_RESOURCE_TYPE_BITMAP, fn, 0, 0, 0);
		wl_title_tile[i].x = 320 - (48 * 4) + i * 48;
		wl_title_tile[i].y = WL_TITLE_LOGO_Y;
		wl_title_tile[i].vy = -1.25;

		/* move every other tile to the top of it's "jump" */
		if(i % 2 == 0)
		{
			while(wl_title_tile[i].vy < 0.0)
			{
				wl_title_tile_logic(&wl_title_tile[i]);
			}
		}
	}

	/* create atlas */
	wl_title_atlas = t3f_create_atlas(512, 512);
	if(wl_title_atlas)
	{
		for(i = 0; i < strlen(wl_title_text); i++)
		{
			t3f_add_bitmap_to_atlas(wl_title_atlas, &wl_title_tile[i].bp, T3F_ATLAS_SPRITE);
		}
	}
}

void wl_title_exit(void)
{
	int i;

	for(i = 0; i < WL_MAX_MENUS; i++)
	{
		if(wl_menu[i])
		{
			t3f_destroy_gui(wl_menu[i]);
		}
	}
	for(i = 0; i < (int)strlen(wl_title_text); i++)
	{
		t3f_destroy_resource(wl_title_tile[i].bp);
	}
}

void wl_title_word_logic(void)
{
	unsigned int i;

	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		wl_title_word[i].x += wl_title_word[i].vx;
		wl_title_word[i].y += wl_title_word[i].vy;
		if(wl_title_word[i].time > 0)
		{
			wl_title_word[i].a += wl_title_word[i].va;
			if(wl_title_word[i].a > 1.0)
			{
				wl_title_word[i].a = 1.0;
			}
		}
		else
		{
			wl_title_word[i].a -= wl_title_word[i].va;
			if(wl_title_word[i].a <= 0.0)
			{
				wl_generate_title_word(&wl_title_word[i]);
			}
		}
		if(wl_title_word[i].time > 0)
		{
			wl_title_word[i].time--;
		}
	}
}

void wl_title_process_menu(void)
{
	unsigned int i, l;
	int x;
	float s = 32.0;

	/* process the GUI */
	wl_last_item = wl_menu[wl_current_menu]->hover_element;
	t3f_process_gui(wl_menu[wl_current_menu], NULL);
	wl_process_particles(0);

	/* handle keyboard input */
	if(t3f_key[ALLEGRO_KEY_UP])
	{
		t3f_select_previous_gui_element(wl_menu[wl_current_menu]);
		t3f_key[ALLEGRO_KEY_UP] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_DOWN])
	{
		t3f_select_next_gui_element(wl_menu[wl_current_menu]);
		t3f_key[ALLEGRO_KEY_DOWN] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_ENTER] || t3f_key[ALLEGRO_KEY_SPACE])
	{
		t3f_activate_selected_gui_element(wl_menu[wl_current_menu], NULL);
		t3f_key[ALLEGRO_KEY_ENTER] = 0;
		t3f_key[ALLEGRO_KEY_SPACE] = 0;
	}
	if(t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		t3f_select_previous_gui_element(wl_menu[wl_current_menu]);
		t3f_activate_selected_gui_element(wl_menu[wl_current_menu], NULL);
		wl_menu[wl_current_menu]->hover_element = -1;
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
	}

	/* generate particles when changing menu selection */
	if(wl_menu[wl_current_menu]->hover_element >= 0 && wl_last_item != wl_menu[wl_current_menu]->hover_element)
	{
		if(wl_menu[wl_current_menu]->element[wl_menu[wl_current_menu]->hover_element].resource == (void **)&wl_font[WL_FONT_16])
		{
			s = 24.0;
		}
		l = wl_menu_get_element_characters(wl_menu[wl_current_menu]->element[wl_menu[wl_current_menu]->hover_element].data);
		for(i = 0; i < l; i++)
		{
			x = wl_menu[wl_current_menu]->ox + wl_menu[wl_current_menu]->element[wl_menu[wl_current_menu]->hover_element].ox;
			if(wl_menu[wl_current_menu]->element[wl_menu[wl_current_menu]->hover_element].flags & T3F_GUI_ELEMENT_CENTRE)
			{
				x -= (l * s) / 2;
			}
			wl_add_particle(0, WL_PARTICLE_TYPE_FLASH, wl_animation[WL_ANIMATION_PARTICLE], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), x + i * s, wl_menu[wl_current_menu]->oy + wl_menu[wl_current_menu]->element[wl_menu[wl_current_menu]->hover_element].oy, 0.0, s, 10 + i * 2, 0);
		}
	}
}

static bool wl_title_want_input(void)
{
	int i;

	if(wl_mouse_clicked)
	{
		for(i = 1; i < T3F_MAX_TOUCHES; i++)
		{
			if(t3f_touch[i].active)
			{
				return false;
			}
		}
	}
	wl_mouse_clicked = false;
	return true;
}

void wl_title_logic(void)
{
	ALLEGRO_EVENT event;
	unsigned int i;
	bool check_profile = false;
	char fnbuf[1024] = {0};

	wl_enter_text();
	if(wl_title_want_input())
	{
		wl_title_process_menu();
	}

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		wl_title_tile_logic(&wl_title_tile[i]);
	}
	wl_title_word_logic();

	/* check for leaderboard events */
	while(!al_event_queue_is_empty(wl_async_queue))
	{
		al_wait_for_event(wl_async_queue, &event);
		if(event.type == ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 's'))
		{
			wl_leaderboard_thread_counter--;
			/* if we are uploading manually, keep going until we get all game modes done */
			if(!wl_menu_upload_helper())
			{
				wl_select_menu(WL_MENU_PROFILE);
			}
		}
		else if(event.type == ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 'f'))
		{
			wl_leaderboard_thread_counter--;
			wl_select_menu(WL_MENU_UPLOAD_FAILED);
		}
		else if(event.type == ALLEGRO_GET_EVENT_TYPE('t', 'r', 'u', 'p'))
		{
			wl_leaderboard_thread_counter--;
			wl_select_menu(WL_MENU_UPLOAD_FAILED);
		}
		else if(event.type == ALLEGRO_GET_EVENT_TYPE('t', 'r', 'd', 's'))
		{
			wl_leaderboard_thread_counter--;
			if(wl_leaderboard_thread_counter == 0)
			{
				wl_create_leaderboard_menu(wl_leaderboard_mode, wl_leaderboard_type);
				wl_select_menu(WL_MENU_LEADERBOARD);
			}
		}
		else if(event.type == ALLEGRO_GET_EVENT_TYPE('t', 'r', 'd', 'f'))
		{
			wl_leaderboard_thread_counter--;
			wl_select_menu(WL_MENU_DOWNLOAD_FAILED);
		}

		/* if any events were ever in the queue, see if we are uploading (wl_leaderboard_mode == -1 is upload indicator) */
		if(wl_leaderboard_mode < 0)
		{
			check_profile = true; // say that we want to compare profile flags to upload flags to see if uploads succeeded
		}
	}

	/* save profiles if the flags have changed due to successful score uploads */
	if(check_profile && wl_current_menu != WL_MENU_EMPTY)
	{
		if(wl_profiles->entry[wl_selected_profile].flags != wl_leaderboard_profile_upload_flags)
		{
			wl_save_profiles(wl_profiles, t3f_get_filename(t3f_data_path, "profiles.dat", fnbuf, 1024));
		}
	}
}

void wl_title_word_render(ALLEGRO_FONT * fp, float strength)
{
	int i;

	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		al_draw_text(fp, al_map_rgba_f(wl_title_word[i].a * strength, wl_title_word[i].a * strength, wl_title_word[i].a * strength, wl_title_word[i].a * strength), wl_title_word[i].x, wl_title_word[i].y, 0, wl_dictionary->word[wl_title_word[i].word].data);
	}
}

void wl_title_render_menu(void)
{
	int h;

	t3f_render_gui(wl_menu[wl_current_menu]);
	wl_render_particles(0, WL_PARTICLE_TYPE_FLASH);
	h = wl_menu[wl_current_menu]->hover_element;
	if(h >= 0 && wl_menu[wl_current_menu]->element[h].description)
	{
		wl_draw_text(wl_font[WL_FONT_24], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 320 + 3, 480 - 32, 3, 3, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_SHADOW, wl_menu[wl_current_menu]->element[h].description);
	}
}

void wl_title_render(void)
{
	unsigned int i, l;
	float min_y = -1.0, max_y = -1.0;

	/* clear entire screen while on options menu in case there are
	 * artifacts from video mode switching */
	if(wl_current_menu == WL_MENU_OPTIONS)
	{
		al_set_clipping_rectangle(0, 0, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
		al_clear_to_color(al_map_rgb_f(0.0, 0.0, 0.0));
		t3f_select_view(t3f_default_view);
	}

	al_clear_to_color(WL_TITLE_BG_COLOR);

	al_hold_bitmap_drawing(true);
	wl_title_word_render(wl_font[WL_FONT_16], 0.125);
	al_hold_bitmap_drawing(false);
	if(wl_current_menu == WL_MENU_MAIN || wl_current_menu == WL_MENU_GAME || wl_current_menu == WL_MENU_PLAYERS || wl_current_menu == WL_MENU_MULTIPLAYER)
	{
		for(i = 0; i < strlen(wl_title_text); i++)
		{
			if(min_y < 0.0 || wl_title_tile[i].y < min_y)
			{
				min_y = wl_title_tile[i].y;
			}
			if(max_y < 0.0 || wl_title_tile[i].y > max_y)
			{
				max_y = wl_title_tile[i].y;
			}
		}
		l = strlen(wl_title_text);
//		al_draw_filled_rectangle(wl_title_tile[0].x - 32, min_y - 32, wl_title_tile[l - 1].x + 48 + 32, max_y + 48 + 32, al_map_rgba_f(0.05, 0.05, 0.05, 0.05));
//		al_draw_filled_rectangle(wl_title_tile[0].x - 16, min_y - 16, wl_title_tile[l - 1].x + 48 + 16, max_y + 48 + 16, al_map_rgba_f(0.05, 0.05, 0.05, 0.05));
		al_hold_bitmap_drawing(true);
		for(i = 0; i < l; i++)
		{
			t3f_draw_bitmap(wl_title_tile[i].bp, al_map_rgba_f(0.0, 0.0, 0.0, 0.25), wl_title_tile[i].x, wl_title_tile[i].y, 32.0, 0);
		}
		for(i = 0; i < l; i++)
		{
			al_draw_bitmap(wl_title_tile[i].bp, wl_title_tile[i].x, wl_title_tile[i].y, 0);
		}
		wl_draw_text(wl_font[WL_FONT_16], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.75), 320 + 3, WL_TITLE_COPYRIGHT_Y, 2, 2, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_OUTLINE | WL_TEXT_FLAG_SHADOW, wl_copyright_message);
	}
	wl_title_render_menu();
	al_hold_bitmap_drawing(false);
}

void wl_title_exit_logic(void)
{
	unsigned int i;

	wl_process_particles(0);

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		wl_title_tile_logic(&wl_title_tile[i]);
	}
	wl_title_word_logic();
	if(wl_fade_tick <= 0)
	{
		t3f_exit();
	}
}

void wl_title_exit_render(void)
{
	wl_title_render();
}

void wl_title_to_game_logic(void)
{
	unsigned int i;

	wl_process_particles(0);

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		wl_title_tile_logic(&wl_title_tile[i]);
	}
	wl_title_word_logic();
	if(wl_fade_tick <= 0)
	{
		if(wl_game.mode == WL_GAME_MODE_MUSIC)
		{
			wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
			wl_reset_particles(0);
			wl_game.retry = false;
			wl_game.regenerate = false;
			wl_game_initialize(WL_LEVEL_LOAD_NORMAL);
		}
		else
		{
			if(wl_game.player[0].start_level > 0)
			{
				wl_fade_to(al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
				wl_reset_particles(0);
				wl_game.retry = false;
				wl_game.regenerate = false;
				wl_game_initialize(WL_LEVEL_LOAD_NORMAL);
				if(wl_game.mode == WL_GAME_MODE_STORY)
				{
					wl_game_scroll_level_name();
				}
			}
			else
			{
				wl_stop_fade();
				wl_reset_particles(0);
				wl_cinema = wl_load_cinema_script("data/scripts/intro.script");
				if(wl_cinema)
				{
					wl_set_cinema_view_callbacks(wl_cinema, wl_game_view_logic, wl_game_view_render);
					wl_game.player[0].start_level = 1;
					wl_game.retry = false;
					wl_game.regenerate = false;
					wl_game_initialize(WL_LEVEL_LOAD_INTRO);
					wl_state = WL_STATE_GAME_INTRO;
				}
			}
		}
	}
}

void wl_title_to_game_render(void)
{
	wl_title_render();
}

void wl_title_credits_logic(void)
{
	unsigned int i;
	bool touched = false;

	for(i = 0; i < T3F_MAX_TOUCHES; i++)
	{
		if(t3f_touch[i].active)
		{
			touched = true;
			break;
		}
	}

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		wl_title_tile_logic(&wl_title_tile[i]);
	}
	wl_title_word_logic();
	wl_credits_logic(&wl_credits);
	if(touched && !wl_mouse_clicked)
	{
		for(i = 0; i < 15; i++)
		{
			wl_credits_logic(&wl_credits);
		}
	}
	if(!touched)
	{
		wl_mouse_clicked = false;
	}
	if(wl_credits.done || t3f_key[ALLEGRO_KEY_ESCAPE] || t3f_key[ALLEGRO_KEY_BACK])
	{
		wl_state = WL_STATE_TITLE;
		wl_mouse_clicked = true;
		t3f_key[ALLEGRO_KEY_ESCAPE] = 0;
		t3f_key[ALLEGRO_KEY_BACK] = 0;
	}
}

void wl_title_credits_render(void)
{
	al_clear_to_color(WL_TITLE_BG_COLOR);
	wl_title_word_render(wl_font[WL_FONT_16], 0.125);
	wl_credits_render(&wl_credits);
}
