#include "t3f/t3f.h"
#include "t3f/resource.h"
#include "defines.h"
#include "data.h"

bool wl_video_mode_okay(int mode)
{
	ALLEGRO_MONITOR_INFO info;

	al_get_monitor_info(0, &info);
	switch(mode)
	{
		case WL_VIDEO_MODE_MEDIUM_WINDOW:
		{
			if(info.x2 - info.x1 > 960 && info.y2 - info.y1 > 720)
			{
				return true;
			}
			return false;
		}
		case WL_VIDEO_MODE_LARGE_WINDOW:
		{
			if(info.x2 - info.x1 > 1280 && info.y2 - info.y1 > 960)
			{
				return true;
			}
			return false;
		}
		default:
		{
			return true;
		}
	}
}

bool wl_set_video_mode(int w, int h, int flags)
{
	ALLEGRO_STATE old_state;

	/* must recreate display if we get error code 2 */
	if(t3f_set_gfx_mode(w, h, flags) == 2)
	{
		al_destroy_bitmap(wl_bitmap[WL_BITMAP_SCRATCH]);
		t3f_unload_resources();
		al_unregister_event_source(t3f_queue, al_get_display_event_source(t3f_display));
		al_destroy_display(t3f_display);
		t3f_display = NULL;
		t3f_set_gfx_mode(640, 480, flags);
		al_register_event_source(t3f_queue, al_get_display_event_source(t3f_display));
		t3f_reload_resources();
		al_store_state(&old_state, ALLEGRO_STATE_NEW_BITMAP_PARAMETERS);
		al_set_new_bitmap_flags(ALLEGRO_MIN_LINEAR | ALLEGRO_MAG_LINEAR | ALLEGRO_NO_PRESERVE_TEXTURE);
		wl_bitmap[WL_BITMAP_SCRATCH] = al_create_bitmap(512, 64);
		al_restore_state(&old_state);
		t3f_default_view->need_update = true;
		t3f_select_view(t3f_default_view);
	}
	return true;
}

void wl_initialize_video_mode(void)
{
	ALLEGRO_MONITOR_INFO info;
	int w, h;

	/* make sure we are using the configured display setting */
	switch(wl_option[WL_OPTION_VIDEO_MODE])
	{
		case WL_VIDEO_MODE_SMALL_WINDOW:
		{
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			al_set_config_value(t3f_config, "T3F", "force_resizable", "false");
			al_set_config_value(t3f_config, "T3F", "display_width", "640");
			al_set_config_value(t3f_config, "T3F", "display_height", "480");
			wl_set_video_mode(640, 480, T3F_FORCE_ASPECT);
			break;
		}
		case WL_VIDEO_MODE_MEDIUM_WINDOW:
		{
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			al_set_config_value(t3f_config, "T3F", "force_resizable", "false");
			al_set_config_value(t3f_config, "T3F", "display_width", "960");
			al_set_config_value(t3f_config, "T3F", "display_height", "720");
			wl_set_video_mode(960, 720, T3F_FORCE_ASPECT);
			break;
		}
		case WL_VIDEO_MODE_LARGE_WINDOW:
		{
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			al_set_config_value(t3f_config, "T3F", "force_resizable", "false");
			al_set_config_value(t3f_config, "T3F", "display_width", "1280");
			al_set_config_value(t3f_config, "T3F", "display_height", "960");
			wl_set_video_mode(1280, 960, T3F_FORCE_ASPECT);
			break;
		}
		case WL_VIDEO_MODE_RESIZEABLE_WINDOW:
		{
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "false");
			al_set_config_value(t3f_config, "T3F", "force_resizable", "true");
			if(t3f_flags & T3F_USE_FULLSCREEN)
			{
				w = 640;
				h = 480;
			}
			else
			{
				w = al_get_display_width(t3f_display);
				h = al_get_display_height(t3f_display);
			}
			wl_set_video_mode(w, h, T3F_FORCE_ASPECT | T3F_RESIZABLE);
			break;
		}
		case WL_VIDEO_MODE_FULL_SCREEN:
		{
			al_get_monitor_info(0, &info);
			al_set_config_value(t3f_config, "T3F", "force_fullscreen", "true");
			al_set_config_value(t3f_config, "T3F", "force_resizable", "false");
			wl_set_video_mode(info.x2 - info.x1, info.y2 - info.y1, T3F_FORCE_ASPECT | T3F_USE_FULLSCREEN);
			break;
		}
	}
	t3f_select_view(t3f_default_view);
}
