#include "t3f/t3f.h"
#include "t3f/music.h"
#include "t3f/sound.h"
#include "t3net/t3net.h"
#include "modules/data.h"
#include "modules/dictionary.h"
#include "modules/particle.h"
#include "modules/effects.h"
#include "modules/achievements.h"
#include "modules/leaderboards.h"
#include "modules/text.h"
#include "modules/music_queue.h"
#include "ui/menu_leaderboards.h"
#include "ui/menu_overlay.h"
#include "ui/menu.h"
#include "instance.h"
#include "game_intro.h"
#include "player.h"
#include "board.h"
#include "game_sprite.h"
#include "game_hazard.h"
#include "game_tally.h"
#include "game_level.h"
#include "game.h"
#include "game_music.h"

void wl_game_generate_powerups(WL_GAME * game)
{
	game->powerups = 0;
	if(game->powerup_guru)
	{
		game->powerup[game->powerups] = WL_POWERUP_GURU;
		game->powerups++;
	}
	if(game->powerup_sneaker)
	{
		game->powerup[game->powerups] = WL_POWERUP_HOURGLASS;
		game->powerups++;
		game->powerup[game->powerups] = WL_POWERUP_HOURGLASS;
		game->powerups++;
		game->powerup[game->powerups] = WL_POWERUP_HOURGLASS;
		game->powerups++;
	}
	if(game->powerup_health)
	{
		game->powerup[game->powerups] = WL_POWERUP_LIFE;
		game->powerups++;
		game->powerup[game->powerups] = WL_POWERUP_LIFE;
		game->powerups++;
		game->powerup[game->powerups] = WL_POWERUP_LIFE;
		game->powerups++;
		game->powerup[game->powerups] = WL_POWERUP_LIFE;
		game->powerups++;
	}
}

int wl_game_figure_base_points(const char * word)
{
	unsigned int i;
	int m = -2;
	int points = 0;

	for(i = 0; i < strlen(word); i++)
	{
		if(wl_letter_probability[word[i] - 'A'] <= 0.2)
		{
			points += 5;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 2.0)
		{
			points += 4;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 4.0)
		{
			points += 3;
		}
		else if(wl_letter_probability[word[i] - 'A'] <= 7.0)
		{
			points += 2;
		}
		else
		{
			points += 1;
		}
		m++;
	}
	points *= m;
	points *= 10;
	return points;
}

int wl_game_figure_total_points(WL_DICTIONARY * dictionary, int w)
{
	int s, m;

	s = wl_game_figure_base_points(dictionary->word[w].data);
	if(dictionary->word[w].rarity <= 0)
	{
		m = 5;
	}
	else if(dictionary->word[w].rarity < 5)
	{
		m = 3;
	}
	else if(dictionary->word[w].rarity < 9)
	{
		m = 2;
	}
	else
	{
		m = 1;
	}
	return s * m;
}

void wl_game_setup_type(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	instance->game.uses_powerups = false;
	instance->game.uses_hazards = false;
	instance->game.uses_life = false;
	instance->game.uses_level_progress = false;
	instance->game.uses_invalid = false;
	switch(instance->game.mode)
	{
		case WL_GAME_MODE_CASUAL:
		{
			instance->game.base_range = 3;
			break;
		}
		case WL_GAME_MODE_PUZZLE:
		{
			instance->game.base_range = 3;
			break;
		}
		case WL_GAME_MODE_GURU:
		{
			instance->game.base_range = WL_MAX_BOARD_WIDTH;
			instance->game.uses_life = true;
			break;
		}
		case WL_GAME_MODE_PARTY:
		{
			break;
		}
		case WL_GAME_MODE_MUSIC:
		{
			instance->game.music_pos = 0.0;
			instance->game.old_music_pos = 0.0;
			instance->game.music_length = 0.0;
			instance->game.fast_forward = false;
			instance->game.music_fade_level = 0.0;
			instance->game.music_done = false;
			break;
		}
		default:
		{
			instance->game.uses_powerups = true;
			instance->game.uses_hazards = true;
			instance->game.uses_life = true;
			instance->game.uses_level_progress = true;
			instance->game.uses_invalid = true;
			instance->game.hazard_count = instance->level_list->entry[instance->game.player[0].level].hazard_delay * 60 + rand() % 300 - 150;
			i = instance->level_list->entry[instance->game.player[0].level].story;
			instance->game.base_range = instance->level_list->story[i].base_range;
			instance->game.powerup_health = instance->level_list->story[i].powerup_health;
			instance->game.powerup_guru = instance->level_list->story[i].powerup_guru;
			instance->game.powerup_sneaker = instance->level_list->story[i].powerup_sneaker;
			break;
		}
	}
}

bool wl_game_initialize(void * data, int flags)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, j;

	t3f_srand(&instance->game.rng_state, time(0));
	srand(time(0));
	wl_reset_particle_list(instance->particle_list[0]);
	wl_setup_board_tiles(data);

	/* when starting a brand new game, regenerate entire contents of game board */
	if(!(flags & WL_GAME_INIT_FLAG_NO_BOARD_GENERATION))
	{
		instance->game.wipe_count = -1;
		for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
		{
			for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
			{
				instance->game.board[i][j].letter = wl_board_generate_letter(false, data);
				instance->game.board[i][j].type = 0;
				instance->game.board[i][j].flags = 0;
			}
		}
	}
	/* begin wiping the current board at continue/retry */
	else if(instance->game.regenerate)
	{
		instance->game.wipe_count = 0;
//		wl_board_regenerate();
		for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
		{
			for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
			{
				instance->game.board[i][j].flags = 0;
				instance->game.board[i][j].type = 0;
			}
		}
	}
	instance->game.board[WL_BOARD_CENTER_Y][WL_BOARD_CENTER_X].letter = ' ';
	instance->game.player[0].level = instance->game.player[0].start_level;
	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_update_hazard_chart(data);
		if(instance->game.player[0].start_level > instance->level_list->story[instance->game.player[0].story].end_level)
		{
			instance->game.player[0].level = instance->level_list->story[instance->game.player[0].story].end_level;
		}
		instance->game.player[0].progress = (instance->game.player[0].start_level - instance->level_list->story[instance->game.player[0].story].start_level) * WL_LEVEL_UP;
	}
	else
	{
		instance->game.player[0].progress = 0;
	}
	wl_game_setup_type(data);
	wl_game_generate_powerups(&instance->game);
	instance->game.powerup_count = WL_POWERUP_COUNT;
	instance->game.player[0].made_progress = false;
	instance->game.player[0].score = 0;
	instance->game.player[0].word_score = 0;
	instance->game.player[0].life = WL_MAX_LIFE;
	instance->game.player[0].lost = false;
	instance->game.player[0].life_fade = 0.0;
	instance->game.player[0].words = 0;
	instance->game.player[0].word_list_count = 0;
	instance->game.player[0].top_word_count = 0;
	instance->game.player[0].word[0] = '\0';
	instance->game.player[0].word_pos = 0;
	instance->game.player[0].word_exists = false;
	instance->game.player[0].x = WL_BOARD_CENTER_X;
	instance->game.player[0].y = WL_BOARD_CENTER_Y;
	instance->game.player[0].sprite.x = instance->game.player[0].x * WL_BOARD_TILE_SIZE;
	instance->game.player[0].sprite.y = instance->game.player[0].y * WL_BOARD_TILE_SIZE;
	instance->game.player[0].sprite.z = -WL_VIRTUAL_DISPLAY_WIDTH / 2;
	instance->game.player[0].sprite.vz = WL_TERMINAL_VELOCITY;
	instance->game.player[0].hover_x = WL_BOARD_CENTER_X;
	instance->game.player[0].hover_y = WL_BOARD_CENTER_Y;
	instance->game.player[0].range = 0;
	instance->game.player[0].powerups = 0;
	instance->game.player[0].words_this_level = 0;
	instance->game.player[0].counter = 0;
	instance->game.player[0].state = WL_PLAYER_STATE_ENTER;
	instance->game.player[0].sprite.type = WL_SPRITE_TYPE_PLAYER;
	instance->game.player[0].sprite.state = 0;
	instance->game.player[0].sprite.active = true;
	instance->game.player[0].click_queue.clicks = 0;
	instance->game.player[0].sprite.collider = t3f_create_collision_object(8, 8, 32 - 8, 32 - 8, 32, 32, 0);
	instance->game.player[0].sprite.cy = 24 + 1;
	instance->game.player[0].move[0] = wl_game_player_pos_to_move(instance->game.player[0].x, instance->game.player[0].y);
	instance->game.player[0].moves = 1;

	/* remember we already did the tutorial when continuing, retrying, etc. */
	if(flags & WL_GAME_INIT_FLAG_NO_LEVEL_LOAD)
	{
		instance->game.player[0].watched_tutorial = false;
	}

	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		instance->game.sprite[i].collider = t3f_create_collision_object(0, 0, 32, 32, 32, 32, 0);
		instance->game.sprite[i].active = false;
	}
	instance->game.guru_active = false;
	instance->game.guru_dir = -1.0;

	/* add play to profile */
	instance->profile->games++;

	/* expand profile word list so we can add the words from this game */
	if(!wl_expand_profile(instance->profile, 1000))
	{
		return false;
	}

	al_stop_timer(t3f_timer);
	if(!(flags & WL_GAME_INIT_FLAG_NO_LEVEL_LOAD))
	{
		if(instance->level[0])
		{
			wl_destroy_level(instance->level[0]);
			instance->level[0] = NULL;
		}
		if(instance->level[1])
		{
			wl_destroy_level(instance->level[1]);
			instance->level[1] = NULL;
		}
		instance->level[0] = wl_load_level(instance->game.player[0].level, 0, data);
		if(!instance->level[0])
		{
			al_start_timer(t3f_timer);
			return false;
		}
	}

	/* generate initial background shapes */
	if(!(flags & WL_GAME_INIT_FLAG_NO_LEVEL_LOAD))
	{
		for(i = 0; i < instance->shape_lists->shapes; i++)
		{
			instance->shape_lists->bg_shape[i].flags = 0;
			instance->shape_lists->fg_shape[i].flags = 0;
		}
		instance->current_bg_shape = 0;
		wl_generate_shape(&instance->shape_lists->bg_shape[0], &instance->level[0]->shape[rand() % instance->level[0]->shapes], &instance->level[0]->shape_image->bitmap, WL_VIRTUAL_DISPLAY_WIDTH, 0, 128.0, -1.0 / 4.0);
		while(instance->shape_lists->bg_shape[instance->current_bg_shape].ox > -64.0)
		{
			instance->current_bg_shape++;
			wl_connect_shape(&instance->shape_lists->bg_shape[instance->current_bg_shape], &instance->shape_lists->bg_shape[instance->current_bg_shape - 1], &instance->level[0]->shape[rand() % instance->level[0]->shapes], &instance->level[0]->shape_image->bitmap);
		}
		instance->current_bg_shape++;
		instance->current_fg_shape = 0;
		wl_generate_shape(&instance->shape_lists->fg_shape[0], &instance->level[0]->shape[rand() % instance->level[0]->shapes], &instance->level[0]->shape_image->bitmap, WL_VIRTUAL_DISPLAY_WIDTH, 40, 0.0, -1.0 / 4.0);
		while(instance->shape_lists->fg_shape[instance->current_fg_shape].ox > -64.0)
		{
			instance->current_fg_shape++;
			wl_connect_shape(&instance->shape_lists->fg_shape[instance->current_fg_shape], &instance->shape_lists->fg_shape[instance->current_fg_shape - 1], &instance->level[0]->shape[rand() % instance->level[0]->shapes], &instance->level[0]->shape_image->bitmap);
		}
		instance->current_fg_shape++;
		instance->level[0]->shapes_used = true;
	}

	instance->word_messages.messages = 0;
	instance->game_messages.messages = 0;

	/* set up HUD */
	instance->game.player[0].progress_indicator = instance->game.player[0].progress;
	instance->game.player[0].life_indicator = 0.0;

	/* set state to start the game */
	instance->game.retry = false;
	instance->game.regenerate = false;
	instance->game.finished = false;
	instance->game.tick = 0;
	instance->game.want_new_level = 0;
	instance->game.word_list_ox = WL_EDGE_RIGHT;
	instance->game.current_player = 0;
	if(instance->game.mode == WL_GAME_MODE_PARTY)
	{
		wl_add_message(&instance->game_messages, WL_MESSAGE_TYPE_SCROLLER, instance->profile->name, instance->font[WL_FONT_32_OUTLINE], 0, data);
		instance->game.state = WL_GAME_STATE_PLAYER_WAIT;
	}
	else
	{
		instance->game.state = WL_GAME_STATE_PLAY;
	}
	if(!flags)
	{
		if(instance->game.mode == WL_GAME_MODE_MUSIC)
		{
			instance->state = WL_STATE_MUSIC;
		}
		else
		{
			instance->state = WL_STATE_GAME;
		}
	}
	if(instance->game.mode >= WL_GAME_MODE_STORY && instance->game.player[0].start_level > instance->level_list->story[instance->game.player[0].story].end_level)
	{
		instance->game.finished = true;
		wl_game_end(data);
	}
	else
	{
		if(!(flags & WL_GAME_INIT_FLAG_NO_MUSIC))
		{
			wl_play_level_music(instance->level[0], data);
		}
	}
	al_start_timer(t3f_timer);
	return true;
}

void wl_game_scroll_level_name(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_add_message(&instance->game_messages, WL_MESSAGE_TYPE_SCROLLER, instance->level[0]->name, instance->font[WL_FONT_32_OUTLINE], 0, data);
}

void wl_game_end_story(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;
	bool exploded = false;
	int bx, by;

	instance->game.board[instance->game.player[0].y][instance->game.player[0].x].letter = ' ';
	instance->game.player[0].state = WL_PLAYER_STATE_EXIT;
	if(instance->game.guru_active)
	{
		instance->game.guru_state = 3;
		instance->game.player[0].powerups &= ~WL_POWERUP_GURU;
		instance->game.player[0].counter = 0;
	}

	_wl_update_high_scores(data);
	wl_game_update_best_flags(data);

	/* destroy sprites */
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		if(instance->game.sprite[i].active)
		{
			if(!exploded)
			{
				t3f_play_sample(instance->sample[WL_SAMPLE_DESTROY], 1.0, 0.0, 1.0);
				exploded = true;
			}

			/* remove tile from beneath sprites that sit on the board */
			if(instance->game.sprite[i].type != WL_SPRITE_TYPE_LASER)
			{
				bx = (int)instance->game.sprite[i].x / WL_BOARD_TILE_SIZE;
				by = (int)instance->game.sprite[i].y / WL_BOARD_TILE_SIZE;
				switch(instance->game.board[by][bx].letter)
				{
					case '?':
					case '!':
					case '_':
					case '0':
					{
						instance->game.board[by][bx].letter = ' ';
						break;
					}
				}
			}
			wl_explode_sprite(&instance->game.sprite[i], data);
		}
	}
	if(instance->game.finished)
	{
		al_stop_timer(t3f_timer);
		t3f_render(true);
		if(instance->demo)
		{
			instance->cinema = wl_load_cinema_script("data/scripts/story_main/demo_ending.script", instance->graphics_scale);
			if(instance->cinema)
			{
				wl_set_cinema_view_callbacks(instance->cinema, wl_ending_view_logic, wl_ending_view_render);
				instance->ending_state = 0;
				wl_select_menu(WL_MENU_DEMO_OVER, data);
				instance->state = WL_STATE_DEMO_WIN;
			}
			else
			{
				wl_title_load_data(data, true);
				wl_select_menu(WL_MENU_MAIN, data);
				instance->state = WL_STATE_TITLE;
			}
		}
		else
		{
			instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].ending_fn, instance->graphics_scale);
			if(instance->cinema)
			{
				wl_set_cinema_view_callbacks(instance->cinema, wl_ending_view_logic, wl_ending_view_render);
				wl_credits_initialize(&instance->credits, 1, data);
				instance->ending_state = 0;
				instance->state = WL_STATE_GAME_WIN;
			}
			else
			{
				wl_title_load_data(data, true);
				wl_select_menu(WL_MENU_MAIN, data);
				instance->state = WL_STATE_TITLE;
			}
		}
		al_start_timer(t3f_timer);
	}
	else
	{
		al_stop_timer(t3f_timer);
		t3f_render(true);
		instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].game_over_fn, instance->graphics_scale);
		if(instance->cinema)
		{
			wl_set_cinema_view_callbacks(instance->cinema, wl_game_view_logic, wl_game_view_render);
		}
		al_start_timer(t3f_timer);
		if(instance->option[WL_OPTION_UPLOAD_SCORES] && instance->game.player[0].word_list_count > 0)
		{
			instance->state = WL_STATE_GAME_OVER_WAIT;
			instance->leaderboard_mode = instance->game.mode;
			instance->leaderboard_type = 0;
			instance->leaderboard_style = 1;
			_wl_unload_leaderboard(instance);
			wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
			wl_download_scores(data);
			wl_select_menu(WL_MENU_DYNAMIC, data);
		}
		else
		{
			instance->state = WL_STATE_GAME_OVER;
			wl_select_menu(WL_MENU_GAME_OVER, data);
		}
	}
}

void wl_game_end(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_game_end_story(data);
	}
	else
	{
		wl_game_tally_score(data);
		instance->state = WL_STATE_GAME_TALLY;
		if(instance->game.mode == WL_GAME_MODE_PUZZLE)
		{
			instance->tally_tick = 60;
		}
		else
		{
			instance->tally_tick = 0;
		}
		instance->game.player[0].hover_x = -1;
		instance->game.player[0].hover_y = -1;
	}
}

void wl_game_update_best_flags(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, c, ci = -1, best = 0;

	/* find the best word in the word list from the game that just ended */
	for(i = 0; i < instance->game.player[0].word_list_count; i++)
	{
		c = wl_game_figure_total_points(instance->dictionary, instance->game.player[0].word_list[i]);
		if(c > best)
		{
			ci = i;
			best = c;
		}
	}
	instance->game.current_best_word = -1;
	if(ci >= 0)
	{
		instance->game.current_best_word = instance->game.player[0].word_list[ci];

		/* compare current best word to profile best word to see if we beat our best */
		if(instance->profile->best_word[instance->game.mode] >= 0)
		{
			if(wl_game_figure_total_points(instance->dictionary, instance->game.player[0].word_list[ci]) > wl_game_figure_total_points(instance->dictionary, instance->profile->best_word[instance->game.mode]))
			{
				instance->profile->best_word[instance->game.mode] = instance->game.player[0].word_list[ci];
				instance->profile->best_word_upload[instance->game.mode] = 1;
			}
		}
		else // we don't have a best word yet so use the best from this game */
		{
			instance->profile->best_word[instance->game.mode] = instance->game.player[0].word_list[ci];
			instance->profile->best_word_upload[instance->game.mode] = 1;
		}
	}
}

void wl_game_exit(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char fnbuf[1024] = {0};
	int i;

	wl_save_profile(instance->profile, t3f_get_filename(t3f_data_path, "profile.dat", fnbuf, 1024));
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		t3f_destroy_collision_object(instance->game.sprite[i].collider);
	}
	for(i = 0; i < 2; i++)
	{
		wl_destroy_level(instance->level[i]);
		instance->level[i] = NULL;
	}
	wl_reset_particle_list(instance->particle_list[0]);
	wl_title_load_data(data, true);
	instance->state = WL_STATE_TITLE;
	wl_select_menu(WL_MENU_MAIN, data);
}

int wl_get_best_word(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, amount;
	int best_amount = 0;
	int best_i = -1;

	if(instance->game.player[0].word_list_count <= 0)
	{
		return -1;
	}
	for(i = 0; i < instance->game.player[0].word_list_count; i++)
	{
		amount = wl_game_figure_total_points(instance->dictionary, instance->game.player[0].word_list[i]);
		if(amount > best_amount)
		{
			best_i = i;
			best_amount = amount;
		}
	}
	return instance->game.player[0].word_list[best_i];
}

void wl_game_guru_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	float a;

	if(instance->game.guru_active)
	{
		switch(instance->game.guru_state)
		{
			case 0:
			{
				break;
			}
			case 1:
			{
				break;
			}
			case 2:
			{
				if(instance->game.guru_timer < 30)
				{
					instance->game.guru_timer++;
					a = (float)instance->game.guru_timer / 240.0;
					instance->game.guru_color = al_map_rgba_f(a, a, a, a);
				}
				break;
			}
			case 3:
			{
				instance->game.guru_timer--;
				a = (float)instance->game.guru_timer / 240.0;
				instance->game.guru_color = al_map_rgba_f(a, a, a, a);
				if(instance->game.guru_timer <= 0)
				{
					instance->game.guru_active = false;
				}
				break;
			}
		}
	}

	/* make guru move up and down */
	instance->game.guru_z += instance->game.guru_vz;
	instance->game.guru_vz += 0.01 * instance->game.guru_dir;
	if(instance->game.guru_dir > 0.0 && instance->game.guru_vz >= 0.25)
	{
		instance->game.guru_dir = -instance->game.guru_dir;
	}
	else if(instance->game.guru_dir < 0.0 && instance->game.guru_vz <= -0.25)
	{
		instance->game.guru_dir = -instance->game.guru_dir;
	}
}

void wl_game_process_overlay_menu(T3F_GUI * mp, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->last_item = mp->hover_element;
	if(instance->input_type == WL_INPUT_TYPE_MOUSE)
	{
		t3f_select_hover_gui_element(mp, t3f_get_mouse_x(), t3f_get_mouse_y());
	}
	else if(instance->input_type == WL_INPUT_TYPE_TOUCH)
	{
		t3f_select_hover_gui_element(mp, -1, -1);
	}
	if(instance->game.mode == WL_GAME_MODE_MUSIC)
	{
		if(instance->input_state.up.pressed || instance->input_state.left.pressed)
		{
			t3f_select_previous_gui_element(mp);
			if(mp->hover_element != instance->last_item)
			{
				t3f_play_sample(instance->sample[WL_SAMPLE_MENU_MOVE], 0.75, 0.0, 1.0);
			}
		}
		else if(instance->input_state.down.pressed || instance->input_state.right.pressed)
		{
			t3f_select_next_gui_element(mp);
			if(mp->hover_element != instance->last_item)
			{
				t3f_play_sample(instance->sample[WL_SAMPLE_MENU_MOVE], 0.75, 0.0, 1.0);
			}
		}
	}
	if(instance->input_state.action.pressed)
	{
		if(instance->input_type == WL_INPUT_TYPE_TOUCH)
		{
			t3f_select_hover_gui_element(mp, t3f_get_touch_x(1), t3f_get_touch_y(1));
		}
		if(mp->hover_element >= 0)
		{
			t3f_play_sample(instance->sample[WL_SAMPLE_MENU_TAP], 1.0, 0.0, 1.0);
		}
		t3f_activate_selected_gui_element(mp, data);
	}
}

void wl_game_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, x, y;

	/* In story mode, preload the next level as soon as we have only a single
	   level's assets active. */
	if(instance->game.mode >= WL_GAME_MODE_STORY && !instance->level[1] && instance->game.player[0].level < instance->level_list->story[instance->game.player[0].story].end_level)
	{
		instance->level[1] = wl_load_level(instance->game.player[0].level + 1, WL_LEVEL_LOAD_FLAG_THREADED, data);
	}

	if(instance->game.wipe_count >= 0)
	{
		instance->game.wipe_count++;
		if(instance->game.wipe_count >= 2 * 15)
		{
			instance->game.wipe_count = -1;
		}
		else if(instance->game.wipe_count % 2 == 0)
		{
			i = instance->game.wipe_count / 2 - 1;
			y = i;
			for(x = 0; x < i + 1; x++)
			{
				if(x < WL_MAX_BOARD_WIDTH && y < WL_MAX_BOARD_HEIGHT && !(x == 3 && y == 3))
				{
					instance->game.board[y][x].letter = wl_board_generate_letter(false, data);
					wl_add_board_particle(al_map_rgb_f(1.0, 1.0, 1.0), x, y, 10, data);
				}
				y--;
			}
		}
	}
	wl_process_particle_list(instance->particle_list[1]);
	if(instance->game.uses_hazards)
	{
		wl_game_hazard_logic(data);
	}
	wl_clear_board_flags(data);
	wl_game_sprite_logic(&instance->game, data);
	wl_board_logic(data);
	wl_game_player_control(data);
	wl_game_player_logic(data);
	wl_player_rules_logic(data);
	wl_game_guru_logic(data);
	wl_game_level_bg_logic(data);

	wl_message_logic(&instance->game_messages, false);
	wl_message_logic(&instance->word_messages, true);

	/* add time to profile */
	instance->profile->time++;

	if(instance->input_state.pause.pressed)
	{
		wl_menu_proc_overlay_menu(data, 0, NULL);
	}
	else
	{
		if(instance->state == WL_STATE_GAME_TALLY)
		{
		}
		else
		{
			wl_game_process_overlay_menu(instance->menu[WL_MENU_GAME_OVERLAY], data);
		}
	}
	if(instance->game.tick > 0)
	{
		instance->game.tick--;
	}
	if(instance->game.tick <= 0 && instance->game.got_new_level && wl_level_transition_complete(data))
	{
		if(instance->level[1])
		{
			wl_destroy_level(instance->level[1]);
			instance->level[1] = NULL;
		}
		instance->game.got_new_level = false;
	}
}

/* render text up to width 'width', removing the last bit and replacing with
 * '...' if it is too long */
static void wl_draw_truncated_textf(T3F_FONT * fp, ALLEGRO_COLOR color, float x, float y, float z, float width, int flags, const char * format, ...)
{
	char buf[1024] = {0};
	bool truncated = false;
	va_list vap;

	va_start(vap, format);
	vsnprintf(buf, 1024, format, vap);
	va_end(vap);

	/* keep taking away the last letter until we are at or below the desired width */
	while(t3f_get_text_width(fp, buf) > width)
	{
		buf[strlen(buf) - 1] = '\0';
		truncated = true;
	}

	/* if we actually truncated, add elipsis */
	if(truncated)
	{
		strncat(buf, "...", 1024);
	}

	t3f_draw_text(fp, color, x, y, z, flags, buf);
}

void wl_game_render_word_list_header(T3F_FONT * font, const char * header, float x, float y, bool shadow)
{
	if(shadow)
	{
		t3f_draw_textf(font, al_map_rgba_f(0.0, 0.0, 0.0, 0.25), x - 16.0, y, 0, T3F_FONT_ALIGN_RIGHT, "%s", header);
	}
	else
	{
		t3f_draw_textf(font, WL_LIFE_FILL_COLOR, x - 16.0, y, -8, T3F_FONT_ALIGN_RIGHT, "%s", header);
	}
}

/* render a word list word, x refers to the center point between the word and the point display */
void wl_game_render_word_list_word(WL_DICTIONARY * dictionary, T3F_FONT * font, int word, float x, float y, float a, bool shadow)
{
	if(shadow)
	{
		wl_draw_truncated_textf(font, al_map_rgba_f(0.0, 0.0, 0.0, 0.25 * a), x - 16.0, y, 0, 192, T3F_FONT_ALIGN_RIGHT, "%s", dictionary->word[word].data);
//		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(0.0, 0.0, 0.0, 0.75 * a), x - 16.0 + 2.0, y + 2.0, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
	}
	else
	{
		wl_draw_truncated_textf(font, al_map_rgba_f(a, a, a, a), x - 16.0, y, -8, 192, T3F_FONT_ALIGN_RIGHT, "%s", dictionary->word[word].data);
//		t3f_draw_textf(wl_ascii_font, al_map_rgba_f(a, a, a, a), x - 16.0, y, 0, 0, 0, T3F_FONT_ALIGN_RIGHT, "%s", wl_dictionary->word[word].data);
	}
}

void wl_game_render_top_words_list(float ox, bool shadow, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	float a, sy, oy;
	int i;

	/* render top words */
	a = 1.0;
	sy = 0.0;
	wl_game_render_word_list_header(instance->font[WL_FONT_ASCII], "Top Words", ox, WL_BOARD_Y + sy, shadow);
	sy += 24;
	if(instance->game.player[instance->game.current_player].top_word_counter > 0)
	{
		oy = 0;
		for(i = 0; i < instance->game.player[instance->game.current_player].top_word_spot; i++)
		{
			a = 1.0;
			wl_game_render_word_list_word(instance->dictionary, instance->font[WL_FONT_ASCII], instance->game.player[instance->game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 - oy, a, shadow);
		}

		/* render inserted word */
		a = 1.0 - (float)instance->game.player[instance->game.current_player].top_word_counter / 24.0;
		wl_game_render_word_list_word(instance->dictionary, instance->font[WL_FONT_ASCII], instance->game.player[instance->game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 - oy, a, shadow);

		/* offset position of words below inserted word only while the counter is active (> 0) */
		if(instance->game.player[instance->game.current_player].top_word_counter > 0 && instance->game.player[instance->game.current_player].top_word_spot < instance->game.player[instance->game.current_player].top_word_count - 1)
		{
			oy = instance->game.player[instance->game.current_player].top_word_counter - 24;
		}
		for(i = instance->game.player[instance->game.current_player].top_word_spot + 1; i < instance->game.player[instance->game.current_player].top_word_count; i++)
		{
			a = 1.0;

			/* fade out word past end of list */
			if(i >= WL_MAX_TOP_WORDS - 1)
			{
				a = (float)instance->game.player[instance->game.current_player].top_word_counter / 24.0;
			}
			wl_game_render_word_list_word(instance->dictionary, instance->font[WL_FONT_ASCII], instance->game.player[instance->game.current_player].top_word[i], ox, WL_BOARD_Y + sy + (i - 1) * 24 - oy, a, shadow);
		}
	}
	else
	{
		oy = 0;
		for(i = 0; i < instance->game.player[instance->game.current_player].top_word_count && i < WL_MAX_TOP_WORDS - 1; i++)
		{
			a = 1.0;
			wl_game_render_word_list_word(instance->dictionary, instance->font[WL_FONT_ASCII], instance->game.player[instance->game.current_player].top_word[i], ox, WL_BOARD_Y + sy + i * 24 + oy, a, shadow);
		}
	}
}

void wl_game_render_recent_words_list(float ox, bool shadow, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	float a, oy;
	int i, start;

	/* render word list */
	a = 1.0;
	oy = 24 * 8;
	start = instance->game.player[0].word_list_count - WL_MAX_WORD_LIST;
	if(instance->game.player[instance->game.current_player].word_list_count > WL_MAX_WORD_LIST && instance->game.player[instance->game.current_player].word_list_counter > 0)
	{
		start--;
		oy = 24 * 8 + instance->game.player[instance->game.current_player].word_list_counter - 24;
	}
	if(start < 0)
	{
		start = 0;
	}
	for(i = start; i < instance->game.player[instance->game.current_player].word_list_count; i++)
	{
		a = 1.0;
		if(instance->game.player[instance->game.current_player].word_list_counter > 0)
		{
			if(instance->game.player[instance->game.current_player].word_list_count > WL_MAX_WORD_LIST && i == start)
			{
				a = (float)instance->game.player[instance->game.current_player].word_list_counter / 24.0;
			}
			else if(i == instance->game.player[instance->game.current_player].word_list_count - 1)
			{
				a = 1.0 - (float)instance->game.player[instance->game.current_player].word_list_counter / 24.0;
			}
		}
		wl_game_render_word_list_word(instance->dictionary, instance->font[WL_FONT_ASCII], instance->game.player[instance->game.current_player].word_list[i], ox, WL_BOARD_Y + (i - start) * 24 + oy, a, shadow);
	}

	/* render recent words header last so diappearing words go beneath it */
	a = 1.0;
	oy = 24 * 7;
	wl_game_render_word_list_header(instance->font[WL_FONT_ASCII], "Recent Words", ox, WL_BOARD_Y + oy, shadow);
}

void wl_game_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, li = 0;
	float a, ox, oy = 0.0, x, w, fi;
	float life_scale;
	float d;
	ALLEGRO_COLOR c1, c2, c3, c4;
	char helper_text[3] = {0};
	float life_rate;

	wl_game_level_bg_render(data);

	al_hold_bitmap_drawing(true);
	wl_board_render(data);
	t3f_set_clipping_rectangle(WL_BOARD_X - 36, WL_BOARD_Y - 36, WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 72, WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE + 72);
	if((instance->game.player[0].state == WL_PLAYER_STATE_NORMAL || instance->game.player[0].state == WL_PLAYER_STATE_JUMP) && instance->game.player[0].hover_x >= 0 && instance->game.player[0].hover_y >= 0)
	{
		if(instance->game.player[0].state == WL_PLAYER_STATE_NORMAL && instance->game.player[0].hover_x == instance->game.player[0].x && instance->game.player[0].hover_y == instance->game.player[0].y)
		{
			if((instance->game.uses_invalid && strlen(instance->game.player[0].word) > 0) || instance->game.player[0].word_exists)
			{
				t3f_draw_animation(instance->animation[WL_ANIMATION_SELECTOR], WL_LIFE_FILL_COLOR, instance->game.tick, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
			}
			else
			{
				t3f_draw_animation(instance->animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(1.0, 0.5, 0.5, 1.0), instance->game.tick, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
			}
		}
		else if(instance->game.player[0].in_range)
		{
			t3f_draw_animation(instance->animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), instance->game.tick, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
		}
		else
		{
			t3f_draw_animation(instance->animation[WL_ANIMATION_SELECTOR], al_map_rgba_f(0.8, 0.8, 0.8, 0.8), instance->game.tick, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE, 0.0, 0);
		}
	}

	/* draw particles */
	wl_render_particle_list(instance->particle_list[1], WL_PARTICLE_TYPE_FLASH);

	/* draw shadows */
	for(i = 0; i < instance->game.sprite_list.sprites; i++)
	{
		wl_sprite_shadow_render(instance->game.sprite_list.sprite[i], data);
	}

	/* draw sprites */
	for(i = 0; i < instance->game.sprite_list.sprites; i++)
	{
		wl_sprite_render(instance->game.sprite_list.sprite[i], data);
	}

	/* draw particles */
	wl_render_particle_list(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD);

	al_hold_bitmap_drawing(false);
	t3f_set_clipping_rectangle(0, 0, 0, 0);

	/* draw guru */
	if(instance->game.guru_active)
	{
		t3f_draw_animation(instance->animation[WL_ANIMATION_GURU], instance->game.guru_color, 0, WL_BOARD_X, WL_BOARD_Y + 16.0 - instance->game.guru_z, 0.0, 0);
	}

	al_hold_bitmap_drawing(true);

	/* draw "HUD" */
	c1 = wl_fade_color(WL_LIFE_BG_COLOR, WL_LIFE_FREEZE_BG_COLOR, instance->game.player[0].life_fade);
	c2 = wl_fade_color(WL_LIFE_FILL_COLOR, WL_LIFE_FREEZE_FILL_COLOR, instance->game.player[0].life_fade);
	if(instance->game.player[0].life <= 10.0 && !(instance->game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		c3 = wl_fade_color(c2, t3f_color_white, 1.0 - (float)(instance->tick % 60) / 60.0);
	}
	else
	{
		c3 = c2;
	}
	if(instance->game.player[0].life <= 0.0 && !instance->game.player[0].lost && instance->state == WL_STATE_GAME)
	{
		c4 = wl_fade_color(c1, t3f_color_white, 1.0 - (float)(instance->tick % 60) / 50.0);
	}
	else
	{
		c4 = c1;
	}
//	al_draw_bitmap(wl_bitmap[WL_BITMAP_LIFE], WL_BOARD_X - 32 - 24, WL_BOARD_Y, 0);
	life_scale = WL_MAX_LIFE / instance->bitmap[WL_BITMAP_LIFE_FILL]->target_height;
	ox = WL_LIFE_BAR_X;
	oy = WL_VIRTUAL_DISPLAY_HEIGHT - (WL_VIRTUAL_DISPLAY_HEIGHT - (WL_BOARD_Y + WL_MAX_BOARD_HEIGHT * WL_BOARD_TILE_SIZE)) / 2 - instance->bitmap[WL_BITMAP_LEVEL]->target_height / 2;
	t3f_draw_bitmap(instance->bitmap[WL_BITMAP_LIFE], c4, WL_LIFE_BAR_X, WL_LIFE_BAR_Y, 0, 0);
	d = instance->bitmap[WL_BITMAP_LIFE_FILL]->target_height - instance->game.player[0].life_indicator / life_scale;
//	al_draw_bitmap_region(wl_bitmap[WL_BITMAP_LIFE_FILL], 0, d, al_get_bitmap_width(wl_bitmap[WL_BITMAP_LIFE_FILL]), al_get_bitmap_height(wl_bitmap[WL_BITMAP_LIFE_FILL]) - d, WL_BOARD_X - 32 - 24, WL_BOARD_Y + d, 0);
	t3f_draw_bitmap_region(instance->bitmap[WL_BITMAP_LIFE_FILL], c3, 0, d, instance->bitmap[WL_BITMAP_LIFE_FILL]->target_width, instance->bitmap[WL_BITMAP_LIFE_FILL]->target_height - d, WL_LIFE_BAR_X, WL_LIFE_BAR_Y + d, 0, 0);
	if(instance->game.uses_life && !(instance->game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		if(instance->game.mode == WL_GAME_MODE_GURU)
		{
			life_rate = WL_LIFE_DECREASE_GURU_MODE;
		}
		else
		{
			life_rate = instance->level_list->entry[instance->game.player[0].level].life_decrease;
		}
		li = 0;
		for(fi = WL_MAX_LIFE - instance->game.player[0].life; fi < WL_MAX_LIFE; fi += life_rate)
		{
//			al_draw_tinted_bitmap(wl_bitmap[WL_BITMAP_LIFE_DEC], al_map_rgba_f(0.0, 0.0, 0.0, 0.125), ox, WL_BOARD_Y + fi / life_scale, 0);
			li++;
		}
	}
	t3f_draw_bitmap(instance->bitmap[WL_BITMAP_LIFE_OVERLAY], t3f_color_white, WL_LIFE_BAR_X, WL_LIFE_BAR_Y, 0, 0);
	t3f_draw_bitmap(instance->bitmap[WL_BITMAP_LEVEL], t3f_color_white, WL_BOARD_X, oy, 0, 0);
	d = (float)instance->bitmap[WL_BITMAP_LEVEL_FILL]->target_width * (float)(instance->game.player[0].progress_indicator % WL_LEVEL_UP) / (float)WL_LEVEL_UP;
	t3f_draw_bitmap_region(instance->bitmap[WL_BITMAP_LEVEL_FILL], t3f_color_white, instance->bitmap[WL_BITMAP_LEVEL_FILL]->target_width - d - 1, 0, d, instance->bitmap[WL_BITMAP_LEVEL_FILL]->target_height, WL_BOARD_X + 1, oy, 0, 0);
	if(instance->game.uses_life && !(instance->game.player[0].powerups & WL_POWERUP_HOURGLASS))
	{
		if(instance->game.player[0].life > 0.0)
		{
			if(instance->game.player[0].life <= WL_MAX_LIFE * 0.5)
			{
				a = 0.125;
			}
			else
			{
				a = (1.0 - ((instance->game.player[0].life - (WL_MAX_LIFE * 0.5))) / (WL_MAX_LIFE * 0.5)) * 0.125;
			}
			t3f_draw_textf(instance->font[WL_FONT_24], al_map_rgba_f(0.0, 0.0, 0.0, a), ox + 16.0, WL_BOARD_Y + (WL_MAX_LIFE - instance->game.player[0].life_indicator) / life_scale - t3f_get_font_line_height(instance->font[WL_FONT_24]), 0, T3F_FONT_ALIGN_CENTER, "%d", li);
		}
	}

	/* draw particles */
	wl_render_particle_list(instance->particle_list[1], WL_PARTICLE_TYPE_ICE);
	wl_render_particle_list(instance->particle_list[1], WL_PARTICLE_TYPE_STEAM);

	/* render current word */
	x = (float)WL_BOARD_X + (float)WL_BOARD_TILE_SIZE * ((float)WL_MAX_BOARD_WIDTH / 2.0);
	oy = WL_BOARD_Y - 52;
	t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(0.0, 0.0, 0.0, 0.25), x, oy, 0, T3F_FONT_ALIGN_CENTER, "( %s )", instance->game.player[0].word);
	if((instance->game.uses_invalid && strlen(instance->game.player[0].word) > 0) || instance->game.player[0].word_exists)
	{
		t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), x, oy, -8, T3F_FONT_ALIGN_CENTER, "( %s )", instance->game.player[0].word);
	}
	else
	{
		t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(1.0, 0.5, 0.5, 1.0), x, oy, -8, T3F_FONT_ALIGN_CENTER, "( %s )", instance->game.player[0].word);
	}

	/* render score */
//	if(instance->game.word_list_ox >= WL_VIRTUAL_DISPLAY_WIDTH)
	{
		w = t3f_get_text_width(instance->font[WL_FONT_ASCII], "Score: 000000");
		t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(0.0, 0.0, 0.0, 0.25), instance->game.word_list_ox - 16 - w, WL_BOARD_Y - 24 * 2, 0, 0, "Score:");
		t3f_draw_textf(instance->font[WL_FONT_ASCII], WL_LIFE_FILL_COLOR, instance->game.word_list_ox - 16 - w - 1, WL_BOARD_Y - 24 * 2, -8, 0, "Score:");
		t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(0.0, 0.0, 0.0, 0.25), instance->game.word_list_ox - 16, WL_BOARD_Y - 24 * 2, 0, T3F_FONT_ALIGN_RIGHT, "%d", instance->game.player[instance->game.current_player].score);
		t3f_draw_textf(instance->font[WL_FONT_ASCII], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), instance->game.word_list_ox - 16, WL_BOARD_Y - 24 * 2, -8, T3F_FONT_ALIGN_RIGHT, "%d", instance->game.player[instance->game.current_player].score);
	}

	if(instance->game.word_list_ox < WL_VIRTUAL_DISPLAY_WIDTH)
	{
		al_hold_bitmap_drawing(false);
		al_draw_filled_rectangle(0, 0, WL_VIRTUAL_DISPLAY_WIDTH, WL_VIRTUAL_DISPLAY_HEIGHT, al_map_rgba_f(0.0, 0.0, 0.0, ((WL_VIRTUAL_DISPLAY_WIDTH - instance->game.word_list_ox) / (WL_VIRTUAL_DISPLAY_WIDTH / 2)) / 2.9));
		al_hold_bitmap_drawing(true);
	}

	wl_game_render_top_words_list(instance->game.word_list_ox, true, data);
	wl_game_render_top_words_list(instance->game.word_list_ox, false, data);
//	if(instance->game.word_list_ox >= WL_VIRTUAL_DISPLAY_WIDTH)
	{
		wl_game_render_recent_words_list(instance->game.word_list_ox, true, data);
		wl_game_render_recent_words_list(instance->game.word_list_ox, false, data);
	}

	/* render touch helper */
	if((instance->game.player[0].state == WL_PLAYER_STATE_NORMAL || instance->game.player[0].state == WL_PLAYER_STATE_JUMP) && instance->game.player[0].hover_x >= 0 && instance->game.player[0].hover_y >= 0)
	{
		if(instance->input_type == WL_INPUT_TYPE_TOUCH && instance->input_state.action.held)
		{
			if(instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter != ' ')
			{
				t3f_draw_bitmap(instance->bitmap[WL_BITMAP_POPUP], t3f_color_white, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE + (WL_BOARD_TILE_SIZE / 2) - instance->bitmap[WL_BITMAP_POPUP]->target_width / 2, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE - instance->bitmap[WL_BITMAP_POPUP]->target_height, 0, 0);

				/* render powerup animation */
				if(instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter == '?')
				{
					t3f_draw_animation(instance->animation[WL_ANIMATION_POWER_GURU], t3f_color_white, 0, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - instance->animation[WL_ANIMATION_POWER_GURU]->data->frame[0]->width / 2, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				else if(instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter == '!')
				{
					t3f_draw_animation(instance->animation[WL_ANIMATION_POWER_HEALTH], t3f_color_white, 0, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - instance->animation[WL_ANIMATION_POWER_HEALTH]->data->frame[0]->width / 2, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				else if(instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter == '_')
				{
					t3f_draw_animation(instance->animation[WL_ANIMATION_POWER_HOURGLASS], t3f_color_white, 0, WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2 - instance->animation[WL_ANIMATION_POWER_HOURGLASS]->data->frame[0]->width / 2, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 8, 0.0, 0);
				}
				/* otherwise it's not a powerup, render character */
				else
				{
					if(instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter == 'Q')
					{
						sprintf(helper_text, "%s", "Qu");
					}
					else
					{
						sprintf(helper_text, "%c", instance->game.board[instance->game.player[0].hover_y][instance->game.player[0].hover_x].letter);
					}
					t3f_draw_text(instance->font[WL_FONT_32], al_map_rgba_f(0.0, 0.0, 0.0, 1.0), WL_BOARD_X + instance->game.player[0].hover_x * WL_BOARD_TILE_SIZE + WL_BOARD_TILE_SIZE / 2, WL_BOARD_Y + instance->game.player[0].hover_y * WL_BOARD_TILE_SIZE - WL_BOARD_TILE_SIZE - 6, 0, T3F_FONT_ALIGN_CENTER, helper_text);
				}
			}
		}
	}

	/* render pause overlay menu */
	if(instance->state == WL_STATE_GAME_TALLY)
	{
	}
	else
	{
		t3f_render_gui(instance->menu[WL_MENU_GAME_OVERLAY], T3F_GUI_USER_FLAG);
		t3f_render_gui(instance->menu[WL_MENU_GAME_OVERLAY], 0);
	}

	/* draw particles */
	wl_render_particle_list(instance->particle_list[1], WL_PARTICLE_TYPE_GLASS);

	wl_render_messages(&instance->game_messages, false, data);
	wl_render_messages(&instance->word_messages, true, data);
	al_hold_bitmap_drawing(false);
}

void wl_game_paused_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema)
	{
		wl_process_cinema(instance->cinema, data);

		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_title_process_menu(instance->current_menu, data);
		}
	}
	else
	{
		wl_title_process_menu(instance->current_menu, data);
	}
}

void wl_game_paused_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema)
	{
		wl_render_cinema(instance->cinema, data);
		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_title_render_menu(data);
		}
	}
	else
	{
		if(instance->game.mode == WL_GAME_MODE_MUSIC)
		{
			wl_game_music_render(data);
		}
		else
		{
			wl_game_render(data);
		}
		al_draw_filled_rectangle(0, 0, t3f_virtual_display_width, t3f_virtual_display_height, al_map_rgba_f(0.0, 0.0, 0.0, 0.5));
		wl_title_render_menu(data);
	}
}

static void _wl_game_back_to_title(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	_wl_update_high_scores(data);
	wl_game_update_best_flags(data);
	if(instance->option[WL_OPTION_UPLOAD_SCORES] && instance->game.player[0].word_list_count > 0)
	{
		instance->leaderboard_mode = instance->game.mode;
		instance->leaderboard_type = 0;
		instance->leaderboard_style = 1;
		_wl_unload_leaderboard(instance);
		wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
		wl_download_scores(data);
		wl_select_menu(WL_MENU_DYNAMIC, data);
		instance->state = WL_STATE_LEADERBOARDS;
	}
	if(instance->game.mode < WL_GAME_MODE_STORY)
	{
		t3f_stop_music();
		instance->state = WL_STATE_GAME_DONE;
	}
	else
	{
		wl_game_exit(data);
		wl_title_load_data(data, true);
		wl_play_music(instance->music_queue, "data/music/theme.ogg");
		if(!instance->option[WL_OPTION_UPLOAD_SCORES] || instance->game.player[0].word_list_count <= 0)
		{
			wl_select_menu(WL_MENU_MAIN, data);
			instance->state = WL_STATE_TITLE;
		}
		else
		{
			wl_select_menu(WL_MENU_DYNAMIC, data);
			instance->state = WL_STATE_LEADERBOARDS;
		}
	}
	wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
}

void wl_game_win_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	int i;
	int frames = 1;

	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(instance->cinema, data);
		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_credits_logic(&instance->credits);
			if(instance->input_state.action.held)
			{
				for(i = 0; i < 15; i++)
				{
					wl_credits_logic(&instance->credits);
				}
			}
		}
	}
	if(instance->ending_state == 0)
	{
		if(instance->input_state.pause.pressed || instance->credits.done)
		{
			wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
			wl_finish_current_song(instance->music_queue, 1.0 / 60.0);
			instance->ending_state = 1;
			t3f_clear_touch_state();
		}
	}
	else
	{
		if(instance->effects_data->fade_tick <= 0)
		{
			_wl_game_back_to_title(data);
		}
	}
}

void wl_game_win_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_render_cinema(instance->cinema, data);
	if(instance->cinema->tick >= instance->cinema->length)
	{
		wl_credits_render(&instance->credits, data);
	}
}

void wl_demo_win_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;
	int frames = 1;

	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(instance->cinema, data);
	}
	if(instance->ending_state == 0)
	{
		if(instance->input_state.pause.pressed || instance->credits.done)
		{
			wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
			instance->ending_state = 1;
			t3f_clear_touch_state();
		}
		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_title_process_menu(instance->current_menu, data);
		}
	}
	else
	{
		if(instance->effects_data->fade_tick <= 0)
		{
			_wl_game_back_to_title(data);
		}
	}
}

void wl_demo_win_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_render_cinema(instance->cinema, data);
	if(instance->cinema->tick >= instance->cinema->length)
	{
		wl_title_render_menu(data);
	}
}
