#include "t3f/t3f.h"
#include "t3f/sound.h"
#include "t3f/rng.h"
#include "modules/data.h"
#include "instance.h"
#include "defines.h"
#include "sprite.h"

static int wl_hazard_chart[10];

void wl_update_hazard_chart(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;
	int pos = 0;

	for(i = 0; i < 10; i++)
	{
		wl_hazard_chart[i] = WL_SPRITE_TYPE_LASER;
	}

	for(i = 0; i < instance->level_list->entry[instance->game.player[0].level].laser_chances; i++)
	{
		if(pos < 10)
		{
			wl_hazard_chart[pos] = WL_SPRITE_TYPE_LASER;
			pos++;
		}
	}
	for(i = 0; i < instance->level_list->entry[instance->game.player[0].level].bomb_chances; i++)
	{
		if(pos < 10)
		{
			wl_hazard_chart[pos] = WL_SPRITE_TYPE_BOMB;
			pos++;
		}
	}
	for(i = 0; i < instance->level_list->entry[instance->game.player[0].level].spike_chances; i++)
	{
		if(pos < 10)
		{
			wl_hazard_chart[pos] = WL_SPRITE_TYPE_SPIKE;
			pos++;
		}
	}
}

static int wl_game_add_sprite(WL_GAME * game)
{
	int i;
	
	for(i = 0; i < WL_MAX_SPRITES; i++)
	{
		if(!game->sprite[i].active)
		{
			return i;
		}
	}
	return -1;
}

void wl_game_initialize_hazard(WL_SPRITE * sp, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, j, r, s, x, y;
	int spike_gen_w;
	float z;
	int spot[WL_MAX_BOARD_WIDTH * WL_MAX_BOARD_HEIGHT];
	int spots = 0;
	
	switch(sp->type)
	{
		/* bombs need to spawn with player in the blast radius */
		case WL_SPRITE_TYPE_BOMB:
		{
			for(i = 0; i < 25; i++)
			{
				x = ((instance->game.player[0].x - 2) + (i % 5));
				y = ((instance->game.player[0].y - 2) + (i / 5));
				if(i != 12 && x >= 0 && x < WL_MAX_BOARD_WIDTH && y >= 0 && y < WL_MAX_BOARD_HEIGHT && !instance->game.board[y][x].used)
				{
					spot[spots] = i;
					spots++;
				}
			}
			if(spots > 0)
			{
				r = rand() % spots;
				for(j = r; j < spots - 1; j++)
				{
					spot[j] = spot[j + 1];
				}
				spots--;
				s = wl_game_add_sprite(&instance->game);
				if(s >= 0)
				{
					sp->timer = 300;
					sp->ftick = 0.0;
					sp->tick = 0;
					sp->type = WL_SPRITE_TYPE_BOMB;
					sp->x = ((instance->game.player[0].x - 2) + (spot[r] % 5)) * WL_BOARD_TILE_SIZE;
					sp->y = ((instance->game.player[0].y - 2) + (spot[r] / 5)) * WL_BOARD_TILE_SIZE;
					sp->z = -WL_VIRTUAL_DISPLAY_WIDTH / 2 - 24;
					sp->z *= 3;
					sp->cy = 24;
					sp->vz = WL_TERMINAL_VELOCITY;
					sp->state = 0;
					sp->active = true;
				}
			}
			break;
		}
		case WL_SPRITE_TYPE_LASER:
		{
			r = rand() % 4;
			switch(r)
			{
				case 0:
				{
					sp->y = instance->game.player[0].y * WL_BOARD_TILE_SIZE;
					sp->x = -36 - 48 - WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE;
					sp->z = 0.0;
					sp->vx = 4.0;
					sp->vy = 0.0;
					sp->timer = 0;
					sp->state = 0;
					break;
				}
				case 1:
				{
					sp->y = instance->game.player[0].y * WL_BOARD_TILE_SIZE;
					sp->x = WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 36 + WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE;
					sp->z = 0.0;
					sp->vx = -4.0;
					sp->vy = 0.0;
					sp->timer = 0;
					sp->state = 0;
					break;
				}
				case 2:
				{
					sp->x = instance->game.player[0].x * WL_BOARD_TILE_SIZE;
					sp->y = -36 - 48 - WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE;
					sp->z = 0.0;
					sp->vy = 4.0;
					sp->vx = 0.0;
					sp->timer = 0;
					sp->state = 1;
					break;
				}
				default:
				{
					sp->x = instance->game.player[0].x * WL_BOARD_TILE_SIZE;
					sp->y = WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE + 36 + WL_MAX_BOARD_WIDTH * WL_BOARD_TILE_SIZE;
					sp->z = 0.0;
					sp->vy = -4.0;
					sp->vx = 0.0;
					sp->timer = 0;
					sp->state = 1;
					break;
				}
			}
			sp->flags = 0;
			t3f_play_sample(instance->sample[WL_SAMPLE_LASER], 1.0, t3f_get_sound_position(instance->game.player[0].sprite.x, instance->game.player[0].sprite.y, sp->x, sp->y), 1.0);
			sp->cy = 24;
			sp->active = true;
			break;
		}
		case WL_SPRITE_TYPE_SPIKE:
		{
			z = -WL_VIRTUAL_DISPLAY_WIDTH - 24;
			z *= 3;
			spike_gen_w = instance->game.player[0].range * 2 - 1;
			
			/* make list of places we want spikes to be able to drop */
			for(i = 0; i < spike_gen_w * spike_gen_w; i++)
			{
				x = ((instance->game.player[0].x - spike_gen_w / 2) + (i % spike_gen_w));
				y = ((instance->game.player[0].y - spike_gen_w / 2) + (i / spike_gen_w));
				if(!(x == instance->game.player[0].x && y == instance->game.player[0].y) && x >= 0 && x < WL_MAX_BOARD_WIDTH && y >= 0 && y < WL_MAX_BOARD_HEIGHT && !instance->game.board[y][x].used)
				{
					spot[spots] = i;
					spots++;
				}
			}
			for(i = 0; i < 3; i++)
			{
				if(spots > 0)
				{
					r = rand() % spots;
					s = wl_game_add_sprite(&instance->game);
					if(s >= 0)
					{
						instance->game.sprite[s].type = WL_SPRITE_TYPE_SPIKE;
						instance->game.sprite[s].x = ((instance->game.player[0].x - spike_gen_w / 2) + (spot[r] % spike_gen_w)) * WL_BOARD_TILE_SIZE;
						instance->game.sprite[s].y = ((instance->game.player[0].y - spike_gen_w / 2) + (spot[r] / spike_gen_w)) * WL_BOARD_TILE_SIZE;
						instance->game.sprite[s].z = z;
						instance->game.sprite[s].cy = 24;
						instance->game.sprite[s].vz = WL_TERMINAL_VELOCITY;
						instance->game.sprite[s].state = 0;
						z -= 8.0;
						instance->game.sprite[s].active = true;
					}
					z += (9 - i) * 16.0;
					for(j = r; j < spots - 1; j++)
					{
						spot[j] = spot[j + 1];
					}
					spots--;
				}
			}
			break;
		}
	}
}

void wl_explode_sprite(WL_SPRITE * sp, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, p;
	
	switch(sp->type)
	{
		case WL_SPRITE_TYPE_BOMB:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba_f(0.1, 0.1, 0.1, 1.0), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
		case WL_SPRITE_TYPE_LASER:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba(255, 158, 28, 255), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
		case WL_SPRITE_TYPE_SPIKE:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba_f(0.7, 0.7, 0.7, 1.0), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_SNEAKER:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_GURU:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba(255, 255, 80, 197), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
		case WL_SPRITE_TYPE_POWERUP_HEALTH:
		{
			for(i = 0; i < 10; i++)
			{
				p = wl_add_particle(instance->particle_list[1], WL_PARTICLE_TYPE_SHARD, instance->animation[WL_ANIMATION_PARTICLE_3], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), sp->x + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, sp->y + WL_BOARD_TILE_SIZE / 2.0 + t3f_drand(&instance->rng_state) * 8.0 - 4.0, -8.0, 0, 60, 0);
				instance->particle_list[1]->particle[p].vx = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vy = t3f_drand(&instance->rng_state) * 4.0 - 2.0;
				instance->particle_list[1]->particle[p].vz = -5.0;
			}
			sp->active = false;
			break;
		}
	}
}

void wl_game_hazard_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;
	
	instance->game.hazard_count--;
	if(instance->game.hazard_count <= 0)
	{
		for(i = 0; i < WL_MAX_SPRITES; i++)
		{
			if(!instance->game.sprite[i].active)
			{
				instance->game.sprite[i].type = wl_hazard_chart[rand() % 10];
				wl_game_initialize_hazard(&instance->game.sprite[i], data);
				break;
			}
		}
		instance->game.hazard_count = instance->level_list->entry[instance->game.player[0].level].hazard_delay * 60 + rand() % 300 - 150;
	}
}
