#include "t3f/music.h"
#include "modules/cinema.h"
#include "modules/data.h"
#include "modules/effects.h"
#include "modules/music_queue.h"
#include "instance.h"
#include "game_sprite.h"
#include "player.h"
#include "game_level.h"
#include "game_music.h"

void wl_game_intro_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int frames = 1;
	int i;

	for(i = 0; i < frames; i++)
	{
		wl_process_cinema(instance->cinema, data);
	}
	if(instance->cinema->tick >= instance->cinema->length || instance->input_state.pause.pressed)
	{
		wl_destroy_cinema(instance->cinema);
		instance->cinema = NULL;
		wl_clear_music_queue(instance->music_queue);
		instance->game.retry = false;
		wl_game_initialize(data, WL_GAME_INIT_FLAG_NO_LEVEL_LOAD | WL_GAME_INIT_FLAG_NO_BOARD_GENERATION);
		
		/* don't scroll level name if we are going to be showing the tutorial */
		if(!wl_player_needs_tutorial(data))
		{
			wl_game_scroll_level_name(data);
		}
		instance->state = WL_STATE_GAME;
		t3f_clear_touch_state();
	}
}

void wl_game_intro_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_render_cinema(instance->cinema, data);
	}
	else if(instance->game.mode == WL_GAME_MODE_MUSIC)
	{
		wl_game_music_render(data);
	}
	else
	{
		wl_game_render(data);
	}
}

void wl_game_view_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->game.player[0].life = 0;
	wl_process_particle_list(instance->particle_list[1]);
	wl_clear_board_flags(data);
	wl_game_sprite_logic(&instance->game, data);
	wl_board_logic(data);
	if(instance->state != WL_STATE_GAME_INTRO)
	{
		wl_game_player_logic(data);
	}
	wl_game_guru_logic(data);
	wl_game_level_bg_logic(data);
}

void wl_game_view_render(void * data)
{
	wl_game_render(data);
}

void wl_game_paused_view_logic(void * data)
{
	wl_game_level_bg_logic(data);
}

void wl_game_continue_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_process_cinema(instance->cinema, data);
	if(instance->cinema->tick >= instance->cinema->length)
	{
		wl_destroy_cinema(instance->cinema);
		instance->cinema = NULL;
		instance->state = WL_STATE_GAME;
	}
}

void wl_game_quit_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_process_cinema(instance->cinema, data);
		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_destroy_cinema(instance->cinema);
			instance->cinema = NULL;
			wl_game_exit(data);
			wl_play_music(instance->music_queue, "data/music/theme.ogg");
			wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
	else
	{
		/* game logic */
		wl_process_particle_list(instance->particle_list[1]);
		wl_clear_board_flags(data);
		wl_game_sprite_logic(&instance->game, data);
		wl_board_logic(data);
		wl_game_player_logic(data);
		wl_game_level_bg_logic(data);
		if(!instance->effects_data->fading)
		{
			wl_game_exit(data);
			wl_play_music(instance->music_queue, "data/music/theme.ogg");
			wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
		}
	}
}

void wl_game_exit_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_process_cinema(instance->cinema, data);
		if(instance->cinema->tick >= instance->cinema->length)
		{
			wl_destroy_cinema(instance->cinema);
			instance->cinema = NULL;
			instance->state = -1;
			wl_game_exit(data);
			t3f_exit();
		}
	}
	else if(instance->game.mode == WL_GAME_MODE_MUSIC)
	{
		if(!instance->effects_data->fading)
		{
			wl_game_exit(data);
			t3f_exit();
		}
	}
	else
	{
		/* game logic */
		wl_process_particle_list(instance->particle_list[1]);
		wl_clear_board_flags(data);
		wl_game_sprite_logic(&instance->game, data);
		wl_board_logic(data);
		wl_game_player_logic(data);
		wl_game_level_bg_logic(data);
		if(!instance->effects_data->fading)
		{
			wl_game_exit(data);
			t3f_exit();
		}
	}
}

void wl_ending_view_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema->scene == 0)
	{
		wl_game_view_logic(data);
	}
	else
	{
		wl_title_word_logic(data);
	}
}

void wl_ending_view_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema->scene == 0)
	{
		wl_game_view_render(data);
	}
	else
	{
		wl_title_word_render(instance->font[WL_FONT_24], 0.125, data);
	}
}
