#include "modules/data.h"
#include "modules/text.h"
#include "modules/leaderboards.h"
#include "ui/menu_leaderboards.h"
#include "instance.h"
#include "defines.h"
#include "game.h"
#include "game_sprite.h"
#include "game_tally.h"
#include "game_level.h"

void wl_game_tally_score(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i, l;
	char buf[32] = {0};
	
	if(instance->game.mode == WL_GAME_MODE_PUZZLE)
	{
		
		instance->game.current_unused_letters = 0;
		
		/* count letters player has picked up and not used */
		l = strlen(instance->game.player[0].word);
		for(i = 0; i < l; i++)
		{
			if(instance->game.player[0].score < 0)
			{
				instance->game.player[0].score = 0;
			}
			instance->game.current_unused_letters++;
			instance->game.player[0].word[0] = '\0';
		}
		
		/* count letters left on the board */
		for(i = 0; i < WL_MAX_BOARD_WIDTH * WL_MAX_BOARD_HEIGHT; i++)
		{
			l = instance->game.board[i / WL_MAX_BOARD_HEIGHT][i % WL_MAX_BOARD_WIDTH].letter;
			if(l >= 'A' && l <= 'Z')
			{
				instance->game.current_unused_letters++;
			}
		}
		if(instance->game.current_unused_letters)
		{
			sprintf(buf, "Unused Tiles: %d", instance->game.current_unused_letters);
			wl_add_message(&instance->word_messages, WL_MESSAGE_TYPE_FLOAT_UP, buf, instance->font[WL_FONT_ASCII], 0, data);
			sprintf(buf, "Penalty: %d", instance->game.current_unused_letters * 100);
			wl_add_message(&instance->word_messages, WL_MESSAGE_TYPE_FLOAT_UP_2, buf, instance->font[WL_FONT_ASCII], 0, data);
			instance->game.player[0].score -= instance->game.current_unused_letters * 100;
			if(instance->game.player[0].score < 0)
			{
				instance->game.player[0].score = 0;
			}
		}
	}
}

void _wl_update_high_scores(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	/* adjust high score when appropriate */
	if(instance->game.player[instance->game.current_player].score > instance->profile->high_score[instance->game.mode])
	{
		instance->profile->high_score[instance->game.mode] = instance->game.player[instance->game.current_player].score;
		instance->profile->high_score_upload[instance->game.mode] = 1;
		switch(instance->game.mode)
		{
			case WL_GAME_MODE_PUZZLE:
			{
				instance->profile->high_score_extra[instance->game.mode] = instance->game.current_unused_letters;
				break;
			}
			default:
			{
				instance->profile->high_score_extra[instance->game.mode] = 0;
			}
		}
	}
}

void wl_game_tally_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	
	/* game logic */
	wl_process_particle_list(instance->particle_list[1]);
	wl_clear_board_flags(data);
	wl_game_sprite_logic(&instance->game, data);
	wl_board_logic(data);
	wl_game_player_logic(data);
	wl_game_level_bg_logic(data);
	wl_message_logic(&instance->word_messages, true);
	instance->tally_tick--;
	if(instance->tally_tick <= 0)
	{
		instance->game.board[instance->game.player[0].y][instance->game.player[0].x].letter = ' ';
		instance->game.player[0].state = WL_PLAYER_STATE_EXIT;

		/* store scores in profile */
		_wl_update_high_scores(data);
		wl_game_update_best_flags(data);
		if(instance->option[WL_OPTION_UPLOAD_SCORES] && instance->game.player[0].word_list_count > 0)
		{
			instance->state = WL_STATE_GAME_DONE;
			instance->leaderboard_mode = instance->game.mode;
			instance->leaderboard_type = 0;
			instance->leaderboard_style = 1;
			_wl_unload_leaderboard(instance);
			wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
			wl_download_scores(data);
			wl_select_menu(WL_MENU_DYNAMIC, data);
		}
		else
		{
			wl_select_menu(WL_MENU_RETRY, data);
			instance->state = WL_STATE_GAME_DONE;
		}
	}
}

void wl_game_tally_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_game_render(data);
	switch(instance->game.mode)
	{
		case WL_GAME_MODE_PARTY:
		{
			break;
		}
	}
}
