#ifndef WL_INSTANCE_H
#define WL_INSTANCE_H

/* frameworks */
#include "t3f/t3f.h"
#include "t3f/input.h"
#include "t3f/bitmap.h"
#include "t3net/leaderboard.h"

/* modules */
#include "modules/level.h"
#include "modules/dictionary.h"
#include "modules/shape.h"
#include "modules/message.h"
#include "modules/credits.h"
#include "modules/profile.h"
#include "modules/cinema.h"
#include "modules/effects.h"
#include "modules/music_queue.h"
#include "modules/text_entry.h"
#include "modules/level_list.h"
#include "modules/character.h"
#include "modules/playlist.h"
#include "modules/input.h"
#include "modules/tile.h"
#include "modules/nine_patch.h"

/* main program */
#include "ui/title.h"
#include "gameplay/game.h"
#include "defines.h"

typedef struct
{
  T3F_BITMAP * bitmap[WL_MAX_BITMAPS];
  T3F_ANIMATION * animation[WL_MAX_ANIMATIONS];
  NINE_PATCH_BITMAP * box_nine_patch;
  WL_TILE * tile[WL_MAX_TILES];
  T3F_FONT * font[WL_MAX_FONTS];
  T3F_FONT * ascii_font;
  ALLEGRO_SAMPLE * sample[WL_MAX_SAMPLES];
  T3F_ATLAS * atlas[WL_MAX_ATLASES];
  WL_EFFECTS_DATA * effects_data;
  WL_MUSIC_QUEUE * music_queue;

  /* configuration */
  float dead_zone;

  WL_DICTIONARY * dictionary;
  unsigned long dictionary_checksum;
  WL_LEVEL * level[2];
  WL_PLAYLIST * playlist;
  int playlist_pos;
  int state;
  int overlay_pause_state;
  int global_pause_state; // underlying state for global pause menu
  int tick;
  int tally_tick;
  int exit_tick;
  bool demo;
  WL_GAME game;
  int current_fg_shape;
  int current_bg_shape;
  WL_MESSAGES game_messages;
  WL_MESSAGES award_messages;
  WL_MESSAGES word_messages;
  int option[WL_MAX_OPTIONS];
  int input_type;
  int input_device;
  void * input_device_handle;
  bool input_device_lost;
  WL_CREDITS credits;
  T3F_RNG_STATE rng_state;
  T3F_INPUT_HANDLER * input_handler;
  WL_INPUT_STATE input_state;
  float graphics_scale;
  bool steam_running;

  T3F_GUI * menu[WL_MAX_MENUS];
  int current_menu;
  int last_menu;
  int last_item;
  char menu_text[256];
  ALLEGRO_USTR * menu_ustr;
  char menu_ustr_buffer[256];

  const char * import_profile_fn;
  WL_PROFILE * import_profile;
  WL_PROFILE * profile;
  T3F_ACHIEVEMENTS_LIST * achievements;

  WL_CINEMA * cinema;
  int ending_state;

  T3F_OBJECT_LOADER * leaderboard_loader;
  char * user_key;
  bool new_user_key;
  T3NET_LEADERBOARD * leaderboard;
  bool * leaderboard_upload_result;
  int leaderboard_mode;
  int leaderboard_type;
  int leaderboard_spot;
  int leaderboard_style;
  bool leaderboard_want_download;

  /* tutorial */
  bool want_tutorial;
  int tutorial_step;
  int game_tutorial_sprite;
//  bool touched;
  float tutorial_text_offset_x;
  float tutorial_text_offset_y;
  float tutorial_text_width;

  /* menus */
  int selected_profile;
  int level_page;
  bool menu_caps;
  char * options_title;
  char * options_done_text;
  bool multiplayer;

  /* particles */
  WL_PARTICLE_LIST * particle_list[2];

  /* shapes */
  WL_SHAPE_LISTS * shape_lists;

  /* text entry */
  WL_TEXT_ENTRY_DATA * text_entry;

  /* title screen */
  WL_TITLE_TILE title_tile[8];
  WL_TITLE_TILE * title_tile_zsort[8];
  WL_TITLE_WORD title_word[WL_MAX_TITLE_WORDS];
  T3F_ATLAS * title_atlas;

  /* content */
  WL_LEVEL_LIST * level_list;
  WL_CHARACTER * character[WL_MAX_CHARACTERS];
  int character_count;

} WL_INSTANCE;

WL_INSTANCE * wl_create_instance(void);
void wl_destroy_instance(WL_INSTANCE * ip);

#endif
