#include "instance.h"
#include "message.h"
#include "data.h"
#include "profile.h"
#include "achievements.h"

bool wl_set_up_achievements(WL_INSTANCE * instance)
{
	instance->achievements = t3f_create_achievements_list(WL_ACHIEVEMENT_COUNT);
	if(!instance->achievements)
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_A_GURU_OF_GURUS, "A_GURU_OF_GURUS", "A Guru of Gurus", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_CONSISTENTLY_WORDY, "CONSISTENTLY_WORDY", "Consistently Wordy", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_ENLIGHTENMENT, "ENLIGHTENMENT", "Enlightenment", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_FOUL_MOUTH, "FOUL_MOUTH", "Foul Mouth", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_GOOD_PUPIL, "GOOD_PUPIL", "Good Pupil", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_HUNDREDFOLD, "HUNDREDFOLD", "Hundredfold", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_LOSER, "LOSER", "Loser", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_NEAR_DEATH_VERBOSITY, "NEAR_DEATH_VERBOSITY", "Near Death Verbosity", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_PUZZLE_SOLVER, "PUZZLE_SOLVER", "Puzzle Solver", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_QUICKIE, "QUICKIE", "Quickie", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_RARE_WORD, "RARE_WORD", "Rare Word", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_SUPERCALIFRAGILISTIC, "SUPERCALIFRAGILISTIC", "Supercalifragilistic", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_TAKING_IT_IN_STRIDE, "TAKING_IT_IN_STRIDE", "Taking It in Stride", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_TO_HEALTH, "TO_HEALTH", "To Health", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_TRUE_ENLIGHTENMENT, "TRUE_ENLIGHTENMENT", "True Enlightenment", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_WORDSMITH, "WORDSMITH", "Wordsmith", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_TRANSCENDENCE, "TRANSCENDENCE", "Beyond Enlightenment", "", 1, false))
	{
		goto fail;
	}
	if(!t3f_set_achievement_details(instance->achievements, WL_ACHIEVEMENT_EPONYMOUS, "EPONYMOUS", "Eponymous", "", 1, false))
	{
		goto fail;
	}

	return true;

	fail:
	{
		if(instance->achievements)
		{
			t3f_destroy_achievements_list(instance->achievements);
		}
		return false;
	}
}

void wl_achievements_logic(WL_PROFILE * profile, int achievement, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int c, i, j;
	
	switch(achievement)
	{
		case WL_ACHIEVEMENT_ENLIGHTENMENT:
		{
			if(!instance->demo && instance->game.finished)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_ENLIGHTENMENT, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_TRUE_ENLIGHTENMENT:
		{
			if(!instance->demo && instance->game.finished && instance->game.player[0].start_level == 0)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_TRUE_ENLIGHTENMENT, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_TRANSCENDENCE:
		{
			if(instance->game.finished && !strcmp(instance->level_list->story[instance->game.player[0].story].name, "GURU STORY"))
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_TRANSCENDENCE, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_HUNDREDFOLD:
		{
			if(instance->game.player[0].word_list_count >= 100)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_HUNDREDFOLD, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_TAKING_IT_IN_STRIDE:
		{
			if(instance->game.player[0].words_this_level >= 30)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_TAKING_IT_IN_STRIDE, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_QUICKIE:
		{
			if(instance->game.player[0].words_this_level < 10)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_QUICKIE, 1);
			}
			break;
		}
		
		/* check this just after grabbing the @ in puzzle mode */
		case WL_ACHIEVEMENT_PUZZLE_SOLVER:
		{
			c = 1;
			if(strlen(instance->game.player[0].word) <= 0)
			{
				for(i = 0; i < WL_MAX_BOARD_HEIGHT; i++)
				{
					for(j = 0; j < WL_MAX_BOARD_WIDTH; j++)
					{
						if(!(i == WL_MAX_BOARD_WIDTH / 2 && j == WL_MAX_BOARD_HEIGHT / 2) && instance->game.board[i][j].letter != ' ')
						{
							c = 0;
						}
					}
				}
				if(c)
				{
					t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_PUZZLE_SOLVER, 1);
				}
			}
			break;
		}
		
		/* check when game ends */
		case WL_ACHIEVEMENT_LOSER:
		{
			if(instance->game.player[0].word_list_count <= 0)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_LOSER, 1);
			}
			break;
		}
		case WL_ACHIEVEMENT_FOUL_MOUTH:
		{
			c = 0;
			
			for(i = 0; i < instance->game.player[0].word_list_count; i++)
			{
				if(!strcasecmp(instance->dictionary->word[instance->game.player[0].word_list[i]].data, "FUCK"))
				{
					c++;
				}
				else if(!strcasecmp(instance->dictionary->word[instance->game.player[0].word_list[i]].data, "SHIT"))
				{
					c++;
				}
				else if(!strcasecmp(instance->dictionary->word[instance->game.player[0].word_list[i]].data, "DAMN"))
				{
					c++;
				}
				else if(!strcasecmp(instance->dictionary->word[instance->game.player[0].word_list[i]].data, "HELL"))
				{
					c++;
				}
			}
			if(c >= 4)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_FOUL_MOUTH, 1);
			}
			break;
		}

		/* completed all tutorials */
		case WL_ACHIEVEMENT_GOOD_PUPIL:
		{
			if(instance->profile->flags & (WL_PROFILE_FLAG_TUTORIAL_BASIC | WL_PROFILE_FLAG_TUTORIAL_STORY | WL_PROFILE_FLAG_TUTORIAL_CASUAL | WL_PROFILE_FLAG_TUTORIAL_PUZZLE | WL_PROFILE_FLAG_TUTORIAL_GURU))
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_GOOD_PUPIL, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_RARE_WORD:
		{
			if(instance->dictionary->word[instance->game.player[0].word_index].rarity <= 0)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_RARE_WORD, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_WORDSMITH:
		{
			if(strlen(instance->dictionary->word[instance->game.player[0].word_index].data) >= 12)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_WORDSMITH, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_SUPERCALIFRAGILISTIC:
		{
			if(strlen(instance->dictionary->word[instance->game.player[0].word_index].data) >= 20)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_SUPERCALIFRAGILISTIC, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_TO_HEALTH:
		{
			if(instance->game.player[0].life >= WL_MAX_LIFE)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_TO_HEALTH, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_A_GURU_OF_GURUS:
		{
			if(instance->game.player[0].words >= 20)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_A_GURU_OF_GURUS, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_NEAR_DEATH_VERBOSITY:
		{
			if(instance->game.player[0].life <= 0)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_NEAR_DEATH_VERBOSITY, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_CONSISTENTLY_WORDY:
		{
			for(i = 0; i < instance->game.player[0].words; i++)
			{
				if(strlen(instance->dictionary->word[instance->game.player[0].word_list[i]].data) < 10)
				{
					break;
				}
			}
			if(i >= 10)
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_CONSISTENTLY_WORDY, 1);
			}
			break;
		}

		case WL_ACHIEVEMENT_EPONYMOUS:
		{
			if(!strcasecmp(instance->dictionary->word[instance->game.player[0].word_index].data, instance->level_list->entry[instance->game.player[0].level].name))
			{
				t3f_update_achievement_progress(instance->achievements, WL_ACHIEVEMENT_EPONYMOUS, 1);
			}
			break;
		}
	}
}
