#include "t3f/t3f.h"
#include "character.h"

static bool _load_character_animation(WL_CHARACTER * character, int state, float scale, ALLEGRO_CONFIG * config, const char * key_name, ALLEGRO_PATH * path)
{
  const char * val;
  char buf[1024];

  val = al_get_config_value(config, "Character", key_name);
  if(val)
  {
    sprintf(buf, "%s#Size %1.1f", val, scale);
    al_set_path_filename(path, buf);
    character->animation[state] = t3f_load_animation(al_path_cstr(path, '/'), T3F_BITMAP_FLAG_PADDED, false);
    if(character->animation[state])
    {
      return true;
    }
  }
  return false;
}

WL_CHARACTER * wl_load_character(const char * fn, float scale)
{
  WL_CHARACTER * character = NULL;
  ALLEGRO_CONFIG * config = NULL;
  ALLEGRO_PATH * path = NULL;
  const char * val;

  character = malloc(sizeof(WL_CHARACTER));
  if(!character)
  {
    goto fail;
  }
  memset(character, 0, sizeof(WL_CHARACTER));

  config = al_load_config_file(fn);
  if(!config)
  {
    goto fail;
  }

  path = al_create_path(fn);
  if(!path)
  {
    goto fail;
  }

  val = al_get_config_value(config, "Character", "name");
  if(!val)
  {
    goto fail;
  }
  character->name = strdup(val);
  if(!character->name)
  {
    goto fail;
  }

  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_NORMAL, scale, config, "state_normal", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_HAPPY, scale,  config, "state_happy", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_SAD, scale, config, "state_sad", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_HURT, scale, config, "state_hurt", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_UP, scale, config, "state_up", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_DOWN, scale, config, "state_down", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_LEFT, scale, config, "state_left", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_RIGHT, scale, config, "state_right", path))
  {
    goto fail;
  }
  if(!_load_character_animation(character, WL_CHARACTER_ANIMATION_SHADOW, scale, config, "shadow", path))
  {
    goto fail;
  }

  /* cleanup */
  al_destroy_path(path);
  al_destroy_config(config);

  return character;

  fail:
  {
    if(path)
    {
      al_destroy_path(path);
    }
    if(config)
    {
      al_destroy_config(config);
    }
    wl_destroy_character(character);

    return NULL;
  }
}

void wl_destroy_character(WL_CHARACTER * cp)
{
  int i;

  if(cp)
  {
    if(cp->name)
    {
      free(cp->name);
    }
    for(i = 0; i < WL_CHARACTER_MAX_ANIMATIONS; i++)
    {
      if(cp->animation[i])
      {
        t3f_destroy_animation(cp->animation[i]);
      }
    }
    free(cp);
  }
}

bool wl_add_character_to_atlas(WL_CHARACTER * cp, T3F_ATLAS * ap)
{
  int i;
  bool ret = true;

  for(i = 0; i < WL_CHARACTER_MAX_ANIMATIONS; i++)
  {
    if(cp->animation[i])
    {
      if(!t3f_add_animation_to_atlas(ap, cp->animation[i], T3F_ATLAS_SPRITE))
      {
        ret = false;
      }
    }
  }

  return ret;
}
