#include "instance.h"
#include "data.h"
#include "credits.h"
#include "text.h"

static void wl_add_credit(WL_CREDITS * cp, const char * name, T3F_FONT * font, ALLEGRO_COLOR color, float y)
{
	if(cp->credits < WL_MAX_CREDITS)
	{
		strcpy(cp->credit[cp->credits].name, name);
		cp->credit[cp->credits].font = font;
		cp->credit[cp->credits].color = color;
		cp->credit[cp->credits].y = y;
		cp->credits++;
	}
}

void wl_credits_initialize(WL_CREDITS * cp, int mode, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	float oy = 0.0;
	float small_space = 36.0;
	float big_space = 128.0;
	ALLEGRO_COLOR title_color = al_map_rgba_f(0.2, 1.0, 0.2, 1.0);
	ALLEGRO_COLOR name_color = al_map_rgba_f(1.0, 1.0, 1.0, 1.0);
	
	cp->credits = 0;
	cp->done = false;
	cp->fast = false;
	cp->y = WL_VIRTUAL_DISPLAY_HEIGHT;
	
	wl_add_credit(cp, "Producer", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;
	
	wl_add_credit(cp, "Game Design", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;
	
	wl_add_credit(cp, "Programming", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Level Art", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Story", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Cinema Art", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Cinema Storyboard", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Cinema Programming", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Additional Graphics", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Sound", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Michael Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Music", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Grady O'Connell", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Lowell J. Stevens", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Testing", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Todd Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Travis Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Terry Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Candace Cope", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Kathy Hancock", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Jason Hale", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Ryan Gumbs", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Dizzy Egg", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "taron", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Matthew Leverton", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Grady O'Connell", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Cameron Priest", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;
	
	wl_add_credit(cp, "Third Party Code", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "zlib", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "PNG", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "FreeType", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "DUMB", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Ogg Vorbis", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "PhysicsFS", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Allegro", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "OpenGL", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "DirectX", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "MacOS SDK", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "MinGW-w64", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Android SDK", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += big_space;

	wl_add_credit(cp, "Third Party Tools", instance->font[WL_FONT_32_OUTLINE], title_color, oy);
	oy += small_space;
	wl_add_credit(cp, "GCC", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "GNU Make", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "MSYS2", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Clang", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "MilkyTracker", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Audacity", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Inkscape", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Tracktion", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "gedit", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Atom", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "Visual Studio Code", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += small_space;
	wl_add_credit(cp, "nano", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	oy += WL_VIRTUAL_DISPLAY_HEIGHT;

	if(mode == 1)
	{
		wl_add_credit(cp, "You Are Truly", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
		oy += small_space;
		wl_add_credit(cp, "Enlightened!", instance->font[WL_FONT_32_OUTLINE], name_color, oy);
	}
}

void wl_credits_logic(WL_CREDITS * cp)
{
	cp->y -= 1.0;
	if(cp->y + cp->credit[cp->credits - 1].y < -40.0)
	{
		cp->done = true;
	}
}

void wl_credits_render(WL_CREDITS * cp, void * data)
{
	int i;
	
	for(i = 0; i < cp->credits; i++)
	{
		if(cp->y + cp->credit[i].y >= t3f_default_view->top - t3f_get_font_line_height(cp->credit[i].font) - 4 && cp->y + cp->credit[i].y < t3f_default_view->bottom)
		{
			wl_draw_text(cp->credit[i].font, cp->credit[i].color, t3f_color_black, al_map_rgba_f(0.0, 0.0, 0.0, 0.25), WL_VIRTUAL_DISPLAY_WIDTH / 2, cp->y + cp->credit[i].y, 8, 8, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_SHADOW, cp->credit[i].name);
		}
	}
}
