#include "t3f/t3f.h"
#include "t3net/t3net.h"
#include "instance.h"
#include "data.h"
#include "leaderboards.h"
#include "gameplay/game.h"

int wl_obfuscate_score(int score)
{
	return score * 3 + 'w' + 'l' + 'e' + 'a' + 'p';
}

int wl_unobfuscate_score(int score)
{
	return (score - 'w' - 'l' - 'e' - 'a' - 'p') / 3;
}

static void * _wl_retrieve_leaderboard_key(void * arg)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)arg;
	const char * url;
	char * user_key = NULL;

	if(arg)
	{
		url = al_get_config_value(t3f_config, "Config", "leaderboard_key_url");
		if(url)
		{
			user_key = t3net_get_new_leaderboard_user_key(url, instance->profile->name);
		}
	}
	return user_key;
}

char * wl_get_leaderboard_user_key(void * data)
{
	const char * val;

	val = al_get_config_value(t3f_user_data, "Leaderboards", "user_key");
	if(val)
	{
		return strdup(val);
	}
	return _wl_retrieve_leaderboard_key(data);
}

bool wl_update_leaderboard_username(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	const char * val;
	const char * url;
	const char * user_key;

	val = al_get_config_value(t3f_user_data, "Leaderboards", "user_name");
	/* if profile name differs from stored user_name, update the remote data */
	if(!val || strcmp(val, instance->profile->name))
	{
		url = al_get_config_value(t3f_config, "Config", "leaderboard_username_url");
		if(url)
		{
			user_key = al_get_config_value(t3f_user_data, "Leaderboards", "user_key");
			if(user_key)
			{
				/* only update stored user name if upload succeeds */
				if(t3net_update_leaderboard_user_name(url, user_key, instance->profile->name))
				{
					al_set_config_value(t3f_user_data, "Leaderboards", "user_name", instance->profile->name);
					return true;
				}
			}
		}
		return false;
	}

	return true;
}

static void _wl_get_mode_name(WL_INSTANCE * instance, char * buffer, const char * prefix, int mode)
{
	int story;

	if(mode >= WL_GAME_MODE_STORY)
	{
		story = mode - WL_GAME_MODE_STORY;
		if(story < instance->level_list->story_count)
		{
			sprintf(buffer, "%sstory_%s", prefix, instance->level_list->story[mode - WL_GAME_MODE_STORY].name);
		}
		else
		{
			sprintf(buffer, "%sstory_none", prefix);
		}
	}
	else if(mode == WL_GAME_MODE_CASUAL)
	{
		sprintf(buffer, "%scasual", prefix);
	}
	else if(mode == WL_GAME_MODE_PUZZLE)
	{
		sprintf(buffer, "%spuzzle", prefix);
	}
	else if(mode == WL_GAME_MODE_GURU)
	{
		sprintf(buffer, "%sguru", prefix);
	}
}

static void * _wl_leaderboard_upload_proc(void * arg)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)arg;
	int i, p;
	char opt[256] = {0};
	char extra[256] = {0};
	char mode_name[256];
	bool * ip;
	const char * val;

	if(arg)
	{
		ip = malloc(sizeof(bool));
		if(ip)
		{
			*ip = false;
		}

		val = al_get_config_value(t3f_config, "Config", "leaderboard_upload_url");
		if(!val)
		{
			return NULL;
		}

		sprintf(opt, "%lu", instance->dictionary_checksum);

		/* upload high scores */
		for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
		{
			_wl_get_mode_name(instance, mode_name, "high_score_", i);
			if(instance->profile->high_score_upload[i])
			{
				sprintf(extra, "%d", instance->profile->high_score_extra[i]);
				if(t3net_upload_score(val, "wordleap", "1.0", mode_name, opt, instance->user_key, wl_obfuscate_score(instance->profile->high_score[i]), extra))
				{
					instance->profile->high_score_upload[i] = 0;
					*ip = true;
				}
			}
		}

		/* upload best words */
		for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
		{
			_wl_get_mode_name(instance, mode_name, "best_word_", i);
			if(instance->profile->best_word_upload[i])
			{
				if(instance->profile->best_word[i] >= 0 && instance->profile->best_word[i] < instance->dictionary->words)
				{
					p = wl_game_figure_total_points(instance->dictionary, instance->profile->best_word[i]);
					sprintf(extra, "%d", instance->profile->best_word[i]);
					if(t3net_upload_score(val, "wordleap", "1.0", mode_name, opt, instance->user_key, wl_obfuscate_score(p), extra))
					{
						instance->profile->best_word_upload[i] = 0;
						*ip = true;
					}
				}
				else
				{
					instance->profile->best_word_upload[i] = 0;
					*ip = true;
				}
			}
		}

		return ip;
	}
	return NULL;
}

void * _wl_leaderboard_download_proc(void * arg)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)arg;
	char * prefix[2] = {"high_score_", "best_word_"};
	char mode_text[256];
	char opt[256] = {0};
	T3NET_LEADERBOARD * lp = NULL;
	const char * val;
	void * ret;
	bool can_download = true;
	int i;

	if(arg)
	{
		/* ensure we have a user key */
		if(!instance->user_key)
		{
			instance->user_key = wl_get_leaderboard_user_key(arg);
			if(instance->user_key)
			{
				instance->new_user_key = true;
			}
		}
		if(!instance->user_key)
		{
			return NULL;
		}

		/* upload unuploaded scores first */
		for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
		{
			if(instance->profile->high_score_upload[i] || instance->profile->best_word_upload[i])
			{
				break;
			}
		}
		if(i < WL_PROFILE_MAX_HIGH_SCORES)
		{
			ret = _wl_leaderboard_upload_proc(arg);
			if(!ret)
			{
				can_download = false;
			}
			else
			{
				free(ret);
			}
		}
		if(!wl_update_leaderboard_username(arg))
		{
			can_download = false;
		}

		if(can_download)
		{
			val = al_get_config_value(t3f_config, "Config", "leaderboard_download_url");
			if(val)
			{
				_wl_get_mode_name(instance, mode_text, prefix[instance->leaderboard_type], instance->leaderboard_mode);
				sprintf(opt, "%lu", instance->dictionary_checksum);
				lp = t3net_get_leaderboard(val, "wordleap", "1.0", mode_text, opt, 10, 0);
			}
			return lp;
		}
	}
	else
	{
		return t3net_create_leaderboard(10);
	}

	return lp;
}

void _wl_leaderboard_destroy_proc(void * data)
{
	T3NET_LEADERBOARD * lp = (T3NET_LEADERBOARD *)data;

	t3net_destroy_leaderboard(lp);
}

bool wl_download_scores(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->leaderboard_want_download = true;

	return true;
}

void wl_free_leaderboard_data(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->leaderboard)
	{
		t3net_destroy_leaderboard(instance->leaderboard);
	}
}

void wl_update_leaderboard(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(t3f_object_ready(instance->leaderboard_loader))
	{
		instance->leaderboard = t3f_fetch_object(instance->leaderboard_loader);
	}
}
