#include "t3f/t3f.h"
#include "t3f/file_utils.h"
#include "defines.h"
#include "level_list.h"

WL_LEVEL_LIST * wl_create_level_list(int max)
{
  WL_LEVEL_LIST * lp;

  lp = malloc(sizeof(WL_LEVEL_LIST));
  if(!lp)
  {
    goto fail;
  }
  memset(lp, 0, sizeof(WL_LEVEL_LIST));

  lp->entry_max = max;
  lp->entry = malloc(sizeof(WL_LEVEL_LIST_ENTRY) * max);
  if(!lp->entry)
  {
    goto fail;
  }
  memset(lp->entry, 0, sizeof(WL_LEVEL_LIST_ENTRY) * max);

  lp->story_max = WL_PROFILE_MAX_STORIES;
  lp->story = malloc(sizeof(WL_LEVEL_LIST_STORY_DATA) * lp->story_max);
  if(!lp->story)
  {
    goto fail;
  }
  memset(lp->story, 0, sizeof(WL_LEVEL_LIST_STORY_DATA) * lp->story_max);

  lp->unlocked_story = malloc(sizeof(bool) * WL_PROFILE_MAX_STORIES);
  if(!lp->unlocked_story)
  {
    goto fail;
  }
  memset(lp->unlocked_story, 0, sizeof(bool) & WL_PROFILE_MAX_STORIES);

  lp->unlocked_index = malloc(sizeof(int) * max);
  if(!lp->unlocked_index)
  {
    goto fail;
  }
  memset(lp->unlocked_index, 0, sizeof(int) * max);

  return lp;

  fail:
  {
    wl_destroy_level_list(lp);
    return NULL;
  }
}
void wl_destroy_level_list(WL_LEVEL_LIST * lp)
{
  int i;

  if(lp)
  {
    if(lp->unlocked_story)
    {
      free(lp->unlocked_story);
    }
    if(lp->unlocked_index)
    {
      free(lp->unlocked_index);
    }
    if(lp->story)
    {
      free(lp->story);
    }
    if(lp->entry)
    {
      for(i = 0; i < lp->entry_count; i++)
      {
        if(lp->entry[i].path)
        {
          free(lp->entry[i].path);
        }
        if(lp->entry[i].name)
        {
          free(lp->entry[i].name);
        }
        if(lp->entry[i].story_name)
        {
          free(lp->entry[i].story_name);
        }
        if(lp->entry[i].unlock_name)
        {
          free(lp->entry[i].unlock_name);
        }
        if(lp->entry[i].character)
        {
          free(lp->entry[i].character);
        }
        if(lp->entry[i].intro_fn)
        {
          free(lp->entry[i].intro_fn);
        }
        if(lp->entry[i].ending_fn)
        {
          free(lp->entry[i].ending_fn);
        }
        if(lp->entry[i].pause_fn)
        {
          free(lp->entry[i].pause_fn);
        }
        if(lp->entry[i].continue_fn)
        {
          free(lp->entry[i].continue_fn);
        }
        if(lp->entry[i].quit_fn)
        {
          free(lp->entry[i].quit_fn);
        }
        if(lp->entry[i].game_over_fn)
        {
          free(lp->entry[i].game_over_fn);
        }
      }
      free(lp->entry);
    }
    free(lp);
  }
}

static bool is_level_file(const char * fn, bool isfolder)
{
  ALLEGRO_PATH * path;
  ALLEGRO_CONFIG * cp;
  const char * val;
  bool ret = false;

  if(!isfolder)
  {
    path = al_create_path(fn);
    if(path)
    {
      if(!strcasecmp(al_get_path_extension(path), ".ini"))
      {
        cp = al_load_config_file(fn);
        if(cp)
        {
          val = al_get_config_value(cp, "Level Data", "name");
          if(val)
          {
            ret = true;
          }
          al_destroy_config(cp);
        }
      }
      al_destroy_path(path);
    }
  }
  return ret;
}

static bool count_level_file(const char * fn, bool isfolder, void * data)
{
  int * c = (int *)data;

  if(is_level_file(fn, isfolder))
  {
    (*c)++;
  }

  return false;
}

static bool add_level_file(const char * fn, bool isfolder, void * data)
{
  WL_LEVEL_LIST * level_list = (WL_LEVEL_LIST *)data;
  ALLEGRO_CONFIG * cp;
  const char * val;

  if(is_level_file(fn, isfolder))
  {
    cp = al_load_config_file(fn);
    if(cp)
    {
      level_list->entry[level_list->entry_count].path = strdup(fn);
      val = al_get_config_value(cp, "Level Data", "name");
      if(val)
      {
        level_list->entry[level_list->entry_count].name = strdup(val);
      }

      /* load story data */
      level_list->entry[level_list->entry_count].slot = WL_MAX_STORY_LEVELS;
      level_list->entry[level_list->entry_count].story = WL_PROFILE_MAX_STORIES;
      level_list->entry[level_list->entry_count].base_range = -1;
      level_list->entry[level_list->entry_count].powerup_health = -1;
      level_list->entry[level_list->entry_count].powerup_guru = -1;
      level_list->entry[level_list->entry_count].powerup_sneaker = -1;
      val = al_get_config_value(cp, "Story Data", "index");
      if(val)
      {
        level_list->entry[level_list->entry_count].story = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "name");
      if(val)
      {
        level_list->entry[level_list->entry_count].story_name = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "unlock_name");
      if(val)
      {
        level_list->entry[level_list->entry_count].unlock_name = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "character");
      if(val)
      {
        level_list->entry[level_list->entry_count].character = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "intro_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].intro_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "ending_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].ending_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "pause_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].pause_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "continue_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].continue_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "quit_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].quit_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "game_over_fn");
      if(val)
      {
        level_list->entry[level_list->entry_count].game_over_fn = strdup(val);
      }
      val = al_get_config_value(cp, "Story Data", "slot");
      if(val)
      {
        level_list->entry[level_list->entry_count].slot = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "life_decrease");
      if(val)
      {
        level_list->entry[level_list->entry_count].life_decrease = atof(val);
      }
      val = al_get_config_value(cp, "Story Data", "hazard_delay");
      if(val)
      {
        level_list->entry[level_list->entry_count].hazard_delay = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "base_range");
      if(val)
      {
        level_list->entry[level_list->entry_count].base_range = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "powerup_health");
      if(val)
      {
        level_list->entry[level_list->entry_count].powerup_health = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "powerup_guru");
      if(val)
      {
        level_list->entry[level_list->entry_count].powerup_guru = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "powerup_sneaker");
      if(val)
      {
        level_list->entry[level_list->entry_count].powerup_sneaker = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "laser_chances");
      if(val)
      {
        level_list->entry[level_list->entry_count].laser_chances = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "bomb_chances");
      if(val)
      {
        level_list->entry[level_list->entry_count].bomb_chances = atoi(val);
      }
      val = al_get_config_value(cp, "Story Data", "spike_chances");
      if(val)
      {
        level_list->entry[level_list->entry_count].spike_chances = atoi(val);
      }
      level_list->entry_count++;
      al_destroy_config(cp);
    }
  }
  return false;
}

int wl_count_installed_levels(void)
{
  int c = 0;

  t3f_scan_files("data/levels", count_level_file, true, &c);

  return c;
}

static int story_mode_slot_compare(const void * e1, const void * e2)
{
	const WL_LEVEL_LIST_ENTRY * entry1 = e1;
	const WL_LEVEL_LIST_ENTRY * entry2 = e2;

  /* sort by story index first */
  if(entry1->story < entry2->story)
  {
    return -1;
  }
  else if(entry1->story > entry2->story)
  {
    return 1;
  }

  /* sort by story slot if index the same */
  else
  {
    if(entry1->slot < entry2->slot)
    {
      return -1;
    }
    else if(entry1->slot > entry2->slot)
    {
      return 1;
    }
  }
	return 0;
}

static int story_mode_name_compare(const void * e1, const void * e2)
{
	const WL_LEVEL_LIST_ENTRY * entry1 = e1;
	const WL_LEVEL_LIST_ENTRY * entry2 = e2;

  return strcasecmp(entry1->name, entry2->name);
}

bool wl_catalog_installed_levels(WL_LEVEL_LIST * lp)
{
  int i;
  int last_index = -1;

  t3f_scan_files("data/levels", add_level_file, true, lp);
	qsort(&lp->entry[0], lp->entry_count, sizeof(WL_LEVEL_LIST_ENTRY), story_mode_slot_compare);
  for(i = 0; i < lp->entry_count; i++)
  {
    if(lp->entry[i].slot >= WL_MAX_STORY_LEVELS)
    {
      break;
    }
  }
  qsort(&lp->entry[i], lp->entry_count - i, sizeof(WL_LEVEL_LIST_ENTRY), story_mode_name_compare);

  /* count number of stories */
  lp->story_count = 0;
  for(i = 0; i < lp->entry_count; i++)
  {
    if(lp->entry[i].slot < WL_MAX_STORY_LEVELS && lp->entry[i].story != last_index)
    {
      lp->story_count++;
      last_index = lp->entry[i].story;
    }
  }

  /* set default story data */
  for(i = 0; i < WL_PROFILE_MAX_STORIES; i++)
  {
    lp->story[i].intro_fn = "data/scripts/story_main/intro.script";
    lp->story[i].ending_fn = "data/scripts/story_main/ending.script";
    lp->story[i].pause_fn = "data/scripts/story_main/pause.script";
    lp->story[i].continue_fn = "data/scripts/story_main/continue.script";
    lp->story[i].quit_fn = "data/scripts/story_main/quit.script";
    lp->story[i].character = "WordLeap Guy";
  }

  /* grab story data */
  for(i = 0; i < lp->entry_count; i++)
  {
    /* only grab story data if story slot is valid */
    if(lp->entry[i].story >= 0 && lp->entry[i].story < WL_PROFILE_MAX_STORIES)
    {
      if(lp->entry[i].story_name)
      {
        lp->story[lp->entry[i].story].name = lp->entry[i].story_name;
      }
      if(lp->entry[i].unlock_name)
      {
        lp->story[lp->entry[i].story].unlock_name = lp->entry[i].unlock_name;
      }
      if(lp->entry[i].intro_fn)
      {
        lp->story[lp->entry[i].story].intro_fn = lp->entry[i].intro_fn;
      }
      if(lp->entry[i].ending_fn)
      {
        lp->story[lp->entry[i].story].ending_fn = lp->entry[i].ending_fn;
      }
      if(lp->entry[i].pause_fn)
      {
        lp->story[lp->entry[i].story].pause_fn = lp->entry[i].pause_fn;
      }
      if(lp->entry[i].continue_fn)
      {
        lp->story[lp->entry[i].story].continue_fn = lp->entry[i].continue_fn;
      }
      if(lp->entry[i].quit_fn)
      {
        lp->story[lp->entry[i].story].quit_fn = lp->entry[i].quit_fn;
      }
      if(lp->entry[i].game_over_fn)
      {
        lp->story[lp->entry[i].story].game_over_fn = lp->entry[i].game_over_fn;
      }
      if(lp->entry[i].character)
      {
        lp->story[lp->entry[i].story].character = lp->entry[i].character;
      }
      if(lp->entry[i].base_range >= 0)
      {
        lp->story[lp->entry[i].story].base_range = lp->entry[i].base_range;
      }
      if(lp->entry[i].powerup_health >= 0)
      {
        lp->story[lp->entry[i].story].powerup_health = lp->entry[i].powerup_health;
      }
      if(lp->entry[i].powerup_guru >= 0)
      {
        lp->story[lp->entry[i].story].powerup_guru = lp->entry[i].powerup_guru;
      }
      if(lp->entry[i].powerup_sneaker >= 0)
      {
        lp->story[lp->entry[i].story].powerup_sneaker = lp->entry[i].powerup_sneaker;
      }
    }
  }

  /* map start and end levels for each story */
	for(i = 0; i < WL_PROFILE_MAX_STORIES; i++)
	{
		lp->story[i].start_level = lp->entry_count;
		lp->story[i].end_level = 0;
	}
	for(i = 0; i < lp->entry_count; i++)
	{
    if(lp->entry[i].story < WL_PROFILE_MAX_STORIES)
    {
      if(lp->entry[i].slot == 0)
      {
        lp->story[lp->entry[i].story].start_level = i;
      }
      lp->story[lp->entry[i].story].end_level = i;
    }
	}

  return true;
}

static WL_LEVEL_LIST * _wl_work_level_list = NULL;

static int _unlocked_story_mode_name_compare(const void * e1, const void * e2)
{
	const int * entry1 = e1;
	const int * entry2 = e2;

  return strcasecmp(_wl_work_level_list->entry[*entry1].name, _wl_work_level_list->entry[*entry2].name);
}

static bool _wl_story_completed(WL_LEVEL_LIST * lp, const char * name, WL_PROFILE * pp)
{
  int i;

  for(i = 0; i < WL_PROFILE_MAX_STORIES; i++)
  {
    if(!strcmp(lp->story[i].name, name))
    {
      break;
    }
  }
  if(i < WL_PROFILE_MAX_STORIES)
  {
    if(pp->progress[i] / WL_LEVEL_UP >= lp->story[i].end_level - lp->story[i].start_level)
    {
      return true;
    }
  }
  return false;
}

static bool _wl_level_unlocked(WL_LEVEL_LIST * lp, int level, WL_PROFILE * pp)
{
  if(pp->flags & WL_PROFILE_FLAG_UNLOCKED)
  {
    return true;
  }
  else if(lp->entry[level].slot >= WL_MAX_STORY_LEVELS)
  {
    return true;
  }
  else if(lp->entry[level].story < WL_PROFILE_MAX_STORIES && (!lp->entry[level].unlock_name || _wl_story_completed(lp, lp->entry[level].unlock_name, pp)))
  {
    if(pp->progress[lp->entry[level].story] / WL_LEVEL_UP >= level - lp->story[lp->entry[level].story].start_level)
    {
      return true;
    }
  }
  return false;
}

/* This should be called before generating level list menus to ensure the most
   up-to-date list. */
void wl_update_unlocked_level_list(WL_LEVEL_LIST * lp, WL_PROFILE * pp)
{
  int i;

  /* flag unlocked stories */
  for(i = 0; i < lp->story_count; i++)
  {
    if(_wl_level_unlocked(lp, lp->story[i].start_level, pp))
    {
      lp->unlocked_story[i] = true;
    }
    else
    {
      lp->unlocked_story[i] = false;
    }
  }

  /* index all unlocked levels */
  lp->unlocked_count = 0;
  for(i = 0; i < lp->entry_count; i++)
  {
    if(_wl_level_unlocked(lp, i, pp))
    {
      lp->unlocked_index[lp->unlocked_count] = i;
      lp->unlocked_count++;
    }
  }

  /* sort unlocked levels alphabetically */
  _wl_work_level_list = lp;
  qsort(lp->unlocked_index, lp->unlocked_count, sizeof(int), _unlocked_story_mode_name_compare);
}
