#include "t3f/t3f.h"
#include "playlist.h"

WL_PLAYLIST * wl_create_playlist(int max)
{
  WL_PLAYLIST * pp;

  pp = malloc(sizeof(WL_PLAYLIST));
  if(!pp)
  {
    goto fail;
  }
  memset(pp, 0, sizeof(WL_PLAYLIST));
  pp->entry = malloc(sizeof(int) * max);
  if(!pp->entry)
  {
    goto fail;
  }
  memset(pp->entry, 0, sizeof(int) * max);
  pp->max = max;
  pp->count = 0;

  return pp;

  fail:
  {
    wl_destroy_playlist(pp);
    return NULL;
  }
}
void wl_destroy_playlist(WL_PLAYLIST * pp)
{
  if(pp)
  {
    if(pp->entry)
    {
      free(pp->entry);
    }
    free(pp);
  }
}

bool wl_add_level_to_playlist(WL_PLAYLIST * pp, int level)
{
  if(pp->count < pp->max)
  {
    pp->entry[pp->count] = level;
    pp->count++;
    return true;
  }
  return false;
}

void wl_clear_playlist(WL_PLAYLIST * pp)
{
  pp->count = 0;
}

static void wl_remove_level_from_playlist(WL_PLAYLIST * pp, int level)
{
  int i;

  if(pp->count > 0)
  {
    for(i = level; i < pp->count - 1; i++)
    {
      pp->entry[i] = pp->entry[i + 1];
    }
    pp->count--;
  }
}

static WL_PLAYLIST * _wl_clone_playlist(WL_PLAYLIST * pp)
{
  WL_PLAYLIST * new_pp;
  int i;

  new_pp = wl_create_playlist(pp->count);
  if(new_pp)
  {
    for(i = 0; i < pp->count; i++)
    {
      new_pp->entry[i] = pp->entry[i];
    }
    new_pp->max = pp->count;
    new_pp->count = pp->count;
    return new_pp;
  }
  return NULL;
}

void wl_shuffle_playlist(WL_PLAYLIST * pp)
{
  WL_PLAYLIST * new_pp;
  T3F_RNG_STATE rng_state;
  int i, entry;

  t3f_srand(&rng_state, time(0));
  new_pp = _wl_clone_playlist(pp);
  if(new_pp)
  {
    for(i = 0; i < pp->count; i++)
    {
      entry = t3f_rand(&rng_state) % new_pp->count;
      pp->entry[i] = new_pp->entry[entry];
      wl_remove_level_from_playlist(new_pp, entry);
    }
    wl_destroy_playlist(new_pp);    
  }
}
