#include "t3f/t3f.h"
#include "profile.h"
#include "defines.h"

WL_PROFILE * wl_create_profile(void)
{
	WL_PROFILE * pp = NULL;
	int i;

	pp = al_malloc(sizeof(WL_PROFILE));
	if(pp)
	{
		memset(pp, 0, sizeof(WL_PROFILE));
		for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
		{
			pp->best_word[i] = -1;
		}
		strcpy(pp->name, "GUEST");
	}
	return pp;
}

void wl_destroy_profile(WL_PROFILE * pp)
{
	if(pp->word)
	{
		al_free(pp->word);
	}
	al_free(pp);
}

static void _wl_setup_legacy_game_mode_map(int * mode_map)
{
	int i;

	for(i = 0; i < WL_PROFILE_MAX_HIGH_SCORES; i++)
	{
		mode_map[i] = WL_PROFILE_MAX_HIGH_SCORES - 1;
	}
	mode_map[0] = WL_GAME_MODE_STORY;
	mode_map[1] = WL_GAME_MODE_CASUAL;
	mode_map[2] = WL_GAME_MODE_PUZZLE;
	mode_map[3] = WL_GAME_MODE_GURU;
}

WL_PROFILE * wl_load_legacy_profiles(const char * fn)
{
	ALLEGRO_FILE * fp = NULL;
	WL_PROFILE * pp = NULL;
	char header[16] = {0};
	int game_mode_map[WL_PROFILE_MAX_HIGH_SCORES];
	int i;
	unsigned int j;
	const ALLEGRO_FILE_INTERFACE * fi;
	int entries;
	int awards;

	_wl_setup_legacy_game_mode_map(game_mode_map);

	fi = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = al_fopen(fn, "rb");
	if(fp)
	{
		al_fread(fp, header, 16);
		if(header[0] == 'W' && header[1] == 'L' && header[2] == 'P')
		{
			entries = al_fread32le(fp);
			if(entries > 0)
			{
				pp = al_malloc(sizeof(WL_PROFILE) * (entries + 1));
				if(pp)
				{
					memset(pp, 0, sizeof(WL_PROFILE) * (entries + 1));
					for(i = 0; i < entries; i++)
					{
						pp[i].word = NULL;
						al_fread(fp, pp[i].name, 256);
						pp[i].words = al_fread32le(fp);
						pp[i].word_size = pp[i].words;
						wl_expand_profile(&pp[i], 0);
						for(j = 0; j < pp[i].words; j++)
						{
							pp[i].word[j] = al_fread32le(fp);
						}
						pp[i].time = al_fread32le(fp);
						pp[i].moves = al_fread32le(fp);
						pp[i].undos = al_fread32le(fp);
						pp[i].wrongs = al_fread32le(fp);
						pp[i].progress[0] = al_fread32le(fp);
						pp[i].games = al_fread32le(fp);
						for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
						{
							pp[i].high_score[game_mode_map[j]] = al_fread32le(fp);
						}
						for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
						{
							pp[i].best_word[game_mode_map[j]] = al_fread32le(fp);
							if(header[15] == 2)
							{
								pp[i].best_word[j] = -1;
							}
						}
						awards = al_fread32le(fp);
						(void)awards;
						pp[i].flags = al_fread32le(fp);
					}
					for(i = 0; i < entries; i++)
					{
						if(strlen(pp[i].name) == 0)
						{
							strcpy(pp[i].name, "ERROR");
						}
					}
				}
			}
		}
		al_fclose(fp);
	}
	al_set_new_file_interface(fi);
	return pp;
}

static void _wl_read_profile_0(WL_PROFILE * pp, ALLEGRO_FILE * fp)
{
	int game_mode_map[64];
	int j;

	_wl_setup_legacy_game_mode_map(game_mode_map);

	pp->word = NULL;
	al_fread(fp, pp->name, 256);
	pp->words = al_fread32le(fp);
	pp->word_size = pp->words;
	wl_expand_profile(pp, 0);
	for(j = 0; j < pp->words; j++)
	{
		pp->word[j] = al_fread32le(fp);
	}
	pp->time = al_fread32le(fp);
	pp->moves = al_fread32le(fp);
	pp->undos = al_fread32le(fp);
	pp->wrongs = al_fread32le(fp);
	for(j = 0; j < WL_PROFILE_MAX_STORIES; j++)
	{
		pp->progress[j] = al_fread32le(fp);
	}
	pp->games = al_fread32le(fp);
	for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
	{
		pp->high_score[game_mode_map[j]] = al_fread32le(fp);
		pp->high_score_extra[game_mode_map[j]] = al_fread32le(fp);
		pp->high_score_upload[game_mode_map[j]] = al_fgetc(fp);
	}
	for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
	{
		pp->best_word[game_mode_map[j]] = al_fread32le(fp);
		pp->best_word_extra[game_mode_map[j]] = al_fread32le(fp);
		pp->best_word_upload[game_mode_map[j]] = al_fgetc(fp);
	}
	pp->flags = al_fread32le(fp);
}

static void _wl_read_profile_1(WL_PROFILE * pp, ALLEGRO_FILE * fp)
{
	int j;

	pp->word = NULL;
	al_fread(fp, pp->name, 256);
	pp->words = al_fread32le(fp);
	pp->word_size = pp->words;
	wl_expand_profile(pp, 0);
	for(j = 0; j < pp->words; j++)
	{
		pp->word[j] = al_fread32le(fp);
	}
	pp->time = al_fread32le(fp);
	pp->moves = al_fread32le(fp);
	pp->undos = al_fread32le(fp);
	pp->wrongs = al_fread32le(fp);
	for(j = 0; j < WL_PROFILE_MAX_STORIES; j++)
	{
		pp->progress[j] = al_fread32le(fp);
	}
	pp->games = al_fread32le(fp);
	for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
	{
		pp->high_score[j] = al_fread32le(fp);
		pp->high_score_extra[j] = al_fread32le(fp);
		pp->high_score_upload[j] = al_fgetc(fp);
	}
	for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
	{
		pp->best_word[j] = al_fread32le(fp);
		pp->best_word_extra[j] = al_fread32le(fp);
		pp->best_word_upload[j] = al_fgetc(fp);
	}
	pp->flags = al_fread32le(fp);
}

WL_PROFILE * wl_load_profile(const char * fn)
{
	ALLEGRO_FILE * fp = NULL;
	WL_PROFILE * pp = NULL;
	char header[16] = {0};
	const ALLEGRO_FILE_INTERFACE * fi;

	fi = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = al_fopen(fn, "rb");
	if(fp)
	{
		pp = wl_create_profile();
		if(pp)
		{
			al_fread(fp, header, 16);
			if(header[0] == 'W' && header[1] == 'L' && header[2] == 'R' && header[3] == 'P')
			{
				switch(header[15])
				{
					case 0:
					{
						_wl_read_profile_0(pp, fp);
						break;
					}
					case 1:
					{
						_wl_read_profile_1(pp, fp);
						break;
					}
				}
			}
		}
		else
		{
			al_free(pp);
			pp = NULL;
		}
		al_fclose(fp);
	}
	al_set_new_file_interface(fi);
	return pp;
}

void wl_save_profile(WL_PROFILE * pp, const char * fn)
{
	ALLEGRO_FILE * fp = NULL;
	char header[16] = {'W', 'L', 'R', 'P'};
	unsigned int j;
	const ALLEGRO_FILE_INTERFACE * fi;

	fi = al_get_new_file_interface();
	al_set_standard_file_interface();

	fp = al_fopen(fn, "wb");
	if(fp)
	{
		header[15] = 1;
		al_fwrite(fp, header, 16);
		al_fwrite(fp, pp->name, 256);
		al_fwrite32le(fp, pp->words);
		for(j = 0; j < pp->words; j++)
		{
			al_fwrite32le(fp, pp->word[j]);
		}
		al_fwrite32le(fp, pp->time);
		al_fwrite32le(fp, pp->moves);
		al_fwrite32le(fp, pp->undos);
		al_fwrite32le(fp, pp->wrongs);
		for(j = 0; j < WL_PROFILE_MAX_STORIES; j++)
		{
			al_fwrite32le(fp, pp->progress[j]);
		}
		al_fwrite32le(fp, pp->games);
		for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
		{
			al_fwrite32le(fp, pp->high_score[j]);
			al_fwrite32le(fp, pp->high_score_extra[j]);
			al_fputc(fp, pp->high_score_upload[j]);
		}
		for(j = 0; j < WL_PROFILE_MAX_HIGH_SCORES; j++)
		{
			al_fwrite32le(fp, pp->best_word[j]);
			al_fwrite32le(fp, pp->best_word_extra[j]);
			al_fputc(fp, pp->best_word_upload[j]);
		}
		al_fwrite32le(fp, pp->flags);
		al_fclose(fp);
	}
	al_set_new_file_interface(fi);
}

bool wl_expand_profile(WL_PROFILE * pp, int amount)
{
	int * word = NULL;
	unsigned int i;

	if(pp->word_size + amount > 0)
	{
		word = al_malloc(sizeof(int) * pp->word_size + amount);
		if(!word)
		{
			return false;
		}
		pp->word_size += amount;

		/* if we have words, copy them and free the old ones */
		if(pp->word)
		{
			for(i = 0; i < pp->words; i++)
			{
				word[i] = pp->word[i];
			}
			al_free(pp->word);
		}
		pp->word = word;
	}

	return true;
}
