#include "t3f/t3f.h"
#include "defines.h"
#include "data.h"
#include "text.h"

const int text_box_margin = 4;

void wl_draw_text(T3F_FONT * fp, ALLEGRO_COLOR color, ALLEGRO_COLOR outline_color, ALLEGRO_COLOR shadow_color, float x, float y, float sx, float sy, int flags, const char * text)
{
	int draw_flags = 0;

	if(flags & WL_TEXT_FLAG_CENTER)
	{
		draw_flags |= T3F_FONT_ALIGN_CENTER;
	}
	
	/* render text */
	if(flags & WL_TEXT_FLAG_SHADOW)
	{
		t3f_draw_text(fp, shadow_color, x, y, 0, draw_flags, text);
	}
	t3f_draw_text(fp, color, x, y, -sx, draw_flags, text);
}

void wl_get_text_lines(WL_TEXT_LINES * lp, const char * text, T3F_FONT * fp, float width)
{
	char current_line[256];
	int current_line_pos = 0;
	int last_space = -1;
	int i;

	memset(lp, 0, sizeof(WL_TEXT_LINES));
	for(i = 0; i < (int)strlen(text); i++)
	{
		current_line[current_line_pos] = text[i];
		current_line[current_line_pos + 1] = '\0';
		if(text[i] == ' ')
		{
			last_space = current_line_pos;
		}
		current_line_pos++;
		
		/* copy line since we encountered a manual new line */
		if(text[i] == '\n')
		{
			current_line[current_line_pos] = '\0';
			strcpy(lp->line[lp->line_count], current_line);
			lp->line_count++;
			current_line_pos = 0;
			current_line[current_line_pos] = '\0';
		}
		
		/* copy this line to our list of lines because it is long enough */
		else if(t3f_get_text_width(fp, current_line) > width - text_box_margin * 2)
		{
			current_line[last_space] = '\0';
			strcpy(lp->line[lp->line_count], current_line);
			while(text[i] != ' ' && i >= 0)
			{
				i--;
			}
			lp->line_count++;
			current_line_pos = 0;
			current_line[current_line_pos] = '\0';
		}
	}
	strcpy(lp->line[lp->line_count], current_line);
	lp->line_count++;
}

void wl_draw_text_box(NINE_PATCH_BITMAP * npbp, T3F_FONT * fp, ALLEGRO_COLOR bg_color, ALLEGRO_COLOR fg_color, float x, float y, float width, int flags, int tick, const char * text)
{
	WL_TEXT_LINES text_lines;
	int i;
	float box_x, box_y;
	int box_height;
	
	/* divide text into lines */
	wl_get_text_lines(&text_lines, text, fp, width);

	box_height = t3f_get_font_line_height(fp) * text_lines.line_count + text_box_margin * 2;
	if(flags & WL_TEXT_BOX_FLAG_CENTER_X)
	{
		box_x = x - width / 2.0;
	}
	else
	{
		box_x = x;
	}
	if(flags & WL_TEXT_BOX_FLAG_CENTER_Y)
	{
		box_y = y - box_height / 2;
	}
	else
	{
		box_y = y;
	}
	draw_nine_patch_bitmap(npbp, t3f_color_white, box_x - 1, box_y - 1, width + 1, box_height + 1);
	for(i = 0; i < text_lines.line_count; i++)
	{
		t3f_draw_text(fp, fg_color, x + text_box_margin, box_y + text_box_margin + i * t3f_get_font_line_height(fp), 0, 0, text_lines.line[i]);
	}
	if(text_lines.line_count > 0 && (tick / 15) % 2 == 0)
	{
		t3f_draw_text(fp, fg_color, x + width - text_box_margin - t3f_get_text_width(fp, "_"), box_y + text_box_margin + (text_lines.line_count - 1) * t3f_get_font_line_height(fp), 0, 0, "_");
	}
}
