#include "t3f/gui.h"
#include "t3f/bitmap.h"
#include "instance.h"
#include "modules/data.h"
#include "modules/effects.h"
#include "modules/text.h"

static T3F_GUI_DRIVER wl_gui_driver;
static void * wl_gui_data = NULL;

int wl_menu_get_element_characters(const char * text)
{
	const ALLEGRO_USTR * ustr = NULL;
	ALLEGRO_USTR_INFO info;
	int i, l, u;

	ustr = al_ref_cstr(&info, text);
	if(!ustr)
	{
		return 0;
	}
	l = al_ustr_length(ustr);
	for(i = 0; i < al_ustr_length(ustr); i++)
	{
		u = al_ustr_get(ustr, i);
		if(u == '\\')
		{
			l--;
		}
	}
	return l;
}

int wl_menu_get_element_characters_u(ALLEGRO_USTR * ustr)
{
	int i, l, u;

	l = al_ustr_length(ustr);
	for(i = 0; i < al_ustr_length(ustr); i++)
	{
		u = al_ustr_get(ustr, i);
		if(u == '\\')
		{
			l--;
		}
	}
	return l;
}

static void wl_menu_get_element_edges(T3F_GUI * pp, int i, int * left, int * top, int * right, int * bottom)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)wl_gui_data;
	int charwidth = 32;

	/* use small font */
	if(pp->element[i].resource == (void **)&instance->font[WL_FONT_16])
	{
		charwidth = 24;
	}
	if(left)
	{
		if(pp->element[i].flags & T3F_GUI_ELEMENT_CENTRE)
		{
			*left = pp->element[i].ox - (charwidth * wl_menu_get_element_characters((char *)pp->element[i].data)) / 2 - 8;
		}
		else
		{
			*left = pp->element[i].ox - 8;
		}
	}
	if(top)
	{
		*top = pp->element[i].oy - 8;
	}
	if(right)
	{
		if(pp->element[i].flags & T3F_GUI_ELEMENT_CENTRE)
		{
			*right = pp->element[i].ox + (charwidth * wl_menu_get_element_characters((char *)pp->element[i].data)) / 2 + 8;
		}
		else
		{
			*right = pp->element[i].ox + charwidth * wl_menu_get_element_characters((char *)pp->element[i].data) + 8;
		}
	}
	if(bottom)
	{
		*bottom = pp->element[i].oy + charwidth + 8;
	}
}

static void wl_menu_render_element(T3F_GUI * pp, int i, bool hover, int flags)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)wl_gui_data;
	ALLEGRO_COLOR color;
	T3F_FONT * f;
	const ALLEGRO_USTR * ustr = NULL;
	ALLEGRO_USTR_INFO info;
	char * text;
	int x, w, z;
	int j, pj, l, fl, u;
	float y_offset = 0.0;
	int tile_index = WL_TILE_MENU_32;

	if(hover)
	{
		color = al_map_rgba(162, 180, 213, 255);
		z = -16.0;
	}
	else if(pp->element[i].flags & T3F_GUI_ELEMENT_STATIC)
	{
		color = al_map_rgba(162, 180, 213, 255);
		z = -24.0;
	}
	else
	{
		color = wl_darken_color(al_map_rgba(162, 180, 213, 255), 0.7);
		z = -16.0;
	}
	text = (char *)(pp->element[i].data);
	if(pp->element[i].resource == (void **)&instance->font[WL_FONT_16])
	{
		w = 24;
		f = instance->font[WL_FONT_16];
		y_offset = 4.0;
		tile_index = WL_TILE_MENU_24;
	}
	else
	{
		w = 32;
		f = instance->font[WL_FONT_24];
		y_offset = 2.0;
		tile_index = WL_TILE_MENU_32;
	}
	if(!text)
	{
		return;
	}
	ustr = al_ref_cstr(&info, text);
	if(!ustr)
	{
		return;
	}
	l = al_ustr_length(ustr);
	fl = l;
	for(j = 0; j < l; j++)
	{
		u = al_ustr_get(ustr, j);

		/* subtract 1 for each '\' character since it is not going to be rendered */
		if(u == '\\')
		{
			fl--;
		}
	}

	x = pp->element[i].ox;
	if(pp->element[i].flags & T3F_GUI_ELEMENT_CENTRE)
	{
		x = pp->element[i].ox - (fl * w) / 2;
	}
	if(flags & T3F_GUI_USER_FLAG)
	{
		wl_draw_tile(instance->tile[tile_index], al_map_rgba_f(0.0, 0.0, 0.0, 0.25), pp->ox + x, pp->oy + pp->element[i].oy, 0.0, fl, 1, 0);
	}
	else
	{
		wl_draw_tile(instance->tile[tile_index], color, pp->ox + x, pp->oy + pp->element[i].oy, z, fl, 1, 0);
		pj = 0;
		for(j = 0; j < l; j++)
		{
			u = al_ustr_get(ustr, j);
			if(u == '\\')
			{
				j++;
				u = al_ustr_get(ustr, j);
				switch(u)
				{
					case '.':
					{
						t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, "...");
						break;
					}
					case '_':
					{
						if((instance->tick / 15) % 2 == 0)
						{
							t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, "_");
						}
						break;
					}
					case '|':
					{
						t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, "| |");
						break;
					}
					case '>':
					{
						t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, ">>");
						break;
					}
					case '+':
					{
						t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, ">|");
						break;
					}
					case '-':
					{
						t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, "|<");
						break;
					}
				}
			}
			else
			{
				al_ustr_set_chr(instance->menu_ustr, 0, u);
				t3f_draw_text(f, pp->element[i].color, pp->ox + x + pj * w + w / 2, pp->oy + pp->element[i].oy + y_offset, z, T3F_FONT_ALIGN_CENTER, al_cstr(instance->menu_ustr));
			}
			pj++;
		}
	}
}

void wl_set_menu_driver(void * data)
{
	wl_gui_driver.get_element_edges = wl_menu_get_element_edges;
	wl_gui_driver.render_element = wl_menu_render_element;
	t3f_set_gui_driver(&wl_gui_driver);
	wl_gui_data = data;
}
