#include "t3f/t3f.h"
#include "modules/cinema.h"
#include "instance.h"
#include "gameplay/game_intro.h"
#include "menu_profile.h"

int wl_menu_proc_main_quit_yes(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->state = WL_STATE_TITLE_EXIT;
	wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	wl_finish_current_song(instance->music_queue, 1.0 / 60.0);
	return 1;
}

int wl_menu_proc_main_quit_no(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_select_menu(instance->last_menu, data);
	return 1;
}

int wl_menu_proc_play_new_ok(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_end_text_entry(instance->text_entry);
	strcpy(instance->profile->name, instance->text_entry->entered_text);
	wl_create_profile_menu(data);
	wl_select_menu(WL_MENU_DYNAMIC, data);
	return 1;
}

#ifdef ALLEGRO_ANDROID
	static void wl_edit_box_new_account_callback(void * data)
	{
		wl_menu_proc_play_new_ok_finish();
	}
#endif

int wl_menu_proc_game_over_retry(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_destroy_cinema(instance->cinema);
	al_stop_timer(t3f_timer);
	instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].continue_fn, instance->graphics_scale);
	if(instance->cinema)
	{
		wl_set_cinema_view_callbacks(instance->cinema, wl_game_view_logic, wl_game_view_render);
		instance->game.player[0].start_level = instance->game.player[0].level;
		instance->state = WL_STATE_GAME_INTRO;
	}
	else
	{
		wl_title_load_data(data, true);
		wl_select_menu(WL_MENU_MAIN, data);
		instance->state = WL_STATE_TITLE;
	}
	al_start_timer(t3f_timer);
	wl_finish_current_song(instance->music_queue, 1.0 / 120.0);
	wl_reset_particle_list(instance->particle_list[0]);
	instance->game.regenerate = true;
	wl_block_cinema_skip();
	return 1;
}

int wl_menu_proc_game_over_give_up(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_destroy_cinema(instance->cinema);
	al_stop_timer(t3f_timer);
	instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].quit_fn, instance->graphics_scale);
	if(instance->cinema)
	{
		wl_reset_particle_list(instance->particle_list[0]);
		instance->menu[instance->current_menu]->hover_element = -1;
		wl_reset_particle_list(instance->particle_list[0]);
		wl_set_cinema_view_callbacks(instance->cinema, wl_game_paused_view_logic, wl_game_render);
		instance->state = WL_STATE_GAME_QUIT;
		wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
		wl_finish_current_song(instance->music_queue, 1.0 / 60.0);
	}
	else
	{
		wl_title_load_data(data, true);
		wl_select_menu(WL_MENU_MAIN, data);
		instance->state = WL_STATE_TITLE;
	}
	al_start_timer(t3f_timer);
	return 1;
}

T3F_GUI * wl_create_game_over_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
  T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, NULL, "", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_game_over_retry, "RETRY", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_game_over_give_up, "GIVE UP", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

  return menu;
}

int wl_menu_proc_demo_over_yes(void * data, int i, void * p)
{
	const char * purchase_link = WL_PURCHASE_LINK_STANDALONE;

  #ifdef T3F_ENABLE_STEAM_INTEGRATION
		WL_INSTANCE * instance = (WL_INSTANCE *)data;
		if(instance->steam_running)
		{
			purchase_link = WL_PURCHASE_LINK_STEAM;
		}
	#else
		#ifdef T3F_ANDROID_GP
			purchase_link = WL_PURCHASE_LINK_GOOGLE_PLAY;
		#endif
	#endif
	t3f_open_url(purchase_link);
	return 1;
}

int wl_menu_proc_demo_over_no(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	instance->ending_state = 1;
	t3f_clear_touch_state();
	return 1;
}

int wl_menu_proc_game_paused_2_continue(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->global_pause_state == WL_STATE_GAME_INTRO || instance->global_pause_state == WL_STATE_GAME_WIN || instance->global_pause_state == WL_STATE_MUSIC)
	{
		t3f_resume_music();
		wl_set_music_fade_speed(instance->music_queue, WL_MUSIC_FADE_SPEED);
	}
	wl_block_cinema_skip();
	wl_reset_particle_list(instance->particle_list[0]);
	instance->state = instance->global_pause_state;
	return 1;
}

int wl_menu_proc_game_paused_continue(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_destroy_cinema(instance->cinema);
		al_stop_timer(t3f_timer);
		instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].continue_fn, instance->graphics_scale);
		if(instance->cinema)
		{
			wl_set_cinema_view_callbacks(instance->cinema, wl_game_paused_view_logic, wl_game_render);
			instance->state = WL_STATE_GAME_CONTINUE;
		}
		else
		{
			wl_title_load_data(data, true);
			wl_select_menu(WL_MENU_MAIN, data);
			instance->state = WL_STATE_TITLE;
		}
		al_start_timer(t3f_timer);
		wl_reset_particle_list(instance->particle_list[0]);
		wl_block_cinema_skip();
	}
	else if(instance->game.mode == WL_GAME_MODE_MUSIC)
	{
		t3f_resume_music();
		wl_set_music_fade_speed(instance->music_queue, WL_MUSIC_FADE_SPEED);
		instance->state = WL_STATE_MUSIC;
	}
	else
	{
		instance->state = WL_STATE_GAME;
	}
	return 1;
}

int wl_menu_proc_game_paused_2_exit(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_reset_particle_list(instance->particle_list[0]);
	wl_finish_current_song(instance->music_queue, 1.0 / 60.0);
	instance->exit_tick = WL_FADE_TICKS;

	return 1;
}

int wl_menu_proc_game_paused_quit(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		wl_destroy_cinema(instance->cinema);
		al_stop_timer(t3f_timer);
		instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].quit_fn, instance->graphics_scale);
		if(instance->cinema)
		{
			wl_set_cinema_view_callbacks(instance->cinema, wl_game_paused_view_logic, wl_game_render);
			instance->state = WL_STATE_GAME_QUIT;
		}
		else
		{
			wl_title_load_data(data, true);
			wl_select_menu(WL_MENU_MAIN, data);
			instance->state = WL_STATE_TITLE;
		}
		al_start_timer(t3f_timer);
		wl_reset_particle_list(instance->particle_list[0]);
		wl_block_cinema_skip();
	}
	else
	{
		if(instance->overlay_pause_state == WL_STATE_MUSIC)
		{
			instance->state = WL_STATE_MUSIC_QUIT;
		}
		else
		{
			instance->state = WL_STATE_GAME_QUIT;
		}
	}
	wl_finish_current_song(instance->music_queue, 1.0 / 60.0);
	wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	return 1;
}

int wl_menu_proc_game_paused_view_tutorial(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema)
	{
		wl_destroy_cinema(instance->cinema);
		instance->cinema = NULL;
	}
	instance->want_tutorial = true;
	wl_game_initialize(data, WL_GAME_INIT_FLAG_NO_LEVEL_LOAD | WL_GAME_INIT_FLAG_NO_MUSIC);
	instance->state = WL_STATE_GAME;

	return 1;
}

int wl_menu_proc_game_retry(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->game.retry = true;
	instance->game.regenerate = true;
	wl_game_initialize(data, WL_GAME_INIT_FLAG_NO_BOARD_GENERATION | WL_GAME_INIT_FLAG_NO_LEVEL_LOAD | WL_GAME_INIT_FLAG_NO_MUSIC);
	instance->state = WL_STATE_GAME;
	return 1;
}

int wl_menu_proc_game_quit(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->state = WL_STATE_GAME_QUIT;
	wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 0.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), 60);
	wl_select_menu(WL_MENU_MAIN, data);
	return 1;
}

int wl_menu_proc_tutorial_skip_yes(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->profile->flags |= WL_PROFILE_FLAG_TUTORIAL_SKIP;
	instance->game.board[instance->game.player[0].y][instance->game.player[0].x].letter = ' ';
	instance->game.player[0].state = WL_PLAYER_STATE_EXIT;
	instance->tutorial_step = 99;
	instance->state = WL_STATE_GAME_TUTORIAL;
	return 1;
}

int wl_menu_proc_tutorial_skip_no(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->state = WL_STATE_GAME_TUTORIAL;
	return 1;
}

int wl_menu_proc_play_new_name(void * data, int i, void * p)
{
	return 1;
}

int wl_menu_proc_play_new_cancel(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_end_text_entry(instance->text_entry);
	wl_create_profile_menu(data);
	return 1;
}

void wl_create_name_entry_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int y;

	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
	}
	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(instance->menu[WL_MENU_DYNAMIC])
	{
		y = 0;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "ENTER NAME", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		y += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_play_new_name, instance->text_entry->entered_text, (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		y += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_play_new_ok, "OK", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		y += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_play_new_cancel, "CANCEL", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, y, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		#ifdef ALLEGRO_ANDROID
			t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, 240);
		#else
			t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);
		#endif
	}
}

T3F_GUI * wl_create_demo_over_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, NULL, "", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_demo_over_yes, "GET THE FULL GAME", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_demo_over_no, "GO TO TITLE SCREEN", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5 + 32, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

T3F_GUI * wl_create_game_paused_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return false;
	}
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_quit, "QUIT", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Quit the Game");
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_view_tutorial, "VIEW TUTORIAL", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "View the Tutorial");
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_continue, "CONTINUE", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 7, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Back to the Game In Progress");
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

T3F_GUI * wl_create_scene_paused_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return false;
	}
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_quit, "QUIT", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Quit the Game");
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_continue, "CONTINUE", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Back to the Game In Progress");
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

T3F_GUI * wl_create_tutorial_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, NULL, "SKIP TUTORIAL?", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_tutorial_skip_yes, "YES", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Skip to the Game");
	t3f_add_gui_text_element(menu, wl_menu_proc_tutorial_skip_no, "NO", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Continue Tutorial");
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

T3F_GUI * wl_create_game_exit_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_2_exit, "EXIT TO OS", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Exit the Game");
	t3f_add_gui_text_element(menu, wl_menu_proc_game_paused_2_continue, "KEEP PLAYING", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_describe_last_gui_element(menu, "Back to the Game In Progress");
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

T3F_GUI * wl_create_quit_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, NULL, "REALLY QUIT?", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
	t3f_add_gui_text_element(menu, wl_menu_proc_main_quit_yes, "YES", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_main_quit_no, "NO", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(menu, WL_TITLE_MENU_TOP, WL_TITLE_MENU_BOTTOM);

	return menu;
}

T3F_GUI * wl_create_retry_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, wl_menu_proc_game_retry, "PLAY AGAIN", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 5, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_game_quit, "QUIT", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 6, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

int wl_menu_proc_profile_unlock_yes(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char fnbuf[1024] = {0};

	instance->profile->flags |= WL_PROFILE_FLAG_UNLOCKED;
	wl_create_profile_menu(data);
	wl_save_profile(instance->profile, t3f_get_filename(t3f_data_path, "profile.dat", fnbuf, 1024));
	wl_update_unlocked_level_list(instance->level_list, instance->profile);
	wl_select_menu(WL_MENU_DYNAMIC, data);
	return 1;
}

int wl_menu_proc_profile_unlock_no(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char fnbuf[1024] = {0};

	instance->profile->flags &= ~WL_PROFILE_FLAG_UNLOCKED;
	wl_create_profile_menu(data);
	wl_save_profile(instance->profile, t3f_get_filename(t3f_data_path, "profile.dat", fnbuf, 1024));
	wl_update_unlocked_level_list(instance->level_list, instance->profile);
	wl_select_menu(WL_MENU_DYNAMIC, data);
	return 1;
}

T3F_GUI * wl_create_profile_unlock_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	T3F_GUI * menu;

	menu = t3f_create_gui(0, 0);
	if(!menu)
	{
		return NULL;
	}
	t3f_add_gui_text_element(menu, NULL, "UNLOCK CONTENT?", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 0, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY | T3F_GUI_ELEMENT_STATIC);
	t3f_add_gui_text_element(menu, wl_menu_proc_profile_unlock_yes, "YES", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 1, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(menu, wl_menu_proc_profile_unlock_no, "NO", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, 48 * 2, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(menu, 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return menu;
}

void wl_show_pause_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->cinema)
	{
		t3f_center_gui(instance->menu[WL_MENU_GAME_PAUSED], 240, WL_VIRTUAL_DISPLAY_HEIGHT);
	}
	else
	{
		t3f_center_gui(instance->menu[WL_MENU_GAME_PAUSED], 0, WL_VIRTUAL_DISPLAY_HEIGHT);
	}
	wl_select_menu(WL_MENU_GAME_PAUSED, data);
	instance->state = WL_STATE_GAME_PAUSED;
}
