#include "t3f/t3f.h"
#include "instance.h"
#include "menu_high_scores.h"
#include "menu_profile.h"

int wl_menu_proc_high_scores_previous_mode(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int last_item = instance->menu[instance->current_menu]->hover_element;

	instance->leaderboard_mode--;
	if(instance->leaderboard_mode < 0)
	{
		instance->leaderboard_mode = WL_GAME_MODE_STORY + instance->level_list->story_count - 1;
	}
	else if(instance->leaderboard_mode > 2 && instance->leaderboard_mode < WL_GAME_MODE_STORY)
	{
		instance->leaderboard_mode = 2;
	}
	wl_create_high_scores_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
	instance->menu[instance->current_menu]->hover_element = last_item;
	wl_reset_particle_list(instance->particle_list[0]);
	return 1;
}

int wl_menu_proc_high_scores_next_mode(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int last_item = instance->menu[instance->current_menu]->hover_element;

	instance->leaderboard_mode++;
	if(instance->leaderboard_mode > WL_GAME_MODE_STORY + instance->level_list->story_count - 1)
	{
		instance->leaderboard_mode = 0;
	}
	else if(instance->leaderboard_mode > 2 && instance->leaderboard_mode < WL_GAME_MODE_STORY)
	{
		instance->leaderboard_mode = WL_GAME_MODE_STORY;
	}
	wl_create_high_scores_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
	instance->menu[instance->current_menu]->hover_element = last_item;
	wl_reset_particle_list(instance->particle_list[0]);
	return 1;
}

int wl_menu_proc_high_scores_done(void * data, int i, void * p)
{
	wl_create_profile_menu(data);
	return 1;
}

bool wl_create_high_scores_menu(int mode, int type, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char * mode_text[] = {"CASUAL MODE", "PUZZLE MODE", "GURU MODE"};
	char mode_buf[256];
	int pos = 0;

	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
	}
	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(!instance->menu[WL_MENU_DYNAMIC])
	{
		return false;
	}

	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, instance->profile->name, (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	pos += 48;

	/* show mode selector */
	if(mode < WL_GAME_MODE_STORY)
	{
		sprintf(mode_buf, "%s", mode_text[mode]);
	}
	else
	{
		sprintf(mode_buf, "%s", instance->level_list->story[mode - WL_GAME_MODE_STORY].name);
	}
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, mode_buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, mode_buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_high_scores_previous_mode, "<", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_high_scores_next_mode, ">", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	pos += 48;

	/* show high score */
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "HIGH SCORE", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	pos += 32;
	sprintf(mode_buf, "%d", instance->profile->high_score[mode]);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, mode_buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	pos += 32;
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "BEST WORD", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	pos += 32;
	if(instance->profile->best_word[mode] >= 0)
	{
		sprintf(mode_buf, "%s", instance->dictionary->word[instance->profile->best_word[mode]].data);
	}
	else
	{
		sprintf(mode_buf, "N/A");
	}
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, mode_buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	pos += 40;
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_high_scores_done, "DONE", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);
	return true;
}
