#include "t3f/t3f.h"
#include "modules/leaderboards.h"
#include "instance.h"
#include "menu.h"
#include "menu_leaderboards.h"

static void _wl_truncate_string(ALLEGRO_USTR * sp, int length)
{
	int i;

	if(al_ustr_length(sp) > (unsigned int)length)
	{
		for(i = length - 2; i >= 0; i--)
		{
			if(al_ustr_get(sp, i) != ' ')
			{
				break;
			}
		}
		length = i + 1;
		al_ustr_truncate(sp, al_ustr_offset(sp, length));
		al_ustr_append_cstr(sp, "\\.");
	}
}

void _wl_unload_leaderboard(WL_INSTANCE * instance)
{
	if(instance->leaderboard && !t3f_object_loading(instance->leaderboard_loader))
	{
		t3net_destroy_leaderboard(instance->leaderboard);
		instance->leaderboard = NULL;
	}
}

/* leaderboard menu */
int wl_menu_proc_leaderboard_previous_mode(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int mode_max = 1;
	int last_item = instance->menu[instance->current_menu]->hover_element;

	if((instance->profile->flags & WL_PROFILE_FLAG_UNLOCKED) ||instance->profile->progress[0] >= WL_LEVEL_UP * (instance->level_list->story[0].end_level - instance->level_list->story[0].start_level + 1))
	{
		mode_max = 2;
	}
	instance->leaderboard_mode--;
	if(instance->leaderboard_mode < 0)
	{
		instance->leaderboard_mode = WL_GAME_MODE_STORY + instance->level_list->story_count - 1;
		while(!instance->level_list->unlocked_story[instance->leaderboard_mode - WL_GAME_MODE_STORY])
		{
			instance->leaderboard_mode--;
		}
	}
	else if(instance->leaderboard_mode > mode_max && instance->leaderboard_mode < WL_GAME_MODE_STORY)
	{
		instance->leaderboard_mode = mode_max;
	}
	_wl_unload_leaderboard(instance);
	wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
	instance->menu[instance->current_menu]->hover_element = last_item;
	wl_download_scores(data);
	wl_reset_particle_list(instance->particle_list[0]);
	return 1;
}

int wl_menu_proc_leaderboard_next_mode(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int mode_max = 1;
	int last_item = instance->menu[instance->current_menu]->hover_element;

	/* adjust 'mode_max' to include Guru mode if it is unlocked */
	if((instance->profile->flags & WL_PROFILE_FLAG_UNLOCKED) ||instance->profile->progress[0] >= WL_LEVEL_UP * (instance->level_list->story[0].end_level - instance->level_list->story[0].start_level + 1))
	{
		mode_max = 2;
	}
	instance->leaderboard_mode++;

	/* cycle through unlocked stories, go back to 0 when we reach the end */
	if(instance->leaderboard_mode >= WL_GAME_MODE_STORY)
	{
		while(instance->leaderboard_mode < WL_GAME_MODE_STORY + instance->level_list->story_count && !instance->level_list->unlocked_story[instance->leaderboard_mode - WL_GAME_MODE_STORY])
		{
			instance->leaderboard_mode++;
		}
		if(instance->leaderboard_mode >= WL_GAME_MODE_STORY + instance->level_list->story_count)
		{
			instance->leaderboard_mode = 0;
		}
	}

	/* skip to story modes if we have exceeded 'mode_max' */
	else if(instance->leaderboard_mode > mode_max && instance->leaderboard_mode < WL_GAME_MODE_STORY)
	{
		instance->leaderboard_mode = WL_GAME_MODE_STORY;
	}

	_wl_unload_leaderboard(instance);
	wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
	instance->menu[instance->current_menu]->hover_element = last_item;
	wl_download_scores(data);
	wl_reset_particle_list(instance->particle_list[0]);
	return 1;
}

int wl_menu_proc_leaderboard_toggle_type(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int last_item = instance->menu[instance->current_menu]->hover_element;

	instance->leaderboard_type--;
	if(instance->leaderboard_type < 0)
	{
		instance->leaderboard_type = 1;
	}
	_wl_unload_leaderboard(instance);
	wl_create_leaderboard_menu(instance->leaderboard_mode, instance->leaderboard_type, data);
	instance->menu[instance->current_menu]->hover_element = last_item;
	wl_download_scores(data);
	wl_reset_particle_list(instance->particle_list[0]);
	return 1;
}

int wl_menu_proc_leaderboards_done(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	if(instance->state == WL_STATE_LEADERBOARDS)
	{
		instance->state = WL_STATE_TITLE;
		wl_select_menu(WL_MENU_GAME, data);
	}
	else if(instance->game.mode >= WL_GAME_MODE_STORY)
	{
		if(instance->state == WL_STATE_GAME_OVER_WAIT)
		{
			instance->state = WL_STATE_GAME_OVER;
			wl_select_menu(WL_MENU_GAME_OVER, data);
		}
		else
		{
			wl_game_exit(data);
			wl_play_music(instance->music_queue, "data/music/theme.ogg");
		}
	}
	else
	{
		wl_select_menu(WL_MENU_RETRY, data);
	}
	return 1;
}

static void _wl_add_leaderboard_entries_to_menu(WL_INSTANCE * instance, int mode, int type, int * pos)
{
	ALLEGRO_USTR * part[3] = {NULL};
	char buf[256] = {0};
	int i, j;
	int spaces;

	for(i = 0; i < instance->leaderboard->entries; i++)
	{
		part[0] = al_ustr_new(instance->leaderboard->entry[i]->name);
		if(type == 1)
		{
			part[1] = al_ustr_new(instance->dictionary->word[atoi(instance->leaderboard->entry[i]->extra)].data);
		}
		else
		{
			sprintf(buf, "%d", wl_unobfuscate_score(instance->leaderboard->entry[i]->score));
			part[1] = al_ustr_new(buf);
		}
		if(type == 1)
		{
			/* ensure enough space to show at least one character of word */
			if(al_ustr_length(part[0]) > 21 - 1 - 2)
			{
				_wl_truncate_string(part[0], 21 - 1 - 2);
			}
			_wl_truncate_string(part[1], 21 - al_ustr_length(part[0]) - 1);
		}
		else
		{
			_wl_truncate_string(part[0], 21 - al_ustr_length(part[1]) - 1);
		}

		/* add spaces to even things out */
		spaces = 21 - (wl_menu_get_element_characters_u(part[0]) + wl_menu_get_element_characters_u(part[1]));
		part[2] = al_ustr_dup(part[0]);
		for(j = 0; j < spaces; j++)
		{
			al_ustr_append_cstr(part[2], " ");
		}
		al_ustr_append(part[2], part[1]);

		if(instance->leaderboard_style == 1 && i == instance->leaderboard_spot)
		{
			t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, (char *)al_cstr(part[2]), (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, *pos, al_map_rgba_f(1.0, 1.0, 1.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		}
		else
		{
			t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, (char *)al_cstr(part[2]), (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, *pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		}
		sprintf(buf, "%s - %d", instance->leaderboard->entry[i]->name, wl_unobfuscate_score(instance->leaderboard->entry[i]->score));

		/* append unused tile count for puzzle mode */
		if(mode == WL_GAME_MODE_PUZZLE && type == 0 && strlen(instance->leaderboard->entry[i]->extra) > 0)
		{
			strcat(buf, " (");
			strcat(buf, instance->leaderboard->entry[i]->extra);
			strcat(buf, " Unused)");
		}
		else if(type == 1)
		{
			strcat(buf, " (");
			strcat(buf, instance->dictionary->word[atoi(instance->leaderboard->entry[i]->extra)].data);
			strcat(buf, ")");
		}
		t3f_describe_last_gui_element(instance->menu[WL_MENU_DYNAMIC], buf);
		for(j = 0; j < 3; j++)
		{
			al_ustr_free(part[j]);
		}
		*pos += 32;
	}
	for(i = instance->leaderboard->entries; i < 10; i++)
	{
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "...", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, *pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		*pos += 32;
	}
}

static void _wl_add_leaderboard_placeholder_entries_to_menu(WL_INSTANCE * instance, int * pos)
{
	int i;

	for(i = 0; i < 10; i++)
	{
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "...", (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, *pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		*pos += 32;
	}
}

bool wl_create_leaderboard_menu(int mode, int type, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char * mode_text[] = {"CASUAL MODE", "PUZZLE MODE", "GURU MODE"};
	char * type_text[] = {"HIGH SCORE", "BEST WORDS"};
	char buf[256];
	int pos = 0;

	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
	}
	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(!instance->menu[WL_MENU_DYNAMIC])
	{
		return false;
	}
	if(mode < WL_GAME_MODE_STORY)
	{
		sprintf(buf, "%s", mode_text[mode]);
	}
	else
	{
		sprintf(buf, "%s", instance->level_list->story[mode - WL_GAME_MODE_STORY].name);
	}
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	if(instance->leaderboard_style == 0)
	{
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_leaderboard_previous_mode, "<", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_leaderboard_next_mode, ">", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	}
	pos += 32;
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, type_text[type], (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_leaderboard_toggle_type, "<", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) - 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_leaderboard_toggle_type, ">", (void **)&instance->font[WL_FONT_16], (WL_VIRTUAL_DISPLAY_WIDTH / 2) + 160, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
	pos += 48;
	if(!instance->leaderboard || t3f_object_loading(instance->leaderboard_loader))
	{
		_wl_add_leaderboard_placeholder_entries_to_menu(instance, &pos);
		strcpy(buf, "CANCEL");
	}
	else
	{
		_wl_add_leaderboard_entries_to_menu(instance, mode, type, &pos);
		strcpy(buf, "DONE");
	}
	pos += 16;
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_leaderboards_done, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);
	return true;
}

int wl_menu_proc_leaderboards_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME, data);
	return 1;
}
