#include "t3f/t3f.h"
#include "instance.h"
#include "menu_common.h"
#include "menu_profile.h"
#include "menu_dialog.h"
#include "menu_high_scores.h"

int wl_menu_proc_profiles_view(void * data, int i, void * p)
{
	wl_create_profile_menu(data);
	wl_select_menu(WL_MENU_DYNAMIC, data);
	return 1;
}

int wl_menu_proc_profile_rename(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_begin_text_entry(instance->text_entry, instance->profile->name);
	wl_create_name_entry_menu(data);
	wl_select_menu(WL_MENU_DYNAMIC, data);
	return 1;
}

int wl_menu_proc_profiles_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME, data);
	return 1;
}

static bool wl_used(int profile, int word, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	for(i = 0; i < word; i++)
	{
		if(instance->profile->word[i] == instance->profile->word[word])
		{
			return false;
		}
	}
	return true;
}

static int wl_profile_count_unique_words(int profile, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	unsigned int i;
	int count = 0;

	for(i = 0; i < instance->profile->words; i++)
	{
		if(wl_used(profile, i, data))
		{
			count++;
		}
	}
	return count;
}

static int _wl_menu_proc_import_profile(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	memcpy(instance->profile, &(instance->import_profile[i - 1]), sizeof(WL_PROFILE));
	instance->profile->flags = 0; // we won't honor the old flags
	free(instance->import_profile);
	instance->import_profile = NULL;

	wl_select_menu(WL_MENU_MAIN, data);

	return 1;
}

static int _wl_menu_proc_import_profile_cancel(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_select_menu(WL_MENU_MAIN, data);
	free(instance->import_profile);
	instance->import_profile = NULL;

	return 1;
}

bool wl_create_import_profile_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int pos = 0;
	int i;

	instance->import_profile = wl_load_legacy_profiles(instance->import_profile_fn);
	if(!instance->import_profile)
	{
		goto fail;
	}

	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(!instance->menu[WL_MENU_DYNAMIC])
	{
		goto fail;
	}
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "IMPORT PROFILE", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
	pos += 48;
	for(i = 0; instance->import_profile[i].word; i++)
	{
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], _wl_menu_proc_import_profile, instance->import_profile[i].name, (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 48;
	}
	t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], _wl_menu_proc_import_profile_cancel, "CANCEL", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
	t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);

	return true;

	fail:
	{
		if(instance->menu[WL_MENU_DYNAMIC])
		{
			t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
			instance->menu[WL_MENU_DYNAMIC] = NULL;
		}
		if(instance->import_profile)
		{
			free(instance->import_profile);
			instance->import_profile = NULL;
		}
		return false;
	}
}

int wl_menu_proc_profile_high_scores(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	instance->leaderboard_mode = 0;
	instance->leaderboard_type = 0;
	instance->leaderboard_style = 0;
	wl_create_high_scores_menu(0, 0, data);
	wl_select_menu(WL_MENU_DYNAMIC, data);

	return 1;
}

int wl_menu_proc_profile_stats_back(void * data, int i, void * p)
{
	wl_create_profile_menu(data);
	return 1;
}

int wl_menu_proc_profile_stats(void * data, int i, void * p)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char buf[256] = {0};
	char buf2[256] = {0};
	int pos = 0;
	int h, m, s;

	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
	}
	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, "STATS", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC);
		pos += 48;

		/* played games */
		sprintf(buf, "GAMES PLAYED   %5d", instance->profile->games);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* play time */
		h = instance->profile->time / 216000;
		m = (instance->profile->time / 3600) % 60;
		s = (instance->profile->time / 60) % 60;
		if(instance->profile->time / 216000 > 0)
		{
			sprintf(buf2, "%3d:%02d:%02d", h, m, s);
		}
		else
		{
			sprintf(buf2, "    %02d:%02d", m, s);
		}
		sprintf(buf, "PLAY TIME  %s", buf2);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total words */
		sprintf(buf, "WORDS         %6lu", instance->profile->words);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* unique words */
		sprintf(buf, "UNIQUE WORDS  %6d", wl_profile_count_unique_words(instance->selected_profile, data));
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total moves */
		sprintf(buf, "MOVES         %6lu", instance->profile->moves);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* total undos */
		sprintf(buf, "UNDOS         %6lu", instance->profile->undos);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 32;

		/* wrong guesses */
		sprintf(buf, "INVALID WORDS %6lu", instance->profile->wrongs);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_nop, buf, (void **)&instance->font[WL_FONT_16], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_COPY);
		pos += 40;

		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_stats_back, "BACK", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);

		wl_select_menu(WL_MENU_DYNAMIC, data);
	}
	return 1;
}

int wl_menu_proc_profile_unlock(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_PROFILE_UNLOCK, data);
	return 1;
}

int wl_menu_proc_profile_back(void * data, int i, void * p)
{
	wl_select_menu(WL_MENU_GAME, data);
	return 1;
}

void wl_create_profile_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char buf[256] = {0};
	int pos = 0;

	if(instance->menu[WL_MENU_DYNAMIC])
	{
		t3f_destroy_gui(instance->menu[WL_MENU_DYNAMIC]);
	}
	instance->menu[WL_MENU_DYNAMIC] = t3f_create_gui(0, 0);
	if(instance->menu[WL_MENU_DYNAMIC])
	{
		sprintf(buf, "%s", instance->profile->name);
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], NULL, buf, (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE | T3F_GUI_ELEMENT_STATIC | T3F_GUI_ELEMENT_COPY);
		pos += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_high_scores, "HIGH SCORES", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(instance->menu[WL_MENU_DYNAMIC], "See Your Personal Bests");
		pos += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_stats, "STATS", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(instance->menu[WL_MENU_DYNAMIC], "Various Game Stats");
		pos += 48;
		if(!instance->steam_running)
		{
			t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_rename, "RENAME", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
			t3f_describe_last_gui_element(instance->menu[WL_MENU_DYNAMIC], "Set Profile Name");
			pos += 48;
		}
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_unlock, "UNLOCK CONTENT", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_describe_last_gui_element(instance->menu[WL_MENU_DYNAMIC], "Unlock All Levels and Game Modes");
		pos += 48;
		t3f_add_gui_text_element(instance->menu[WL_MENU_DYNAMIC], wl_menu_proc_profile_back, "BACK", (void **)&instance->font[WL_FONT_24], WL_VIRTUAL_DISPLAY_WIDTH / 2, pos, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), T3F_GUI_ELEMENT_CENTRE);
		t3f_center_gui(instance->menu[WL_MENU_DYNAMIC], 0, WL_VIRTUAL_DISPLAY_HEIGHT);
	}
}
