#include "t3f/gui.h"
#include "t3f/sound.h"
#include "t3f/music.h"
#include "t3f/resource.h"
#include "t3f/rng.h"
#include "t3f/draw.h"
#include "instance.h"
#include "modules/data.h"
#include "modules/text_entry.h"
#include "modules/effects.h"
#include "modules/text.h"
#include "modules/leaderboards.h"
#include "gameplay/game_intro.h"
#include "menu.h"
#include "menu_main.h"
#include "menu_game.h"
#include "menu_dialog.h"
#include "menu_overlay.h"

static char * wl_title_text = "wordleap";

void wl_generate_title_word(WL_TITLE_WORD * wp, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	while(1)
	{
		wp->word = rand() % instance->dictionary->words;
		if(!instance->dictionary->word[wp->word].censored)
		{
			break;
		}
	}
	wp->x = t3f_drand(&instance->rng_state) * (WL_VIRTUAL_DISPLAY_WIDTH + 160.0) - 160.0;
	wp->vx = t3f_drand(&instance->rng_state) * 2.0 - 1.0;
	wp->y = t3f_drand(&instance->rng_state) * (WL_VIRTUAL_DISPLAY_HEIGHT + 160.0) - 160.0;
	wp->vy = t3f_drand(&instance->rng_state) * 2.0 - 1.0;
	wp->a = 0.0;
	wp->va = 0.01;
	wp->time = rand() % 180 + 60;
}

void wl_title_tile_logic(WL_TITLE_TILE * tp, bool noy, bool noz, bool finish)
{
	if(!noy)
	{
		tp->y += tp->vy;
		tp->vy += 0.125;
		if(tp->y >= WL_TITLE_LOGO_Y)
		{
			tp->y = WL_TITLE_LOGO_Y;
			tp->vy = -1.25;
		}
	}
	if(!noz)
	{
		tp->z += tp->vz;
		if(tp->gravity)
		{
			tp->vz += 0.5;
		}
		if(tp->z >= 0)
		{
			tp->z = 0;
			tp->vz = -5;
			if(finish)
			{
				tp->vz = -15;
				tp->gravity = false;
			}
		}
		else if(tp->vz > 20.0)
		{
			tp->vz = 20.0;
		}
	}
}

/* qsort function for depth-sorting sprites by y-coordinate */
static int _wl_title_tile_sorter(const void * item_1, const void * item_2)
{
	WL_TITLE_TILE * sp1 = *(WL_TITLE_TILE **)(item_1);
	WL_TITLE_TILE * sp2 = *(WL_TITLE_TILE **)(item_2);

	if(sp2->z < sp1->z)
	{
		return -1;
	}
	else if(sp2->z > sp1->z)
	{
		return 1;
	}
	return 0;
}

/* sort the sprite list using the above helper function */
static void _wl_zsort_title_tiles(WL_INSTANCE * instance)
{
   qsort(instance->title_tile_zsort, 8, sizeof(WL_TITLE_TILE *), _wl_title_tile_sorter);
}

void _wl_animate_title_logo(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		wl_title_tile_logic(&instance->title_tile[i], true, false, instance->state == WL_STATE_TITLE_EXIT);
	}
	_wl_zsort_title_tiles(instance);
}

void _wl_draw_title_bg(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	t3f_draw_bitmap(instance->bitmap[WL_BITMAP_TITLE_BG], t3f_color_white, WL_VIRTUAL_DISPLAY_WIDTH / 2 - instance->bitmap[WL_BITMAP_TITLE_BG]->target_width / 2, WL_VIRTUAL_DISPLAY_HEIGHT / 2 - instance->bitmap[WL_BITMAP_TITLE_BG]->target_height / 2, 0, 0);

	al_hold_bitmap_drawing(true);
	wl_title_word_render(instance->font[WL_FONT_24], 0.125, instance);
	al_hold_bitmap_drawing(false);

	t3f_draw_bitmap(instance->bitmap[WL_BITMAP_TITLE_FG], t3f_color_white, WL_VIRTUAL_DISPLAY_WIDTH / 2 - instance->bitmap[WL_BITMAP_TITLE_FG]->target_width / 2, WL_VIRTUAL_DISPLAY_HEIGHT / 2 - instance->bitmap[WL_BITMAP_TITLE_FG]->target_height / 2, 0, 0);
}

static void _wl_draw_title_logo(WL_INSTANCE * instance)
{
	int i, l;
	float a;

		l = strlen(wl_title_text);
	al_hold_bitmap_drawing(true);
	for(i = 0; i < l; i++)
	{
		if(instance->title_tile_zsort[i]->z < -200.0)
		{
			a = 0.0;
		}
		else if(instance->title_tile_zsort[i]->z < -30.0)
		{
			a = 0.25 * (200.0 + instance->title_tile_zsort[i]->z) / 170.0;
		}
		else
		{
			a = 0.25;
		}
		t3f_draw_bitmap(instance->title_tile_zsort[i]->bp, al_map_rgba_f(0.0, 0.0, 0.0, a), instance->title_tile_zsort[i]->x, instance->title_tile_zsort[i]->y, 0.0, 0);
	}
	for(i = 0; i < l; i++)
	{
		t3f_draw_bitmap(instance->title_tile_zsort[i]->bp, t3f_color_white, instance->title_tile_zsort[i]->x, instance->title_tile_zsort[i]->y, instance->title_tile_zsort[i]->z, 0);
	}
}

/* helper for moving between menus */
void wl_select_menu(int menu, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int current_menu = instance->current_menu;

	if(menu < 0)
	{
		instance->current_menu = instance->last_menu;
		instance->last_menu = current_menu;
	}
	else
	{
		if(menu != instance->current_menu)
		{
			instance->last_menu = instance->current_menu;
			instance->current_menu = menu;
		}
	}
	instance->menu[instance->current_menu]->hover_element = -1;
	wl_reset_particle_list(instance->particle_list[0]);
}

bool wl_title_load_data(void * data, bool pause_timer)
{
 	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	char fn[1024]= {0};
	int i;

	if(pause_timer)
	{
		al_stop_timer(t3f_timer);
	}
	sprintf(fn, "data/graphics/core/title_bg.ini#Size %1.1f", instance->graphics_scale);
	instance->bitmap[WL_BITMAP_TITLE_BG] = t3f_load_bitmap(fn, 0, false);
	if(!instance->bitmap[WL_BITMAP_TITLE_BG])
	{
		goto fail;
	}
	sprintf(fn, "data/graphics/core/title_fg.ini#Size %1.1f", instance->graphics_scale);
	instance->bitmap[WL_BITMAP_TITLE_FG] = t3f_load_bitmap(fn, 0, false);
	if(!instance->bitmap[WL_BITMAP_TITLE_FG])
	{
		goto fail;
	}
	/* create title tiles */
	for(i = 0; i < strlen(wl_title_text); i++)
	{
		sprintf(fn, "data/graphics/core/logo_%c.ini#Size %1.1f", wl_title_text[i], instance->graphics_scale);
		instance->title_tile[i].bp = t3f_load_bitmap(fn, T3F_BITMAP_FLAG_PADDED, false);
		if(!instance->title_tile[i].bp)
		{
			goto fail;
		}
	}

	/* create atlas */
	instance->title_atlas = t3f_create_atlas(512 * instance->graphics_scale, 512 * instance->graphics_scale);
	if(!instance->title_atlas)
	{
		goto fail;
	}
	for(i = 0; i < strlen(wl_title_text); i++)
	{
		t3f_add_bitmap_to_atlas(instance->title_atlas, &instance->title_tile[i].bp->bitmap, T3F_ATLAS_SPRITE);
	}
	if(pause_timer)
	{
		al_start_timer(t3f_timer);
	}
	return true;

	fail:
	{
		wl_title_free_data(data);
		if(pause_timer)
		{
			al_start_timer(t3f_timer);
		}
		return false;
	}
}

void wl_title_free_data(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	for(i = 0; i < strlen(wl_title_text); i++)
	{
		if(instance->title_tile[i].bp)
		{
			t3f_destroy_bitmap(instance->title_tile[i].bp);
			instance->title_tile[i].bp = NULL;
		}
	}
	if(instance->title_atlas)
	{
		t3f_destroy_atlas(instance->title_atlas);
		instance->title_atlas = NULL;
	}
	if(instance->bitmap[WL_BITMAP_TITLE_BG])
	{
		t3f_destroy_bitmap(instance->bitmap[WL_BITMAP_TITLE_BG]);
		instance->bitmap[WL_BITMAP_TITLE_BG] = NULL;
	}
	if(instance->bitmap[WL_BITMAP_TITLE_FG])
	{
		t3f_destroy_bitmap(instance->bitmap[WL_BITMAP_TITLE_FG]);
		instance->bitmap[WL_BITMAP_TITLE_FG] = NULL;
	}
}

bool wl_title_init(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	wl_set_menu_driver(data);
	instance->menu_ustr = al_ustr_new_from_buffer(instance->menu_ustr_buffer, 256);

	instance->menu[WL_MENU_EMPTY] = t3f_create_gui(0, 0);
	if(!instance->menu[WL_MENU_EMPTY])
	{
		return false;
	}
	instance->menu[WL_MENU_MAIN] = wl_create_main_menu(data);
	if(!instance->menu[WL_MENU_MAIN])
	{
		return false;
	}
	instance->menu[WL_MENU_GAME] = wl_create_game_menu(data);
	if(!instance->menu[WL_MENU_GAME])
	{
		return false;
	}
	instance->menu[WL_MENU_GAME_OVER] = wl_create_game_over_menu(data);
	if(!instance->menu[WL_MENU_GAME_OVER])
	{
		return false;
	}
	instance->menu[WL_MENU_DEMO_OVER] = wl_create_demo_over_menu(data);
	if(!instance->menu[WL_MENU_DEMO_OVER])
	{
		return false;
	}
	instance->menu[WL_MENU_GAME_PAUSED] = wl_create_game_paused_menu(data);
	if(!instance->menu[WL_MENU_GAME_PAUSED])
	{
		return false;
	}
	instance->menu[WL_MENU_SCENE_PAUSED] = wl_create_scene_paused_menu(data);
	if(!instance->menu[WL_MENU_GAME_PAUSED])
	{
		return false;
	}
	instance->menu[WL_MENU_TUTORIAL] = wl_create_tutorial_menu(data);
	if(!instance->menu[WL_MENU_TUTORIAL])
	{
		return false;
	}
	instance->menu[WL_MENU_GAME_EXIT] = wl_create_game_exit_menu(data);
	if(!instance->menu[WL_MENU_GAME_EXIT])
	{
		return false;
	}
	instance->menu[WL_MENU_QUIT] = wl_create_quit_menu(data);
	if(!instance->menu[WL_MENU_QUIT])
	{
		return false;
	}
	instance->menu[WL_MENU_RETRY] = wl_create_retry_menu(data);
	if(!instance->menu[WL_MENU_RETRY])
	{
		return false;
	}
	instance->menu[WL_MENU_PROFILE_UNLOCK] = wl_create_profile_unlock_menu(data);
	if(!instance->menu[WL_MENU_PROFILE_UNLOCK])
	{
		return false;
	}
	instance->menu[WL_MENU_GAME_OVERLAY] = wl_create_game_overlay_menu(data);
	if(!instance->menu[WL_MENU_GAME_OVERLAY])
	{
		return false;
	}
	instance->menu[WL_MENU_MUSIC_OVERLAY] = wl_create_music_overlay_menu(data);
	if(!instance->menu[WL_MENU_MUSIC_OVERLAY])
	{
		return false;
	}

	/* create title words */
	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		wl_generate_title_word(&instance->title_word[i], data);
	}

	/* create title tiles */
	for(i = 0; i < strlen(wl_title_text); i++)
	{
		instance->title_tile[i].x = WL_VIRTUAL_DISPLAY_WIDTH / 2 - (48 * 4) + i * 48;
		instance->title_tile[i].y = WL_TITLE_LOGO_Y;
		instance->title_tile[i].vz = 20.0;
		instance->title_tile[i].z = -WL_VIRTUAL_DISPLAY_WIDTH / 2.0 - 20.0 * (float)i * 10.5;
		instance->title_tile[i].vy = -1.25;
		instance->title_tile[i].gravity = true;

		/* move every other tile to the top of it's "jump" */
		if(i % 2 == 0)
		{
			while(instance->title_tile[i].vy < 0.0)
			{
				wl_title_tile_logic(&instance->title_tile[i], false, true, false);
			}
		}
		instance->title_tile_zsort[i] = &instance->title_tile[i];
	}

	return true;
}

void wl_title_exit(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	for(i = 0; i < WL_MAX_MENUS; i++)
	{
		if(instance->menu[i])
		{
			t3f_destroy_gui(instance->menu[i]);
		}
	}
}

void wl_title_word_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	unsigned int i;

	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		instance->title_word[i].x += instance->title_word[i].vx;
		instance->title_word[i].y += instance->title_word[i].vy;
		if(instance->title_word[i].time > 0)
		{
			instance->title_word[i].a += instance->title_word[i].va;
			if(instance->title_word[i].a > 1.0)
			{
				instance->title_word[i].a = 1.0;
			}
		}
		else
		{
			instance->title_word[i].a -= instance->title_word[i].va;
			if(instance->title_word[i].a <= 0.0)
			{
				wl_generate_title_word(&instance->title_word[i], data);
			}
		}
		if(instance->title_word[i].time > 0)
		{
			instance->title_word[i].time--;
		}
	}
}

void wl_title_process_menu(int menu, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	bool sound = false;

	instance->last_item = instance->menu[menu]->hover_element;
	if(instance->input_type == WL_INPUT_TYPE_MOUSE)
	{
		t3f_select_hover_gui_element(instance->menu[menu], t3f_get_mouse_x(), t3f_get_mouse_y());
	}
	else if(instance->input_type == WL_INPUT_TYPE_TOUCH)
	{
		t3f_select_hover_gui_element(instance->menu[menu], -1, -1);
	}
	if(instance->input_state.up.pressed || instance->input_state.left.pressed)
	{
		t3f_select_previous_gui_element(instance->menu[menu]);
		if(instance->menu[menu]->hover_element != instance->last_item)
		{
			t3f_play_sample(instance->sample[WL_SAMPLE_MENU_MOVE], 0.75, 0.0, 1.0);
		}
	}
	if(instance->input_state.down.pressed || instance->input_state.right.pressed)
	{
		t3f_select_next_gui_element(instance->menu[menu]);
		if(instance->menu[menu]->hover_element != instance->last_item)
		{
			t3f_play_sample(instance->sample[WL_SAMPLE_MENU_MOVE], 0.75, 0.0, 1.0);
		}
	}
	if(instance->input_state.action.pressed)
	{
		if(instance->input_type == WL_INPUT_TYPE_TOUCH)
		{
			t3f_select_hover_gui_element(instance->menu[menu], t3f_get_touch_x(1), t3f_get_touch_y(1));
		}
		if(instance->menu[menu]->hover_element >= 0)
		{
			sound = true;
		}
		t3f_activate_selected_gui_element(instance->menu[menu], data);
		if(sound)
		{
			t3f_play_sample(instance->sample[WL_SAMPLE_MENU_TAP], 1.0, 0.0, 1.0);
		}
		t3f_clear_touch_state();
	}
	if(instance->input_state.cancel.pressed)
	{
		instance->menu[menu]->hover_element = -1;
		t3f_select_previous_gui_element(instance->menu[menu]);
		t3f_activate_selected_gui_element(instance->menu[menu], data);
		instance->menu[menu]->hover_element = -1;
	}
}

void wl_title_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_enter_text(instance->text_entry, data);
	wl_title_process_menu(instance->current_menu, data);

	if(!(instance->current_menu == WL_MENU_OPTIONS && instance->option[WL_OPTION_FIRST_RUN]))
	{
		_wl_animate_title_logo(instance);
	}
	wl_title_word_logic(data);
}

void wl_title_word_render(T3F_FONT * fp, float strength, void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int i;

	for(i = 0; i < WL_MAX_TITLE_WORDS; i++)
	{
		t3f_draw_text(fp, wl_alpha_color(t3f_color_black, instance->title_word[i].a * strength), instance->title_word[i].x, instance->title_word[i].y, 0, 0, instance->dictionary->word[instance->title_word[i].word].data);
	}
}

void wl_title_render_menu(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	int h;

	t3f_render_gui(instance->menu[instance->current_menu], T3F_GUI_USER_FLAG);
	t3f_render_gui(instance->menu[instance->current_menu], (instance->input_type == WL_INPUT_TYPE_TOUCH) ? T3F_GUI_NO_MOUSE : 0);
	wl_render_particle_list(instance->particle_list[0], WL_PARTICLE_TYPE_FLASH);
	h = instance->menu[instance->current_menu]->hover_element;
	if(h >= 0 && instance->menu[instance->current_menu]->element[h].description)
	{
		wl_draw_text(instance->font[WL_FONT_24_OUTLINE], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.25), WL_VIRTUAL_DISPLAY_WIDTH / 2 + 3, WL_VIRTUAL_DISPLAY_HEIGHT - 32, 8, 8, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_SHADOW, instance->menu[instance->current_menu]->element[h].description);
	}
}

void wl_title_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	unsigned int i;
	float min_y = -1.0, max_y = -1.0;

	/* clear entire screen while on options menu in case there are
	 * artifacts from video mode switching */
	if(instance->current_menu == WL_MENU_OPTIONS)
	{
		al_set_clipping_rectangle(0, 0, al_get_display_width(t3f_display), al_get_display_height(t3f_display));
		al_clear_to_color(al_map_rgb_f(0.0, 0.0, 0.0));
		t3f_select_view(t3f_default_view);
	}

	_wl_draw_title_bg(instance);
	if(instance->current_menu == WL_MENU_MAIN || instance->current_menu == WL_MENU_GAME || instance->current_menu == WL_MENU_QUIT)
	{
		for(i = 0; i < strlen(wl_title_text); i++)
		{
			if(min_y < 0.0 || instance->title_tile[i].y < min_y)
			{
				min_y = instance->title_tile[i].y;
			}
			if(max_y < 0.0 || instance->title_tile[i].y > max_y)
			{
				max_y = instance->title_tile[i].y;
			}
		}
		_wl_draw_title_logo(instance);
		wl_draw_text(instance->font[WL_FONT_16_OUTLINE], al_map_rgba_f(1.0, 1.0, 1.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.25), WL_VIRTUAL_DISPLAY_WIDTH / 2 + 3, WL_TITLE_COPYRIGHT_Y, 8, 8, WL_TEXT_FLAG_CENTER | WL_TEXT_FLAG_SHADOW, T3F_APP_COPYRIGHT);
	}
	wl_title_render_menu(data);
	al_hold_bitmap_drawing(false);
}

void wl_title_exit_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_process_particle_list(instance->particle_list[0]);

	_wl_animate_title_logo(instance);
	wl_title_word_logic(data);
	if(instance->effects_data->fade_tick <= 0)
	{
		t3f_exit();
	}
}

void wl_title_exit_render(void * data)
{
	wl_title_render(data);
}

void wl_title_to_game_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	wl_process_particle_list(instance->particle_list[0]);

	_wl_animate_title_logo(instance);
	wl_title_word_logic(data);
	if(instance->effects_data->fade_tick <= 0)
	{
		wl_title_free_data(data);
		if(instance->game.mode == WL_GAME_MODE_MUSIC)
		{
			wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
			wl_reset_particle_list(instance->particle_list[0]);
			instance->game.retry = false;
			instance->game.regenerate = false;
			wl_game_initialize(data, 0);
		}
		else
		{
			if(instance->game.player[0].start_level >= 0)
			{
				wl_fade_to(instance->effects_data, al_map_rgba_f(0.0, 0.0, 0.0, 1.0), al_map_rgba_f(0.0, 0.0, 0.0, 0.0), 60);
				wl_reset_particle_list(instance->particle_list[0]);
				instance->game.retry = false;
				instance->game.regenerate = false;
				wl_game_initialize(data, 0);
				if(instance->game.mode >= WL_GAME_MODE_STORY)
				{
					if(!wl_player_needs_tutorial(data))
					{
						wl_game_scroll_level_name(data);
					}
				}
			}
			else
			{
				instance->game.player[0].start_level = instance->level_list->story[instance->game.player[0].story].start_level;
				wl_stop_fade(instance->effects_data);
				wl_reset_particle_list(instance->particle_list[0]);
				instance->cinema = wl_load_cinema_script(instance->level_list->story[instance->game.player[0].story].intro_fn, instance->graphics_scale);
				if(instance->cinema)
				{
					wl_set_cinema_view_callbacks(instance->cinema, wl_game_view_logic, wl_game_view_render);
					instance->game.retry = false;
					instance->game.regenerate = false;
					wl_game_initialize(data, WL_GAME_INIT_FLAG_NO_MUSIC);
					instance->state = WL_STATE_GAME_INTRO;
				}
				else
				{
					wl_game_initialize(data, 0);
					instance->state = WL_STATE_GAME;
				}
			}
		}
	}
}

void wl_title_to_game_render(void * data)
{
	wl_title_render(data);
}

void wl_title_credits_logic(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;
	unsigned int i;

	_wl_animate_title_logo(instance);
	wl_title_word_logic(data);
	wl_credits_logic(&instance->credits);
	if(instance->input_state.action.pressed)
	{
		instance->credits.fast = true;
	}
	if(instance->input_state.action.released)
	{
		instance->credits.fast = false;
	}
	if(instance->credits.fast)
	{
		for(i = 0; i < 15; i++)
		{
			wl_credits_logic(&instance->credits);
		}
	}
	if(instance->credits.done || instance->input_state.pause.pressed)
	{
		t3f_clear_touch_state();
		wl_select_menu(WL_MENU_MAIN, data);
		instance->state = WL_STATE_TITLE;
	}
}

void wl_title_credits_render(void * data)
{
	WL_INSTANCE * instance = (WL_INSTANCE *)data;

	_wl_draw_title_bg(instance);
	wl_credits_render(&instance->credits, data);
}
